<?php
/**
 * Syntax CMS User Config
 *
 * These are constants that will probably change for each instance. Only on
 * special occasions will you want to go into static.conf.php.
 */


// The NAME of the Site.  This is used in page titles.
define('SITE_NAME', 'Syntax Site');

// the tag line for the site, mainly for atom/rss feeds
define('SITE_TAGLINE', 'A Syntax Site');

define('ROWS_PER_PAGE', 10);
/*
* 11/3/2004 OAM: Auth configurations has been moved to 
* SITE_CFG_PATH . /auth.conf.php
*/


/*********************************************************************
* CUSTOM Classes
*
* If you have classes that you need throughout your site or that
* extend or override Syntax CMS behavior, put them in private/user
* and include them here, following the example for StxCustomUtilities:
**********************************************************************/
include_once(SITE_USR_PATH . '/CustomUtilities.php');


/* setup the browse functionality common to all pages
* A sample browseBy specification is below. The key of the browseBy array is the
* pick table to use to get options. Note that you can ommit all the parameters and
* the function defaults to using the name field for name, value and weight for sorting.
* $browseby = array(
*    'topics' => array (
*        'nameField' => 'name',
*        'labelField'=> 'name',
*        'order'     => 'weight',
*        'listName'  => 'technology'
*  ),
*    'issues' => array (),
*  ),
*);
*/
$browseby = array(
    'topics' => array('listName' => 'topic'),
    'issues' => array('listName' => 'issue')
);
$browseArray = F1CMS::buildBrowseArrays($browseby);

/**
* FACETS replace the browseby arrays and are used more consistently
* to enforce facetted classification via the general module (in the
* list and browse capabilities.  This array holds some basic information
* about the typesfields used as global facets.  Hence you should make
* sure that you only use fields that are named and defined the same
* way across content types. 
*
* The key to the array is a identifier for documentation only.  Each element
* is an hash that has to have an element named 'field' which indicates 
* the name of the typesfield to use.  You can also specifiy an element named 
* label that is used to output the Facet's name (in select menus, or detail
* pages for example.
*
* EX:
* $facets = array (
*       'topic' => array('field' => 'topic', 'label'=>'Site Topics'),
*       'state' => array('field' => 'state', 'label'=>'US State'),
* );
*/
$facets = array(
    'Topic' => array('field' => 'topic'),
    'Issue' => array('field' => 'issue'),
);
StxPrefs::setFacets($facets);

/*
* Seup the Modules used by this site.
* -----------------------------------
* 
* Modules may be applications or content datatypes or both.  In some cases you may want to allow
* people to browse modules directly (the ModuleRequest class is responsible for translating REQUEST_URI into
* the correct module request) and in others you may want to only access the modules from within other scripts.
* 
* There's nothing to prevent modules from overlapping in terms of functionality -- or the datatypes they use,
* etc.  BUT there's also nothing that allows different modules to share library files (unless you put them
* in the SITE_LIB_PATH directory).    
*/

// The Generic Module
Modules::addModule('general', 'Site Content');
Modules::addCapability('general', 'list');
Modules::addCapability('general', 'detail');
Modules::addCapability('general', 'archive');
Modules::addCapability('general', 'browse');
Modules::addCapability('general', 'search');
Modules::addCapability('general', 'send_file');
Modules::setAttribute('general', 'cache', false);
Modules::regGenericModule('general');    

// =============================================================================
// Site Map module
Modules::addModule('sitemap', 'Site Map');
//    attributes
Modules::setAttribute('sitemap', 'content_label', 'Site Map');
Modules::setAttribute('sitemap', 'datatype', DATATYPE_SECTION);    
Modules::setAttribute('sitemap', 'cache', true);
Modules::setAttribute('sitemap', 'cache_key', 'nav');
Modules::setAttribute('sitemap', 'navigation', array('Site Map' => 'list')); // 'label'=>'action'
Modules::setAttribute('sitemap', 'descend_hidden', true); //true or false
Modules::setAttribute('sitemap', 'action_alias', array('map' => 'list'));
//    capabilities
Modules::addCapability('sitemap', 'list');

// =============================================================================
// Generic Feedback email form module
Modules::addModule('feedback', 'Feedback');
// attributes
Modules::setAttribute('feedback', 'cache', false);
Modules::setAttribute('feedback', 'default_action', 'input');
Modules::setAttribute('feedback', 'public_exposure', true);
Modules::addCapability('feedback', 'input');

// The Core Datatypes
// ------------------
    
// General Articles, Documents, and Links/Resources
// This is a globbing module.
Modules::addModule('article', 'Document Library');    
Modules::setAttribute('article', 'content_label', 'Articles');    
Modules::setAttribute('article', 'datatype', array(DATATYPE_GENERAL_ARTICLE, DATATYPE_DOCUMENT, DATATYPE_RESOURCE));
Modules::setAttribute('article', 'public_exposure', true);
Modules::setAttribute('article', 'cache', true);
Modules::setAttribute('article', 'default_action', 'list');
Modules::setAttribute('article', 'searchable', true);

// =============================================================================
// Improved Search module
Modules::addModule('search', 'Search');
//    attributes
Modules::setAttribute('search', 'content_label', 'Search');
Modules::setAttribute('search', 'cache', false);
Modules::addCapability('search', 'results');
Modules::setAttribute('search', 'default_action', 'results');

// =============================================================================
// Section module
Modules::addModule('section', 'Section');

//    attributes
Modules::setAttribute('section', 'content_label', 'Section');
Modules::setAttribute('section', 'datatype', DATATYPE_SECTION);	
Modules::setAttribute('section', 'cache', false);
Modules::setAttribute('section', 'public_exposure', true);
Modules::setAttribute('section', 'searchable', true);
//    capabilities
Modules::addCapability('section', 'list');
Modules::addCapability('section', 'detail');

// Ad Banner [not publicly exposed]
Modules::addModule('ad', 'Ad Banner');
Modules::setAttribute('ad', 'datatype', DATATYPE_AD_BANNER);
Modules::setAttribute('ad', 'cache', false);
Modules::addCapability('ad', 'image');
Modules::addCapability('ad', 'list');

// Module for RSS Feeds
Modules::addModule('feed', 'Syndication');
// attributes
Modules::setAttribute('feed', 'content_label', 'Syndication Feeds');
//Modules::setAttribute('feed', 'datatype', DATATYPE_SYNDICATION_FEED);
Modules::setAttribute('feed', 'public_exposure', true);
Modules::setAttribute('feed', 'cache', false);
Modules::setAttribute('feed', 'searchable', false);
Modules::setAttribute('feed', 'default_action', 'list');
// capabilities
Modules::addCapability('feed', 'list');
Modules::addCapability('feed', 'read');
Modules::addCapability('feed', 'syndicate');
// The Applications
// ----------------
// Directory (of Organizations and/or People)
Modules::addModule('directory', "Member Directory");
Modules::setAttribute('directory', 'content_label', 'Members');
Modules::setAttribute('directory', 'datatype', array(DATATYPE_PERSON, DATATYPE_ORGANIZATION));	
Modules::setAttribute('directory', 'navigation', array('Member Directory' => 'list'));//  'label'=>'action'	
Modules::setAttribute('directory', 'public_exposure', true);
Modules::setAttribute('directory', 'cache', false);
Modules::addCapability('directory', 'list');
Modules::addCapability('directory', 'filters');
Modules::addCapability('directory', 'subscribe');
Modules::addCapability('directory', 'register');

// Calendar
// --------    
Modules::addModule('calendar', 'Event Calendar');
Modules::addCapability('calendar', 'list');
Modules::addCapability('calendar', 'calendar_widget');
Modules::addCapability('calendar', 'month');
Modules::setAttribute('calendar', 'content_label', 'Events');    
Modules::setAttribute('calendar', 'datatype', DATATYPE_EVENT);
Modules::setAttribute('calendar', 'navigation', array('Event Calendar' => 'list')); // 'label'=>'action'
Modules::setAttribute('calendar', 'public_exposure', true);
Modules::setAttribute('calendar', 'cache', true);
Modules::setAttribute('calendar', 'default_action', 'list');
Modules::setAttribute('calendar', 'searchable', true);

// Blog Module
Modules::addModule('blog', 'Blog');
Modules::setAttribute('blog', 'public_exposure', true);
Modules::setAttribute('blog', 'cache', false);
Modules::setAttribute('blog', 'navigation', array('Blog' => 'list_posts'));
Modules::setAttribute('blog', 'datatype', DATATYPE_BLOG_POST);
Modules::setAttribute('blog', 'default_action', 'list_posts');
Modules::addCapability('blog', 'list_posts');
Modules::addCapability('blog', 'list_comments');
Modules::addCapability('blog', 'submit_post');
Modules::regFallbackProvider('blog', 'feed', 'syndicate');


// Which datatypes are used by modules
// -----------------------------------                            
        
define('CONTENT_DATATYPES_LIST', implode(',', Modules::aggregateAttribute('datatype', Modules::findModules('public_exposure', true))));    

// slightly different list (i.e. people & orgs are hidden)
define('BROWSE_CONTENT_DATATYPES_LIST', implode(',', Modules::aggregateAttribute('datatype', array('article', 'calendar'))));



// Formats used by strftime() -- which is locale-sensitive.
// However, this file should be swapped out with a locale-specific file if
// this site becomes multi-lingual / multi-locale. (to handle correct
// ordering, for example, of day / month  date elements.

/*

These are the codes for strftime()
 
%a - abbreviated weekday name according to the current locale 
%A - full weekday name according to the current locale 
%b - abbreviated month name according to the current locale 
%B - full month name according to the current locale 
%c - preferred date and time representation for the current locale 
%C - century number (the year divided by 100 and truncated to an integer, range 00 to 99) 
%d - day of the month as a decimal number (range 01 to 31) 
%D - same as %m/%d/%y 
%e - day of the month as a decimal number, a single digit is preceded by a space (range ' 1' to '31') 
%g - like %G, but without the century. 
%G - The 4-digit year corresponding to the ISO week number (see %V). This has the same format and value as %Y, except that if the ISO week number belongs to the previous or next year, that year is used instead. 
%h - same as %b 
%H - hour as a decimal number using a 24-hour clock (range 00 to 23) 
%I - hour as a decimal number using a 12-hour clock (range 01 to 12) 
%j - day of the year as a decimal number (range 001 to 366) 
%m - month as a decimal number (range 01 to 12) 
%M - minute as a decimal number 
%n - newline character 
%p - either `am' or `pm' according to the given time value, or the corresponding strings for the current locale 
%r - time in a.m. and p.m. notation 
%R - time in 24 hour notation 
%S - second as a decimal number 
%t - tab character 
%T - current time, equal to %H:%M:%S 
%u - weekday as a decimal number [1,7], with 1 representing Monday 
%U - week number of the current year as a decimal number, starting with the first Sunday as the first day of the first week 
%V - The ISO 8601:1988 week number of the current year as a decimal number, range 01 to 53, where week 1 is the first week that has at least 4 days in the current year, and with Monday as the first day of the week. (Use %G or %g for the year component that corresponds to the week number for the specified timestamp.) 
%W - week number of the current year as a decimal number, starting with the first Monday as the first day of the first week 
%w - day of the week as a decimal, Sunday being 0 
%x - preferred date representation for the current locale without the time 
%X - preferred time representation for the current locale without the date 
%y - year as a decimal number without a century (range 00 to 99) 
%Y - year as a decimal number including the century 
%Z - time zone or name or abbreviation 
%% - a literal `%' character 
*/

define('LOCALE_DATE_ABBREV', '%D');
define('LOCALE_DATE_STD', '%e %b %Y');
define('LOCALE_DATE_LONG', '%e %B %Y');
define('LOCALE_DATE_SQL', '%Y-%m-%d');
define('LOCALE_DATE_DAYMONTH', '%e %B');
define('LOCALE_DATE_MONTHYEAR', '%B %Y');
define('LOCALE_DATETIME_LONG', '%e %B %Y, %H:%M');
define('LOCALE_DATETIME_SQL', '%Y-%m-%d %H:%M');
define('LOCALE_TIME_STD', '%H:%M');
?>
