<?php

/*
* Main init file.  This file invokes the other init.?.php files which
* perform the various initialization functions.  The reason for breaking
* these out into separate files, is that in many cases you may only want
* to initialize certain components.  -- e.g. files.php does not need
* to load the entire request framework, it just needs the database to check
* a permission before returning a file.
*/
ob_start();

// INIT {{{

if ( isset($_GET['debug']) )
{
    define('DEBUG', true);
}
else
{
    define('DEBUG', false);
}
    
    
// File Locations
// ------------------									
require_once(dirname(__FILE__) . '/init.urls.php');

ini_set('session.save_path', SITE_TMP_PATH .DIRECTORY_SEPARATOR .'session');

// Include Classes and Libraries

// Load Content Fetching Classes
// (including content Modules)
// -----------------------------

// Site Libraries
// ==============

include_once('PEAR.php');
include_once(SITE_LIB_PATH . '/syntaxcms/Site.class.php');					
include_once(SITE_LIB_PATH . '/syntaxcms/F1CMS.class.php');
include_once(SITE_LIB_PATH . '/syntaxcms/SectionNavigator.php');
include_once(SITE_LIB_PATH . '/syntaxcms/SectionNavigatorOptimized.php');
include_once(SITE_LIB_PATH . '/syntaxcms/PathInfo.php');
include_once(SITE_LIB_PATH . '/syntaxcms/F1CMS_Cache.class.php');
include_once(SITE_LIB_PATH . '/syntaxcms/functions.lib.php');
include_once(SITE_LIB_PATH . '/syntaxcms/StxUtilities.php');
include_once(SITE_LIB_PATH . '/syntaxcms/StxPrefs.php');

// Content Modules
// ===============
    
include_once(SITE_LIB_PATH . '/syntaxcms/Modules.class.php');

// Database & PHlexDB Setup
// ------------------------	

require_once(SITE_INI_PATH . '/init.pxdb.php');


// NAVIGATION -- dependent classes for the Section datatype
// --------------------------------------------------------

// Include class dependencies
pxdb_import('content.output.pxdb_record');

// Set up a constant to use in place of datatype number

define('DATATYPE_SECTION', 9);

// Register the Document class as the pxdb_record class to use for datatype DATATYPE_SECTION

pxdb::register_class('Section', 'pxdb_record', DATATYPE_SECTION);	

include_once(SITE_LIB_PATH . '/syntaxcms/Section.class.php');


// Set Error Handling
// ------------------	
// OAM - May 19, 2004: Removed custom error handling because it added more overhead
// than it is worth.  Sandy aggreed.
require_once('PEAR.php');

$Site =& Site::singleton();

// Logging
include_once(SITE_LIB_PATH . '/syntaxcms/Log.class.php');


// Shutdown
// --------
$Site =& Site::singleton();
register_shutdown_function(array($Site, 'shutdown'));

// Load Configuration Files
// ------------------------

// some user-defined constants, e.g. SITE_NAME
require_once(SITE_CFG_PATH . '/user.conf.php');
// static, non-user constants
require_once(SITE_CFG_PATH . '/static.conf.php');

// Caching
// -------

// PEAR Cache
// ==========

if (isset($_GET['nocache']))
{
    define('CACHING_ENABLED', false);
}
else
{
    define('CACHING_ENABLED', true);
}

$Cache =& new F1CMS_Cache(CACHE_STORAGE_CLASS, array('cache_dir' => CACHE_DIR));
$Cache->caching = CACHING_ENABLED;
Site::setCache($Cache);	

if (isset($_GET['flushcache']))
{
    $Cache->flush();
}	
    

// Authentication
// --------------		
require_once(dirname(__FILE__) . '/init.auth.php');
        
// Parse Request & Load Script
// ---------------------------

$Request_classname = ucfirst($_GET['navtype']).'Request';

if ( file_exists(SITE_LIB_PATH . "/syntaxcms/Request/$Request_classname.class.php") )
{
    require_once( SITE_LIB_PATH . "/syntaxcms/Request/$Request_classname.class.php" );
    $Request =& new $Request_classname();

    // top-level request is stored in Site object
    Site::setRequest($Request); 
}
else
{
   trigger_error("Invalid navigation type: '$navtype'", E_USER_ERROR);
}

if ( isset($_GET['request']) )
{
    $Request->setPath($_GET['request']);
}

$Request->isIncluded(false);


if ( isset($_GET['login']) && !Site::getUser())
{	
    include_once(SITE_TPL_PATH . '/header.tpl');			
        
    $auth =& pxdb_auth::singleton('public_auth');
    $auth->showlogin(SITE_TPL_PATH . '/login.tpl', array('action' => $_SERVER['SCRIPT_URI'].'?login=1', 'login_ref'=>$_SERVER['SCRIPT_URI']));		
        
    include_once(SITE_TPL_PATH . '/footer.tpl');		
    exit();		
}	

?>