<?php
// $Id: pxdb_user.class.php,v 1.5 2005/12/16 20:11:42 omerida Exp $

pxdb_import('content.output.pxdb_record');

/**
* User class for Syntax authentication framework.
* This class exposes an API that supports the core methods of the AuthDB class, written by Nyk Cowham for 
* Forum One Communications, and licensed under the GPL.  This class extends the pxdb_record class and provides
* some authentication-specific functionality.
* 
* This class is organized so that you can override the methods:
*     sql_groups()     -- get group ids and names for current user
*     sql_user()        -- get username
*     sql_uid()        -- get user id
*
* @author Hans Lellelid <hans@velum.net>
* @package pxdb.auth
*/
class pxdb_user extends pxdb_record
{

    /**
    * @var integer
    * @access private
    */
    var $default_ugid = 0;
    
    /**
    * @var string
    * @access private
    */
    var $default_groupname = 'Everybody';
    
    /**
    * @var string
    */
    var $username_col = 'idnum';
    
    /**
    * Username.
    * @var string
    * @access private
    */
    var $user;
        
    /**
    * The encrypted username.
    * @var string
    */
    var $encrypted_user;
    
    /**
    * DB function for encryption.
    * @var string
    */    
    var $sql_encryption_function = 'MD5';
    
    /**
    * Array of user/group ids that user belongs to.
    * @var array
    * @access public
    */
    var $ugidarray = array();
    
    /**
    * Array of group names.
    * @var array
    * @access public
    */
    var $grouparray = array();
    
    /**
    * The SQL-ready list of groups (based on grouparray above).
    * @var string
    */
    var $groups;
    
    /**
    * The SQL-ready list of user and group IDs (based on $ugidarray above)
    * @var string
    */
    var $ugids;
    
    /**
    * ADOdb Connection.
    * @var object
    * @access public
    */
    var $db;
    
    /**
    * Array of admin groups.
    * @var array
    */
    var $admin_gids;
    
    /**
    * Constructor.
    * @param int $id UserID for user record.
    * @access public
    */
    function pxdb_user($id = null) 
    {    
        // prefs used throughout (including by read() method invoked form pxdb_record() constructor
        $this->prefs =& pxdb_prefs::singleton();
        
        // will call ->read() method
        parent::pxdb_record($id);                        
        
        // initialize to default admin group (usually 1)
        $this->admin_gids = $this->prefs->get_array('gid_admin');
        $this->set_datatype($this->prefs->get_pref('dt_user'));
    }    

    /**
    * Attempts to return a reference to an already-instantiated user class.
    * Will create a new user class only if one does not already exist for current db.
    * @param int $id The UserID to instantiate.
    * @param boolean $instantiate_new Whether to create a new instance if one doesn't exist (default: true)
    * @param boolean $destroy Whether to destroy the current singleton.
    * @return object pxdb_user or NULL if it's being destroyed.
    * @access public
    */    
    function &singleton($id = null, $instantiate_new = true, $destroy = false)
    {
        static $instance = null;
        
        if ($destroy == true)
        {
            $instance = null;
        }
        
        if ($instance === null && $instantiate_new)
        {
            $instance = new pxdb_user($id);
        }

        return $instance;
    }

    /**
    * Will return an object if already instantiated.
    * This method calls singleton with a FALSE second parameter.  Both singleton
    * and this method need to use one method for the static variable to work.
    * @return object pxdb_user
    */
    function &existant() 
    {
        $user = pxdb_user::singleton($id = null, $new = false);         
        return  $user;   
    }
    
    /**
    * Destroys the current singleton, if set.
    * @return NULL (value of singleton instance will be NULL again)
    * @since 0.0.4
    * @access public
    */
    function destroy()
    {
        return pxdb_user::singleton($id = null, $new = false, $destroy = true);
    }
    
     /**
    * Set the column name to use for the username.
    * @param string $col The column name.
    * @access public
    */
     function set_username_col($col)
     {
        $this->username_col = $col;        
        // reload $user var    
        $this->user = $this->get_field($this->username_col);
    }
    
    /**
    * Adds an admin group to the admin_gidarray array.
    * @param mixed Group name or id to add to array.
    * @access public
    */
    function add_admin_group($group) 
    {
        if (!is_numeric($group))
        {
            // lookup gid        
            $sql = "
SELECT id
FROM records
WHERE name = " . $this->db->qstr($group) . "
    AND datatype IN (" . $this->prefs->get_list('dt_group') . ")";
            $rs = $this->db->Execute($sql);
            $group = $rs->fields['id'];
        }        
        $this->admin_gids[] = $group;
    }
        
    /**
    * Read in data from db.
    * @return boolean True/false on success.
    * @access public
    */
    function read() 
    {
        parent::read(); // load the record
        $this->get_groups(); // load groups user belongs to
        $this->user = $this->get_field_strict($this->username_col);
    }
 
    /**
    * The SQL that gets the groups for this user.
    * This method can be overridden by child classes that wish to redefine the relationshps between users
    * and groups -- e.g. to have all group-user relationships be defined on a per-project basis.
    * @return string SQL used to fetch group ids and names for given user.
    * @access public
    */
    function sql_groups() 
    {        
        return "
SELECT DISTINCT groups.id,groups.name 
FROM records groups
LEFT JOIN
    primaryrel xref ON groups.id = xref.id1
    AND xref.relid IN (" . $this->prefs->get_list('rel_user_group') . ")
LEFT JOIN
    records user on user.id = xref.id2
WHERE
    user.id = ".$this->get_id()."
    AND groups.datatype IN (" . $this->prefs->get_list('dt_group') . ")"; 
    }

    /**
    * Returns the current username.
    * @return string Username
    * @access public
    */
    function get_user() 
    {
        return $this->user;
    }

    /**
    * [Legacy] Provides an alias to get_id()
    * @return int id
    * @access public
    */
    function get_uid() 
    {
        return $this->get_id();
    }
    
    /**
    * A Public boolean test method - see documentation
    * @return boolean
    * @access public
    */
    function is_admin() 
    {    
        // are any of the values in $admin_gids *also* present
        // in the user's ugid array  ($ugidarray)?
        return (array_intersect($this->admin_gids, $this->ugidarray) ? true : false);
    }

    /**
    * A Public boolean test method - see documentation
    * @return boolean
    * @access public
    */
    function not_admin() 
    {
        return (!$this->is_admin()); // Bitwise XOR mask
    }

    /**
    * Returns whether user is member of passed group [id].
    * @param mixed $group The group id (or name) OR array of these that we are checking against.
    * @param string $behavior 'any' or 'all' -- how the membership is treated -- i.e. does user have to be in any of groups specified, or all?
    * @return boolean Whether $group is in $grouparray.
    * @access public
    */
    function is_member($group, $behavior = 'any') 
    {
        if (!is_array($group)) 
        {
            $group = preg_split("/[\s,]/", $group);
        };
            
        switch (strtolower($behavior))
        {
            case 'any':
                if (is_numeric($group[0]))
                {
                    return (sizeof(array_intersect($this->ugidarray, $group)) > 0);
                }
                else
                {
                    $matches = sizeof(array_intersect($this->grouparray, $group));
                    if ( 1 < $matches )
                    {
                        trigger_error("More than one group name matches request for '$group'.", E_USER_WARNING);                        
                        return true;
                    } 
                    elseif ( 0 < $matches )
                    {
                        return true;
                    }
                    else
                    {
                        return false;
                    }                        
                } // if is_numeric
            break;
            case 'all':
                if (is_numeric($group[0]))
                {
                    return (sizeof(array_diff($group, $this->ugidarray)) == 0);
                }
                else
                {
                    return (sizeof(array_intersect($this->grouparray, $group)) == 0);
                } 
            break;
            
            default:            
                trigger_error("Invalid behavior '$behavior'.", E_USER_WARNING);
                return false;
            break;
                
        } // switch
    } // is_member

    /**
    * Returns true if user is not in passed group [name].
    * @param string $group The group name to check.
    * @return boolean Whether $group is not in $grouparray.
    * @access public
    */
    function not_member($group) 
    {
        return (!$this->is_member($group)); // Bitwise XOR mask
    }

    /**
    * Sets the ugidarray and grouparray with values for current user.
    * @access public
    */
    function get_groups() 
    {    
        $this->ugidarray[]  = $this->get_default_ugid(); //add default ugid
        $this->grouparray[] = $this->get_default_groupname(); // add default group name
        
        if ($this->get_id())
        {
            $rs = $this->db->Execute($this->sql_groups());
            while (!$rs->EOF)
            {            
                $this->ugidarray[] = $rs->fields['id'];    // push id col
                //OAM, change grouparray key to be the group id
                $this->grouparray[$rs->fields['id']] = $rs->fields['name']; // push name col
                $rs->MoveNext();
            }
            
            //add userid to ugidarray
            if ($this->get_uid())
            {
                $this->ugidarray[] = $this->get_uid();
            }
        }
    }
    
    /**
    *  A Public accessor method - see documentation
    * Although you can invoke this explicitly, most of the time
    * you are probably better off letting it be implicitly
    * invoked from get_sql_groups() or get_sql_ugids()
    * @access public
    */
    function build_sql() 
    {
        if (!$this->grouparray)
        {
            $this->get_groups();
        }
        $this->ugids  = join(',', $this->ugidarray);
        $this->groups = join(',', $this->grouparray);                
    }

    /**
    * A Public accessor method - see documentation
    * @return string List of group names.
    * @access public
    */
    function get_sql_groups() 
    {
        if (!$this->groups)
        {
            $this->build_sql();
        }
        return $this->groups;
    }

    /**
    * A Public accessor method - see documentation
    * @return string List of user-group IDs.
    * @access public
    */
    function get_sql_ugids() 
    {
        if (!$this->ugids)
        {
            $this->build_sql();
        }
        return $this->ugids;
    }

    /**
    * Set Default UserGroup ID
    * 
    * Set the default UserGroup ID in case you use something different
    * than 0 as your default group ID.
    * 
    * @access public
    * @param int $ugid The default User Group ID
    * @return integer 0 if no value passed
    */
    function set_default_ugid($ugid) 
    {
        if ( '' !== $ugid )
        {
            $this->default_ugid = $ugid;
        }
        else
        {
            return 0;
        }
    }

    /**
    * Get Default User Group ID
    * 
    * Get the currently set default usergroup ID.
    * See set method above.
    * 
    * @access public
    * @return integer The ID of the current default group
    */
    function get_default_ugid() 
    {
        return $this->default_ugid;
    }
    
    /**
    * Set Default Group Name
    * 
    * Set the default group name to something other than
    * 'Everybody' if your project has something different.
    * 
    * @access public
    * @return integer 0 if no value passed
    * @param $groupname string The name of the default group
    */
    function set_default_groupname($groupname) 
    {
        if ( '' !== $groupname )
        {
            $this->default_groupname = $groupname;
        }
        else
        {
            return 0;
        }
    }
    
    /**
    * Get Default Group Name
    * 
    * Gets the currently set name for the default group
    * 
    * @access public
    * @return string The current default group name
    */
    function get_default_groupname() 
    {
        return $this->default_groupname;
    }
  
    /**
    * Public method for getting group array
    *
    * Added by OAM
    * @access public
    * @return array
    */
    function get_group_array()
    {
        return $this->grouparray;
    }
    
    /**
    * Public method for getting ugid array
    *
    * Added by OAM
    * @access public
    * @return array
    */
    function get_ugid_array()
    {
        return $this->ugidarray;
    }
} //end of class specification
?>