<?php
// $Id: pxdb_commit.class.php,v 1.26 2006/04/12 15:35:16 sandys1 Exp $

pxdb_import('content.input.pxdb_confront');
pxdb_import('metadata.pxdb_typesfield');
pxdb_import('metadata.pxdb_dbfield');
pxdb_import('content.input.pxdb_validator');
pxdb_import('content.output.pxdb_record');
pxdb_import('events.pxdb_event_dispatcher');

/**
* This is the class that is responsible for writing content to the records table (and to lookup tables).
* 
* This class has some overlap with the pxdb_record class, but the focus of this class is *writing* data
* to the database, within the system of privileges.  The default use for this class is on the receiving
* end of a pxdb_input form. -- i.e. when this class is instantiated, it will seek to import any vars submitted
* by the pxdb_input form (including uploaded files).  Before being written to the database, variables are
* processed, then validated using defined plugins or registered processor & validator functions.  Records are 
* inserted with 'read,write' privileges for the user inserting the record, for any admin user (pref: gid_admin) and
* a default 'read' value for Everyone.  Additionally, if approve_disabled pref is not set then the record is marked
* approved only if a approve_auto is set (either generally or for the datatype).   Auto-generated fields
* are calculated during the 'cleanup' phase.  Additionally if the record being written is a 'user' or 'group' type
* record (as defined with the dt_user and dt_group pxdb_pref preferences) then the usergroup table is also
* automatically populated (e.g. for authentication with mod_auth_mysql) during the 'cleanup' phase.
*            
* @package pxdb.content.input
* @author Hans Lellelid <hans@velum.net>
* @version $Revision: 1.26 $ 
* @access public
*/
class pxdb_commit extends pxdb_confront 
{
    /**
    * Relationships that will be saved to db.
    * @var array
    * @access private
    */        
    var $_related = array(); // arr of relationships to insert -- [reltype] [relid] [related_id]

    /**
    * Fields that will be saved to db.
    * @var array
    * @access private
    */
    var $_record = array(); // arr of columns/fields to insert    
    
    /**
    * Saved copy of pre-change record.
    * @var array
    * @access private
    */
    var $_original_record;
    
    /**
    * The default priv assigned for everyone (gid 0).
    * @var string
    * @access public
    */
    var $default_priv = 'read,write'; // the default priv that is assigned to ugid '0'
    
    /**
    * The MySQL function to use to encrypt passwords.
    * @var string
    * @access private
    */
    var $mysql_password_function = 'ENCRYPT';

    /**
    * Files that were uploaded.
    * @var array
    * @access private
    */
    var $_uploaded_files = array();
    
    /**
    * records columns that were modified by current save() action.
    * @var array
    * @access private
    */
    var $_modified_fields = array();
    
    /**
    * Whether authentication module must be bound.
    * @var boolean
    * @access public
    */    
    var $auth_required = false;
    
    /**
    * The type of save operation that was performed.
    * (e.g. insert or update)
    * @var string
    * @access public
    */
    var $save_type = null;
    
    /**
    called by php's unserialize function
    */
    function __wakeup()
    {                
        $this->db = &pxdb::dbh();   
    }
    
    /**
    * constructor
    * @param integer $id The record id.
    * @access public
    */
    function pxdb_commit($id = null) 
    {
        // parent constructor        
        parent::pxdb_confront();    
                    
        // Instantiate validator class (even if it's not used)
        // OAM IMPORTANT!: call_user_func apaparently can not return a reference 
        // to a statically called method, you'll have to leave this eval in            
        eval("\$this->validator = &".pxdb::get_class('pxdb_validator')."::singleton();");
        
        //$validator = pxdb::get_class('pxdb_validator');       
        //$this->validator =& call_user_func(array($validator, 'singleton'));
        
        // set variables that hold info about record we are modifying            
        $this->datatype = $this->_params['dt']; // needed for add & mod
        $this->record_id = ($id !== null ? $id : $this->_params['id']); // needed for mod & del
        
        // if record_id is defined but datatype is not -- look up datatype        
        if (!empty($this->record_id) && empty($this->datatype)) 
        {            
            $this->datatype = $this->lookup_datatype($this->record_id);
        }        

        // grab the contents of current record -- used for noting which fields are changed
        $this->read();
    }

    /**
    * Get the record id.
    * @return integer The record id.
    * @access public
    */
    function get_id() 
    {
        return $this->record_id;
    }
    
    /**
    * Sets the record id.
    * @param integer $id The record id.
    * @access public
    */
    function set_id($id) 
    {
        $this->record_id = $id;

        // OAM: added to make sure these two are always in sync        
        $this->datatype = $this->lookup_datatype($this->record_id);
    }
    
    /**
    * Returns the datatype for a given id
    * @access public
    * @return int datatype ID
    * @param int $rec_id Record ID
    */
    function lookup_datatype($rec_id)
    {
        $sql = 'SELECT datatype FROM records WHERE id = ' . $rec_id;
        $rs = $this->db->Execute($sql);
        return $rs->fields['datatype'];
    }
    
    /**
    * Get the datatype.
    * @return integer The datatype.
    * @access public
    */
    function get_datatype()
    {
        return $this->datatype;
    }
    
    /**
    * Sets the datatype.
    * @param integer $dt The datatype.
    * @access public
    */
    function set_datatype($dt)
    {
        $this->datatype = $dt;    
        $this->_params['dt'] = $dt;
    }    
            
    /**
    * Reads in data from database.
    * @access public
    */    
    function read() 
    {
        if ($this->get_id())
        {        
            $sql = "SELECT * FROM records WHERE id = " . $this->get_id();
            $rs = $this->db->Execute($sql);
            $this->_original_record = $rs->fields;
        }
    }
    
    /**
    * Imports variables and uploaded files from global scope.
    * @param array $source The location of source data to use. (for upward compatibility with pxdb::import())
    * @access public
    */    
    function import($source = null) 
    {
        parent::import($source);
        
        // uploaded files present somewhat of a special exception
        // that must be handled separately.
        $this->_import_uploaded_files();
    }
    
    /**
    * Wrapper function for form validation class' function (if present).
    * This method is public and can be invoked separately.  If you invoke this method
    * you can expect to find the validation errors (array) in the global variable $pxdb['validator'].
    * This array is automatically read-in by the pxdb_input class.
    * @return boolean True on success and false on error.
    * @access public
    */
    function validate() 
    {        
        if ($this->debug) 
        {
            pxdb::print_debug('Validating Data...', 'bookmark'); 
        }
        
        $failed = false;
        
        if ($this->validator)
        {
            // SFS change to while
            if (!empty($this->_data))
            {            
                while (list($fname, $value) = each($this->_data))
                {
                    $tfid = $this->lookup_tfid($fname);
                    
                    // 1) Post-process the data
                    
                    // SFS Handle case of built-in non-db field that pxdb stupidly imports
                    if (!$tfid && 'password_verify' != $fname) 
                    {                 
                        trigger_error("Can't identify '$fname'; skipping validation.", E_USER_WARNING);                    
                    } 
                    else 
                    {                
                        $this->_data[$fname] = $this->validator->process($tfid, $fname, $this->_data[$fname]);
                        // 2) validate the data
                        if ($this->validator->validate($tfid, $fname, $this->_data[$fname])) 
                        {
                            /* we want to complete even if val fails */
                            $failed = true;  
                        }                    
                    }
                    
                } // end while
                reset($this->_data); // reset since it's an object property
            }
            
            if ($failed) 
            {
                $this->errors['validator'] = "Problems were found with the data you entered."; 
            }
            
            if ($this->debug)
            {
                pxdb::print_debug('Errors found, adding them to $GLOBALS['.PXDB_GLOBAL_VAR.']', 'bookmark');                
                echo 'commit:validate<br>'; 
                echo '<pre>'; 
                print_r($GLOBALS[PXDB_GLOBAL_VAR]); 
                echo '</pre>';
            }
            return !$failed;
        }
        else
        {
            return true;
        }
    }    

    /**
    * Gets a list of fields that were updated.
    * This will be set after the save method has been called.  Note: it's not particularly
    * useful after an insert(), since all fields will have been modified.
    * @return array Fields that were modified.
    * @access public
    */
    function get_modified_fields() 
    {        
        return array_unique($this->_modified_fields);        
    }    
    
    /*
    * Prepares data to be written to the database.
    * This function creates the $record and $related arrays which contain values
    * to be written to the records table and to relationship (e.g. primaryrel, pickrel)
    * tables.  This method prepares 99% of data to be written.  The one glaring exception
    * is uploaded files.  pxdb convention has uploaded files being named with the ID of the
    * record.  Since record IDs are not necessarily known until after having been created,
    * the file handling is done in the _cleanup() method.  This method just puts a holder
    * string in the value of the upload column.
    * @return boolean on success.
    * @access private
    */
    function _prepare() 
    {
        if ($this->debug) 
        { 
            pxdb::print_debug('Preparing Data...', 'bookmark'); 
        }
        
        if ($this->debug)
        {
            echo '<pre>'; print_r($this->_data); echo '</pre>'; 
        }
        // loop through the data that was submitted
        // and set values in $records[] and $relationships[] arrays.        
        // SFS while for speed!
        if (!empty($this->_data))
        {
            while (list($fname, $value) = each($this->_data))
            {
                // lookup a typesfields id based on a dbfield fname
                $tfid = $this->lookup_tfid($fname);
                    
                // if found, continue -- (some may not be found:  e.g. password_verify)                
                if ($tfid) 
                {                        
                    // instantiate pxdb_typesfields object with that id
                    $objf =& new pxdb_typesfield($tfid);                                
    
                    // make sure the user (AuthDB) is authorized to update this record [they should be since presumably pxdb_input displayed it for them]
                    if (!$this->is_flevel_authorized($objf->get_id(), 'write')) 
                    {
                        $this->errors[] = "You are not authorized to update field. (typesfields id = ".$objf->get_id().")";
                        return false;
                    }                
    
                    // populate the $_record and $_related arrays with info for this typesfields row
    
                    switch ($objf->get_field_type()) 
                    {
                        case "simple":
                        
                            $dbf = new pxdb_dbfield($objf->get_fieldid());
    
                            switch ($dbf->get_field_type()) 
                            {
                                case "pick":
    
                                    // this is really the same thing as the default 
                                    // case -- but we'll leave it as a separate option,
                                    // in case it gets special handling later.
                                    $field = $dbf->get_fname();
                                    $this->_record[$field] = $this->db->Quote($value);
    
                                    // has it changed?
                                    if ($this->is_changed($field, $value)) 
                                    {
                                        $this->_modified_fields[] = $field;                                        
                                    }                                    
    
    
                                break;
    
                                case "pickid":
    
                                    // SFS - Treat multiple and single pickids the same
                                    $tmpA['reltype'] = 'pickrel';
                                    $tmpA['relid'] = $dbf->get_id(); // fid in this case
                                    $tmpA['related_id'] = $value; // could be an array also
                                    $this->_related[] = $tmpA;
    
                                break;
    
                                case "upload":
                                    /* 
                                    We really can't do anything about files until we know the record_id -- and
                                    we won't know that until everything else has been done.
                                    */    
    
                                    $upload_column = ($dbf->get_upload_column() ? $dbf->get_upload_column() : $dbf->get_fname());
                                    
                                    // if a file was actually uploaded then we want to add some placeholder
                                    // text to the records row (until we determine the actual url)
     
                                    if (isset($value['tmp']) && isset($value['name'])) 
                                    {
                                        // save a placeholder in $_record array
                                        // this will eventually hold the URL
                                        $this->_record[$upload_column] = $this->db->Quote("[placeholder]"); 
    
                                        // we can assume if we are here that they have specified a 
                                        // new file to upload -- and that therefore,
                                        // this field has changed.
                                        $this->_modified_fields[] = $upload_column;
    
                                        // save the uploaded file information (the array) as a new element to the $_uploaded_files array
                                        $this->_uploaded_files[$upload_column] = $value;                                        
                                    } 
                                    else
                                    {
                                        // the field is empty (but part of the form) which
                                        // means that the file has been removed. -- so se the record column
                                        // and modified_fields                                    
    
                                        // has the field changed?
                                        if ($this->is_changed($upload_column, $value)) 
                                        {
                                            $this->_modified_records[] = $upload_column;
                                        }
    
                                        // add quoted version to records table
                                        $this->_record[$upload_column] = $this->db->Quote($value);
                                    }
    
                                break;
    
                                case "pass2":
                                
                                    // pass2 is just for validation / verification
                                    // but we treat it just like password as far as writing it to the database.
                                    // no break;
                                case "password":
    
                                    // if it's changed then encrypt it -- otherwise leave it as it is.
                                    $field = $dbf->get_fname();
    
                                    if ($this->is_changed($field, $value)) 
                                    {
                                        // we will encrypt the password
                                        $this->_record[$field] = $this->mysql_password_function . "(".$this->db->Quote($value).")";
    
                                        // add to changed array
                                        $this->_modified_records[] = $field;
                                    }
                                    else 
                                    {
                                        // just re-write the encrypted password to the database
                                        $this->_record[$field] = $this->db->Quote($value);
                                    }
                                    
                                break;
    
                                default:
    
                                    $field = $dbf->get_fname();
    
                                    // has the field changed?
                                    if ($this->is_changed($field, $value)) 
                                    {
                                        $this->_modified_records[] = $field;
                                    }
    
                                    // add quoted version to records table
                                    $this->_record[$field] = $this->db->Quote($value);
                                    
                               break;
                            } // switch get_dbfield_field_type
    
                        break; // end simple case
    
                        case "relationship":
    
                            // right now only primaryrel
                            $tmpA['reltype'] = 'primaryrel';
                            $tmpA['relid']   = $objf->get_relid();
                            $tmpA['related_id'] = $value; // might be an array, might be empty
                            $this->_related[] = $tmpA;
    
                        break;
    
                    } // switch // get object field type
                    
                } // if ($tfid)                    
                                                                        
            } // while (main loop through data)
            reset($this->_data); // reset since is object property
        } // end if !empty
        return true;
        
    }

    /**
    * Find out whether or not a field was modified (from db version).
    * This method compares the value entered with $_original_record and returns
    * true if they are not equal and false if they are.
    * @param string $field The field we are examining.
    * @param mixed $value The new value for the field.
    * @return boolean true/false if changed
    * @access public 
    */
    function is_changed($field, $value) 
    {        
        //$this->_record[$field] != $this->_original_record[$field]        
        return ($value != $this->_original_record[$field]);
    }

    /**
    * Returns the type of save operation that was performed.
    * @return string 'insert' or 'update'
    * @access public
    */ 
    function get_save_type()
    {
        return $this->save_type;
    }
    
    /**
    * Main workhorse function of the pxdb_adm class.
    * This method loads up the $related and $record arrays, then calls the methods that perform the db updates
    * and inserts.
    * @return boolean on success.
    * @access public
    */    
    function save() 
    {
        // we need to look at the values in the $_imported array
        // most of these can simply be inserted into records,
        // but oref_* has to be handled separately -- i.e. related.
        
        // in some cases there is no corresponding dbfield (i.e. in primaryrel relationships)
        // and in others the dbfield type will imply that we need to add values to another table
        // (e.g. in the case of pickrel relationships)

        // 0) check for necessary environment
        // ----------------------------------
        if (!$this->datatype || empty($this->datatype)) 
        {
            trigger_error("You cannot call the save() method without supplying the datatype of object being added / modified.", E_USER_WARNING);
            return false;
        }

        if ('update' == $this->save_type)
        {
            $this->send_event_notification(SYNTAX_EVENT_PREUPDATE);
        }

        // 1) run post_processors and validator functions
        if (!$this->validate())
        {
            return false;
        }

        // 2) Loop through form data and build the $record array and $related array
        if (!$this->_prepare())
        {
            return false;
        }
        
        // 3) now write the record
        if (!$this->_write())
        {
            return false;
        }

        // 4) now add the relationships
        if (!$this->_relate())
        {
            return false;
        }
        
        // 5) now cleanup & move any affiliated items (e.g. files)
        if (!$this->_cleanup())
        {
            return false;
        }
        
        // send event notification
        switch ($this->save_type)
        {
            case 'update':
            
                $this->send_event_notification(SYNTAX_EVENT_UPDATE);
            
            break;

            case 'insert':
            
                $this->send_event_notification(SYNTAX_EVENT_INSERT);
            
            break;
        } // end switch

        return $this->record_id;
        
    } // function
    
    /**
    * Performs any post-write functions.
    * This function was created by the need to rename files with the record id -- which
    * can only be done once the record has been written (in the case of adding records). This function 
    * could call any other functions that depended on the record being written.
    * @return boolean True/False on success.
    * @access private
    */
    function _cleanup() 
    {
        if ($this->debug)
        {
            pxdb::print_debug('Cleaning up...', 'bookmark'); 
        }
        
        // if there are any uploaded files they need to be moved
        
        // rename & move uploaded files        
        if (!$this->_handle_uploaded_files()) return false;
    
        if (!$this->_calculate_autogen_fields()) return false;
        
        if (!$this->_populate_usergroup()) return false;
        
        return true;
    }
    
    /**
    * Calculate any autogen fields in current fields.
    * This method relies on pxdb_input specifying a ['autogen'] array containing identifiers for autogen fields that
    * should be handled by this script.  This script processes the autogen field from typesfields and updates the 
    * records table with the results.
    * @return boolean True/false on success.
    * @access private
    */
    function _calculate_autogen_fields() 
    {
        // 1) get autogen fields for current datatype
                
        // I think the answer is that the input class should do the work -- 
        // in terms of creating an 'autogen' field that contains the identifiers
        // for the fields that need to be calculated.
        
        if (isset($this->_params['autogen'])) 
        {
            if ($this->debug) 
            {
                pxdb::print_debug('Detected autogen fields; Executing generator scripts.', 'bookmark'); 
            }
            
            // initialize a Smarty object -- Smarty will be the new 
            // parser of autogen fields.
            
            // possible replacement values include (are limited to?)
            // current row
            
            $rec = new pxdb_record($this->record_id);
            // SFS at war for while!
            while (list(, $field) = each($this->_params['autogen']))
            {
                // perform a lookup on the field -- to get the autogen value
                $tfid = $this->lookup_tfid($field);    
                
                if ($tfid) 
                {                
                    $objf = new pxdb_typesfield($tfid);                                                                
                    $dbf = new pxdb_dbfield($objf->get_fieldid());
                    
                    if (pxdb::plugin_exists('generators', $objf->get_autogen())) 
                    {
                        // a) is there a plugin with the given name  (?) -- default
                        $data = array();
                        
                        $data['record'] = &$rec;
                        $data['column'] = $dbf->get_fname();
                        $data['record_id'] = $this->record_id;
                        $data['datatype'] = $this->datatype;
                        
                        if ($this->debug) 
                        {
                            pxdb::print_debug(' executing generator: '.$objf->get_autogen(), 'bookmark'); 
                        }                                                            
                        $processed = $this->_include_generator($objf->get_autogen(), $data);
                    }
                    else 
                    {
                        // b) replace {} with field values                        
                        $processed = pxdb_tools::populate($objf->get_autogen(), $rec->fields);                        
                    }        
                    
                    // Update records column with generated value                    
                    $sql = "UPDATE records SET " . PXSQL_L . $dbf->get_fname() . PXSQL_R
                        . " = " . $this->db->qstr($processed)
                        . " WHERE id = ".$this->record_id;
                    $this->db->Execute($sql);
                } // if ($tfid)
            } // end while
            reset($this->_params['autogen']); // reset since its object property
        } // if ($this->_data['autogen']
        return true;
    }
    
    /**
    * Include an autogen calculator.
    * @param string $plugin_data The plugin name for the generator we want to include and any params.
    * @param array $data Hash of data to pass to autogen function.
    * @access private
    * @return mixed
    */
    function _include_generator($plugin_data, &$data) 
    {
        $plugin_info = pxdb::parse_plugin($plugin_data);            
            
        $plugin = $plugin_info['script'];
        $params = $plugin_info['params'];
        // build the path to the plugin
        $gpath = PXDB_PLUGPATH . "/generators/$plugin.php";
        
        if (file_exists($gpath)) 
        {
            extract($data); // extract $record and $column into namespace
            
            // simplifies testing of validator components if we don't have to build validators with object referneces
            $db =& $this->db;             
            
            include($gpath);
            
            return $value;        
        }
        else 
        {            
            trigger_error("Generator not found at specified path: $gpath", E_USER_WARNING);            
            return false;
        }
    }

    /**
    * Renames and moves uploaded files from temporary location to datastore.
    * This function requires that you have set a path for your files in the pxdb_prefs table 
    * (pref_name = 'datastore'; pref_key = 'path' and pref_key = 'url' for the filesystem path and relative URL
    * respectively). If no datastore pref is found, then it will try to put files to directory where current
    * script (class file) is located -- and that probably won't work.
    *
    * OAM - Aug 2005.  Updated and created a constant PXDB_FILESTORE_PATH that is set in config/initialize.php.
    * @access private
    * @return boolean
    */
    function _handle_uploaded_files()
    {
        if ($this->debug) 
        {
            pxdb::print_debug('Renaming and moving uploaded files.', 'bookmark'); 
        }
    
        if (empty($this->_uploaded_files))
        {
            return true;
        }

        // SFS change to while
        while (list($upload_column, $file) = each($this->_uploaded_files))
        {
            // 1) build new filename 
            $filename = $file['name'];
            $filename = preg_replace("/^[\d]+_/", "", $filename); // strip out any old ID_ from filename
            // SFS add: bugfix to identical file uploaded to multiple fields
            $filename = str_replace($upload_column.'_', '', $filename); // strip out old column name
            $filename = preg_replace("/[^\w\.]+/", "_", $filename); // remove special chars
            $filename = $this->record_id . '_' . $upload_column . '_' . $filename; // prepend new record id

            // 2) get datastore information for file
            $prefix_path = $this->_get_filestore_paths();
            $prefix_url  = $this->prefs->get_pref('datastore', 'url');

            if (false === $prefix_path || !$prefix_url) 
            {
                trigger_error("No datastore path or prefix URL found for uploaded files.", E_USER_WARNING);
                return false;
            }
            
            // 3) copy all files to new location
            
            // In cases where no new file was uploaded, the file will be copied to itself.
            // (because 'tmp' is the path) -- this should be fine and makes for a simpler
            // widget.
            // SFS while for ... wait for it ... speed!
            while (list(, $path) = each($prefix_path))
            {
                $new_full_path = "$path/$filename";

                if ($this->debug) 
                {
                    pxdb::print_debug("Copying file... $file[tmp] -> $new_full_path"); 
                }

                // copy the file to store path
                // OAM TODO: should use safer move_uploaded_file PHP function which does
                // a few more checks to make sure it is an uploaded file
                if (@copy($file['tmp'], $new_full_path)) 
                {
                    if ($file['tmp'] != $new_full_path) { unlink($file['tmp']); }
                }
                else 
                {
                    trigger_error("Could not copy $file[tmp] to $new_full_path", E_USER_WARNING);

                    if (!file_exists($file['tmp'])) 
                    {
                        trigger_error("File $file[tmp] doesn't exist.", E_USER_WARNING); 
                    }
                    return false;
                }

            }

            // 4) update the db with new info for file
            $new_url = "$prefix_url/$filename";

            $sql = "UPDATE records SET $upload_column = ".$this->db->Quote($new_url)." WHERE id = $this->record_id";
            if ($this->debug) 
            {
                pxdb::print_debug($sql, 'db-update'); 
            }
            $this->db->Execute($sql);
        
        } // end while
        reset($this->_uploaded_files); // reset since we traversed an object property
        
        return true;
    }
    
    /**
    * Returns an array with one or more PXDB_FILESTORE_PATH entries. Returns false if none found.
    *
    * @access private
    * @return array Array of filestore path strings, or boolean false if none found
    * @author Sandy Smith <ssmith@forumone.com>
    */
    function _get_filestore_paths()
    {
        if (defined('PXDB_FILESTORE_PATH'))
        {
            $prefix_paths = PXDB_FILESTORE_PATH;
        
            if (false !== strpos($prefix_paths, '|'))
            {
                $prefix_paths = explode('|', $prefix_paths);
            }
            elseif ($prefix_paths)
            {
                $prefix_paths = array($prefix_paths);
            }
            else
            {
                $prefix_paths = array();
            }
        }
        else
        {   
            $prefix_paths = array();
        }
        
        if (0 == count($prefix_paths))
        {
            $prefix_paths = false;
        }
        
        return $prefix_paths;
    }
    

    /**
    * Adds user idnum and group name to the usergroup table.
    * The usergroup table is a hack so that you can authenticate against a Syntax instance
    * using a tool like mod_auth_mysql -- which requires a more straightforward user-group relationship
    * table than the primaryrel table can provide.
    * @return boolean
    * @access private
    */
    function _populate_usergroup() 
    {
        // $this->get_datatype() will give current dt.  
        // if current object is a user, then we want to get the idnum for current user
        // if current object is a group, then we want to get the record name.
        $dt_this  = $this->get_datatype();
        $dt_user  = $this->prefs->get_array('dt_user');
        $dt_group = $this->prefs->get_array('dt_group');
        
        if (in_array($dt_this, array_merge($dt_user, $dt_group))) 
        {
            if ($this->debug) 
            {
                pxdb::print_debug('Detected that you are updating a user or group; populating \'usergroup\' table.', 'bookmark'); 
            }
            
            if (in_array($dt_this, $dt_user)) 
            {
                $thisval = 'idnum';
                $thiscol = 'idnum';
                $relval = 'name';
                $relcol = 'groups';
            }
            elseif (in_array($dt_this, $dt_group)) 
            {
                $thisval = 'name';
                $thiscol = 'groups';
                $relval = 'idnum';
                $relcol = 'idnum';
            }
        
            $thisRec = new pxdb_record($this->get_id());
            ${$thisval} = $thisRec->get_field($thisval);
            
            // in case there are many, get it as an array
            $rel_user_group = $this->prefs->get_array('rel_user_group');
        
            // loop through the relationships -- look for one that matches
            // the rel_user_group pref.
            // SFS lova da while
            while (list(, $rel) = each($this->_related))
            {
                // if it matches reltype & relid                
                if ($rel['reltype'] == 'primaryrel' && in_array($rel['relid'], $rel_user_group)) 
                {
                    if (!is_array($rel['related_id'])) 
                    {
                        $rel['related_id'] = array($rel['related_id']); 
                    }
                    
                    foreach($rel['related_id'] as $related_id) 
                    {
                        if ($related_id) 
                        {
                            // instantiate the related record
                            $relRec = &new pxdb_record($related_id);
                            ${$relval} = $relRec->get_field($relval);                        
    
                            $sql = "SELECT COUNT(*) FROM usergroup WHERE $thiscol=".$this->db->qstr(${$thisval})." AND $relcol=".$this->db->qstr(${$relval});
                            if ($this->db->GetOne($sql) == 0) 
                            {
                                $sql = "INSERT INTO usergroup ($thiscol, $relcol) VALUES (".$this->db->qstr(${$thisval}).", ".$this->db->qstr(${$relval}).")";
                                $this->db->Execute($sql);
                                if ($this->debug) { pxdb::print_debug($sql, 'db-update'); }
                            } // end if getOne
                            unset($relRec);
                        }
                    } // related_id foreach                    
                } // if relevant                        
            } // rel while
            reset($this->_related); // reset since it's an object property
            // get some preferences that we will use for populating the user->group table    
        }
        return true;
    }

    /**
    * Adds a primaryrel relationship to current record.
    * @param mixed $related_id This is the record ID (or array of IDs) we are relating the current record to.
    * @param int $relid This is the relid.
    * @access public
    */    
    function add_related($related_id, $relid) 
    {
        // no need to normalize $related_id (i.e. make it an array) as this is done by _write_relationship        
        $this->_write_relationship('primaryrel', $related_id, $relid);                    
    }

    /**
    * Adds a secondaryrel relationship to current record.
    * @param mixed $related_id This is the record ID (or array of IDs) we are relating the current record to.
    * @access public
    */    
    function add_secondaryrel_related($related_id) 
    {        
        // no need to normalize $related_id (i.e. make it an array) as this is done by _write_relationship        
        $this->_write_relationship('secondaryrel', $related_id);                    
    }
    
    /**
    * Adds a noderel child to current record.
    * @param mixed $related_id This is the record ID (or array of IDs) we are relating the current record to.
    * @param int $noderel_id The noderel ID of the noderel relationship type. Defaults to null for backward compatibility
    * @access public
    */
    function add_child($related_id, $noderel_id = null)
    {
        // no need to normalize $related_id (i.e. make it an array) as this is done by _write_relationship
        $this->_write_relationship('child', $related_id, $noderel_id);
    }

    /**
    * Adds a noderel parent to current record.
    * @param mixed $related_id This is the record ID (or array of IDs) we are relating the current record to.
    * @param int $noderel_id The noderel ID of the noderel relationship type. Defaults to null for backward compatibility
    * @access public
    */
    function add_parent($related_id, $noderel_id = null)
    {
        // no need to normalize $related_id (i.e. make it an array) as this is done by _write_relationship
        $this->_write_relationship('parent', $related_id, $noderel_id);
    }
    
    /**
    * Broker function for writing relationships.
    * This function loops through the related array and calls the _clear_relationship() and write_relationship() 
    * methods which actually performs the db queries.
    * @return boolean
    * @access private
    */
    function _relate() 
    {            
        if ($this->debug) 
        {
            pxdb::print_debug('Setting Relationships...', 'bookmark'); 
        }
        
        $failure = false;
            
        if (empty($this->_related)) 
        {
            // nothing to relate, why continue?
            if ($this->debug) 
            {
                pxdb::print_debug('Relationships empty');
            }
            return (!$failure);
        }
                
        // SFS once again with the while
        while (list(, $rel) = each($this->_related))
        {
            $this->_clear_relationship($rel['reltype'], $rel['related_id'], $rel['relid']);
            
            if (!$this->_write_relationship($rel['reltype'], $rel['related_id'], $rel['relid'])) 
            {
                $failure = true;
            }
        } // end while
        reset($this->_related); // reset because it's an object property
        return (!$failure);
    }
    
    /**
    * Clears a relationship from the database.
    * This is only for relationships which are being set through the HTML form -- which means right now
    * only primaryrel and pickrel actually can be cleared.
    * @param string $reltype This is text describing the relationship: 'primaryrel', 'pickrel', 'noderel', 'secondaryrel'
    * @param mixed $related_id This is the record ID (or array of IDs) we are relating the current record to.
    * @param int $relid This is the relid (if type is primaryrel) or the f_id (if pickrel).
    * @access private
    * @return boolean
    */    
    function _clear_relationship($reltype, $related_id, $relid = null) 
    {
        if (!$this->record_id) 
        {
            trigger_error("Cannot clear a relationship without having instantiated with a record id or already invoked save() method for new record (in the case of an INSERT).", E_USER_ERROR);
            return false;
        }
        
        switch($reltype) 
        {
            case "primaryrel":

                $sql = 'DELETE FROM primaryrel WHERE relid = '.$relid.' AND (id1 = '.$this->record_id.' OR id2 ='. $this->record_id.')';
                if ($this->debug) 
                {
                    pxdb::print_debug($sql, 'db-update'); 
                }
                $this->db->Execute($sql);

            break;
            
            case "pickrel":
                        
                $sql = 'DELETE FROM pickrel WHERE recid = '.$this->record_id.' AND f_id = '.$relid;
                if ($this->debug) 
                {
                    pxdb::print_debug($sql, 'db-update'); 
                }
                $this->db->Execute($sql);
            
            break;

            case "secondaryrel":            
                // do nothing                
            break;

            case "child":
                // do nothing
            break;

            case "parent":
                // do nothing
            break;                    
        
        }
        return true;
    }    
    
    /**
    * Relates $this->record_id to a related id.
    * Note that the record must have already been written before this function can be called; i.e. if we are inserting
    * a record we need to know the record id before we can then relate that record.
    *
    * @param string $reltype This is text describing the relationship: 'primaryrel', 'pickrel', 'noderel', 'secondaryrel'
    * @param mixed $related_id This is the record ID (or array of IDs) we are relating the current record to.
    * @param int $relid This is the relid (if type is primaryrel) or the f_id (if pickrel).
    * @access private
    */
    function _write_relationship($reltype, $related_id, $relid = null) 
    {
        if (!$this->record_id) 
        {
            trigger_error("You cannot write a relationship without having instantiated with a record id or saved your new record (in the case of an INSERT).", E_USER_ERROR);
            return false;
        }
        
        if (!is_array($related_id)) 
        {
            $related_id = preg_split("/[,\s]/", $related_id); 
        }
        
        switch($reltype) 
        {
            case "primaryrel":
            
                // SFS while change
                while (list(, $id_piece) = each($related_id))
                {
                    if ($id_piece) 
                    {
                        // first select for existing matches before inserting
                        $sql = "SELECT COUNT(*) FROM primaryrel WHERE relid=$relid AND id1={$this->record_id} AND id2=$id_piece";
                        if (0 == $this->db->GetOne($sql)) 
                        {
                            $sql = "INSERT INTO primaryrel (relid, id1, id2) VALUES ($relid, {$this->record_id}, $id_piece)";
                            if ($this->debug) 
                            {
                                pxdb::print_debug($sql, 'db-update'); 
                            }
                            $this->db->Execute($sql);
                        }
                        $sql = "SELECT COUNT(*) FROM primaryrel WHERE relid=$relid AND id2={$this->record_id} AND id1=$id_piece";
                        if (0 == $this->db->GetOne($sql)) 
                        {
                            $sql = "INSERT INTO primaryrel (relid, id2, id1) VALUES ($relid, $this->record_id, $id_piece)";
                            if ($this->debug) 
                            {
                                pxdb::print_debug($sql, 'db-update'); 
                            }
                            $this->db->Execute($sql);
                        }
                    }
                } // end whie
                    
            break;
            
            case "pickrel":
            
                // SFS while
                while (list(, $id_piece) = each($related_id))
                {
                    if ($id_piece) 
                    {
                        // f_id, p_id, recid
                        $sql = "SELECT COUNT(*) FROM pickrel WHERE f_id=$relid AND recid={$this->record_id} AND p_id=$id_piece";
                        if (0 == $this->db->GetOne($sql))
                        {
                            $sql = "INSERT INTO pickrel (f_id, recid, p_id) VALUES ($relid, {$this->record_id}, $id_piece)";
                            if ($this->debug) 
                            {
                                pxdb::print_debug($sql, 'db-update'); 
                            }
                            $this->db->Execute($sql);
                        }
                    }
                } // end while
                
            break;
            
            case "child":

                // Adds related records as children
                // SFS while change
                while (list(, $id_piece) = each($related_id))
                {
                    if ($id_piece)
                    {
                        $sql = "SELECT COUNT(*) FROM noderel WHERE child=$id_piece AND parent={$this->record_id}";
                        // SFS added to support noderel IDs
                        if (is_numeric($relid)) 
                        {
                            $sql .= " AND noderelid = $relid";
                        }
                        
                        if (0==$this->db->GetOne($sql))
                        {
                            // SFS added to support noderel IDs
                            if (is_numeric($relid))
                            {
                                $sql = "INSERT INTO noderel (child, parent, noderelid) VALUES ($id_piece, {$this->record_id}, $relid)";
                            }
                            else
                            {
                                $sql = "INSERT INTO noderel (child, parent) VALUES ($id_piece, {$this->record_id})";
                            }
                            
                            if ($this->debug)
                            {
                                pxdb::print_debug($sql, 'db-update'); 
                            }
                            $this->db->Execute($sql);
                        }
                    } // end if
                } // end while
                $this->send_event_notification(SYNTAX_EVENT_PUBLISH);

            break;

            
            case "parent":

                // Adds related records as parent
                // SFS while change
                while (list(, $id_piece) = each($related_id))
                {
                    if ($id_piece)
                    {
                        $sql = "SELECT COUNT(*) FROM noderel WHERE parent=$id_piece AND child={$this->record_id}";
                        // SFS added to support noderel IDs
                        if (is_numeric($relid)) 
                        {
                            $sql .= " AND noderelid = $relid";
                        }
                        
                        if (0==$this->db->GetOne($sql))
                        {
                            // SFS added to support noderel IDs
                            if (is_numeric($relid))
                            {
                                $sql = "INSERT INTO noderel (parent, child, noderelid) VALUES ($id_piece, {$this->record_id}, $relid)";
                            }
                            else
                            {
                                $sql = "INSERT INTO noderel (parent, child) VALUES ($id_piece, {$this->record_id})";
                            }
                            
                            if ($this->debug)
                            {
                                pxdb::print_debug($sql, 'db-update');
                            }
                            $this->db->Execute($sql);
                        }
                    } // if id_piece
                }
                $this->send_event_notification(SYNTAX_EVENT_PUBLISH);

            break;
            
            case "secondaryrel":
            
                // basically primaryrel w/o the relid
                // SFS while change        
                while (list(, $id_piece) = each($related_id))
                {
                    if ($id_piece) 
                    {                    
                        $sql = "SELECT COUNT(*) FROM secondaryrel WHERE id1={$this->record_id} AND id2=$id_piece";                        
                        if (0 == $this->db->GetOne($sql)) 
                        {
                            $sql = "INSERT INTO secondaryrel (id1, id2) VALUES ({$this->record_id}, $id_piece)";
                            if ($this->debug) 
                            {
                                pxdb::print_debug($sql, 'db-update'); 
                            }
                            $this->db->Execute($sql);
                        }
                        
                        $sql = "SELECT COUNT(*) FROM secondaryrel WHERE id2={$this->record_id} AND id1=$id_piece";
                        if (0==$this->db->GetOne($sql)) 
                        {
                            $sql = "INSERT INTO secondaryrel (id2, id1) VALUES ($this->record_id, $id_piece)";
                            if ($this->debug) 
                            {
                                pxdb::print_debug($sql, 'db-update'); 
                            }
                            $this->db->Execute($sql);
                        }
                    } // if $id_piece
                }
                                
            break;
                                
        }
        return true;
    }    
    
    /**
    * Performs the update / insert query.
    * @return boolean true/false on success.
    * @access private
    */
    function _write()
    {
        if (true == $this->debug)
        {
            pxdb::print_debug('Writing Data...', 'bookmark'); 
        }
        
        // writes contents of $record to records table                
        $fields = array_keys($this->_record);        
        $values = array_values($this->_record);
        $now = time(); // this is the timestamp we will insert for modified column (and created column if this is an INSERT).
        
        // get approval information        
        $approved = 0; // default
        
        if ($this->prefs->get_pref('approve_disable'))
        {            
            $approved = 1;
        } 
        else if ($approve_auto_ugids = $this->prefs->get_array('approve_auto', $this->datatype)) 
        {
            if ($this->auth) 
            {
                $ugidarr = $this->auth->ugidarray;
            }
            else
            {
                $ugidarr = array(0); // the default ugid
            }
            
            if (sizeof(array_intersect($approve_auto_ugids, $ugidarr))) 
            {
                $approved = 1;    
            }                
        }
        
        if ($this->record_id) 
        {
            $this->save_type = 'update'; // public attribute -- for post-procesing
            // update            
            $sql = "UPDATE records SET modified = NOW(), approved = $approved\n\t";
            // OAM: fixed so that update does not fail if you only want to update
            // a pickrel or primaryrel field
            if (0 < sizeof($fields))
            {
                $sql .= ', ';
                for ($c=0; $c < sizeof($fields); $c++) 
                {            
                    $sql_bit[] = "$fields[$c] = $values[$c]"; 
                }
                
                $sql .= join(',  ' , $sql_bit);
            }
                        
            $sql .= " WHERE id = ".$this->record_id;
                    
            if ($this->debug)
            {
                pxdb::print_debug($sql, 'db-update');    
            }                
        
            $this->db->Execute($sql);            
        }
        else 
        {            
            $this->save_type = 'insert'; // for post-procesing
            
            // insert            
            // error if there is no datatype        
            if (!$this->datatype) 
            { 
                trigger_error("Cannot insert a record without a datatype.", E_USER_WARNING);
                return false;
            }
            
            $baseFields = array(
                'created'=> 'NOW()',
                'modified'=> 'NOW()',
                'datatype'=>$this->datatype,
                'approved'=>$approved,
           );
            
            // SFS while change
            while(list($field, $value) = each($baseFields))
            {
                if (!in_array($field, $fields))
                {
                    $fields[] = $field;
                    $values[] = $value;
                }
            }
            $sql = "INSERT INTO records (" . implode(", ", $fields) . ") "
                    . " VALUES (" . implode(", ", $values) . ")";
            
            if ($this->debug)
            {
                pxdb::print_debug($sql, 'db-update'); 
            }
            
            $this->db->Execute($sql);
            $this->set_id($this->db->Insert_ID());
            
            // insert privs
            $this->_add_privs();
        }
        return $this->get_id();
    }
    
    /**
    * Approve a record.
    * Tests to make sure that bound user has write access to this record & appropriate priv
    * level to approve records of this datatype.  -- Assuming yes, sets the approved column to 1.
    * @return boolean, false on error
    * @access public
    */
    function approve() 
    {
        return $this->set_approval_status(1);
    }

    /**
    * Un-approve a record.
    * Tests to make sure that bound user has write access to this record & appropriate priv
    * level to approve records of this datatype.  -- Assuming yes, sets the approved column to 0.
    * @return boolean, false on error
    * @access public
    */
    function unapprove() 
    {
        return $this->set_approval_status(0);
    }
    
    /**
    * Sets the approval bit for a record.
    * @param boolean $bit Approval bit for record.
    * @access public
    */
    function set_approval_status($bit)
    {
        if (!$this->record_id) 
        {
            trigger_error("Cannot approve/unapprove a record without knowing the record id.", E_USER_ERROR);
            return false;
        }
        
        if ($this->is_rlevel_authorized ($this->record_id, 'write'))
        {
            if ($this->auth && $this->auth->not_admin()) 
            {
                if ($this->auth) 
                {
                    $ugidarr = $this->auth->ugidarray;
                }
                else
                {
                    $ugidarr = array(0); // the default ugid
                }
                            
                // OAM: replaced old code that used preferences to restrict approval to ugid
                // with datatype_privs instead
                //
                // DATATYPE PRIVS
                pxdb_import('metadata.pxdb_datatype_privs');    
                $datatype_privs = new pxdb_datatype_privs($ugidarr);
                
                if (empty($this->datatype))
                {
                    $this->datatype = $this->lookup_datatype($this->record_id);
                }
                
                if (!$datatype_privs->is_allowed($this->datatype, DATATYPE_PRIV_EDIT))
                {                    
                    trigger_error("User is not authorized to approve/unapprove this record (you must set datatype priv).", E_USER_WARNING);
                    return false;
                }
                    
            } // if auth is set && is_admin is false
                
            // otherwise
            // apparently this record does not requre approval, so we'll just 
            // allow it to be approved.
                                                        
            $sql = "UPDATE records SET approved = $bit WHERE id = $this->record_id";
            
            if ($this->debug) 
            {
                pxdb::print_debug($sql, 'db-update'); 
            }
            $rs = $this->db->Execute($sql);        
                            
            if (1 == $bit)
            {                                        
                $this->send_event_notification(SYNTAX_EVENT_APPROVE);
            } 
            else 
            {
                $this->send_event_notification(SYNTAX_EVENT_UNAPPROVE);                    
            }
            
            return ($rs != false);
        }
        else
        {
            trigger_error("User is not authorized to write to this record.", E_USER_WARNING);
            return false;
        }
    }
    
    /**
    * Add (or sets) a field to be written to db.
    * This is for manually writing a record to the database -- bypassing any input forms.  Note
    * that unless you set a record_id, it is assumed that you are inserting a new record.
    * The datatype must be set before this function is invoked. -- You can do this with the set_datatype()
    * method -- or it will happen automatically if you are editing an existing record.
    * 
    * You may gets strange behavior if you are trying to overwrite a value set via a form and you do not 
    * use an identifier to specify the field. 
    * 
    * @param string $field The fname -- or identifier -- of column we are adding.
    * @param mixed $value The value for the new parameter..
    * @param boolean $quote Whether to quote the string (assumes that all values should be quoted)
    * @return boolean true/false on success.
    * @access public
    */
    function set_field($field, $value, $quote = true)
    {
        // Has a datatype been defined?        
        if (!$this->datatype) 
        {
            trigger_error("You must first define a datatype before adding fields.", E_USER_WARNING);
            return false;
        }
        
        // Get the typesfield based on the passed field name & the current datatype.
        $tfid = $this->lookup_tfid($field);
                
        if ($tfid) 
        {                    
            $objf =& new pxdb_typesfield($tfid);

            if ('relationship' == $objf->get_field_type())
            {
                if ($this->debug) 
                {
                    pxdb::print_debug("Setting primaryrel relationship for recid $value."); 
                }        
                // add value as ID for related record
                $relA = array('reltype'=>'primaryrel', 'related_id'=>$value, 'relid'=>$objf->get_relid());
                $this->_related[] = $relA;                
            } 
            else 
            {            
                $dbf = new pxdb_dbfield($objf->get_fieldid());
                
                if ($dbf->get_field_type() == 'pickid') 
                {
                    // SFS - take out recording single pickids in records table
                    $tmpA['reltype'] = 'pickrel';
                    $tmpA['relid'] = $dbf->get_id(); // fid in this case
                    $tmpA['related_id'] = $value; // could be an array also                    
                    $this->_related[] = $tmpA;                    
                } 
                else 
                {
                    // it is a typesfield, but not a pickid
                    $this->_record[$dbf->get_fname()] = ($quote ? $this->db->Quote($value) : $value);
                }
            }    
        } 
        else 
        {
            // not a typesfield     -- so we can't add it.            
            trigger_error("Cannot set field $field (most likely it is not defined in datatype $this->datatype).", E_USER_WARNING);
            return false;            
        }
    }
    
    /**
    * Checks to see if current user has specified access to specified record.
    * @param integer $record_id The ID of the record we are checking.
    * @return boolean Whether or not current user has specified privs.
    * @access public
    */
    function is_rlevel_authorized($record_id) 
    {    
        if ($this->auth) 
        {
            // if authentication module is bou
            if ($this->auth->is_admin()) 
            {
                return true;
            } 
            else 
            {
                return pxdb_privs::is_writeable('record', $record_id, $this->auth->get_sql_ugids());
            }
        } 
        elseif (!$this->auth_required) 
        {        
            return true;            
        }
        else
        {        
            return false; // denied
        }            
    }
    
    /**
    * Checks to see if current user has specified access to specified field.
    * @param integer $field_id The typesfield id of the field we are checking.
    * @return boolean Whether a user has specified access to specified field
    * @access public
    */
    function is_flevel_authorized($field_id) 
    {
        if ($this->auth) 
        {
            return pxdb_privs::is_writeable('field', $field_id, $this->auth->get_sql_ugids());
        } 
        elseif (!$this->auth_required) 
        {
            return true;
        } 
        else 
        {
            // denied
            return false;
        }
    }

    /**
    * Sets the priv that is assigned by default to everyone.
    * If empty, then no priv w/ ugid '0' will be inserted.  The default is 'read,write'.
    * @param mixed $priv String or array containing the privileges to set.
    * @access public
    */
    function set_default_priv($priv) 
    {
        $priv = (is_array($priv) ? implode(",", $priv) : $priv);        
        $this->default_priv = $priv;
    }
    
    /**
    * Adds privs to 'priv' table on insert of new record.
    * @access private
    */    
    function _add_privs() 
    {
        // 1) add for current user
        if ($this->auth) 
        {            
            $this->grant_priv($this->auth->get_uid(), 'read,write');
        }
        
        // 2) grant read/write to admin
        if ($admin_groups = $this->prefs->get_array('gid_admin')) 
        {
            while (list(, $gid) = each($admin_groups))
            {
                if ($gid !== '') 
                {
                    $this->grant_priv($gid, 'read,write');
                }                
            }
        }

        // 3) grant read/write access to everyone
        if ($this->default_priv) 
        {
            $this->grant_priv('0', $this->default_priv);
        }
    }

    /**
    * Function to grant privileges to a record.
    *
    * @param int $ugid The user/group id (from records table)
    * @param mixed $priv A string or array containing privilegs e.g. 'read,write' or array('read', 'write').
    * @return boolean True or False (with $errors[] added)
    * @access public
    */
    function grant_priv($ugid, $priv) 
    {        
        // knows which field we are dealing with from $_imported array.                
        if (!is_array($priv)) 
        {
            $priv = explode(',',$priv); 
        } //normalize
        
        if (!$priv) 
        {        
            trigger_error("You must give either read or write privs to the user/group you just added.", E_USER_WARNING, 'pxdb_commi::grant_priv()');
            return false;                
        } 
        else 
        {
            // OAM: should this check what revoke returns and return true false based on that?            
            pxdb_privs::grant('record', $this->record_id, $ugid, $priv);
            return true;
        }    
    }

    /**
    * Function to revoke privileges to a record
    * 
    * By default it uses values in $_imported array, but can also be passed parameters -- for 
    * revoking privilegs internally.
    *
    * @param int $ugid The user/group id (from records table)
    * @return boolean True or False (with $errors[] added)
    * @access public
    */
    function revoke_priv($ugid) 
    {        
        // OAM: should this check what revoke returns and return true false based on that?
        pxdb_privs::revoke('record', $this->record_id, $ugid);    
        return true;    
    }
    
    /**
    * Removes related [primaryrel] records from the database.
    * This function must be called
    * manually. -- or you must override this class and redefine delete() to call this function *first*. 
    * @param mixed $relid The integer relid or array of relids (only applicable to primaryrel relationships).
    * @param mixed $dt The datatype of related records.
    * @return boolean true/false on success.
    * @access public
    */
    function delete_related($relid, $dt = null) 
    {
        // normalize
        if (!is_array($relid)) 
        {
            $relid = preg_split("/[,\s]/", $relid); 
        }
        
        if (!is_array($dt)) 
        {
            $dt = preg_split("/[,\s]/", $dt); 
        }
        
        $dt_list = implode(',', $dt);
        
        if (!$this->record_id) 
        {
            trigger_error("Cannot delete related items without knowing record id.", E_USER_ERROR);
            return false;
        }
                
        // SFS while
        while (list(, $rel) = each($relid))
        {
            $sql = "
SELECT DISTINCT
    id2
FROM
    primaryrel prel
INNER JOIN
    records r ON r.id = prel.id2 " . ($dt ? " AND r.datatype IN ($dt_list)" : "") . "
WHERE
    prel.relid = $rel
    AND prel.id1 = $this->record_id";
                    
            if ($this->debug)
            {
                pxdb::print_debug($sql, 'db-select'); 
            }
            
            $rs =& $this->db->Execute($sql);
            
            if ($this->is_rlevel_authorized($rs->fields['id2'], 'write')) 
            {
                $sql = "DELETE FROM records WHERE id = ".$rs->fields['id2'];
                if ($this->debug) 
                {
                    pxdb::print_debug($sql, 'db-update'); 
                }
                $this->db->Execute($sql);
            } 
            else
            {
                trigger_error("User is not authorized to delete related record: ".$rs->fields['id2'], E_USER_WARNING);
                return false;
            }
        }
        return true;
    } // function
    
    /**
    * Removes related [secondaryrel] records from the database.
    * This function must be called
    * manually. -- or you must override this class and redefine delete() to call this function *first*. 
    * @param mixed $dt The integer relid or array of relids (only applicable to primaryrel relationships).
    * @return boolean true/false on success.
    * @access public
    */
    function delete_secondaryrel_related($dt = null) 
    {
        if (!is_array($dt)) 
        {
            $dt = preg_split("/[,\s]/", $dt); 
        }        
        
        $dt_list = implode(',', $dt);
        
        if (!$this->record_id) 
        {
            trigger_error("Cannot delete related items without knowing record id.", E_USER_ERROR);
            return false;
        }
    
        $sql = "
SELECT DISTINCT
    id2
FROM
    primaryrel prel 
INNER JOIN
    records r ON r.id = prel.id2 " . ($dt ? " AND r.datatype IN ($dt_list)" : "") . "
WHERE
    prel.id1 = $this->record_id";
                
        pxdb::print_debug($sql, 'blue');
        
        $rs =& $this->db->Execute($sql);
        
        if ($this->is_rlevel_authorized($rs->fields['id2'], 'write')) 
        {        
            $sql = "DELETE FROM records WHERE id = ".$rs->fields['id2'];
            if ($this->debug) 
            {
                pxdb::print_debug($sql, 'db-update'); 
            }
            $this->db->Execute($sql);        
        } 
        else
        {
            trigger_error("User is not authorized to delete related record: ".$rs->fields['id2'], E_USER_WARNING);
        }            
    }
    
    /**
    * Deletes children elements from noderel and [optionally] from records tables.
    * Note that this will recursively delete *all* rows from noderel that are subordinate (children) nodes
    * of this record in the noderel table.  This is different from standard delete() method in that, the standard method
    * only removes the top node, essentially shifting everything up in the heirarchy.  You may also pass 
    * a non false variable  (i.e. type matters) as the first paramter to tell it to also purge the
    * corresponding rows from records table. (dangerous!)
    * relationship
    * @param boolean $purge_records Whether to remove records from records table (in addition to noderel).
    * @param int $parent Used by the recursion to track the current $parent. Defaults to $this->record_id if false.
    * @return Array hash containing number of rows deleted (affected) for each table.
    * @access public
    */
    function delete_children($purge_records = false, $parent = null) 
    {
        if (!$this->record_id) 
        {
            if ($this->is_rlevel_authorized($this->record_id, 'write')) 
            {
                triger_error("User is not authorized to delete children of this record.", E_USER_ERROR);
                return false;
            }
        }

        if (null == $parent)
        {
            $parent = $this->record_id; 
        }        
        
        $sql = "SELECT * FROM noderel WHERE parent = $parent";
        $child_rs = $this->db->Execute($sql);
        
        while(!$child_rs->EOF) 
        {
            // 1) recurse looking for children of the given record.
            if (!$this->delete_children($purge_records, $child_rs->fields['child'])) 
            {
                // if we encounter an error  anywhere along the way we want to stop executing
                return false;
            }
            
            // at this point we assume that the function has gone down and finally exited after finding
            // no children
            // so, we delete the current child record
            $sql = "DELETE FROM noderel WHERE child = ".$child_rs->fields['child'];
            if ($this->debug) 
            {
                pxdb::print_debug($sql, 'db-update'); 
            }
            $this->db->Execute($sql);
                
            // if purge_records then delete related records too
            
            if ($purge_records) 
            {
                if ($this->is_rlevel_authorized($child_rs->fields['child'], 'write')) 
                {
                    $sql = "DELETE FROM records WHERE id = ".$child_rs->fields['child'];
                    if ($this->debug) 
                    {
                        pxdb::print_debug($sql, 'db-update'); 
                    }
                    $this->db->Execute($sql);                            
                } 
                else 
                {
                    trigger_error("You are not authorized to delete [child] record: ".$child_rs->fields['child'], E_USER_WARNING);
                    return false;
                }    
            }
            $child_rs->MoveNext();
        }
        return true;
    }
    
    /**
    * Deletes the record and rows from relationships tables.
    * This method does not delete any related or children records.  If you want to do this, you must either
    * call those delete methods individually, or extend this class and override this method to call those delete
    * methods before calling parent::delete().
    * @access public
    * @return mixed Boolean false on failure, array of affected rows on success
    */ 
    function delete() 
    {
        if (!$this->record_id) 
        {
            trigger_error("Cannot delete a record without knowing the record id.", E_USER_ERROR);
            return false;
        }
        
        $affected = array();
            
        if ($this->is_rlevel_authorized ($this->record_id, 'write')) 
        {
            // send event notification
            $this->send_event_notification(SYNTAX_EVENT_PREDELETE);
            
            $dt_this = $this->get_datatype();
            $dt_user = $this->prefs->get_array('dt_user');
            $dt_group = $this->prefs->get_array('dt_group');
            
            if (in_array($dt_this, array_merge($dt_user, $dt_group))) 
            {
                if (in_array($dt_this, $dt_user)) 
                {
                    $col = 'idnum';
                    $val = 'idnum';
                } 
                elseif (in_array($dt_this, $dt_group)) 
                {
                    $col = 'groups';
                    $val = 'name';
                }    
                
                $thisRec = new pxdb_record($this->get_id());
                ${$val} = $thisRec->get_field($val);
                
                $sql = "DELETE FROM usergroup WHERE $col = ".$this->db->qstr(${$val});
                $this->db->Execute($sql);
                $affected['usergroup'] = $this->db->Affected_Rows();
            }                    

            // $this->record_id is the record we are deleting.

            $sql = "DELETE FROM records WHERE id = $this->record_id";
            if ($this->debug) 
            {
                pxdb::print_debug($sql, 'db-update'); 
            }
            $this->db->Execute($sql);
            $affected['records'] = $this->db->Affected_Rows();

            // primaryrel
            $sql = "DELETE FROM primaryrel WHERE id1 = $this->record_id OR id2 = $this->record_id";
            if ($this->debug) 
            {
                pxdb::print_debug($sql, 'db-update'); 
            }
            $this->db->Execute($sql);
            $affected['primaryrel'] = $this->db->Affected_Rows();

            // secondaryrel
            $sql = "DELETE FROM secondaryrel WHERE id1 = $this->record_id OR id2 = $this->record_id";
            if ($this->debug) 
            {
                pxdb::print_debug($sql, 'db-update'); 
            }
            $this->db->Execute($sql);
            $affected['secondaryrel'] = $this->db->Affected_Rows();

            // noderel
            $sql = "DELETE FROM noderel WHERE parent = $this->record_id";
            if ($this->debug) 
            {
                pxdb::print_debug($sql, 'db-update'); 
            }
            $this->db->Execute($sql); 
            $affected['noderel'] = $this->db->Affected_Rows();

            // pickrel
            $sql = "DELETE FROM pickrel WHERE recid = $this->record_id";
            if ($this->debug) 
            {
                pxdb::print_debug($sql, 'db-update'); 
            }
            $this->db->Execute($sql);        
            $affected['pickrel'] = $this->db->Affected_Rows();

            // priv            
            pxdb_privs::revoke('record', $this->record_id);
            
            // delete any uploaded files
            $filestore_paths = $this->_get_filestore_paths();
            if (false !== $filestore_paths)
            {
                // if we have path(s), loop through and look for matching files
                while (list(, $path) = each($filestore_paths))
                {
                    $files = glob($path . '/' . $this->record_id . '_*');
                    if (false !== $files)
                    {
                        // if files are found, loop through and delete
                        while (list(, $file) = each($files))
                        {
                            if (false === unlink($file))
                            {
                                trigger_error('Failed to delete ' . $file, E_USER_WARNING);
                            }  /// end if file not deleted
                        } // end while
                    } // end if files returned
                } // end loop through filestore paths
            } // end if filestore paths set
            
            // send event notification
            $this->send_event_notification(SYNTAX_EVENT_DELETE);
            
            return $affected;
        }
        else 
        {
            trigger_error("User is not authorized to delete this record.", E_USER_WARNING);
            return false;
        }
    }
    
    /**
    * Sends the specified trigger, and relevant record information to the event dispatcher
    *
    * @param string trigger name
    * @access private
    */
    function send_event_notification($trigger)
    {
         $syntax_event_dispatcher =& pxdb_event_dispatcher::singleton();
         $params = array(
            'id' => $this->record_id,
            'datatype' => $this->datatype,
         );
         $syntax_event_dispatcher->notify($trigger, $params);
    }
} // class
?>