<?php
// $Id: pxdb_confront.class.php,v 1.6 2005/10/17 05:40:30 sandys1 Exp $

pxdb_import('content.pxdb_data');

/**
* This class contains common functionality for classes in confront subpackage.
* @package pxdb.content.input
* @author Hans Lellelid <hans@velum.net>
* @access public
* @version 1.0.1
*/
class pxdb_confront extends pxdb_data
{
    /**
    * Vars imported from global / $_REQUEST scope.
    * @var array
    * @access private
    */
    var $_imported;

    /**
    * Validator instance.
    * @var pxdb_validator
    * @access public
    */
    var $validator;
    
    /**
    * Stores the form fields that are of "upload" type.
    * @var array
    * @access private
    */
    var $_upload_fields;

    /**
    * pxdb parameters
    * @var array
    * @access private
    */
    var $_params;

    /**
    * Paramters for widgets (from pxdb[widget] array).
    * @var array
    * @access private
    */
    var $_widget_params;
        
    /**
    * pxdb data
    * @var array
    * @access private
    */
    var $_data;

    /**
    * The template for form element names.
    * @var string
    * @access public
    */
    var $data_name_structure;

    /**
    * The template for form parameters.
    * @var string
    * @access public
    */
    var $param_name_structure;
        
    /**
    * The template for widget form element names.
    * @var string
    * @access public
    */
    var $widget_param_name_structure;

    /**
    * Calls parent constructor.
    * @access public
    */
    function pxdb_confront() 
    {    
        parent::pxdb_data();

        // prefs class is heavily used here, so set the instance here
        $this->prefs =& pxdb_prefs::singleton();
        
        // import data
        $this->import();
        
        // default _name_structure values
        $this->data_name_structure = PXDB_GLOBAL_VAR . '[data][{name}]';
        $this->param_name_structure = PXDB_GLOBAL_VAR . '[params][{name}]';
        $this->widget_param_name_structure = PXDB_GLOBAL_VAR . '[widget][{name}]';        
    }

    /**
    * Builds parameter form element name based on passed varname.
    * @param string $type The type of element this is: e.g. data, param, widget_param
    * @param string $name The name to insert into the template.
    * @return string
    * @access private
    */
    function _get_form_element_name($type, $name) 
    {
        $structvar = $type . '_name_structure';
        
        if ($this->$structvar)
        {
            return str_replace("{name}", $name, $this->$structvar);
        }
        else
        {
            return $name;        
        }
    }

    /**
    * Invoke a user-registered method.
    * Currently simply a wrapper for PHP's call_user_func() function.
    * @param mixed $function The function name or array of object reference and method name.
    * @param mixed $data The data that needs to be passed to the function.
    * @param object $object Some Hans idea of something to pass.
    * @return string The widget created by function call.
    * @access private
    */
    function _invoke_user_method($function, $data, &$object) 
    {
        return call_user_func($function, $data, $object);        
    }

    /**
    * Function that imports data from global namespace into class.
    * @param array $source The location of source data to use. (for upward compatibility with pxdb::import())
    * @access public
    */
    function import($source = null) 
    {
        // perform basic import function
        $this->_imported = pxdb::import($source);
        
        // the pxdb::import() method has already done the work
        // pxdb_adm / pxdb_input uses a convention of separating
        // between params and data arrays.
        if (isset($this->_imported['params']))
        {
            $this->_params = (array) $this->_imported['params'];
        }
        
        if (isset($this->_imported['data']))
        {
            $this->_data = (array) $this->_imported['data'];
        }
        
        if (isset($this->_imported['widget']))
        {
            $this->_widget_params = (array) $this->_imported['widget'];
        }
        // note _import_uploaded_files() is called only by subclasses that 
        // need that functionality (e.g. pxdb_widgest and pxdb_validator DO NOT)
    }

    /**
    * Imports special exception data related to file uploads.
    * This function is used both by pxdb_commit and pxdb_input (on page reload any specified
    * files are sent to the server).
    * @access private
    * @return boolean Success or failure
    */
    function _import_uploaded_files() 
    {        
        // Make sure prefs are set.        
        if (defined('PXDB_FILESTORE_PATH'))
        {
            $prefix_path = PXDB_FILESTORE_PATH;
        }
        
        if (!$prefix_path) 
        {
            trigger_error("You have not set the pxdb_pref for datastore[path] or datastore[url]. File uploads will not work.", E_USER_WARNING);
        } 
        else 
        {
            $prefix_path .= '/tmp';
            if(!is_dir($prefix_path))
            {
                if (! @mkdir($prefix_path, 0777)) 
                {                   
                    trigger_error("Unable to create $prefix_path directory.", E_USER_WARNING);
                }
            }
        }

        // Grab uploaded files
        if ($_FILES)
        {
            $i = 0;
            foreach ($_FILES['pxdb']['name']['data'] as $var => $file_name) 
            {
                if ($file_name) 
                {
                    // move to new templorary location
                    // (the value from $pxdb array is the temporary location of file)

                    // build new location
                    $tmp_path = $prefix_path . '/' . $i . $file_name;

                    if ($this->debug)
                    {
                        pxdb::print_debug("Moving ".$_FILES['pxdb']['tmp_name']['data'][$var]." to $tmp_path");
                    }

                    if (!move_uploaded_file($_FILES['pxdb']['tmp_name']['data'][$var], $tmp_path) 
                        && !file_exists($tmp_path)) 
                    {
                          pxdb::trigger_error("Could not move ".$_FILES['pxdb']['tmp_name']['data'][$var]." to $tmp_path", E_USER_WARNING);
                          return false;
                    }

                    // othewise assume it worked
                    $file_data = array();

                    // build an array of all info for the uploaded file
                    $file_data['tmp'] = $tmp_path; // use the new path
                    $file_data['name'] = $_FILES['pxdb']['name']['data'][$var];
                    $file_data['type'] = $_FILES['pxdb']['type']['data'][$var];
                    $file_data['size'] = $_FILES['pxdb']['size']['data'][$var];

                    // remove the column from input array
                    unset($this->_data[$var]);

                    // new upload files *should* have the new_ prefix, which must be removed.
                    $new_fname = preg_replace("/^new_/", "", $var);

                    // now reset the value at $this->_imported[$var] to new array
                    $this->_data[$new_fname] = $file_data;
                }
                else
                {
                    // (no file uploaded)
                    // remove unused key
                    unset($this->_data[$var]); // unset the original new_file value.
                }
                $i++;
            }
        } // if
        return true;
    }

    /**
    * This function is for adding a form validating class to the input form / commit class.
    * 
    * The class must have been instantiated, and must conform to a standard pxdb_validator interface.
    * This interface can be seen in the pxdb_validator class, which you may wish to extend / override.
    *
    * @param object $validator The class used to validate data (must conform to interface used above);
    * @return boolean
    * @access public
    */
    function bind_validator(&$validator) 
    {
        // the class check must have been instantiated
        $this->validator = &$validator;
                
        if ($validator !== null)
        {
            // this allows one to bind nothing to remove the validator
            // this is important since there is a default validator
            // automatically instantiated.  If you really want no validation
            // you need to bind FALSE as a validator.
            if ($missing = array_diff(array('validate', 'process', 'get_error'), get_class_methods(get_class($this->validator))))
            {            
                pxdb::trigger_error("You did bind a valitation class, but it apparently doesn't have the required methods: " . implode(", ", $missing), E_USER_ERROR);
                return false;
            }
            else
            {
                return true;
            }
        }
    }
} // class
?>