<?php
// $Id: pxdb_input.class.php,v 1.16 2005/10/17 05:40:30 sandys1 Exp $

pxdb_import('metadata.pxdb_typesfield');
pxdb_import('metadata.pxdb_dbfield');
pxdb_import('metadata.pxdb_viewfield');
pxdb_import('content.input.pxdb_confront');
pxdb_import('content.input.pxdb_widgets');
pxdb_import('content.input.pxdb_validator');

/**
* This is the class that is responsible for creating pxdb input forms.
*
* This class displays an input form for entering data into a datatype or a "view" (subset of a datatype, with
* customized labels).  This class uses Smarty templates to format the display of the form.  It also reads in
* the global variable (default: $pxdb) to gather any validation errors that were added by the pxdb_commit class.
* All fields are displayed according to the privileges defined in the field_priv table.  Additionally, fields
* may be hidden or locked (made read-only) by the hide_field() and lock_field() methods, but fields hidden or
* locked by insufficient privileges may not be revelaed or made writable.  You may also disable auto-generation
* (i.e. make auto-generated fields editable) on a field-by-field basis or on a global basis using the
* disable_autogen() method.
*
* @package pxdb.content.input
* @author Hans Lellelid
* @version $Revision: 1.16 $
* @access public
*/
class pxdb_input extends pxdb_confront
{
    /**
    * Is authentication required?
    * @var boolean
    * @access public
    */
    var $auth_required = false;

    /**
    * Current record id.
    * @var integer
    * @access public
    */
    var $record_id;

    /**
    * JavaScript that goes into form submit button.
    * @var string
    * @access public
    */
    var $submit_jscript = '';

    /**
    * Current request variables.
    * @var array
    * @access public
    */
    var $current_vars;

    /**
    * pxdb paramter variables.
    * @var array
    * @access public
    */
    var $param_vars;

    /**
    * The name of the form.
    * @var string
    * @access private
    */
    var $form_name;

    /**
    * The action of the form.
    * @var string
    * @access private
    */
    var $form_action;

    /**
    * @var array
    * @access private
    */
    var $_labels;

    /**
    * @var array
    * @access private
    */
    var $_descriptions;

    /**
    * @var array
    * @access private
    */
    var $_names;

    /**
    * @var array
    * @access private
    */
    var $_values;

    /**
    * @var array
    * @access private
    */
    var $_options;

    /**
    * holds a compiled form of all attributes for fields
    * @var array
    * @access private
    */
    var $_fields;

    /**
    * Holds the original record (if we are modifying a record).
    * @var array
    * @access private
    */
    var $_record;

    /**
    * Whether or not to save pxdb[preserved_vars] array.
    * @var boolean
    * @access public
    */
    var $preserve_vars = true;

    /**
    * Holds user-specified list of fields that are to be hidden.
    * @var array
    * @access public
    */
    var $hidden_fields;

    /**
    * Holds user-specified list of fields that are to be read-only ('locked').
    * @var array
    * @access public
    */
    var $locked_fields;

    /**
    * Holds user-specified list of autogen fields that are to be modified.
    * @var array
    * @access public
    */
    var $autogen_disabled_fields;

    /**
    * Whether auto-generation is disabled for the entire object.
    * @var boolean
    * @access public
    */
    var $autogen_disabled = false;

    /**
    * Holds widgets
    * @var array
    * @access public
    */
    var $widgets;
    
    /**
    * Label for submit button
    *
    * @var string
    * @access public
    */
    var $submit_label = 'Submit Form';
    
    /**
    * Constructor
    * @param integer $dt Datatype for this input class.
    * @param integer $record_id ID of existing record.
    */
    function pxdb_input($dt, $record_id = null)
    {
        parent::pxdb_confront();

        // set up datatype & record_id based on imported (or passed) vars.\
        $this->datatype = ($dt ? $dt : $this->_params['dt']);
        
        if ($record_id)
        {
            $this->record_id = $record_id;
        }
        elseif (isset($this->_params['id']))
        {
            $this->record_id = $this->_params['id'];
        }
        
        if (!$this->datatype) 
        {
            trigger_error("You cannot instantiate pxdb_input w/o specifying the datatype.", E_USER_ERROR, 'pxdb_input::pxdb_input()');
        }

        // initialize other arrays
        $this->hidden_fields = array();
        $this->locked_fields = array();
        $this->autogen_disabled_fields = array();

        // setup Smarty
        $this->tpl = &pxdb::tpl();
        
        // set some variables that will get put on the form
        $this->current_vars = $this->get_current_vars();
        $this->param_vars = $this->get_param_vars();

        // set a default form name & action
        $this->set_form_name('showform_' . $this->datatype);
        $this->set_form_action($_SERVER['PHP_SELF']);

        // populate the _record array from the database.
        $this->read();
    }

    /**
    * Updates class vars from the database.
    * Sets the contents of $_record[] if $record_id is set.
    * @access public
    */
    function read()
    {
        if ($this->record_id) 
        {
            $sql = "select * from records where id = $this->record_id";
            $record_rs = $this->db->Execute($sql);
            $this->_record = $record_rs->fields;
        }
    }

    /**
    * Imports variables and uploaded files from global scope.
    * @param array $source The location of source data to use. (for upward compatibility with pxdb::import())
    * @access public
    */
    function import($source = null)
    {
        parent::import($source);

        // uploaded files present somewhat of a special exception
        // that must be handled separately.
        $this->_import_uploaded_files();
    }

    /**
    * Specifies the widget class that this class should use.
    * The widget class must extend pxdb_widget and therefore must
    * supply all methods that pxdb_widget supplies. (No checking happens to ensure this.)
    * @param object pxdb_widgets $widgets pxdb_widgets Object
    * @access public
    */
    function bind_widgets(&$widgets)
    {
        $this->widgets = &$widgets;
    }


    /**
    * Set whether authentication is required.
    * If auth_required is true, then you will not be able display
    * a form without binding an authentication object.
    * @param $bit boolean value to set auth_requred to.
    * @access public
    */
    function set_auth_required($bit = true)
    {
        $this->auth_required = $bit;
    }

    /**
    * Sets a default value for a field.
    * @param string $field
    * @param mixed $value
    * @access public
    */
    function set_field_default($field, $value)
    {
        $tfid = $this->lookup_tfid($field);

        if ($tfid) 
        {
            $objf = &new pxdb_typesfield($tfid);
            $key = $this->_get_name($objf);

            if (!isset($this->_data[$key])) 
            {
                $this->_data[$key] = $value;
            }
        } 
        else 
        {
            trigger_error("Cannot find field: $field.", E_USER_WARNING);
        }
    }

    /**
    * Sets the pick options for a picklist (primaryrel or pickid type).
    * This function does NOT check to see that the field you are setting options for is a field that
    * actually can use options (e.g. a relationship or pick list).  $options value is expected to be an
    * array of hashes ['value'] and ['label'] or a simple array.
    * @param string $field The field name (identifier)
    * @param array $options Hash (value,label) of options to display.
    * @access public
    */
    function set_field_options($field, $options)
    {
        $tfid = $this->lookup_tfid($field);
        
        if ($tfid) 
        {
            $this->_options[$tfid] = $options;
        } 
        else 
        {
            trigger_error("Cannot find field: $field.", E_USER_WARNING, 'pxdb_input::set_field_options()');
        }
    }

    /**
    * Gets the current value(s) for the typesfield row.
    * First it checks to see if there is already a value set for this element,
    * e.g. a widget is re-loading the form, so the data is not written to the db,
    * but is being preserved in the form.
    *
    * @param object pxdb_typesfield $objf typesfield Object current form element.
    * @return mixed The value(s) for the specified field.
    * @access private
    */
    function _get_value(&$objf)
    {
        // value may have already been imported from global namespace
        $varname = $this->_get_name($objf);

        if (isset($this->_data[$varname])) 
        {
            if($this->debug) 
            {
                pxdb::print_debug("Found existing value for $varname: ".$this->_data[$varname], 'db-select');
            }

            $this->_values[$objf->get_id()] = $this->_data[$varname];
        }
        else
        {
            if ($this->debug) 
            {
                pxdb::print_debug("Did not find an existing value for $varname. ", 'bookmark');
            }

            if ($this->_record['id']) 
            {
                // i.e. they are modifying a record
                switch ($objf->get_field_type()) 
                {
                    // simple field or relationship?
                    case "simple":

                        $dbf = new pxdb_dbfield($objf->get_fieldid());

                        switch ($dbf->get_field_type()) 
                        {

                            case "pickid":

                                // SFS treat single and multiple the same
                                $sql = "SELECT p_id FROM pickrel WHERE f_id = ".$dbf->get_id()." AND recid = ".$this->_record['id'];
                                $this->_values[$objf->get_id()] = (array) $this->db->GetCol($sql);
                                
                            break;

                            case "upload":

                                // get 'name', 'type', 'size', and 'tmp' ? of upload file.

                                $upload_column = ($dbf->get_upload_column() ? $dbf->get_upload_column() : $dbf->get_fname());

                                $url = $this->_record[$upload_column];

                                if (!empty($url)) 
                                {
                                    // get file path
                                    $file_path = PXDB_FILESTORE_PATH . str_replace($this->prefs->get_pref('datastore', 'url'), '', $url);

                                    if (file_exists($file_path)) 
                                    {
                                        $file = array(
                                            'name' => basename($file_path),
                                            'size' => filesize($file_path),
                                            'type' => filetype($file_path)
                                       );
                                        $this->_values[$objf->get_id()] = $file;
                                    }
                                    else
                                    {
                                        // it's a URL or something
                                        $this->_values[$objf->get_id()] = $this->_record[$dbf->get_fname()];
                                    }

                                }
                            break;

                            default:
                            
                                // not a relationship, add to records table
                                if (isset($this->_record[$dbf->get_fname()]))
                                {
                                    $this->_values[$objf->get_id()] = $this->_record[$dbf->get_fname()];
                                }

                            break;

                        } // switch

                    break;

                    case "relationship":
                                                        
                        // it's a primaryrel relationship
                        $sql = "
SELECT DISTINCT
    xref.id2, weights.weight, r.name
FROM
    primaryrel xref
INNER JOIN
    records r ON r.id = xref.id2
LEFT JOIN
    weights
    ON weights.relid = ".$objf->get_relid()."
    AND weights.recid = xref.id2
    AND weights.parentid = ".$this->_record['id']."
WHERE
    xref.relid = ".$objf->get_relid()."
    AND xref.id1 = ".$this->_record['id']."
    AND r.datatype IN (".$objf->get_otype().")
ORDER BY
    weights.weight, r.name";
                        $this->_values[$objf->get_id()] = $this->db->GetCol($sql);

                    break;

                } // switch
            } // if(record_id
        } // if(isset($this->_imported[...

        if (isset($this->_values[$objf->get_id()]))
        {
            return $this->_values[$objf->get_id()];
        }
    }

    /**
    * Gets the name for the form element.
    * This is always the identifier from typesfields.
    * @param object pxdb_typesfield $objf The pxdb_typesfields object.
    * @return string The name for the field.
    * @access private
    */
    function _get_name(&$objf)
    {
        $this->_names[$objf->get_id()] = $objf->get_identifier();
        return $this->_names[$objf->get_id()];
    }

    /**
    * Gets the possible values for the field.
    * This is only relevant for relationships and pick-driven fields.
    * @param object pxdb_typesfield $objf The pxdb_typesfields object.
    * @return array The value options for this field.
    * @access private
    */
    function _get_options(&$objf)
    {
        // only calculate options if they have not already been set (e.g. by set_field_options() method)

        if(!isset($this->_options[$objf->get_id()])) 
        {
            // are we dealing with a relationship or a simple field?

            switch ($objf->get_field_type()) 
            {
                case "simple":

                    $dbf = new pxdb_dbfield($objf->get_fieldid());

                    switch ($dbf->get_field_type()) 
                    {
                        case "pick":

                            switch($dbf->get_pick_source()) 
                            {

                                case "table":
                                // no break
                                case "vartable":

                                    $pick_namefield = ($dbf->get_pick_column() ? $dbf->get_pick_column() : "name");
                                    $pick_table = $objf->replace_tags($dbf->get_pick_table());
                                    $pick_orderby = pxdb::get_pick_orderby($pick_table);    // label?

                                    $sql = "
SELECT DISTINCT
    pick.$pick_namefield as value,
    pick.name as label
FROM
    $pick_table pick 
ORDER BY
    pick.$pick_orderby, pick.name";
                                    $this->_options[$objf->get_id()] = $this->db->GetAll($sql);

                                break;

                                case "sql":

                                    $sql = trim($dbf->get_pick_sql(), "[]");
                                    $this->_options[$objf->get_id()] = $this->db->GetAll($sql);

                                break;

                            } // switch()

                        break;

                        case "pickid":

                            switch($dbf->get_pick_source()) {

                                case "table":
                                // no break
                                case "vartable":

                                    $pick_idfield = ($dbf->get_pick_column() ? $dbf->get_pick_column() : "id");
                                    $pick_table = $objf->replace_tags($dbf->get_pick_table());
                                    $pick_orderby = pxdb::get_pick_orderby($pick_table, 'label');

                                    $sql = "
SELECT DISTINCT
    pick.$pick_idfield AS value,
    pick.name AS label
FROM
    $pick_table pick
ORDER BY
    pick.$pick_orderby, pick.name";

                                    $this->_options[$objf->get_id()] = $this->db->GetAll($sql);

                                break;

                                case "sql":

                                    $sql = trim($dbf->get_pick_sql(), "[]");
                                    $this->_options[$objf->get_id()] = $this->db->GetAll($sql);

                                break;

                            } // switch

                        break;

                    } // switch

                break;

                case "relationship":

                    // it's a primaryrel relationship

                    // There's some debate over whether this should exclude the current datatype...
                    // my thinking for now is YES.  Because one of the powerful things about these relationships
                    // is being able to display them in both datatypes.
                    //
                    // We should create a new relationship type that doesn't care about datatypes.
                    
                    // NOTE: <nyk@forumone.com> 2004.07.30
                    // We want to be able to have an object be related to othe objects of the same datatype
                    // so we have decided to overrule this so you can specify a datatype that can relate to
					// other objects of the same datatype. However, we do exclude the current record because
					// there are no	circumstances where you want an individual record to relate to itself
					// (it is implicitly always related to itself.
                    
					$exclude_self = ($this->record_id) ? "AND id <> " . $this->record_id : "";
                    $sql = "
SELECT DISTINCT
    id as value, name as label
FROM
    records
WHERE
    datatype IN (" . $objf->get_otype() . ") $exclude_self
ORDER BY
    name";
                    $this->_options[$objf->get_id()] = $this->db->GetAll($sql);

                break;

            } // switch()
        } // if ! isset($this->_options [...]
        
        if (isset($this->_options[$objf->get_id()]))
        {
            return $this->_options[$objf->get_id()];
        }
    }

    /**
    * Get the label for this field.
    * @return string The form label.
    * @param object pxdb_typesfield $objf The pxdb_typesfields object.
    * @access private
    */
    function _get_label(&$objf)
    {
        $this->_labels[$objf->get_id()] = ($objf->get_displayname() ? $objf->get_displayname() : $objf->get_identifier());
        return $this->_labels[$objf->get_id()];
    }

    /**
    * Get the description for this field.
    * @return string The form description.
    * @param object pxdb_typesfield $objf The pxdb_typesfields object.
    * @access private
    */
    function _get_description(&$objf)
    {
        $this->_descriptions[$objf->get_id()] = $objf->get_description();
        return $this->_descriptions[$objf->get_id()];
    }
    
    /**
    * Gets a <label> tag given a couple of parameters
    *
    * @access private
    * @return string HTML of <label> tag plus label text
    * @param string $form_name Name of the form
    * @param string $field_name Name of the field (not the HTML element name)
    * @param string $label Text of the label
    */
    function _get_label_tag($form_name, $element_name, $label)
    {
        return '<label for="' . $form_name . '_' . $element_name . '">' . $label . '</label>';
    }

    /**
    * SQL to get the full object, sorted by 'orderby' column.
    * @return string SQL statement
    * @access private
    */
    function _sql_get_object()
    {
        $sql = "
SELECT
    tf.id
FROM
    typesfields tf
WHERE
    tf.typeid = {$this->datatype}
    AND (tf.fieldid <> 0 OR tf.relid <> 0)
ORDER BY
    tf.orderby";
        return $sql;
    }

    /**
    * Gets the fields for a given view. WARNING: set any default values
    * BEFORE calling this function. We may need to deprecate this method
    * in favor of simply setting an instance variable with a view and folding
    * this functionality into get_fields().
    * @param string $view_name The id of the view we want to display.
    * @return array Array of all typesfields rows for specified view.
    * @access public
    */
    function get_view($view_name)
    {
        pxdb_import('metadata.pxdb_view');
        
        if ($view = &pxdb_view::factory($this->datatype, $view_name))
        {
            $fields = $this->_get_fields($view->arr_typesfields());
            return $fields;
        }
        else
        {
            trigger_error("View '$view_name' does not exist for this object", E_USER_WARNING);
            return false;
        }
    }

    /**
    * Get all (or a specific subset of) the fields for this datatype.
    * @return array Array of all rows found by executing the SQL in _sql_get_object method.
    * @access public
    */
    function get_fields()
    {
        $sql = $this->_sql_get_object();
        return $this->_get_fields($this->db->GetAll($sql));
    }

    /**
    * Builds a multidimensional array of all the fiels in this pxdb object.
    * The array that this function builds is sent to get_widget() method of
    * pxdb_widgets class.
    * @param array $tf_rows Rows from typesfields.
    * @return array
    * @access public
    */
    function _get_fields($tf_rows)
    {
        // if not already bound:
        if (!$this->validator)
        {
            // Instantiate validator class
            //$validator = pxdb::get_class('pxdb_validator');       
            //$this->validator =& call_user_func(array($validator, 'singleton'));
            // OAM IMPORTANT!: call_user_func apaparently can not return a reference 
            // to a statically called method, you'll have to leave this eval in            
            eval("\$this->validator = &".pxdb::get_class('pxdb_validator')."::singleton();");
        }

        // if not already bound:
        if (!$this->widgets)
        {
            // OAM IMPORTANT!: call_user_func apaparently can not return a reference 
            // to a statically called method, you'll have to leave this eval in            
            eval("\$this->widgets = &".pxdb::get_class('pxdb_widgets')."::singleton();");
        }

        $this->_fields = array();

        //reset($tf_rows);
        //while(list(,$row) = each($tf_rows))
        foreach ($tf_rows as $row) 
        {
            $objf = new pxdb_typesfield($row['id']);

            if ($this->auth)
            {
                $priv = array(
                    'read'  => pxdb_privs::is_readable('field', $row['id'], $this->auth->get_sql_ugids()),
                    'write' => pxdb_privs::is_writeable('field', $row['id'], $this->auth->get_sql_ugids())
                );
            }
            elseif (!$this->auth_required) 
            {
                $priv = array('read' => 1, 'write' => 1);
            }
            else
            {
                $priv = array('read' => 0, 'write' => 0);
            } // end if this auth

            if (in_array($objf->get_id(), $this->locked_fields))
            {
                $priv['write'] = 0;
            }

            if (in_array($objf->get_id(), $this->hidden_fields))
            {
                $priv['read'] = 0;
                $priv['write'] = 0;
            }

            if ($objf->get_autogen() && ($this->autogen_disabled == false && !in_array($objf->get_id(), $this->autogen_disabled_fields)))
            {
                // this field is auto-genearted which means it is not displayed
                $this->param_vars['autogen'][] = $this->_get_name($objf);
            }
            elseif($priv['write'] || ($priv['read'] && $this->record_id)) 
            {
                // if write priv then always display -- if read AND we are modifying, then display [as read-only]

                if (isset($row['viewfield']))
                {
                    $vf = new pxdb_viewfield($row['viewfield']);
                    $label = ($vf->get_label() ? $vf->get_label() : $this->_get_label($objf));
                    $description = $vf->get_description();
                }
                else 
                {
                    $label = $this->_get_label($objf);
                    $description = $this->_get_description($objf);
                }

                $field['label']         = $this->_get_label_tag($this->get_form_name(), $this->_get_name($objf), $label);
                $field['description']   = $description;
                $field['name']          = $this->_get_name($objf);
                $field['value']         = $this->_get_value($objf);
                $field['type']          = $objf->get_field_metatype();
                $field['options']       = $this->_get_options($objf);
                $field['multiple']      = $objf->is_multiple();
                $field['required']      = $objf->is_required();

                //oam: for validating required uploads, we don't look in the validator for the error                
                if ('upload' == $field['type'] &&
                      isset($GLOBALS[PXDB_GLOBAL_VAR]['validator'][$field['name']]))
                {
                    $field['error'] = $GLOBALS[PXDB_GLOBAL_VAR]['validator'][$field['name']];
                }
                else 
                {                    
                    $field['error'] = $this->validator->get_error($field['name']);
                }

                $field['id']        = $objf->get_id();
                $field['priv']      = $priv;
                $field['widget']    = $objf->get_widget();
                
                // SFS hack to avoid blank issue
                if ('pass2' == $field['type'] && false === empty($field['error']))
                {
                    $field['value'] = '';
                }
                
                // use the array we've compiled to get the widget
                // -- rewrite the widget key with new value                
                $field['widget'] = $this->widgets->get_widget($field, $this);
                // get_widget method also adds 2 new attributes to $field array:
                //    - element_name (full name of form element)
                //    - form_name (name of form)

                // use the fieldname (i.e. identifier or dbfield) as the key.
                // this makes it easier for template designers to customize the flow of the application
                $this->_fields[$field['name']] = $field;
            }
            
            reset($tf_rows);
        }
        
        return $this->_fields;
    }

    /**
    * Recursive function to create hidden HTML form elements for a name/value pair.
    * The recursion is necessary if $value is an array
    * @param string $name Element name
    * @param mixed $value Element value
    * @param string $prefix Internally used to build an associative array of elements for PHP to process
    * @return string The completed hidden <input> element
    * @access private
    */
    function _create_elements($name, $value, $prefix = '')
    {
        $formstr = "";

        if (is_array($value)) 
        {
            $prefix = ($prefix ? "$prefix[$name]" : $name);

            foreach ($value as $n2 => $v2)
            {
                $formstr .= $this->_create_elements($n2, $v2, $prefix);
            }
        }
        else
        {
            $elname = ($prefix ? $prefix.'['.$name.']' : $name); // that $prefix[$name] doesn't fully resolve here is intentional

            $formstr .= "<input type=\"hidden\" name=\"$elname\" value=\"$value\" />\n";
        }

        return $formstr;
    }

    /**
    * Creates hidden HTML form elements to with currereturn $formstr; nt FORM values.
    * @return string The form elements (input type=hidden)
    * @access public
    */
    function pass_current_vars()
    {
        if (!isset($this->current_vars)) 
        {
            $this->current_vars = $this->get_current_vars(); 
        }

        $formstr = '';
        
        while (list($varname, $value) = each($this->current_vars))
        {
            $formstr .= $this->_create_elements($varname, $value);
        }

        reset($this->current_vars);
        return $formstr;
    }

    /**
    * Gets an array of current, non-pxdb form variables.
    * @return array Key-value pairs
    * @access public
    */
    function get_current_vars()
    {
        $current_vars = array();

        if (isset($this->_imported['preserved_vars']))
        {
            // if not the first time re-loading this page, the vars will be stored.
            $current_vars = $this->_imported['preserved_vars'];
        }
        else 
        {
            reset($_REQUEST);
            while(list($varname, $value)  = each($_REQUEST))            
            {
                if ('pxdb' != $varname) 
                {
                    $current_vars[$varname] = $value;
                }
            }

        }

        return $current_vars;
    }

    /**
    * Adds or replaces the value for a var (hidden form element) to showform form.
    * Note: this will overwrite vars of the same name that are part
    * of the post/get vars that call invoking script.
    * @param string $name The name of the variable to add.as
    * @param mixed $value The value of the variable.
    * @access public
    */
    function set_form_var($name, $value)
    {
        $this->current_vars[$name] = $value; 
    }

    /**
    * Removes a var (hidden form element) from showform form.
    * This is useful if you don't want all of the current vars being
    * passed to $PHP_SELF every time the page reloads. (e.g. if the environment
    * would cause an action to be performed.)
    * @param string $name The name of the variable to remove.
    * @access public
    */
    function unset_form_var($name)
    {
        unset($this->current_vars[$name]);
    }

    /**
    * Create HTML form elements for parameters.
    * @return string The HTML form elements
    * @access public
    */
    function pass_param_vars()
    {
        // parameters to pxdb_adm class.
        $formstr = '';
        foreach($this->param_vars as $varname => $value) 
        {
            $formstr .= $this->_create_elements($this->_get_form_element_name('param', $varname), $value);
        }
        return $formstr;
    }

    /**
    * Get array of parameter vals to pass to pxdb_adm class.
    * @return array Key-value pairs
    * @access public
    */
    function get_param_vars()
    {
        $param_vars = array();
        $param_vars['id'] = $this->record_id;
        $param_vars['dt'] = $this->datatype;
        $param_vars['autogen'] = array();
        return $param_vars;
    }

    /**
    * Hides a specified field from view.
    * This function is used to individually/manually override the settings for the current
    * object.  In this way you can make a field hidden for one particular object
    * form.
    * @param string $field The fieldname (identifier or dbfield name) of field you want to hide.
    * @return boolean True/false on success of finding the matchings typesfield id.
    * @access public
    */
    function hide_field($field)
    {
        $tfid = $this->lookup_tfid($field);
        if ($tfid) 
        {
            $this->hidden_fields[] = $tfid;
            return true;
        }
        else
        {
            return false;
        }
    }

    /**
    * Makes a specified field read-only ("locked").
    * This function is used to individually/manually override the settings for the current
    * object.  In this way you can make a field read-only for one particular object
    * form.
    * @param string $field The fieldname (identifier or dbfield name) of field you want to hide.
    * @return boolean True/false on success of finding the matchings typesfield id.
    * @access public
    */
    function lock_field($field)
    {
        $tfid = $this->lookup_tfid($field);
        
        if ($tfid) 
        {
            $this->locked_fields[] = $tfid;
            return true;
        }
        else 
        {
            return false;
        }
    }

    /**
    * Disables auto-generation for a field (or for the entire object).
    * By default autogen fields will not appear on input forms (because the values are calculated automatically
    * once the form is submitted).  Disabling autogen will allow the value to be manipulated.
    *
    * @param string $field The fieldname (identifier or dbfield name) of field you want to reveal.
    * @return boolean True/false on success of finding the matching typesfield id.
    * @access public
    */
    function disable_autogen($field = null)
    {
        if ($field == null) 
        {
            $this->autogen_disabled = true;
        } 
        else 
        {
            $tfid = $this->lookup_tfid($field);
            if ($tfid) 
            {
                $this->autogen_disabled_fields[] = $tfid;
                return true;
            } 
            else 
            {
                return false;
            }
        }
    }
    
    /**
    * Display the pxdb input form.
    * If no template path is given, this function will load _default/showform.tpl.
    * @param string $template The path of the template to use.
    * @param array $additional_vars Hash of additional vars that should be set for the template to use.
    * @access public
    */
    function showform($template = null, $additional_vars = null)
    {
        $template = $template ? $template : 'showform.tpl';

        if($additional_vars) 
        {
            $this->tpl->assign($additional_vars); // this is an alternative way to assign when using assoc arrays
        }

        // if we haven't already fetched the fields
        if(!$this->_fields)
        {
            $this->get_fields(); 
        }
        
        $this->tpl->assign('fields', $this->_fields);
        
        if (!empty($this->_record))
        {
            $this->tpl->assign('record', $this->_record);
        } // end if

        //print "<pre>"; print_r($this->_fields); print "</pre>";
        // template should provide a way to display errors                
        $this->tpl->assign('errors', $this->errors);

        $this->tpl->assign('form_action', $this->get_form_action());
        $this->tpl->assign('form_name', $this->get_form_name());

        $this->tpl->assign('current_vars', $this->pass_current_vars());
        $this->tpl->assign('param_vars', $this->pass_param_vars());

        $this->tpl->assign('click_submit_jscript', $this->submit_jscript);
        $this->tpl->assign('onsubmit_jscript', $this->widgets->get_onsubmit_value_calculators_jscript($this->form_name));
        
        $this->tpl->assign('submit_label', $this->submit_label);

        $this->tpl->display($template);

        // clear any validator errors from session after having displayed
        // the form.
        if (true === PXDB_USE_SESSION) 
        {
            unset($_SESSION[PXDB_GLOBAL_VAR]['validator']);
        }
    }

    /**
    * Set the action for the form.
    * @param string $action The action for the form
    * @access public
    */
    function set_form_action($action)
    {
        $this->form_action = $action;
    }

    /**
    * Get the form action.
    * @return string The form action
    * @access public
    */
    function get_form_action()
    {
        return $this->form_action;
    }

    /**
    * Set the name for the showform.
    * @param string $form_name Name of the form.
    * @access public
    */
    function set_form_name($form_name)
    {
        $this->form_name = $form_name;
    }

    /**
    * Gets the form name.
    * @access public
    * @return string Form name.
    */
    function get_form_name()
    {
        return $this->form_name;
    }

    /**
    * Change the label used for the submit button
    * @access public
    * @param string $label
    */
    function set_submit_label($label)
    {
        $this->submit_label = trim($label);
    }
    
    /**
    * Specify variables that should be passed to form action on submit.
    * JavaScript is used to set form elements on click of submit button.
    * @param array $params Hash of varname => value that will be set (by JavaScript) on submit.
    * @access public
    */
    function set_submit_vars($params)
    {
        while(list($param, $value) = each($params))
        {
            // javascript code that will be called onClick="" of submit button
            $this->submit_jscript .= "this.form['$param'].value='$value'; ";
            // note: if it wasn't in the current_vars we should probably add the hidden elements also.
            if (!in_array($param, array_keys($this->current_vars))) 
            {
                // empty, but set -- so that the form element will exist.
                $this->current_vars[$param] = ''; 
            }
        }
    }
} // class
?>