<?php
// $Id: pxdb_picktable.class.php,v 1.2 2006/02/28 15:50:11 omerida Exp $
pxdb_import('metadata.object_cache');
/**
* Handles listing and outputting of pick tables and their contents.  
* To access additional fields of your table beyond id and name, you
* have to specify the field list in access_table()
* @package pxdb.content
* @version $Revision: 1.2 $
* @author Sandy Smith <ssmith@forumone.com>
*/
class pxdb_picktable
{
	/**
	* ADOConnection instance
	* @var object ADOConnection
	* @access private
	*/
	var $_db;

	/**
	* The record set of current fetch.
	* @var object ADORecordSet
	* @access private
	*/
	var $_rs = null;
	
	/**
	* The SQL that generated the current record set.
	* @var string
	* @access private
	*/
	var $_sql = null;
	
	/**
	* Stores order by for SQL.
	* @var string
	* @access private
	*/
	var $_orderby;
	
	/**
	* List of picktables
	* @var array
	* @access private
	*/
	var $_tables = array();
	
	/**
	* List of picktable columns
	* @var array
	* @access private
	*/
	var $_columns = array();
		
	/**
	* Constructor
	*
	* @param string $table_name Optional name of table to be accessed
	* @access public
	*/
    function pxdb_picktable($table_name=null) 
	{	    		
		if (! $this->_tables = pxdb::get_nonpxdb_tables())
		{
		    trigger_error("pxdb_picktable->pxdb_picktable() could not get the non-pxdb tables", E_USER_WARNING);
        }
		if($table_name !== null) { $this->set_table($table_name); }		
		$this->debug = PXDB_DEBUG;		
		$this->_db = &pxdb::dbh();
	}
	
	/**
	* Sets the picktable to be accessed.
	*
	* @access public
	* @return boolean
	* @param string $table The name of the table to be accessed
	*/
	function set_table($table = null)
	{
		if (null === $table)
		{
		    trigger_error("pxdb_picktable->set_table() requires a table name", E_USER_WARNING);
            return false;
        }
        if (! in_array($table, $this->_tables))
        {
		    trigger_error("pxdb_picktable->set_table() $table is not in the list of picktables", E_USER_WARNING);
            return false;
        }
		$this->table = $table;
		return true;
	}
	
		
	/**
	* Gets the name of the picktable that is being accessed.
	*
	* @access public
	* @return string
	*/
	function get_table() 
	{		
		return $this->table;
	}
	
	/**
	* Dump the entire picktable into an array.
	*
	* @access public
	* @param string $field_list optional list of fields & aliases to be returned

	* @return array
	*/
	function get_picktable_array($table = null) 
	{
		$test = $this->access_picktable($table);
		if (false === $test)
		{
		    trigger_error("pxdb_picktable->get_picktable_array(): Couldn't access the picktable.", E_USER_WARNING);
            return false;
        }
        return $test->GetAssoc();
	}
	
	/**
	* Dump the entire picktable into an array formatted for use with listWidget
	*
	* @access public
	* @author Nyk Cowham <nyk@forumone.com>
	* @param string $table optional name of the picktable to be accessed
	* @return array
	*/
    function get_listwidget_array($table = null, $value_field = 'id')
    {
        $test = $this->access_picktable($table, $value_field . ', name');

        if (false === $test)
        {
            trigger_error("pxdb_picktable->get_listwidget_array(): Couldn't access the picktable.", E_USER_WARNING );
            return false;
        }
        $opts = array();

        while ($row = $test->FetchRow())
        {            
            array_push($opts, array('label' => $row['name'], 'value' => $row[$value_field]));
        }
        return $opts;

    }
	
	/**
	* Set the SQL and execute to get a resultset for the picktable
	*
	* @access public
	* @param string $table optional name of the picktable to be accessed
	* @param string $field_list optional list of fields & aliases to be returned
	* @return object ADOResultSet on success, boolean false on failure
	*/
	function access_picktable($table = null, $field_list = null) 
	{
	    $field_list = ($field_list) ? $field_list : 'id, name';
	    // if they pass a table, try to set it
		if (null !== $table)
		{
		    if (false === $this->set_table($table))
		    {
		        trigger_error("pxdb_picktable->access_picktable() needs a valid picktable to be set, your input is invalid", E_USER_WARNING);
                return false;
            }
        }
        // if we don't get a table, we're in trouble
        if (! $this->get_table())
        {
            trigger_error("pxdb_picktable->access_picktable() needs a valid picktable to be set; none apparently was", E_USER_WARNING);
            return false;
        }
        
        $this->_sql = "SELECT $field_list FROM ".$this->get_table();
        
        $orderby = $this->get_orderby();
        if ($orderby)
        {
            $orderby = split(',', $orderby);
            
            // make sure these are really fields in this picktable and the query
            // won't barf on us
            if (is_array($orderby))
            {
                foreach ($orderby as $orderby_field)
                {
                    if (! in_array(trim($orderby_field), $this->get_fields($this->get_table())))
                    {
                        trigger_error("pxdb_picktable->access_picktable() needs a valid orderby to be set, your orderby is invalid", E_USER_WARNING);
                        return false;
                    }
                }
                $orderby = join(',', $orderby);
            }
            else
            {
                if (! in_array($orderby, $this->get_fields($this->get_table())))
                {
                    trigger_error("pxdb_picktable->access_picktable() needs a valid orderby to be set, your orderby is invalid", E_USER_WARNING);
                    return false;
                }
            }
            // ok, they should be good
            $this->_sql .= " ORDER BY $orderby";
        }
        
        $this->_rs =& $this->_db->Execute($this->_sql);
        if (false === $this->_rs)
        {
            trigger_error("pxdb_picktable->access_picktable() failed for some reason: ".$this->_db->ErrorMsg(), E_USER_WARNING);
            return false;
        }
        return $this->get_resultset();
	}

	/**
	* Set the order of fields (one or more, in a comma-delimited list) to order results by
	*
	* @param string $orderby The SQL to order the results
	* @access public
	* @return boolean
	*/
	function set_orderby($orderby = null) 
	{
		if (null === $orderby)
		{
		    trigger_error("pxdb_picktable->set_orderby() requires one or more fieldnames in a comma-delimited list to order results by", E_USER_WARNING);
            return false;
        }
        $this->_orderby = $orderby;
        return true;
	}

	/**
	* Set the order of fields (one or more, in a comma-delimited list) to order results by
	*
	* @param string $orderby The SQL to order the results
	* @access public
	* @return boolean
	*/
	function get_orderby() 
	{
        return $this->_orderby;
	}

	/**
	* This method allows you to get a reference to the resultset object.
	*
	* @return boolean True/false on success.
	* @access public
	*/
	function get_resultset() 
	{
		if (! is_object($this->_rs))
		{ 
			trigger_error("You probably haven't executed the picktable yet", E_USER_WARNING);
			return false;
		}
		return $this->_rs;
	}

	/**
	* Looks up the columns of a picktable based on a passed table name.
	*
	* @param string $table optional name of the picktable to be accessed
	* @return array Array of column names for that picktable.
	* @access public
	*/
	function get_fields($table = null) 
	{
	    // if they pass a table, try to set it
		if (null !== $table)
		{
		    if (false === $this->set_table($table))
		    {
		        trigger_error("pxdb_picktable->get_fields() needs a valid picktable to be set, your input is invalid", E_USER_WARNING);
                return false;
            }
        }
        // if we don't get a table, we're in trouble
        if (! $this->get_table())
        {
            trigger_error("pxdb_picktable->get_fields() needs a valid picktable to be set; none apparently was", E_USER_WARNING);
            return false;
        }
        // get a cache object and the column names of the current picktable
        // warning: saveTableColumns() saves a [$value] = $value structured array
        $object_cache = &object_cache::singleton();
        if (! $this->_columns = $object_cache->getTableColumns( $this->get_table() ) )
        {
            $object_cache->saveTableColumns($this->get_table(), $this->_db->MetaColumnNames($this->get_table()));
            $this->_columns = $object_cache->getTableColumns( $this->get_table() );
        }
		// only return the values
		return array_values($this->_columns);	
	}
	
	/**
	* Takes a label and looks up the value (id) associated with it. Useful for
	* situations where you need to build links to related objects.
	*
	* @access public
	* @return integer value (id) of a given label, false if not found
	* @param string $label The label given from a pxdb_record output class
	* @param string $table The name of the table to lookup from (optional)
	*/
	function lookup_value($label, $table = null)
	{
	    if (! $label)
	    {
	        trigger_error('lookup_value(): you must pass a label.', E_USER_WARNING);
            return false;
        }
        $array = $this->get_picktable_array($table);
        if (false === $array)
        {
            trigger_error('lookup_value() had a problem getting the array for that table.', E_USER_WARNING);
            return false;
        }
        return array_search($label, $array);
	}

	/**
	* Takes a pick value (id) and looks up the label associated with it. Useful for
	* situations where you need to do a table lookup.
	*
	* @access public
	* @return string value (label) of a given id, false if not found
	* @param integer $id The value given from a pxdb_record output class
	* @param string $table The name of the table to lookup from (optional)
	*/	
	function lookup_label($value, $table = null)
	{
	    if (! $value)
	    {
	        trigger_error('lookup_label(): you must pass an ID.', E_USER_WARNING);
            return false;
        }
        
        // $rs = $this->access_picktable($table);
        // $array = $rs->GetAssoc();
        
        $array = $this->get_picktable_array($table);
        
        if (false === $array)
        {
            trigger_error('lookup_label() had a problem getting the array for that table.', E_USER_WARNING);
            return false;
        }
        return $array[$value];
	}	

    /**
    * Convenience function for setting the table to use a name based on a datatype identifier (id)
    *
    * @param integer datatype id
    */
    function use_datatype_types($datatype)
    {
        $dt = new pxdb_datatype($datatype);
        
        $table = $dt->get_type().'types';
        
        $this->set_table($table);
        
        
    }

}

?>
