<?php
// $Id: pxdb_picktable_tree.class.php,v 1.2 2005/02/16 03:31:48 omerida Exp $
pxdb_import('content.output.pxdb_picktable');
pxdb_import('metadata.object_cache');
/**
* Extends a normal picktable by allowing rows to point to another pick option as a parent
* @package pxdb.content
* @version $Revision: 1.2 $
* @author Oscar Merida ( oscar@oscarm.org )
*/
class pxdb_picktable_tree extends pxdb_picktable
{

    /*
    cache multiple calls to get the tree / properties
    */
    var $_tree_rows;
    var $_children;
    
	/**
	* Dump the entire picktable into an array formatted for use with listWidget
	*
	* @access public
	* @author Nyk Cowham <nyk@forumone.com>
	* @param string $table optional name of the picktable to be accessed
	* @return array
	*/
    function get_listwidget_array($table = null, $depth_marker = '&nbsp;&nbsp;')

    {
        $test = $this->access_picktable_tree();

        if (false === $test)
        {
            trigger_error("pxdb_picktable->get_listwidget_array(): Couldn't access the picktable.", E_USER_WARNING, __FILE__.__LINE__);
            return false;
        }

        $opts = array();                
        while( list( ,$row ) = each( $test ) )
        {                
            if ( !isset( $depth_label[ $row['depth'] ] ) )
            {
                $depth_label[ $row['depth'] ] = '';
                for ( $i=0; $i<$row['depth']; $i++ )
                {
                    $depth_label[ $row['depth'] ] .= $depth_marker;
                }                
            }
            
            array_push($opts, array('label' => $depth_label[ $row['depth'] ].$row['name'], 'value' => $row['id']));
        }  

        return $opts;

    }
    
    /**
    *  returns array of children for a pick option
    *
    * @access private
    */    
    function get_children(  $parent )
    {
        if ( !empty( $this->_children[ $parent ] ) )
        {
            return $this->_children[ $parent ];
        } else {
            $dbh =& pxdb::dbh();
    
            $sql = "select * from ".$this->get_table()." where parent='$parent' order by name";
            $rs = $dbh->GetAll( $sql );
            
            $this->_children[ $parent ] = $rs;
            
            return $rs;
        }
    }
 
    /**
    *  returns array of options that have children
    *
    * @access private
    */
    function get_breeder_ids()
    {
        $dbh =& pxdb::dbh();

        $sql = "select distinct parent from ".$this->get_table()." order by parent";
        $rs = $dbh->GetAll( $sql );
        return $rs;
    }
 
    /**
    * Returns the rows in the picktable as an array where the labels are indented  
    *
    * @return array
    * @access public
    */
    function access_picktable_tree()
    {
        if ( !empty( $this->_tree_rows ) )
        {
            return $this->_tree_rows;
        }
        
        $breeders = $this->get_breeder_ids();

        while( list( ,$bred ) = each( $breeders ) )
        {
            $children[ $bred['parent'] ] = $this->get_children( $bred['parent'] );    
        }
                        
        $rows = $this->get_rows( 0, $children );
        
        $this->_tree_rows = $rows;
        return $rows;
    }
    
    /**
    * recursive function to get all the children under a pick
    *
    * @access public    
    * @param integer pickid to get descendants for
    * @param array children array
    * @param integer current depth
    * @return array    
    */    
    function &get_rows( $pickid, $children, $depth=0)
    {                        
        $my_kids = $children[ $pickid ];
        $curr_depth = $depth;
        if ( !empty( $my_kids ) )
        {
            while( list( ,$child ) = each( $my_kids ) )
            {                   
                $rows[ $child['id'] ] = array( 
                    'id' => $child['id'],
                    'name'=> $child['name'],
                    'depth'=>$curr_depth,          
                    'parent'=>$pickid
                );
                
                $ngen = $this->get_rows( $child['id'], $children, ($depth+1) );
                if ( !empty( $ngen ) )
                {                    
                    while( list(, $grandchild ) = each( $ngen ) )
                    {                        
                        $rows[ $grandchild['id'] ] = array( 
                            'id'=>$grandchild['id'],
                            'name'=>$grandchild['name'],
                            'depth'=> $grandchild['depth'],             
                            'parent'=>$grandchild['parent']
                        );
                    } // end while
                } // end if no grandkids
            } // end while kids
        } // end if kids

        return $rows;
        
    }    
    
    /**
    * tests if a node has any children under neath it.
    *
    * @param integer node id
    * @access public
    * @return boolean
    */
    function has_children( $node )
    {
       $children = $this->get_children( $node );
       return ( 0 < count( $children ) );        
    }
}
?>
