<?php
// $Id: pxdb_record.class.php,v 1.10 2005/10/17 05:40:30 sandys1 Exp $

pxdb_import('content.pxdb_data');
pxdb_import('metadata.pxdb_typesfield');
pxdb_import('metadata.pxdb_dbfield');

/**
* Class for accessing Syntax record data.
* This class simplifieds retrieval of Syntax content by automatically looking up pickid fields and by returning 
* arrays of record objects for get_field() requests for relationship fields.
* @package pxdb.content.output
* @version $Revision: 1.10 $
* @author Hans Lellelid <hans@velum.net>
* @access public
*/
class pxdb_record extends pxdb_data 
{
    /**
    * @var array
    * @access public
    */
    var $fields;
    
    /**
    * @var integer
    * @access public
    */
    var $record_id;
    
    /**
    * Constructor.
    * Can be passed an id or a recordset row (array) -- to eliminate the need for unnecessary
    * db queries.
    * @param mixed $data The id of record to look up or row of table.
    */
    function pxdb_record($data = null) 
    {        
        parent::pxdb_data();                                        
        
        if ($data !== null) 
        {
            if (is_numeric($data))
            {
                $this->set_id($data);
                $this->read();            
            }
            elseif (is_array($data)) 
            {
                // I'd like to change this for efficiency ... am somewhat comforted that the
                // pref returned by get_pxdb_reserved_fields() will be cached...
                // Record needs to check and make sure that it is being instantiated with more than
                // the reserved fields (i.e. 'id','datatype','created','modified','approved'
                if (array_diff(array_keys($data),pxdb::get_pxdb_reserved_fields())) 
                {
                    // $data contains more than reserved fields
                    $this->set_id($data['id']);
                    $this->set_datatype($data['datatype']);
                    $this->fields = $data;
                }
                elseif (isset($data['id'])) 
                {
                    $this->set_id($data['id']);
                    $this->read();
                }
                else
                {
                    pxdb::trigger_error("Row passed to pxdb_record does not contain ID.", E_USER_ERROR, get_class($this).'::__constructor()', __LINE__, __FILE__);
                }                
            }
            else
            {
                pxdb::trigger_error("Invalid row data passed to pxdb_record.", E_USER_ERROR, get_class($this).'::__constructor()', __LINE__, __FILE__);
            }                    
        } // if not null
        
        //oam: load the reserved fields once here instead of on each call to get_field
        $a = pxdb::get_pxdb_reserved_fields();
        while (list(, $field) = each($a))
        {
            $this->_pxdb_reserved_fields[$field] = $field;
        }
    } // function

    /**
    * Get record id.
    * @return integer
    * @access public
    */    
    function get_id() 
    {
        return $this->record_id;
    }

    /**
    * Set record id.
    * @param integer $id
    * @access public
    */        
    function set_id($id) 
    {
        $this->record_id = $id;
    }
    
    /**
    * Reads data from db into class.
    * @return boolean on success or reading record.
    * @access public
    */    
    function read() 
    {
        if ($this->get_id()) 
        {
            // 1) get datatype            
            $sql = 'SELECT datatype FROM records WHERE id = '.$this->get_id();                        
            $this->set_datatype($this->db->GetOne($sql));
            if (!$this->get_datatype()) 
            {
                // record matching specified ID was not found.
                $this->fields = array();
                return false;
            }
                            
            // 2) now get the fields
            $sql = 'SELECT '.$this->sql_aliased_columns().' FROM records WHERE id = '.$this->get_id();
            $rs = $this->db->Execute($sql);
            $this->fields = $rs->fields;    
            return true;
        }
        else
        {
            return false;
        }
    }
    
    /**
    * Gets related records.
    * @param mixed $relid Relid or array of relids.
    * @param mixed $dt Datatype or array of datatypes.
    * @param string $orderby ORDER BY
    * @param boolean $multiple
    * @return array Array of pxdb_record objects (returned by reference).
    * @access public
    */
    function &get_related($relid, $dt = null, $orderby = 'name', $multiple = false) 
    {    
        // normalize input -- make both arrays ($dt, only if entered)
        if (!is_array($relid))
        {
            $relid = array($relid); 
        }
        
        if (null !== $dt && !is_array($dt))
        {
            $dt = array($dt);
        }
                
        if (false === $multiple) 
        {
            // lookup whether this record is multiple
            $sql = "SELECT mul FROM typesfields WHERE relid IN (".implode(',', $relid).") AND typeid = ".$this->fields['datatype'];
            $mul_rs = $this->db->Execute($sql);
            $multiple = ($mul_rs->fields['mul'] == 'Y' ? true : false);
        }        
        // return an array of pxdb_record objects
        
        $relid_list = implode(',', $relid);
        
        $sql = "
SELECT DISTINCT
    rel.id as related_id, rel.datatype as related_dt
FROM
    records r
INNER JOIN
    primaryrel xref
        ON xref.relid IN ($relid_list)
        AND xref.id1 = r.id
INNER JOIN
    records rel
        ON rel.id = xref.id2
        ".($dt ? 'AND rel.datatype IN (' . implode(',', $dt) . ')' : '')."
LEFT JOIN
    weights
        ON weights.relid IN ($relid_list)
        AND weights.recid = rel.id
        AND weights.parentid = ".$this->get_id()."                
WHERE
    r.id = $this->record_id
    ".(!$this->show_unapproved ? "AND rel.approved = 1" : "")."
ORDER BY
    weights.weight, rel.$orderby";
                
        $rs = $this->db->Execute($sql);    
        if ($multiple) 
        {
            $records = array();
            while (!$rs->EOF) 
            {
                eval("\$records[] = new ".pxdb::get_class('pxdb_record', $rs->fields['related_dt'])."(\$rs->fields['related_id']);");
                $rs->MoveNext();
            }            
            return $records;    
        }
        else
        {
            $record = null;
            if (!$rs->EOF) 
            {
                eval("\$record = new ".pxdb::get_class('pxdb_record', $rs->fields['related_dt'])."(\$rs->fields['related_id']);");        
            }            
            return $record;
        }                
    }

    /**
    * Set show_unapproved explicitly
    * 
    * @param boolean $bool
    * @access public
    */
    function set_show_unapproved($bool)
    {
        $this->show_unapproved = $bool;
    }
    
    /**
    * Builds an array of values from [primaryrel] related records.
    * This is for convenience.
    * @param string $field The field to lookup.
    * @param integer $relid
    * @param mixed $dt Datatype or array of datatypes
    * @param string $orderby ORDER BY
    * @param boolean $multiple
    * @return array Array of $field value for related records.
    * @access public
    */
    function get_related_values($field, $relid, $dt = null, $orderby = 'name', $multiple = false)
    {
        if (!is_array($relid)) 
        {
            $relid = array($relid);
        }
        
        if ($multiple === false) 
        {
            // lookup whether this record is multiple
            $sql = "SELECT mul FROM typesfields WHERE relid IN (".implode(',', $relid).") AND typeid = ".$this->fields['datatype'];
            $mul_rs = $this->db->Execute($sql);
            $multiple = ($mul_rs->fields['mul'] == 'Y' ? true : false);
        }
        
        $recs =& $this->get_related($relid, $dt, $orderby, $multiple);
        if ($multiple) 
        {
            $values = array();        
            for($i = 0, $z = sizeof($recs); $i < $z; $i++) 
            {
                $rec =& $recs[$i]; // assigning objects by ref
                $values[] = $rec->get_field($field);
            }
            return $values;        
        }
        else
        {
            if ($recs) 
            {
                return $recs->get_field($field);
            }
            else
            {
                return null;
            }
        }
    }
    
    /**
    * Gets secondaryrel related records.
    * @param mixed $dt Datatype or array of datatypes.
    * @param string $orderby ORDER BY
    * @return array Array of pxdb_record objects.
    * @access public
    */
    function &get_secondaryrel_related($dt = null, $orderby = 'name') 
    {    
        // normalize input
        if ($dt)
        {
            if (!is_array($dt)) 
            {
                $dt = array($dt);
            }
        }
                
        // return an array of pxdb_record objects
        $sql = "
SELECT DISTINCT
    rel.id as related_id, rel.datatype as related_dt
FROM
    records r
INNER JOIN
    secondaryrel xref
        ON xref.id1 = r.id
INNER JOIN
    records rel
        ON rel.id = xref.id2
        ".($dt ? 'AND rel.datatype IN (' . implode(',', $dt) . ')' : '')."
WHERE
    r.id = $this->record_id
    ".(!$this->show_unapproved ? "AND rel.approved = 1" : "")."
ORDER BY
    rel.$orderby";
        
        $rs = $this->db->Execute($sql);    
        $records = array();
        while (!$rs->EOF) 
        {
            eval("\$records[] = new ".pxdb::get_class('pxdb_record', $rs->fields['related_dt'])."(\$rs->fields['related_id']);");
            $rs->MoveNext();
        }
        $rs->close();
        return $records;
    }

    /**
    * Builds a list of values from [secondaryrel] related records.
    * This is for convenience.
    * @param string $field The field to lookup.
    * @param mixed $dt Datatype or array of datatypes.
    * @param string $orderby ORDER BY
    * @return array Array of $field value for related records.
    * @access public
    */
    function get_secondaryrel_related_values($field, $dt = null, $orderby = 'name')
    {    
        $recs =& $this->get_secondaryrel_related($dt, $orderby);
        $values = array();        
        for($i = 0, $z = sizeof($recs); $i < $z; $i++) 
        {
            $rec =& $recs[$i]; // assigning objects by ref
            $values[] = $rec->get_field($field);
        }
        return $values;
    }
    
    /**
    * Gets children [noderel] records by noderelid.
    * @param int $noderel_id Noderel ID of children to get
    * @param string $orderby Field to order results by
    * @return array Array of pxdb_record objects.
    * @access public
    */
    function &get_children_noderelid($noderel_id = NODEREL_SECTION_TO_SECTION, $orderby = 'name') 
    {
        // return an array of pxdb_record objects
        $sql = '
SELECT DISTINCT
    rel.id as related_id, rel.datatype as related_dt
FROM
    noderel xref
INNER JOIN
    records rel on rel.id = xref.child';
        if (!$this->show_unapproved)
        {
            $sql .= '
    AND rel.approved = 1';
        }
        $sql .= '
WHERE
    xref.parent = ' . $this->record_id . '
    AND xref.noderelid = ' . $noderel_id . '
ORDER BY
    rel.' . $orderby;
        $rs = $this->db->Execute($sql);
        $records = array();
                
        if (0 < $rs->RowCount())
        {
            while (!$rs->EOF) 
            {
                $pxdb_class = pxdb::get_class('pxdb_record', $rs->fields['related_dt']);
                $records[] = new $pxdb_class($rs->fields['related_id']);
                $rs->MoveNext();                
            }
        }
        $rs->close();
        
        return $records;
    }
    
    /**
    * Gets children [noderel] records.
    * @param mixed $dt Datatype or array of datatypes.
    * @param string $orderby Field to order results by
    * @return array Array of pxdb_record objects.
    * @access public
    */
    function &get_children($dt = null, $orderby = 'name') 
    {    
        // normalize input
        if ($dt && !is_array($dt)) 
        {
            $dt = array($dt);            
        }
        
        $sql = '
SELECT DISTINCT
    rel.id as related_id, rel.datatype as related_dt
FROM
    noderel xref
INNER JOIN
    records rel on rel.id = xref.child
        ';
        if (null !== $dt)
        {
            $sql .= ' and rel.datatype in ('.join(',',$dt).')';
        }
        $sql.= '
WHERE
    xref.parent = '.$this->record_id;
        
        if (!$this->show_unapproved)
        {
            $sql.='
    AND rel.approved = 1';
        }
        $sql .= '
ORDER BY
    rel.'.$orderby;
        $rs = $this->db->Execute($sql);
        $records = array();
                
        if (0 < $rs->RowCount())
        {
            while (!$rs->EOF) 
            {
                $pxdb_class = pxdb::get_class('pxdb_record', $rs->fields['related_dt']);
                //eval("\$records[] = new ".$pxdb_class."(\$rs->fields['related_id']);");                                
                $records[] = new $pxdb_class($rs->fields['related_id']);
                $rs->MoveNext();                
            }
        }
        $rs->close();
        
        return $records;
    }
    
    /**
    * Builds an array of values from [noderel children] related records.
    * This is for convenience.
    * @param string $field The field to lookup.
    * @param mixed $dt Datatype or array of datatypes.
    * @param string $orderby ORDER BY
    * @return array Array of $field value for related records.
    * @access public
    */
    function get_children_values($field, $dt = null, $orderby = 'name')
    {    
        $recs =& $this->get_children($dt, $orderby);
        $values = array();        
        for($i = 0, $z = count($recs); $i < $z; $i++) 
        {
            $rec =& $recs[$i]; // assigning objects by ref
            $values[] = $rec->get_field($field);
        }
        return $values;
    }
    
    /**
    * Gets parent [noderel] records.
    * @param mixed $relid Relid or array of relids.
    * @param mixed $dt Datatype or array of datatypes.
    * @return array Array of pxdb_record objects.
    */
    function &get_parents($dt = null, $orderby = 'name') 
    {    
        // normalize input
        if ($dt)
        {
            if (!is_array($dt)) 
            {
                $dt = array($dt);
            }
        }
                
        // return an array of pxdb_record objects
        $sql = "
SELECT DISTINCT
    rel.id as related_id, rel.datatype as related_dt
FROM
    records r
INNER JOIN
    noderel xref ON xref.child = r.id
INNER JOIN
    records rel ON rel.id = xref.parent
    ".($dt ? 'AND rel.datatype IN (' . implode(',', $dt) . ')' : '')."
WHERE
    r.id = $this->record_id
    ".(!$this->show_unapproved ? "AND rel.approved = 1" : "")."
ORDER BY
    rel.$orderby";
        
        $rs = $this->db->Execute($sql);    
        $records = array();
        while (!$rs->EOF) 
        {
            eval("\$records[] = new ".pxdb::get_class('pxdb_record', $rs->fields['related_dt'])."(\$rs->fields['related_id']);");
            $rs->MoveNext();
        }
        $rs->close();
        
        return $records;
    }

    /**
    * Builds an array of values from [noderel parents] related records.
    * This is for convenience.
    * @param string $field The field to lookup.
    * @param mixed $dt Datatype or array of datatypes.
    * @param string $orderby ORDER BY
    * @return array Array of $field value for related records.
    * @access public
    */
    function get_parents_values($field, $dt = null, $orderby = 'name')
    {    
        $recs =& $this->get_parents($dt, $orderby);
        $values = array();        
        for($i = 0, $z = count($recs); $i < $z; $i++) 
        {
            $rec =& $recs[$i]; // assigning objects by ref
            $values[] = $rec->get_field($field);
        }
        return $values;
    }
    
    /**
    * The strict version of get_field()--performs no lookups.
    * Use this function, e.g., if you want to make sure that you get an ID returned
    * when getting the value for a pickid column.
    * @param string $field The fieldname for which to get value.
    * @return mixed False on error (e.g. field doesn't exist) otherwise value.
    * @access public
    */
    function get_field_strict($field) 
    {        
        if (isset($this->fields[$field])) 
        {
            return $this->fields[$field];
        }
        else
        {
            //oam - maybe we set the custom field after this?            
            @pxdb::trigger_error("No field '$field' found in record columns.", E_USER_NOTICE);            
            return false;
        }        
    }
    
    /**
    * Smart lookup function for getting field values.
    * @param string $field The column name (dbfield) or typesfields displayname of field to fetch.
    * @param string $option If pick, field name to return or 'get_pickids' to ensure ids are returned. If relationship, field name to return.
    * @return mixed False on error, otherwise value.
    * @access public
    */
    function &get_field($field, $option = null) 
    {                
        if (!$this->fields)
        {
            $return = false;
            return $return;
        }
        
        // SFS change--make this more accurate by forcing it to check typesfields
        // every time. Now that we cache these lookups it isn't that expensive.
        // Makes it safe for when you change the definition of a field in dbfields.
               
        //if (in_array($field,  $this->_pxdb_reserved_fields))
        if (!empty($this->_pxdb_reserved_fields[$field]))
        {                
            // if it's a reserved var -- e.g. 'datatype' return the value loaded
            return $this->fields[$field];
        }
        else
        {
            // perform the more tedious lookups to return a smart value for this field
            // first tries to lookup as uniquename, then as dbfield fname, then as displayname
            $tfid = $this->lookup_tfid($field, $this->get_datatype());
                
            if ($tfid)
            {
                $objf = new pxdb_typesfield($tfid);
                $dbf = new pxdb_dbfield($objf->get_fieldid());
                
                // this could be a relationship
                if ($objf->get_field_type() == 'relationship')
                {
                    // it's a relationship, get related fields
                    if ($option != null && $option !== 'get_pickids')
                    {
                        // $option is name of field to get
                        return $this->get_related_values($option, $objf->get_relid(), null, 'name', $objf->is_multiple()); 
                    }
                    else
                    {                        
                        return $this->get_related($objf->get_relid(), null, 'name', $objf->is_multiple());
                    }
                }
                else
                {
                    // it's a simple field
                     switch ($dbf->get_field_type())
                     {
                        case "pickid":
                            
                            switch ($dbf->get_pick_source())
                            {
                                case "vartable":
                                    // this is handled by the picktable condx assign
                                                    
                                case "table":
                                    
                                    $pick_table = ($dbf->get_pick_source() == 'table' ? $dbf->get_pick_table() : $objf->replace_tags($dbf->get_pick_table()));
                                    $pickcol = ($dbf->get_pick_column() ? $dbf->get_pick_column() : 'id');
                                    
                                    if ($option != null && $option !== 'get_pickids')
                                    {
                                        $valcol = $option;
                                    }
                                    else
                                    {
                                        $valcol = 'name';
                                    }
                                    
                                    $pick_orderby = pxdb::get_pick_orderby($pick_table, $valcol);
                                    
                                    if ($option == 'get_pickids')
                                    {
                                        // SFS treat single and multiple the same
                                        // join on pickrel
                                        $sql = "
SELECT
    pick.id, pick.$valcol AS value
FROM
    $pick_table pick
INNER JOIN
    pickrel prel
        ON prel.p_id = pick.$pickcol
        AND prel.f_id = ".$objf->get_fieldid()."
        AND prel.recid = ".$this->record_id."
ORDER BY
    pick.$pick_orderby, pick.name
                                        ";
                                        $vals = $this->db->GetAll($sql);
                                        while (list(, $row) = each($vals))
                                        {
                                            $values[$row['id']] = $row['value'];   
                                        }
                                        
                                        return $values;
                                    }
                                    else
                                    {
                                        // SFS multiple and single get treated the same
                                        // join on pickrel
                                        $sql = "
SELECT
    pick.$pickcol as pick_id, pick.$valcol AS value 
FROM
    $pick_table pick
INNER JOIN
    pickrel prel
        ON prel.p_id = pick.$pickcol
        AND prel.f_id = ".$objf->get_fieldid()."
        AND prel.recid = ".$this->record_id."
ORDER BY
    pick.$pick_orderby, pick.name
                                        ";
                                        $pick_idx = 'pick_id';
                                        $q = $this->db->execute($sql);
                                        
                                        while ($row = $q->FetchRow())
                                        {
                                            $values[$row[$pick_idx]] = $row['value'];
                                        }
                                        
                                        // if there's only one element, then it doesn't need to be an array
                                        // unles it's multiple in which case, we should always return an array.
                                        if (isset($values) && sizeof($values) == 1 && !$objf->is_multiple()) 
                                        {                                            
                                            $values = array_pop($values); 
                                        }
                                    }
                                    
                                    return $values;
                                    
                                break;
                                
                                case "sql":
                                                                        
                                    // what this needs to return, then, is an instantiated records row
                                    // from the p_id in pickrel
                                    // ... or an array of such objects
                                    
                                    $sql = "SELECT r.id FROM records r INNER JOIN pickrel prel ON prel.p_id = r.id AND prel.f_id = ".$objf->get_fieldid()." AND prel.recid = ".$this->record_id;
                                    $rs = $this->db->Execute($sql);
                                    
                                    $values = array();
                                    while (!$rs->EOF)
                                    {        
                                        $values[] = new pxdb_record($rs->fields['id']);                            
                                        $rs->MoveNext();
                                    }
                                    
                                    if (sizeof($values) == 1 && !$objf->is_multiple()) 
                                    { 
                                        $values =& $values[0]; 
                                    }
                                    
                                    
                                break;
                            
                            } // switch pick source
                            
                        break;
                        
                        default:
                            // just grab the text straight out of fields
                            return $this->fields[$objf->get_identifier()];
                            
                    } // switch
                } // if is relationship
            }
            else
            {
                // it's not a dbfield but may be a column (e.g. datatype, created, id)                
                if (!$this->has_field($field)) 
                {
                    trigger_error("Could not find a matching field for this object: $field", E_USER_WARNING);
                    $return = false;
                    return $return;
                }
                else
                {
                    return $this->fields[$field];
                }
            }
        } // if in_array($field, $fields... etc.
    } // function

    /**
    * Checks if a record has a specific field
    *
    * @param string $field name
    * @return boolean
    * @access public
    */
    function has_field($field)
    {
        if (empty($this->_field_keys))
        {
            $this->_field_keys = array_flip(array_keys($this->fields));
        }
        
        return isset($this->_field_keys[$field]);
    }
    
    /**
    * Gets all privs for current row.
    * @return array Array of Hash name=> ugid=> priv=>
    * @access public
    */
    function get_all_privs() 
    {    
        return pxdb_privs::get('record', $this->get_id());        
    }

    /**
    * Gets privileges for specified user.
    * @return array Hash name=>user name, ugid => user id, priv => user privs.
    * @access public
    */    
    function get_user_privs() 
    {
        $prefs =& pxdb_prefs::singleton();
        $sql = 'SELECT id FROM records WHERE datatype IN (' . $prefs->get_list('dt_user') .')';
        $ugids = $this->db->GetCol($sql);
        $u_privs = pxdb_privs::get('record', $this->get_id(), $ugids);
        return $u_privs;    
    }
    
    /**
    * Gets privileges for specified group.
    * @return Array Hash name=>group name, ugid => group id, priv => group privs.'
    * @access public
    */    
    function get_group_privs() 
    {
        $prefs =& pxdb_prefs::singleton();
        $sql = 'SELECT id FROM records WHERE datatype IN (' . $prefs->get_list('dt_group') . ')';
        $ugids = $this->db->GetCol($sql);
        $g_privs = pxdb_privs::get('record', $this->get_id(), $ugids);
        return $g_privs;
    }
    
    /**
    * Gets privileges for ugid 0.
    * @return array Hash name=>'Everyone', ugid => '0', priv => global privs.
    * @access public
    */
    function get_global_privs() 
    {
        // special case -- the '0' everyone case
        $privs = pxdb_privs::get('record', $this->get_id(), 0);
        $global_privs = null;
        if (sizeof($privs)) 
        {
            // there should only be one element at the intersection of id & ugid
            $global_privs = array_pop($privs);
            $global_privs['name'] = 'Everyone';
        }
        
        return $global_privs;
    }
    
    /**
    * Checks to see if bound pxdb_user can read specified record.
    * @return boolean
    * @access public
    */
    function is_authorized()
    {                
        // oam - fixed, if no auth module is bound, then it checks the global_privs for the record        
        if (empty($this->auth)) 
        {
            return pxdb_privs::is_readable('record', $this->get_id(), 0); 
        }
        else
        {
            return pxdb_privs::is_readable('record', $this->get_id(), $this->auth->get_sql_ugids());
        }
                    
    }
}
?>