<?php
// $Id: pxdb_data.class.php,v 1.4 2005/10/17 05:40:30 sandys1 Exp $

pxdb_import('pxdb_privs');
pxdb_import('metadata.object_cache');
/**
* An abstract class that contains a shared collection of attributes and methods used by data classes.
* @package pxdb.content
* @version $Revision: 1.4 $
* @author Hans Lellelid <hans@velum.net>
*/
class pxdb_data
{

    /**
    * Debug switch.
    * @var boolean
    * @access public
    */
    var $debug;

    
    /**
    * Errors array.
    * @var array
    * @access public
    */
    var $errors;

    /**
    * The datatype(s) for this search class.
    * @var mixed
    * @access public
    */
    var $datatype;
    
    /**
    * Authentication instance
    * @var object
    * @access public
    */
    var $auth;

    /**
    * Smarty template instance
    * @var object
    * @access public
    */
    var $tpl;
    
    /**
    * Whether to allow unapproved results.
    * @var boolean
    * @access public
    */
    var $show_unapproved = false;

    /**
    * ADOConnection instance
    * @var object
    * @access public
    */
    var $db;

    /**
    * Array of user-defined minimal columns.
    *
    * @var array
    * @access public
    */
    var $user_minimal_columns;
    
    /**
    * Constructor
    * @param mixed $dt Datatype
    * @access public
    */
    function pxdb_data($dt = null) 
    {                
        if ($dt !== null)
        {
            $this->set_datatype($dt);
        }        
        $this->debug = PXDB_DEBUG;        
        $this->db = &pxdb::dbh();
        $this->auth = call_user_func(array(pxdb::get_class('pxdb_user'), 'existant')); 
    }
        
    /**
    * Sets whether unapproved items should be returned
    * @param bool $bool
    * @access public
    */
    function set_show_unapproved($bool)
    {
        $this->show_unapproved = $bool;
    }
    
    /**
    * Gets the datatype(s).
    * @return mixed
    * @access public
    */
    function get_datatype() 
    {        
        return $this->datatype;
    }
    
    /**
    * Returns a $delim-separated list of datatypes.
    * Because a data object may be set up to retrieve data from more than one datatype, $datatype
    * is always an array -- making this method necessary.
    * @return string delim-separated list of datatype(s).
    * @access public
    */
    function get_datatype_list($delim = ',') 
    {
        return implode($delim, $this->get_datatype_array());
    }
    
    /**
    * Gets the datatypes in array form.
    * @return array
    * @access public
    */
    function get_datatype_array() 
    {        
        if (empty($this->datatype))
        {
            return array();
        }
        elseif (!is_array($this->datatype))
        {
            return explode(',', $this->datatype);
        }
        else
        {
            return $this->datatype;
        }
    }
    
    /**
    * Sets the datatype.
    * @param mixed $dt Integer, array of integers, or comma-separated string of integers
    * @access public
    */
    function set_datatype($dt) 
    {        
        $this->datatype = $dt;
    }

    /**
    * Bind your own Smarty object.
    * @param object Smarty $tpl The Smarty object to use instead of built-in.
    * @access public
    */
    function bind_tpl(&$tpl) 
    {
        $this->tpl = &pxdb::tpl($tpl);
    }

    /**
    * Bind an authentication object.
    * This method allows you to bind an authentication class to an input / commit class. This method checks
    * to make sure that the authentication class provides a recognizable interface.
    *
    * @param object pxdb_auth $auth The authentication object.
    * @return boolean True/false on success.
    * @access public
    */
    function bind_auth(&$auth) 
    {
        if ($auth)
        { 
            if (array_diff(array('get_sql_ugids', 'get_uid'), get_class_methods(get_class($auth))))
            {
                pxdb::trigger_error("You did bind a valitation class, but it is missing required methods.", E_USER_WARNING);
                return false;
            }
            else
            {
                $this->auth = &$auth;
                return true;
            }
        }
        else
        {
            // allows one to bind nothing authentication module
            $this->auth = null;
            return true; // SFS since this is an allowed operation
        }
    }

    /**
    * Looks up a dbfield fname based on a passed typesfield identifer.
    *
    * @param string $fieldname The identifier or dbfield fname.
    * @return string The record column name that corresonds to the specified field.
    * @access public
    */
    function lookup_colname($fieldname) 
    {
    
        $dt_list = $this->get_datatype_list();
        $object_cache = &object_cache::singleton();
        if (! $columns = $object_cache->getTableColumns('records'))
        {
            $object_cache->saveTableColumns('records', $this->db->MetaColumnNames('records'));
            $columns = $object_cache->getTableColumns('records');
        }
        
        // 1) assume it's an identifier
        $sql = "
SELECT  df.fname
FROM    typesfields tf
INNER JOIN dbfields df ON df.id = tf.fieldid
WHERE   " . ($dt_list ? "tf.typeid IN ($dt_list)
AND" : "") . " tf.identifier LIKE " . $this->db->qstr($fieldname);
        
        $rs = $this->db->Execute($sql);                
    
        
        if ($rs->RecordCount())
        {
            $field = $rs->fields['fname'];    
        }
        else
        {
            // assume the fieldname passed is a dbfield name
            $field = $fieldname;
        }
        
        if (! empty($columns[$fieldname]))
        {
            return $fieldname;            
        }
        else
        {
            return false;
        }
    }

    /**
    * Looks up a typesfields id based on the typesfield uniquename or dbfield name.
    * This is necessary because we only know the unique name from looking at the form that was submitted.
    * @param string $fieldname The fname from dbfields or the oref_relid name.
    * @param mixed $datatype The datatype(s) to use for lookup.
    * @return integer The typesfields ID that corresonds to that uniquename.
    * @access private
    */
    function lookup_tfid($fieldname, $datatype = null) 
    {
        // 0) if fieldname passed is numeric, then we can assume that we
        // are being passed the typesfields id.
        if (is_numeric($fieldname))
        {
            return $fieldname;
        }
        else
        {
            // otherwise (most cases) :
            if ($datatype !== null)
            {
                if (!is_array($datatype))
                { // normalize
                    $datatype = array($datatype); 
                } 
                $dt_list = implode(',', $datatype);
            }
            else
            {
                // usual...
                if ($this->get_datatype())
                { 
                    $dt_list = $this->get_datatype_list(); 
                }
                else
                { 
                    $dt_list = ''; 
                }
            }
            
            // SFS added caching
            // 1) assume it's an identifier
            $cache =& object_cache::singleton();
            if (! $results = $cache->getTfIdByIdentifiers($dt_list, $fieldname))
            {
                // oam changed identifier operation from LIKE to =
                $sql = "
SELECT
    tf.id
FROM
    typesfields tf
WHERE
    " . ($dt_list ? "tf.typeid IN ($dt_list)
    AND" : "") . " tf.identifier=" . $this->db->qstr($fieldname);
                
                $tf_rs = &$this->db->Execute($sql);
                if ($tf_rs->EOF)
                {
                    $cache->saveTfIdByIdentifiers($dt_list, $fieldname, null, 0);
                }
                else
                {
                    $cache->saveTfIdByIdentifiers($dt_list, $fieldname, $tf_rs->fields['id'], $tf_rs->RecordCount());
                }
                $results = $cache->getTfIdByIdentifiers($dt_list, $fieldname);
            }
    
            // 2) if not found, assume it's a dbfield
            if (0 === $results['num_rows'] && ! ($results = $cache->getTfIdByDbFields($dt_list, $fieldname)))
            {
                $sql = "
SELECT
    tf.id
FROM
    typesfields tf
INNER JOIN
    dbfields df ON df.id = tf.fieldid
WHERE
    " . ($dt_list ? "tf.typeid IN ($dt_list)
    AND" : "") . " df.fname = " . $this->db->qstr($fieldname);
                $tf_rs = &$this->db->Execute($sql);
                if ($tf_rs->EOF)
                {
                    $cache->saveTfIdByDbFields($dt_list, $fieldname, null, 0);
                }
                else
                {
                    $cache->saveTfIdByDbFields($dt_list, $fieldname, $tf_rs->fields['id'], $tf_rs->RecordCount());
                }
                $results = $cache->getTfIdByDbFields($dt_list, $fieldname);
            }
        
            if (1 < $results['num_rows'] && false === strpos($dt_list, ','))
            {
                pxdb::trigger_error("Unreliable results: different typesfields rows match your lookup on $fieldname", E_USER_NOTICE, 'pxdb_data::lookup_field()');
            }
            return $results['id'];
        }

    }

    /**
    * Finds identifiers from typesfields that correspond to a *column* in records table.
    * Gets all column names from records and corresponding identifiers from typesfields.
    * @return array Array of hash [fname] / [identifier] 
    * @access public
    */
    function get_column_identifiers() 
    {        
        $cache =& object_cache::singleton();
        if (! $results = $cache->getColumnIdentifiers($this->get_datatype_list()))
        {
            // SFS - exclude pickids
            $sql = "
SELECT
    df.fname, 
    tf.identifier 
FROM
    typesfields tf 
INNER JOIN
    dbfields df ON df.id = tf.fieldid 
WHERE
    tf.typeid IN (" . $this->get_datatype_list() . ")
    AND df.ftype NOT LIKE 'pickid %'
ORDER BY tf.orderby
            ";
            $cache->saveColumnIdentifiers($this->get_datatype_list(), $this->db->GetAll($sql));
            $results = $cache->getColumnIdentifiers($this->get_datatype_list());
        }
        return $results;        
    }    
    
    /**
    * Get identifiers from typesfields for current datatype.
    * @return array ? from ADOdb GetCol() method
    * @access public
    */
    function get_identifiers() 
    {
        // datatype, modified, created, id                        
        $sql = "
SELECT
    tf.identifier 
FROM
    typesfields tf                         
WHERE
    tf.typeid IN (" . $this->get_datatype_list() . ")
ORDER BY
    tf.orderby
        ";
        return $this->db->GetCol($sql);    
    }
    
    /**
    * Adds user defined fields to the minial columns used to generate SQL when working with more than one datatype
    *
    * @param array $arr, array of field names (database fields)
    * @access public
    */
    function set_user_minimal_columns($arr)
    {
        $this->user_minimal_columns = $arr;        
    }
    
    /**
    * Gets a list of minimal column names (pxdb reserved columns).
    * @param string $alias Alias to records table.
    * @return string SQL
    * @access public
    */
    function sql_minimal_columns($alias = '')
    {
        if (!empty($this->user_minimal_columns))
        {
            $reserved_fields = array_merge($this->user_minimal_columns, pxdb::get_pxdb_reserved_fields());
        }
        else
        {
            $reserved_fields = pxdb::get_pxdb_reserved_fields();
        }        
        $sql = '';
        $i = 0;
        while (list($r_key, $r_field) = each($reserved_fields))
        {
            $sql .= ($i++ ? ', ' : ' '). PXSQL_L . $alias . PXSQL_R . '.' . PXSQL_L . $r_field . PXSQL_R;
        }
        return $sql;
    }
    
    /**
    * Gets a list of fields to select from the records table.
    * This list is first a list of all fields (i.e. '*') and secondly a list of the aliased
    * fields.  Note that if there is a field in the aliased list it will override the value 
    * from the first list ('*').
    * 
    * @param string $alias The alias to the records table.
    * @return string The SQL to insert after the SELECT clause.
    * @access public
    */
    function sql_aliased_columns($alias = '') 
    {
        $pfx = ($alias ? $alias . '.' : '');
        
        // grab all the identifiers that have a dbfield correspondent for current datatypes
        $fields = $this->get_column_identifiers();        
        
        // get the reserved fields -- from pxdb_prefs table
        $reserved_fields = pxdb::get_pxdb_reserved_fields();    
        
        // check against actual columns in records
        $object_cache = &object_cache::singleton();
        if (! $records_columns = $object_cache->getTableColumns('records'))
        {
            $object_cache->saveTableColumns('records', $this->db->MetaColumnNames('records'));
            $records_columns = $object_cache->getTableColumns('records');
        }
        
        // add defaults -- e.g. datatype, id, modified, created
        $i = 0;
        $sql = '';
        while (list(, $reserved_field) = each($reserved_fields))
        {            
            if (!empty($records_columns[$reserved_field]))
            {
                $sql .= ($i++ ? ', ' : ' ');    
                $sql .= $pfx . $reserved_field;                    
            }
        }
                
        $fields = (array) $fields;
        while (list(, $field) = each($fields))
        {                
            if (!empty($records_columns[$field['fname']]))
            {
                $sql .= ($i++ ? ', ' : ' ');                    
                $sql .=  $pfx . PXSQL_L . $field['fname'] . PXSQL_R . ($field['identifier'] && $field['identifier'] != $field['fname'] ? ' AS ' . PXSQL_L . $field['identifier'] . PXSQL_R : '');
            }
        }
        return $sql;
    }
}
?>