<?php
/**
* Listenerers implement custom behaviors when a specific event occurs
*
* Event listeners are classes defined in PXDB_EVENTPATH that extend the base
* pxdb_event_listener class.  A single event listener should be able to 
* respond to multiple events (add, approve, application, etc...)
*
* An event listener should define the 
* @author Oscar Merida <oscar@oscarm.org>
* @package pxdb.events
* @version $Revision: 1.1 $ 
* @access public
*/
class pxdb_event_listener
{    
    /**
    * private array holds the events that trigger a listener to act    
    */
    var $_triggers = array(); 
    
    /**
    * Sets event name as a trigger for execute
    *
    * Conditions is an array where the key is a condition that the event
    * dispatcher can test for and the value is the values match it.
    * For now, one recognized condition is named 'datatype' and conditions
    * can hold all the datatype ids that would call execute()
    */    
    function addTrigger( $event_name )
    {
        $this->_triggers[ $event_name ] = TRUE;
    }
    
    /**
    * Returns an array of all the triggers for an array
    *
    * @return array
    * @access public
    */
    function getTriggers()
    {
        if ( !empty( $this->_triggers ) && is_array( $this->_triggers ) )
        {
            return array_keys( $this->_triggers ); 
        } 
        else 
        {
            return null;
        }
    }
    /**
    * Abstract method to be implemented by child class.
    *
    * Init method should set all the triggers a listener looks for.
    *
    * @abstract
    */
    function init()
    {
        
    }
    
    /**
    * Abstract method to be implemented by child class.
    *
    * Execute method for a listener is called when the event_dispatcher's notify method is called.
    *
    * @param string trigger name
    * @param array paramaters
    * @abstract
    */
    function execute( $trigger, $parameters )
    {
        trigger_error( get_class($this).' execute() called but it has not been extended.', E_USER_WARNING );
        return false;
    }
}
?>
