<?php
/**
* Caches calls to common object definition Syntax classes (datatypes)
*
* Stores object definitons in one array in memory
*
* @author Oscar Merida
*/

class object_cache {

    /**
    * @var array
    * @access private
    */
    var $datatypes = array();
    
    /**
    * @var array
    * @access private
    */
    var $typesFields = array();
    
    /**
    * @var array
    * @access private
    */
    var $typeFieldId = array();
    
    /**
    * @var array
    * @access private
    */
    var $dbFields = array();
    
    /**
    * @var array
    * @access private
    */
    var $tableColumns;
    
    /**
     * @var array $tfIdByIdentifiers used by pxdb_data
     * @access private
     */
    var $tfIdByIdentifiers;
    
    /**
     * @var array $tfIdbyDbFields used by pxdb_data
     * @access private
     */
    var $tfIdbyDbFields;
    
    /**
    * @var array $columnIdentifiers used by pxdb_data
    * @access private
    */
    var $columnIdentifiers;
    
    /**
     * @var array $dbTables An array of all the tables in the database
     * @access private
     */
    var $dbTables;
    
    
    /**
	* Returns a reference to an already-instantiated object_cache class.
	*
    * @access public
	* @return object object_cache
	*/	
    function &singleton()
    {
        static $instance;
		
        if (!isset($instance)) {
            $instance = new object_cache();
        }

        return $instance;
    }
    
    /**
    * Constructor
    *
    * @access public
    * @return void
    */
    function object_cache()
    {
    	
    }
    
    /**
    * Gets information about a given datatype
    *
    * @access public
    * @return array Reference to the info for a given datatype
    * @param 
    */
    function getDatatype( $dt )
    {        
        if( !empty( $this->datatypes[ $dt ] ) )
        {            
            return $this->datatypes[ $dt ];
        }
        else 
        {
            return false;
        }
    }
    
    function saveDatatype( $dt, $fields )
    {        
        $this->datatypes[ $dt ] = $fields;
        
    }
    
    function &getTypesFields( $tfid )
    {        
        if ( !empty( $this->typesFields[ $tfid ] ) )
        {            
            return $this->typesFields[ $tfid ];   
        }
        else
        {
            // avoid notices
            $return = false;
            return $return;
        }
    }
    
    function saveTypesFields( $tfid, $fields )
    {        
        $this->typesFields[ $tfid ] = $fields;   
    }
    
        
    function getDbField( $dbfid )
    {                
        if ( !empty( $this->dbFields[ $dbfid ] ) )
        {                
            return $this->dbFields[ $dbfid ];   
        }
        else 
        {
            return false;
        }
    }
    
    function saveDbField( $dbfid, $fields )
    {        
        $this->dbFields[ $dbfid ] = $fields;   
    }
    
    function getTypeFieldId( $datatype, $identifier )
    {        
        if ( !empty( $this->typeFieldId[ $datatype ][ $identifier ] ) )
        {
            return $this->typeFieldId[ $datatype ][ $identifier ];
        } else {
            return false;
        }
    }
    
    function saveTypeFieldId( $tfid, $datatype, $identifier )
    {     
        $this->typeFieldId[ $datatype ][ $identifier ] = $tfid;
    }
    
    /**
    * Get column names for a table
    *
    * @access public
    * @return array Reference to an array of column names
    * @param string $table Name of the desired table
    */
    function getTableColumns($table)
    {
        if ( !empty( $this->tableColumns[ $table ] ) )
        {
            return $this->tableColumns[ $table ];
        } else {
            return false;
        }
    }

    /**
    * Saves the columns of a table, passed as an array of strings
    *
    * @access public
    * @return void
    * @param string $table The name of the table
    * @param array $columns The names of the columns
    */
    function saveTableColumns($table, $columns)
    {     
        //oam: make columns a hash so that instead of in_array to 
        // see if a field is in a table we can do isset $columns[$field];
        
        while ( list($key,$val) = each( $columns ) )
        {
            $save[ $val ] = $val;
        }
        
        $this->tableColumns[ $table ] = $save;
    }
    
    /**
    * Get Typesfield IDs (and the count of rows) for a set of
    * datatypes, searching by typesfield identifier. Used by 
    * pxdb_data::lookup_tfid()
    *
    * @access public
    * @return array Array of column names
    * @param string $datatype_list The datatype list in string form
    * @param string $fieldname The name of the identifier searched for
    */
    function getTfIdByIdentifiers($datatype_list, $fieldname)
    {
        if ( !empty( $this->tfIdByIdentifiers[ $datatype_list ][ $fieldname ] ) )
        {
            return $this->tfIdByIdentifiers[ $datatype_list ][ $fieldname ];
        }
        else 
        {
            return false;
        }
    }

    /**
    * Sets Typesfield IDs (and the count of rows) for a set of
    * datatypes, searching by typesfield identifier. Used by 
    * pxdb_data::lookup_tfid()
    *
    * @access public
    * @return void
    * @param string $datatype_list The datatype list in string form
    * @param string $fieldname The identifier searched for
    * @param int $typesfield_id The id of the typesfield row found
    * @param int $num_rows The number of rows returned by the search
    */
    function saveTfIdByIdentifiers($datatype_list, $fieldname, $typesfield_id, $num_rows)
    {     
        $this->tfIdByIdentifiers[ $datatype_list ][ $fieldname ][ 'id' ] = $typesfield_id;
        $this->tfIdByIdentifiers[ $datatype_list ][ $fieldname ][ 'num_rows' ] = $num_rows;
    }
    
    /**
    * Get Typesfield IDs (and the count of rows) for a set of
    * datatypes, searching by dbfield name. Used by 
    * pxdb_data::lookup_tfid()
    *
    * @access public
    * @return array Reference to an array of column names
    * @param string $datatype_list The datatype list in string form
    * @param string $dbfield The name of the identifier searched for
    */
    function getTfIdByDbFields($datatype_list, $dbfield)
    {
        if ( !empty( $this->tfIdByDbFields[ $datatype_list ][ $dbfield ] ) )
        {
            return $this->tfIdByDbFields[ $datatype_list ][ $dbfield ];
        } else {
            return false;
        }
    }

    /**
    * Sets Typesfield IDs (and the count of rows) for a set of
    * datatypes, searching by dbfield name. Used by 
    * pxdb_data::lookup_tfid()
    *
    * @access public
    * @return void
    * @param string $datatype_list The datatype list in string form
    * @param string $dbfield The dbfield searched for
    * @param int $typesfield_id The id of the typesfield row found
    * @param int $num_rows The number of rows returned by the search
    */
    function saveTfIdByDbFields($datatype_list, $dbfield, $typesfield_id, $num_rows)
    {     
        $this->tfIdByDbFields[ $datatype_list ][ $dbfield ][ 'id' ] = $typesfield_id;
        $this->tfIdByDbFields[ $datatype_list ][ $dbfield ][ 'num_rows' ] = $num_rows;
    }
    
    /**
    * Get dbfields fname and typesfield identifier for a set of
    * datatypes. Used by pxdb_data::get_column_identifiers()
    *
    * @access public
    * @return array Array of column names
    * @param string $datatype_list The datatype list in string form
    */
    function getColumnIdentifiers($datatype_list)
    {
        if ( !empty( $this->columnIdentifiers[ $datatype_list ] ) )
        {
            return $this->columnIdentifiers[ $datatype_list ];
        } else {
            return false;
        }
    }

    /**
    * Set dbfields fname and typesfield identifier for a set of
    * datatypes. Used by pxdb_data::get_column_identifiers()
    *
    * @access public
    * @return void
    * @param string $datatype_list The datatype list in string form
    * @param array $column_identifiers hash of fnames and identifiers
    */
    function saveColumnIdentifiers($datatype_list, $column_identifiers)
    {     
        $this->columnIdentifiers[ $datatype_list ] = $column_identifiers;
    }
    
    /**
     * Get the tables in the database from the cache
     *
     * @access public
     * @return array Array of tables, false if nothing there
     *
     */
    function getDbTables()
    {
    	if ( !empty( $this->dbTables ) )
    	{
    	    return $this->dbTables;
    	}
    	else
    	{
    	    return false;
    	}
    }
    
    /**
     * Saves the list of tables in the database
     *
     * @access public
     * @return void
     * @param array $db_tables Array of names of tables in the database
     *
     */
    function saveDbTables( $db_tables )
    {
    	$this->dbTables = $db_tables;
    }
    
}
?>