<?php
// $Id: pxdb_basis.class.php,v 1.3 2005/12/28 17:11:57 omerida Exp $

pxdb_import('metadata.object_cache');
/**
* This class contains methods to derive metadata from an Syntax instance.
*
* This is the most generic/general purpose of the metadata classes.  This
* class instantiates via the fetch_*() methods the other metadata objects.
* 
* In addition to instantiating other classes, this class contains the methods that manage
* macro-level metadata.
* 
* @version $Revision: 1.3 $
* @package pxdb.metadata
* @author Hans Lellelid <hans@velum.net>
* @access public
*/
class pxdb_basis
{
	/**
	* Debug switch.
	* @var boolean
	* @access public
	*/
	var $debug; // bool var to track debugging

	/**
	* Errors array.
	* @var array
	* @access public
	*/
	var $errors; // assoc array of errors

	/**
	* Vars imported from global / $_REQUEST scope.
	* @var array
	* @access protected
	*/
	var $_imported;
	
	/**
	* Fields array (used by subclasses).
	* @var array
	* @access private
	*/
	var $_fields;

	/**
	* Array of fields load().
	* @var array
	* @access private
	*/
	var $_old_fields;

	/**
	* Calls parent constructor.
	*/
	function pxdb_basis() 
	{
		$this->db = &pxdb::dbh();
		$this->prefs = &pxdb_prefs::singleton();
		$this->import();
	}
	
	/**
	* Performs the basic task of loading data into the $_imported var.
	* Sub-classes that override this method should always provide this minimal
	* function signature and should begin by invoke the parent import() method.
	* @param array $source The source values to import.
	*/	
	function import($source=null)
	{
		$this->_imported = pxdb::import($source);
	}

	/**
	* Loads vars from global scope into current class.
	* Uses the set_*() methods of the current class to load values for every
	* column of table -- if that value is in the $_imported array.
	* @param string $table The table for which to load values.
	* @access public
	*/
	function load($table) 
	{
	
		// get current vars by simply looking up columns in current table
		
		if (!$table) 
        { 
			trigger_error("Cannot load() with no table defined -- in class: ".get_class($this));
			return false;
		}
		
		// 1) save a pre-load copy of the vars (e.g. dbfields needs to know)
		
			$this->_old_fields = $this->_fields;
		
		// 2) load new ones from global scope
		
        $object_cache = &object_cache::singleton();
        if (! $cols = $object_cache->getTableColumns( $table ) )
        {
		    $object_cache->saveTableColumns($table, $this->db->MetaColumnNames($table));
		    $cols = $object_cache->getTableColumns( $table );
		}
		$class_methods = get_class_methods(get_class($this));
		
		while ( list( $colname, $col ) = each( $cols ) )
		{
			if ( isset( $this->_imported[ $col ] ) )
			{
				// calculate method name based on var
				$method = 'set_' . $col;
				// if it exists, invoke it	
				if(in_array($method, $class_methods)) 
                {
					$this->$method($this->_imported[$col]);				
				}
                else 
                {			
					trigger_error("Could not find class method: $method", E_USER_WARNING);			
				}
				
			} // if isset()		
			
		} // foreach
		
	}
	
	/**
	* Fetch all datatypes rows and instantiate as pxdb_datatype objects.
	* @param $reset If not null, will restart the loop through datatypes rows.
	* @return object the instantiated pxdb_datatype object.
	* @access public
	*/	
	function &fetch_datatype($reset=null)
	{
		static $rs=null;

		if (!$rs || null !== $reset) 
        {
    		// execute a query i
    		$sql = "SELECT id FROM datatypes ORDER BY weight";
    		$rs = $this->db->Execute($sql); // can't assign static vars by reference
		}

		$dt = null; // initialize

		if (!$rs->EOF) 
        {
			$dt = &new pxdb_datatype($rs->fields['id']);
			$rs->MoveNext();
		}

		return $dt;

	}

	/**
	* Fetch all relationships rows and instantiate as pxdb_relationship objects.
	* @param $reset If not null, will restart the loop through relationships rows.
	* @return object the instantiated pxdb_relationship object.
	* @access public
	*/
	function &fetch_relationship($reset=null) 
	{

		static $rs=null;

		if (!$rs || null !== $reset) 
        {
    		// execute a query
    		$sql = "SELECT id FROM relationships ORDER BY weight";
    		$rs = $this->db->Execute($sql); // can't assign static vars by reference			
		} 

		$rel = null; // initialize

		if (!$rs->EOF) 
        {
			$rel =& new pxdb_relationship($rs->fields['id']);
			$rs->MoveNext();
		}
		
		return $rel;
	}
		
	/**
	* Fetch all dbfields rows and instantiate as pxdb_dbfield objects.
	* @param $reset If not null, will restart the loop through dbfields rows.
	* @return object the instantiated pxdb_dbfield object.
	* @access public
	*/	
	function &fetch_dbfield($reset=null) 
	{
	
		static $rs=null;
		
		if (!$rs || $reset !== null) 
        {
    		// execute a query i
    		$sql = "SELECT id FROM dbfields ORDER BY fname";
    		$rs = $this->db->Execute($sql); // can't assign static vars by reference
		} 
		
		$dbf = null; // initialize
		
		if (!$rs->EOF) 
        {
			$dbf = &new pxdb_dbfield($rs->fields['id']);
			$rs->MoveNext();
		}
		
		return $dbf;
	}

	/**
	* Fetch all typesfields rows and instantiate as pxdb_typesfield objects.
	* @param $dt The datatype of object we are working with.
	* @param $reset If not null, will restart the loop through typesfields rows.
	* @return object the instantiated pxdb_typesfield object.
	* @access public
	*/
	function &fetch_typesfield($dt, $reset=null) 
	{
	
		static $rs=null;
		
		if ( !$rs || $reset !== null) 
        {
    		// execute a query i
    		$sql = "SELECT id FROM typesfields WHERE typeid = '$dt' ORDER BY orderby";
    		$rs = $this->db->Execute($sql); // can't assign static vars by reference
		}
		
		$obj = null; // initialize
		
		if ( !$rs->EOF) 
        {
			$obj =& new pxdb_typesfield($rs->fields['id']);
			$rs->MoveNext();
		}
		
		return $obj;
	}
	
	/**
	* Fetch all views rows and instantiate as pxdb_view objects.
	* @param $dt The datatype of object we are working with.
	* @param $reset If not null, will restart the loop through typesfields rows.
	* @return object the instantiated pxdb_typesfield object.
	* @access public
	*/
	function &fetch_view($dt, $reset=null) 
	{
	
		static $rs=null;
		
		if (!$rs || $reset !== null) 
        {
    		// execute a query i
    		$sql = "SELECT id FROM ".PXSQL_L."views".PXSQL_R." WHERE datatype = $dt ORDER BY ".PXSQL_L."view".PXSQL_R;
    		$rs = $this->db->Execute($sql); // can't assign static vars by reference
		}
		
		$obj = null; // initialize
		
		if (!$rs->EOF) 
        {
			$obj =& new pxdb_view($rs->fields['id']);
			$rs->MoveNext();
		}
		
		return $obj;
	}
	
	/**
	* Fetch all viewfields rows and instantiate as pxdb_viewfield objects.
	* @param $v The view of object we are working with.
	* @param $reset If not null, will restart the loop through typesfields rows.
	* @return object the instantiated pxdb_typesfield object.
	* @access public
	*/
	function &fetch_viewfield($v, $reset=null) 
	{
	
		static $rs=null;
		
		if (!$rs || $reset !== null) 
        {
    		// execute a query i
    		$sql = "SELECT id FROM viewfields WHERE viewid = $v ORDER BY orderby";
    		$rs = $this->db->Execute($sql); // can't assign static vars by reference
		}
		
		$obj = null; // initialize
		
		if ( !$rs->EOF) 
        {
			$obj =& new pxdb_viewfield($rs->fields['id']);
			$rs->MoveNext();
		}
		
		return $obj;
	}
		
	
	/**
	* Get a list of dbfields for this pxdb instance.
	* @param string $mask The value to return for custom_field.
	* @return array dbfields.
	* @access public
	*/
	function arr_dbfields($mask='{fname}') 
	{
	
		$column = pxdb_sql::sqlize_mask($mask);
		
		$sql = "SELECT		*,
							$column as custom_field
        		FROM		dbfields
        		ORDER BY	fname";
		
		return $this->db->GetAll($sql);
	}
	
		
	/**
	* Get all datatypes for this pxdb instance.
	* @param string $mask The value to return for custom_field.
	* @param string $constraint_sql WHERE-clause SQL to constrain results.
    * @param string order by fields(s), defaults to weight
	* @access public
	*/
	function arr_datatypes($mask='{type}', $constraint_sql=null, $orderby = 'weight' ) 
	{
	
		$column = pxdb_sql::sqlize_mask($mask);
	        
		$sql = "SELECT		*,
							$column AS custom_field
        		FROM		datatypes
				".($constraint_sql ? "WHERE $constraint_sql" : "")."
        		ORDER BY	".$orderby;
		
		 return $this->db->GetAll($sql);
	}

	/**
	* Get all parent datatypes for a given datatype.
	* @param integer $dt Datatype
	* @param string $mask The value to return for custom_field.
	* @param string $constraint_sql WHERE-clause SQL to constrain results.
	* @access public
	*/
	function arr_parent_datatypes($dt, $mask='{type}', $constraint_sql=null) 
	{
	
		$column = pxdb_sql::sqlize_mask($mask, 'parent_dt');
	
		// get distinct relids
		$sql = "SELECT		parent_dt.*,
							$column AS custom_field
        		FROM		datatypes child_dt
							INNER JOIN datatypesre rel ON rel.relid = child_dt.parent_relid
							INNER JOIN datatypes parent_dt ON ((rel.dt1 = child_dt.id AND parent_dt.id = rel.dt2) OR (rel.dt2 = child_dt.id AND parent_dt.id = rel.dt1))
				WHERE		child_dt.id = $dt							
							".($constraint_sql ? "AND $constraint_sql" : "")."
        		ORDER BY	parent_dt.weight";
		
		return $this->db->GetAll($sql);
	}

	/**
	* Get all child datatypes for a given datatype.
	* @param integer $dt Datatype
	* @param string $mask The value to return for custom_field.
	* @param string $constraint_sql WHERE-clause SQL to constrain results.
	* @access public
	*/
	function arr_child_datatypes($dt, $mask='{type}', $constraint_sql=null) 
	{
	
		$column = pxdb_sql::sqlize_mask($mask, 'child_dt');
	
		$sql = "SELECT		child_dt.*,
							$column AS custom_field
        		FROM		datatypes child_dt
							INNER JOIN datatypesrel rel ON rel.relid = child_dt.parent_relid
							INNER JOIN datatypes parent_dt ON ((rel.dt2 = parent_dt.id AND child_dt.id = rel.dt1))
				WHERE		parent_dt.id = $dt							
							".($constraint_sql ? "AND $constraint_sql" : "")."
        		ORDER BY	child_dt.weight";
		
		return $this->db->GetAll($sql);
	}
		
	/**
	* Gets the relationships for this pxdb instance.
	* @param string $mask The value to return for custom_field.
	* @param string $constraint_sql WHERE-clause SQL to constrain results.
	* @return array relid and description of relationships.
	* @access public
	*/
	function arr_relationships($mask='{description}', $constraint_sql=null) 
	{
		
		$column = pxdb_sql::sqlize_mask($mask);
		
		$sql = "SELECT		id
        		FROM		relationships
				".($constraint_sql ? "WHERE $constraint_sql" : "")."
				ORDER BY	weight";		
		
		return $this->db->GetAll($sql);		
	}
	
	/**
	* Gets the relationships for this pxdb instance.
	* @param integer $dt Datatype to fetch typesfields for.
	* @param string $mask The value to return for custom_field.
	* @param string $constraint_sql WHERE-clause SQL to constrain results.
	* @return array relid and description of relationships.
	* @access public
	*/
	function arr_typesfields($dt, $mask='{identifier}', $constraint_sql=null) 
	{
		
		$column = pxdb_sql::sqlize_mask($mask);
		
		$sql = "SELECT		*,
							$column AS custom_field
        		FROM		typesfields
				WHERE		typeid = $dt
							".($constraint_sql ? "AND $constraint_sql" : "")."
        		ORDER BY	orderby";		
		
		return $this->db->GetAll($sql);
		
	}
	
	/**
	* Get all views for a specific datatype.
	* @param integer $dt The datatype for which to fetch views.
	* @param string $mask The value to return for custom_field.
	* @param string $constraint_sql WHERE-clause SQL to constrain results.
	* @access public
	*/
	function arr_views($dt, $mask='{view}', $constraint_sql=null) 
	{
	
		$column = pxdb_sql::sqlize_mask($mask);
	
		$sql = "SELECT		*,
							$column AS custom_field
        		FROM		views
				WHERE		datatype = $dt
							".($constraint_sql ? "AND $constraint_sql" : "")."
				ORDER BY	".PXSQL_L."view".PXSQL_R;
		
		return $this->db->GetAll($sql);		
	}

	/**
	* Get all viewfields for a specific view.
	* @param integer $dt The datatype for which to fetch views.
	* @param string $mask The value to return for custom_field.
	* @param string $constraint_sql WHERE-clause SQL to constrain results.
	* @access public
	*/
	function arr_viewfields($v, $mask='{label}', $constraint_sql=null) 
	{
	
		$column = pxdb_sql::sqlize_mask($mask);
	
		$sql = "SELECT		*,
							$column AS custom_field
        		FROM		viewfields
				WHERE		viewid = $v
							".($constraint_sql ? "WHERE $constraint_sql" : "")."
				ORDER BY	orderby";
		
		return $this->db->GetAll($sql);
	}
	
	
	/**
	* Abstract function for re-ordering weighted rows.
	* This function goes through the table assigning weights of interval 5 to all the rows
	* matching constraints.
	* @param array $params Hash of variables needed: table, id, dir, id_col, weight_col, constraint_sql.
	*/
	function reorder($params) 
	{
		
		// for curr datatype
		extract($params);
		
		if (empty($constraint_sql)) 
        {
            $constraint_sql = '1'; 
        }
		
		$sql = "SELECT $id_col, $weight_col FROM $table WHERE ($constraint_sql) ORDER BY $weight_col ASC";
		
		$rs = &$this->db->Execute($sql);
		
		for($oby=0; !$rs->EOF; $oby += 5) 
        {
            // precautionary
			$sql = "UPDATE $table SET $weight_col = $oby WHERE $id_col = " . $rs->fields[$id_col] . " AND ($constraint_sql)";
			$this->db->Execute($sql);			
			$rs->MoveNext();
		}

	}
	
	/**
	* Abstract function for promoting or demoting a row.
	* @param array $params Hash of variables needed: table, id, dir, id_col, weight_col, constraint_sql.
	* @return boolean True or False on success of move.
	*/
	function move($params) 
	{
		extract($params);

		$dir = strtolower($dir); // 'UP' -> 'up'
		
		if (empty($constraint_sql)) 
        {
            $constraint_sql = '1'; 
        }
		
		$sql = "SELECT $weight_col FROM $table WHERE $id_col = $id AND ($constraint_sql)";
		$rs = $this->db->Execute($sql);
		
		$curr_weight = $rs->fields[$weight_col];
		
		switch($dir) 
        {
			case 'up':				
				$new_sql = "SELECT $id_col, $weight_col FROM $table WHERE $weight_col < $curr_weight AND ($constraint_sql) ORDER BY $weight_col DESC";				
			break;			
			
			case 'down':			
				$new_sql = "SELECT $id_col, $weight_col FROM $table WHERE $weight_col > $curr_weight AND ($constraint_sql) ORDER BY $weight_col ASC";				
			break;		
		}
		
		$new_rs = &$this->db->Execute($new_sql);
		
		if (!$new_rs->EOF) 
        {
			// if it found a higher or lower order by value, then it
			// should swap them.
			$sql = "UPDATE $table SET $weight_col = $curr_weight WHERE $id_col = ".$new_rs->fields[$id_col];
			$this->db->Execute($sql);
			
			$sql = "UPDATE $table SET $weight_col = ".$new_rs->fields[$weight_col]." WHERE $id_col = $id";
			$this->db->Execute($sql);
			
			return true;
		}
        else 
        {
			
			// first check to make sure that there aren't any duplicates
			$sql = "SELECT COUNT(*) AS count FROM $table WHERE $weight_col = $curr_weight AND ($constraint_sql)";
			$check_rs = &$this->db->Execute($sql);
			
			// if dup orderby nums were found -- then re-organize
			if ( 1 < $check_rs->fields['count'] ) 
            {
			
				$this->reorder($params);
				
				// and run this function again ...  RECURSIVE
				$this->move($params);
			
				return true;
			}
            else 
            {
				// wrap around 
				switch($dir) 
                {		
				// if we were moving up & it didn't have anywhere to go, put it at the end of the list.
					case 'up':				
						$new_sql = "SELECT $id_col, $weight_col FROM $table ORDER BY $weight_col DESC";
						$new_rs = &$this->db->Execute($new_sql);
						$new_weight = $new_rs->fields[$weight_col] + 5;
					break;								
				// if we were moving down & there was no where to go, put it back at the beginning of the list.
					case 'down':			
						$new_sql = "SELECT $id_col, $weight_col FROM $table ORDER BY $weight_col ASC";
						$new_rs = &$this->db->Execute($new_sql);
						$new_weight = $new_rs->fields[$weight_col] - 5; // may be < 0, but that's ok -- it'll get fixed in re-order			
					break;		
				}
				
				$sql = "UPDATE $table SET $weight_col = $new_weight WHERE $id_col = $id";
				$this->db->Execute($sql);				
				
				return true;
			}
		} 
	}
	
		
	/**
	* Gets the groups from the database.
	* As specified by  the group_dt values in preferences table.
	* 
	* @return Array of id and gname for all groups.
	*/	
	function get_groups() 
	{
		$sql = "SELECT id, IF(LENGTH(name), name, idnum) as gname
				FROM records 
				WHERE 	datatype IN (".$this->prefs->get_list('dt_group').") AND (name <> '' OR idnum <> '')
				ORDER BY gname";
	
		return $this->db->GetAll($sql);
	}
	
	/**
	* Gets the users from the database.
	* As specified by the user_dt values in preferences table.
	* @return Array of id and uname for all users.
	*/
	function get_users() 
	{
		$sql = "SELECT id, IF(LENGTH(lname), CONCAT(lname, ', ', fname), IF(LENGTH(name), name, idnum)) as uname
				FROM records 
				WHERE 	datatype IN (".$this->prefs->get_list('dt_user').") AND (name <> '' OR lname <> '' OR idnum <> '')
				ORDER BY uname";	
		
		return $this->db->GetAll($sql);		
	}
	
}
?>
