<?php
// $Id: pxdb_datatype.class.php,v 1.4 2005/08/16 04:35:58 sandys1 Exp $

pxdb_import('metadata.pxdb_basis');
pxdb_import('metadata.pxdb_typesfield');
pxdb_import('metadata.pxdb_view');
pxdb_import('metadata.object_cache');
/**
* This class represtents a row of the datatypes table.  Implements object_cache
* to reduce database load
*
* @version $Revision: 1.4 $
* @package pxdb.metadata
* @author Hans Lellelid <hans@velum.net>
* @access public
*/
class pxdb_datatype extends pxdb_basis 
{
            
    /**
    * Constructor: perform db query to get full row based on the relationship id.
    * @param integer $dt The datatype id.
    */    
    function pxdb_datatype($dt=null) 
    {
        parent::pxdb_basis();        
        
        $this->set_id($dt);        
        $this->read();        
        
    }
    
    /**
    * Updates class vars from the database.
    * @access public
    */
    function read() 
    {    
        $object_cache = &object_cache::singleton();
        
        if ( $dt = $this->get_id()) 
        {
            
            if ( $datatype = $object_cache->getDatatype( $dt ) )
            {
                $this->_fields = $datatype;
            }
            else 
            {
                $sql = "select * from datatypes where id = ".$dt;
                $rs = $this->db->Execute($sql);
                
                $object_cache->saveDatatype( $dt, $rs->fields );
                $this->_fields = $rs->fields;        
            }                    
        }
    }
    

    /** 
    * Load override: loads values from global scope into the class.
    * @access public
    */
    function load() 
    {
        parent::load('datatypes');
    }

    /**
    * @return integer
    * @access public
    */
    function get_id() 
    {
        return $this->_fields['id'];
    }
    
    /**
    * @param integer $id
    * @access public
    */
    function set_id($id) 
    {
        $this->_fields['id'] = $id;
    }
    
    /**
    * @return string Datatype name
    * @access public
    */
    function get_type() 
    {
        return $this->_fields['type'];
    }

    /**
    * @param integer $type
    * @access public
    */    
    function set_type($type) 
    {
        $this->_fields['type'] = $type;
    }
    
    /**
    * @return string 'Y'/'N'
    * @access public
    */
    function get_expose() 
    {
        $expose = $this->_fields['expose'];
        if (!$expose) 
        {
            $expose = 'N'; 
        }
        return $expose;
    }
    
    /**
    * @param string $expose 'Y'/'N'
    * @access public
    */
    function set_expose($expose) 
    {
        $this->_fields['expose'] = $expose;
    }
    
    /**
    * @return integer
    * @access public
    */
    function get_weight() 
    {
        return $this->_fields['weight'];
    }
    
    /**
    * @access public
    */
    function set_weight($weight) 
    {
        $this->_fields['weight'] = $weight;
    }

    /**
    * @access public
    */
    function get_label() 
    {
        if (true === isset($this->_fields['label']))
        {
            return $this->_fields['label'];
        }
    }
    
    /**
    * @access public
    */
    function set_label($label) 
    {
        $this->_fields['label'] = $label;
    }

    /**
    * @access public
    */
    function get_description() 
    {
        return $this->_fields['description'];
    }
    
    /**
    * @access public
    */
    function set_description($description) 
    {
        $this->_fields['description'] = $description;
    }

    /**
    * @return integer Parent datatype ID
    * @access public
    */
    function get_parent_relid() 
    {
        if (true === isset($this->_fields['parent_relid']))
        {
            return $this->_fields['parent_relid'];
        }
    }
    
    /**
    * @param integer $parent Parent datatype ID
    * @access public
    */
    function set_parent_relid($parent_relid) 
    {
        $this->_fields['parent_relid'] = $parent_relid;
    }

    /**
    * @return string
    * @access public
    */
    function get_weighted() 
    {
        if (true === isset($this->_fields['weighted']))
        {
            return $this->_fields['weighted'];
        }
    }
    
    /**
    * @param string $weighted 'Y'/'N'
    * @access public
    */
    function set_weighted($weighted) 
    {
        $this->_fields['weighted'] = $weighted;
    }    
    
    /**
    * Wrapper for parent method.
    * @param boolean $reset Whether to start over.
    * @return object pxdb_typesfield object
    */
    function &fetch_typesfield($reset=null)
    {
        return parent::fetch_typesfield($this->get_id(), $reset);
    }

    /**
    * Wrapper for parent method.
    * @param boolean $reset Whether to start over.
    * @return object pxdb_typesfield object
    */
    function &fetch_view($reset=null)
    {
        return parent::fetch_view($this->get_id(), $reset);
    }
        
    /**
    * Returns an array of typesfields for this object.
    * @param string $mask The value to return for custom_field.
    * @param string $constraint_sql WHERE-clause SQL to constrain results.
    * @return array 2-dim array of typsefields.
    */
    function arr_typesfields($mask='{identifier}', $constraint_sql=null) 
    {            
        return parent::arr_typesfields($this->get_id(), $mask, $constraint_sql );        
    }
    
    /**
    * Get all views for a specific datatype.
    * @param string $mask The value to return for custom_field.
    * @param string $constraint_sql WHERE-clause SQL to constrain results.
    * @access public
    */
    function arr_views($mask='{view}', $constraint_sql=null) 
    {    
        return parent::arr_views($this->get_id(), $mask, $constraint_sql);
    }

    
    /**
    * Finds dbfields that are in use by this object.
    * @param string $mask The value to return for custom_field.
    * @param string $constraint_sql WHERE-clause SQL to constrain results.
    * @access public
    */
    function arr_dbfields($mask='{fname}', $constraint_sql=null) 
    {
        $column = pxdb_sql::sqlize_mask($mask, 'dbf');
        
        $sql = "SELECT     DISTINCT dbf.*, 
                        $column as custom_field
                FROM     dbfields dbf
                        inner join typesfields tf ON tf.fieldid = dbf.id
                WHERE     tf.typeid = ".$this->get_id()."
                        ".($constraint_sql ? "AND $constraint_sql" : "");
        
        $rs = $this->db->Execute($sql);        
        return $rs->GetArray();
    }

    /**
    * Finds dbfields that are in use by this object.
    * @param string $mask The value to return for custom_field.
    * @param string $constraint_sql WHERE-clause SQL to constrain results.
    * @access public
    */
    function arr_relationships($mask='{description}', $constraint_sql=null) 
    {
        
        $column = pxdb_sql::sqlize_mask($mask, 'rel');
        
        $sql = "SELECT DISTINCT rel.*,
                        $column as custom_field
                FROM     datatypesrel rel
                        inner join typesfields tf ON tf.relid = rel.relid
                WHERE     tf.typeid = ".$this->get_id()."
                        ".($constraint_sql ? "AND $constraint_sql" : "");
                                        
        return $this->db->GetAll($sql);
    }
    
    /**
    * Get all child datatypes for a given datatype.
    * @param string $mask The value to return for custom_field.
    * @param string $constraint_sql WHERE-clause SQL to constrain results.
    * @access public
    */
    function arr_child_datatypes($mask='{type}', $constraint_sql=null) 
    {
        return parent::arr_child_datatypes($this->get_id(), $mask, $constraint_sql);
    }

    /**
    * Get all parent datatypes for a given datatype.
    * @param string $mask The value to return for custom_field.
    * @param string $constraint_sql WHERE-clause SQL to constrain results.
    * @access public
    */
    function arr_parent_datatypes($mask='{type}', $constraint_sql=null) 
    {
        return parent::arr_parent_datatypes($this->get_id(), $mask, $constraint_sql);
    }    
        
    /**
    * Re-orders the relationship weights.
    * This is a cleanup method called by move() when there seem to be problems in the ordering
    * of rows.
    */
    function reorder() 
    {
    
        // for curr datatype
        $params = array('id' => $this->get_id(),
                        'table' => 'datatypes',
                        'constraint_sql' => '',
                        'id_col' => 'id',
                        'weight_col' => 'weight',
                        'dir' => $dir 
                        );

        // invoke abstract parent method
        parent::reorder($params);
    }

    /**
    * Promote / demote (change 'weight') of datatype row.
    * This function changes the order in which datatypes are displayed (on the list, in pull-downs, etc.).
    * @access public
    */
    function move($dir) 
    {
    
        // 1) do we need to clean up order of fields?
        $sql = "select d.weight from datatypes d inner join datatypes d2 on d2.id <> d.id and d2.weight = d.weight";
        $test_rs = $this->db->Execute($sql);
        if (!$test_rs->EOF) 
        {
            $this->reorder();
        }
        
        $params = array('id' => $this->get_id(),
                        'table' => 'datatypes',
                        'constraint_sql' => '',
                        'id_col' => 'id',
                        'weight_col' => 'weight',
                        'dir' => $dir 
                        );
        
        // call the abstract method
        parent::move($params);
        // get the weight of the current field.
        // swap with next higher or lower field
    }

    /**
    * The main function to handle inserts & updates.
    * Be sure you call load() before invoking this method if you want it to pull values
    * from the global scope.
    * @access public
    */
    function save() 
    {
        
        if ($this->get_id()) 
        {
        
            $sql = "UPDATE datatypes SET
                            type = ".$this->db->Quote($this->get_type()).",
                            expose = ".$this->db->Quote($this->get_expose()).",
                            label = ".$this->db->Quote($this->get_label()).",
                            description = ".$this->db->Quote($this->get_description()).",
                            parent_relid = ".$this->db->Quote($this->get_parent_relid()).",
                            weighted = ".$this->db->Quote($this->get_weighted())."
                    WHERE id = ".$this->get_id();
                    
            $this->db->Execute($sql);
        }
        else 
        {
            $sql = "INSERT INTO datatypes (type, expose, weight, label, description, parent_relid, weighted) VALUES (
                            ".$this->db->Quote($this->get_type()).",
                            ".$this->db->Quote($this->get_expose()).",
                             ".$this->_new_weight().",
                            ".$this->db->Quote($this->get_label()).",
                            ".$this->db->Quote($this->get_description()).",
                            ".$this->db->Quote($this->get_parent_relid()).",
                            ".$this->db->Quote($this->get_weighted()).")";
        
            $this->db->Execute($sql);
            $this->set_id($this->db->Insert_ID());
            
            //OAM - default dbasis to insert name, sunrise, sunset fields into typesfields
            //OAM - come back and lookup field ids
            $id = $this->get_id();
            $name = new pxdb_typesfield();
                $name->set_typeid( $id );
                $name->set_displayname( 'Name' );
                $name->set_identifier( 'name' );
                $name->set_req('Y');
                $name->set_fieldid( 2 ); // 2 is name field
            $name_id = $name->save();
                
            $sunrise = new pxdb_typesfield();
                $sunrise->set_typeid( $id );
                $sunrise->set_displayname( 'Go live' );
                $sunrise->set_identifier( 'sunrise' );
                $sunrise->set_req('N');
                $sunrise->set_fieldid( 47 ); // 47 is sunrise field
            $sunrise->save();
            
            $sunset = new pxdb_typesfield();
                $sunset->set_typeid( $id );
                $sunset->set_displayname( 'Expire' );
                $sunset->set_identifier( 'sunset' );
                $sunset->set_req('N');
                $sunset->set_widget('optional_date');
                $sunset->set_fieldid( 48 ); // 48 is sunset field
            $sunset->save();
            
            // oam default dbasis to grant all datatype privs to admin group
            pxdb_import( 'metadata.pxdb_datatype_privs' );
            pxdb_import( 'pxdb_prefs' );
            $prefs =& pxdb_prefs::singleton();
            $admin_ugid = $prefs->get_pref('gid_admin');
            
            $obj_privs =& new pxdb_datatype_privs($admin_ugid);
            
            $defined_privs = pxdb_datatype_privs::get_priv_columns();
            
            if ( 0 < count($defined_privs) )
            {
                $priv_array = array();
                while ( list( ,$privc )  = each( $defined_privs ) )
                {
                    $priv_array[$privc] = TRUE;
                }
                
                $obj_privs->grant( $id, $admin_ugid, $priv_array );
            }
            
            //OAM default dbasis to create 'admin' view and populate it
            //    with the name field.
            pxdb_import( 'metadata.pxdb_view' );
            pxdb_import( 'metadata.pxdb_viewfield' );
            $view = new pxdb_view();
            $view->set_view('admin');
            $view->set_datatype( $id );
            $view->set_label('Admin');
            $view->set_description('Controls columns used in admin list view.');
            $view_id = $view->save();
            
            $vf = new pxdb_viewfield();
            $vf->set_viewid( $view_id );
            $vf->set_typesfield( $name_id );
            $vf->save();
            // reread record -- just to be sure everything is up-to-date.
            // SFS - not necessary for updated record, only populating id in
            // new record.
            $this->read();            
        }        
        
        if ($this->debug) 
        {
            pxdb::print_debug($sql, 'db-update'); 
        }
        
        return $this->get_id();
    }                
    

    /**
    * This function figures out what the next (max+5) weight should be.
    * @access private
    */
    function _new_weight() 
    {    
        $sql = "select weight from datatypes order by weight desc";
        $rs = $this->db->Execute($sql);
        return ($rs->fields['weight'] + 5);
    }

    
    /**
    * Function to delete a relationship entry.
    * @param integer $id The ID of the record we wish to remove.
    * @access public
    */
    function delete($id=null) 
    {    
        $db =& pxdb::dbh();
        $id = ($id ? $id : $this->get_id());

        if (!$id) 
        {
            trigger_error("You must instantiate pxdb_datatype with the id of the field you wish to delete -- or pass an id to delete() method.", E_USER_WARNING);
            return false;
        }
        
        $sql = "DELETE FROM datatypes WHERE id = $id"; // note: could be deleting more than one record.
        if ($this->debug) 
        {
            pxdb::print_debug($sql, 'db-update'); 
        }
        
        $db->Execute($sql);
        
        $sql = "SELECT id FROM typesfields WHERE typeid = $id";
        $rs = $db->Execute($sql);
        
        while (!$rs->EOF) 
        {
            pxdb_typesfield::delete($rs->fields['id']);    
            $rs->MoveNext();
        }
        
        $sql = "SELECT id FROM views WHERE datatype = $id";
        $rs = $db->Execute($sql);
        while (!$rs->EOF) 
        {
            pxdb_view::delete($rs->fields['id']);    
            $rs->MoveNext();
        }        
        
        return true;
        
    }


} // class

?>
