<?php
// $Id: pxdb_dbfield.class.php,v 1.6 2006/02/07 18:42:04 omerida Exp $

pxdb_import('metadata.pxdb_basis');
pxdb_import('metadata.object_cache');

/**
* This class represtents a row of the dbfields table.
*
* @version $Revision: 1.6 $
* @package pxdb.metadata
* @author Hans Lellelid <hans@velum.net>
* @access public
*/
class pxdb_dbfield extends pxdb_basis 
{
	
	/**
	* Constructor: perform db query to get full row based on the dbfield name or id.
	* @param $db The ADOdb connection to use.
	* @param $f A field id or [unique] fname.
	*/	
	function pxdb_dbfield($id=null) 
	{
		
		parent::pxdb_basis();
		
		$this->set_id($id);
		$this->read();

	}

	/**
	* Updates class vars from the database.
	* @access public
	*/
	function read() 
	{	
        $object_cache =& object_cache::singleton();
        
		if ( $dbfid = $this->get_id()) 
        {			    		
            
            if ( $dbfield = $object_cache->getDbField( $dbfid ) )
            {                
                $this->_fields = $dbfield;                    
            }
            else 
            {
                if ( is_numeric($dbfid) )
                {
                    $sql = "select * from dbfields where id = ".$dbfid;
                }
                else
                {
                    $sql = "select * from dbfields where fname = " . $this->db->quote($dbfid);
                }
                $rs = &$this->db->Execute($sql);
                $this->_fields = $rs->fields;		
                                
                $object_cache->saveDbField( $dbfid, $this->_fields );
            }             
		}					
	}

	
	/** 
	* Load subclass: loads values from global scope into the class.
	* @access public
	*/
	function load() 
	{
		parent::load('dbfields'); 
	}

	/**
	* @access public
	*/
	function get_id() 
	{
		return $this->_fields['id'];
	}
	
	/**
	* @access public
	*/
	function set_id($id) 
	{	
		$this->_fields['id'] = $id;
	}
	
	/**
	* @access public
	*/
	function get_fname() 
	{
        if ( isset($this->_fields['fname']) )
        {
            return $this->_fields['fname'];
        }
	}
	
	/**
	* @access public
	*/
	function set_fname($fname) 
	{
		$this->_fields['fname'] = $fname;
	}
	
	/**
	* @access public
	*/
	function get_ftype() 
	{
        if ( isset( $this->_fields['ftype'] ) )
        {
            return $this->_fields['ftype'];
        }
	}

	/**
	* @access public
	*/
	function set_ftype($ftype) 
	{
		$this->_fields['ftype'] = $ftype;
	}

	/**
	* Gets the field type.
	* This function returns the first part of the ftype column.
	* @return string The first word of ftype column.
	* @access public
	*/
	function get_field_type() 
	{
		return $this->_parts($this->get_ftype(), 0);
	}
	
	
	
	/**
	* Get pick source from ftype.
	* Only applies to dbfields where ftype is pickid or pick
	*
	* Modified: Joe Khoobyar <joe@ankhcraft.com>
	* Fixed bugs in regexps.
	*
	* @access public
	*/
	function get_pick_source() 
	{
	
		$param = $this->_parts($this->get_ftype(), 1);		
		
        // oam: moved this to the front, if param is empty, no need to do preg_match at all
        if ( empty( $param) )
        {
            $type = false;
        }
        else if(preg_match("/\\\$\([^\)]+\)/", $param)) 
        {
            //if it starts with $( any non-closing paren to a )		
			$type = "vartable";
		}
        else if(preg_match("/\\[[^\\]]+\\]/", $param)) 
        {
			$type = "sql";
		}
        else if (!empty($param)) 
        {
			$type = "table";
		}
        else 
        {
			$type = false;
		}
		
		return $type;

	}
	
	/**
	* Returns the 2nd component of the pick/pickid column type.
	* @access public
	*/
	function get_pick_table() 
	{
		// assuming current dbfield has a table defined
		
		return $this->_parts($this->get_ftype(), 1);
	}
	
	/**
	* Returns the 3rd parameter in pick/picked column type.
	* @access public
	*/ 
	function get_pick_column() 
	{
		// assuming current dbfield has a table defined
		
		return $this->_parts($this->get_ftype(), 2);
	}
	
	/**
	* Returns SQL component of pick/pickid column type.
	* Currently this means that it just returns 2nd component, but
	* this definition may expand to include checking to ensure it is
	* SQL, etc.
	* @access public
	*/
	function get_pick_sql() 
	{
		return $this->_parts($this->get_ftype(), 1);
	}
	
	/**
	* Returns the field for an upload column type.
	* Currently this means 2nd parameter, but maybe we should also expand this 
	* to include verification that this column is an upload type
	*/
	function get_upload_column() 
	{
		return $this->_parts($this->get_ftype(), 1);
	}
	
	/**
	* Break an a string into parts.
	* This is generic, but in this object is used to break 
	* $ftype into components.
	*
	* Modified: Joe Khoobyar <joe@ankhcraft.com>
	* Added support for [] quoted sql as an atom (part) in the regexp.
	*
	* @access private
	*/
	function _parts($str, $param_num=false) 
	{				
		$matches = array();
		preg_match_all("/\\[[^\\]]+\\]|[^\\s]+/", $str, $matches);
			
		$parts = $matches[0];
		
		if ( $param_num === false ) 
        {
			return $parts;
		}
        else if ( isset( $parts[$param_num] ) )
        {
			return $parts[$param_num];
		}	
	}
	
	/**
	* Finds objects (datatypes) that are depending on a certain dbfield.
	* This is used, e.g., when wanting to display a popup warning when a certain
	* dbfield is deleted.
	* @access public
	*/
	function get_depending_objects() 
	{			
		
		$sql = "SELECT DISTINCT typeid as id, dt.type as name
				FROM 	typesfields tf
						inner join datatypes dt on dt.id = tf.typeid 
				WHERE tf.fieldid = $this->fields[id]";
		
		$rs = &$this->db->Execute($sql);		
		return $rs->GetArray();
	}

	
	
	/**
	* Determines whether or not the passed var is the name of a current table.
	* @param $table The name of the table to check for.
	* @return boolean
	* @access private
	*/
	function _has_table() 
	{
			
		// 1) check that there is a second param for this ftype
		
		$table = $this->_parts($this->get_ftype(), 1);
		
		if (!($table = $this->get_ftype(1)))
        {
            return false;
        }
		
		// 2) check that the table exists				
		
        $object_cache =& object_cache::singleton();
        if (! ($tables = $object_cache->getDbTables()) )
        {
            $object_cache->saveDBTables( $this->db->MetaTables() );
            $tables = $object_cache->getDbTables();
        }
		return in_array($table, $tables);

	}
	
	/**
	* This is for checking whether ftype params are tables or not.
	*
	* Joe Khoobyar <joe@ankhcraft.com>
	* Fixed bugs in regexps.
	*/
	function _appears_as_table($table) 
	{	
		if (preg_match("/\$\\([^\\)]+\\)/", $table) || preg_match("/\\[[^\\]]+\\]/", $table)) 
        {
			return false;
		}
        else 
        {
			return true;
		}
	}
	
	/**
	* The broker function for _insert and _update.
	* Make sure you call load() before invoking this method if you want it to 
	* load vars from global scope.
	* @access public
	*/
	function save() 
	{
		// set default prefs that are important
		// to modification of database column types.
        if ($this->get_id())
        {		
            $this->_update();
        }
        else 
        {
            $this->_insert();
        }
		
		// read in class vars from db
        $this->read();

		return $this->get_id();	
	}


	/**
	* Modify a dbfeld entry.
	* Checks to make sure that a different (diff id) column doesn't exist with same name.
	* @param $fname The new name of the column.
	* @param $ftype The new metatype of the column.
	* @return boolean True on success; false on error.
	*/
	function _update() 
	{

		// paramters have already been loaded with ->load() method
        $ftype = $this->get_ftype();
        $fname = $this->get_fname();

		// used for transactions (where supported)
		$ok = true;
			
		// check to make sure the column doesn't already exist elsewhere
		$sql = "SELECT COUNT(*) FROM dbfields WHERE fname = '$fname' AND id <> ".$this->get_id();

		if ($this->debug) 
        {
            pxdb::print_debug($sql, 'blue');
        }
        
		$found = $this->db->GetOne($sql);	
	
		if (!$found) 
        {
            // the entered data is unique			
			$this->db->BeginTrans();
							
			// 1) first update the entry			
			$sql = "UPDATE dbfields SET fname = '$fname', ftype ='$ftype' WHERE id = ".$this->get_id();			
			if ($this->debug) 
            {
                pxdb::print_debug($sql, 'db-update'); 
            }			
			$ok = $this->db->Execute($sql);			
			
			// make old vars accessible
			$old = $this->_old_fields;
						
			// 2) catch the special exceptions -- and modify the structure of db.											
				
			switch ($this->get_field_type()) 
            {				
				case "pick":
				// or
				case "pickid":

					// 1) add the pick table if necessary						
                    $object_cache =& object_cache::singleton();
                    if (! ($tables = $object_cache->getDbTables()) )
                    {
                        $object_cache->saveDBTables( $this->db->MetaTables() );
                        $tables = $object_cache->getDbTables();
                    }
                    
                    if( !in_array($this->get_pick_table(), $tables) && "table" == $this->get_pick_source() ) 
                    {
                        // create pick table
                        pxdb::create_pick_table($this->get_pick_table());
                    }
						
                    // make sure the column exists...
                    $object_cache = &object_cache::singleton();
                    if (! ($records_columns = $object_cache->getTableColumns('records')))
                    {
                        $object_cache->saveTableColumns('records', $this->db->MetaColumnNames('records'));                            $records_columns = $object_cache->getTableColumns('records');
                    }
                    
                    if (! empty($records_columns[$this->get_fname()]))
                    {
                        // 2) then modify column in records table																
                        // coltype is gathered from RDBMS-specific constant
                        $coltype = constant('PXSQL_TYPE_'.strtoupper($this->get_field_type()));
    
                        if (!defined($coltype)) 
                        {
                            trigger_error("Undefined column type: $coltype.", E_USER_WARNING);
                            $ok = false;
                        }	
                        
                        if ($ok) 
                        {
                            $ok = $this->_change_column($old['fname'], $this->get_fname(), $coltype);
                        }
                    }
                    else 
                    {
                        // add to records table
								
                        // coltype is gathered from RDBMS-specific constant
                        $coltype = constant('PXSQL_TYPE_'.strtoupper($this->get_field_type()));
                        if (!defined($coltype)) 
                        {
                            trigger_error("Undefined column type: $coltype.", E_USER_WARNING);
                            $ok = false;
                        }	
                        
                        $sql = "ALTER TABLE records ADD COLUMN ".PXSQL_L.$this->get_fname().PXSQL_R." $coltype";
                        if ($this->debug) 
                        {
                            pxdb::print_debug($sql, 'db-update'); 
                        }
                        if ($ok) 
                        {
                            $ok = $this->db->Execute($sql);
                        }
                    }

                break;

				case "upload":
															
					if ( $this->get_upload_column() )
					{
												
						// if a colname was specified, then 
						// we don't have to do anything -- except check that [1] is a field that already exists in table.
                        $object_cache =& object_cache::singleton();
                        if (! ($cols = $object_cache->getTableColumns('records')))
                        {
                            $object_cache->saveTableColumns('records', $this->db->MetaColumnNames('records'));
                            $cols = $object_cache->getTableColumns('records');
                        }

						if ( empty( $cols[ $this->get_upload_column() ] ) )
						{							
							trigger_error("You must specify a field (column) name that exists for the upload field type.", E_USER_WARNING);
							return false;
						}
					}
                    else
                    {
						// no colname was specified -- so we'll modify the col w/ name of dbfield
						// coltype is gathered from RDBMS-specific constant
						$coltype = constant('PXSQL_TYPE_'.strtoupper($this->get_field_type()));
						if (!defined($coltype)) 
                        {
                            trigger_error("Undefined column type: $coltype.", E_USER_WARNING);
                            $ok = false;
						}
						
						if ($ok) 
                        {
                            $ok = $this->_change_column($old['fname'], $this->get_fname(), $coltype);
                        }
					}
				break;
			
				default:
				
					// coltype is gathered from RDBMS-specific constant
					$coltype = constant('PXSQL_TYPE_'.strtoupper($this->get_field_type()));
					if (!defined($coltype)) 
                    {
						trigger_error("Undefined column type: $coltype.", E_USER_WARNING);
						$ok = false;
					}
						
					if ($ok) 
                    {
                        $ok = $this->_change_column($old['fname'], $this->get_fname(), $coltype);
                    }
                break;
											
			}

			$this->db->CommitTrans($ok);
			
			return $ok;
		
		}
        else 
        {
			trigger_error("A field named ".$this->get_fname()." already exists.", E_USER_WARNING);
			return false;
		}
		
	}
	
	/**
	* Wrapper for column renaming/changing functionality.
	* @param string $old_name Original column name
	* @param string $new_name New column name
	* @param string $new_type New column type
	* @return boolean Success of operation
	*/
	function _change_column($old_name, $new_name, $new_type, $table='records')
	{			
		
		$this->db->BeginTrans();
		
		// 1) rename column (cannot rely on SQL-92)
        $sql = sprintf(PXSQL_ALTER_RENAME_COLUMN, $table, $old_name, $new_name);
        $ok = $this->db->Execute($sql);
        if ($this->debug) 
        {
            pxdb::print_debug($sql, 'db-update'); 
        }
        
		// 2) modify column type (SQL-92 -- hopefully)
        $sql = "ALTER TABLE $table ALTER COLUMN $new_name $new_type"; // new_name because it's been renamed
        if ($ok) 
        {
            $ok = $this->db->Execute($sql);
        }
        
		if ($this->debug) 
        {
            pxdb::print_debug($sql, 'db-update'); 
        }
			
		$this->db->CommitTrans($ok);
			
		return $ok;
	}
		
	/**
	* Add a new dbfeld entry.
	* First checks to make sure that the dbfield w/ same name doesn't already exist.
	* @return boolean True on success; false on error.
	* @access private
	*/
	function _insert() 
	{
		
		// paramters have already been loaded with ->load() method.		
        $fname = $this->get_fname();
        $ftype = $this->get_ftype();
	
		// check to make sure the col doesn't already exist
		$sql = "SELECT * FROM dbfields WHERE fname = '$fname'";
		if ($this->debug) 
        {
            pxdb::print_debug($sql, 'blue'); 
        }
		$rs =& $this->db->Execute($sql);

		if($rs->EOF) 
        { // the entered data is unique
            $ok = true;
            $this->db->BeginTrans();
												
			// 1) first create entry in dbfields table.
            $sql = "INSERT INTO dbfields (fname, ftype) VALUES ('$fname', '$ftype')";
            if ($this->debug) 
            {
                pxdb::print_debug($sql, 'db-update'); 
            }
            if ($ok) 
            {
                $ok = $this->db->Execute($sql);
            }
        
            $this->set_id($this->db->Insert_ID());
				
			// there are some special case column types that have implications other than the standard												
			switch($this->get_field_type()) 
            {
				
				case "pick":
				// or
				case "pickid":
				
					// [1] is the table name / col in db
                    // coltype is gathered from RDBMS-specific constant
                    $coltype = constant('PXSQL_TYPE_'.strtoupper($this->get_field_type()));
        
                    if (!defined($coltype))
                    {
                        trigger_error("Undefined column type: $coltype.", E_USER_WARNING);
                        $ok = false;
                    }	
                    
                    // add to records table
                    $sql = "ALTER TABLE records ADD COLUMN ".$this->get_fname()." $coltype";
                    if ($this->debug) 
                    {
                        pxdb::print_debug($sql, 'db-update'); 
                    }
                    
                    if ($ok) 
                    {
                        $ok = $this->db->Execute($sql);
                    }
                
                    // create pick table
                    pxdb::create_pick_table($this->get_pick_table());
						
				break;
				
				case "upload":
					if (	$this->get_upload_column() ) 
                    {												
						// even if a colname was specified we need to create a column
						// with the same name -- here we just make sure the specified column
						// does match a db column.									
                        $object_cache =& object_cache::singleton();
                        if (! ($cols = $object_cache->getTableColumns('records')))
                        {
                            $object_cache->saveTableColumns('records', $this->db->MetaColumnNames('records'));
                            $cols = $object_cache->getTableColumns('records');
                        }
						
						if ( empty( $cols[ $this->get_upload_column() ] ) )
						{							
							trigger_error("You must specify a field (column) name that exists for the upload field type.", E_USER_WARNING);
							return false;
						}
					
					}
					
					// coltype is gathered from RDBMS-specific constant
					$coltype = constant('PXSQL_TYPE_'.strtoupper($this->get_field_type()));
					
					//  add a column to record table
					$sql = "ALTER TABLE records ADD COLUMN ".PXSQL_L."$fname".PXSQL_R." $coltype";
					if ($this->debug) 
                    {
                        pxdb::print_debug($sql, 'db-update'); 
                    }
                    
					if ($ok) 
                    {
                        $ok = $this->db->Execute($sql);
                    }
																						
				break;
			
				default:
					// coltype is gathered from RDBMS-specific constant
					$coltype = constant('PXSQL_TYPE_'.strtoupper($this->get_field_type()));
					
					// the default is to add a column to record table
					$sql = "ALTER TABLE records ADD COLUMN ".PXSQL_L."$fname".PXSQL_R." $coltype";
					if ($this->debug) 
                    {
                        pxdb::print_debug($sql, 'db-update'); 
                    }
                    
					if ($ok) 
                    {
                        $ok = $this->db->Execute($sql);
                    }
                break;
			}
			
			$this->db->CommitTrans($ok);
										
			return $ok;
		
		}
        else 
        {
			trigger_error("A field named $fname already exists.", E_USER_WARNING, 'pxdb_dbfield::_insert()');
			return false;
		
		}
	}
	
	/**
	* Function to delete a dbfield entry, the associated column, and any related tables.
	* Because of the complexity of the delete() method for dbfields, you cannot call this method
	* without instantiating the class. (i.e. pxdb_dbfield::delete() will not work.)
	* @param $complete_removal Whether or not to remove related components (e.g. a table in the case of dbfield being pick/pickid)
	*/
	function delete($complete_removal=false) 
	{
		// load parameters or values from import	
		// remove the record from dbfields table.
        $sql = "DELETE FROM dbfields WHERE id = ".$this->get_id();
        if ($this->debug) 
        {
            pxdb::print_debug($sql, 'db-update'); 
        }
        $this->db->Execute($sql);
        
        $object_cache = &object_cache::singleton();
        if (! ($records_columns = $object_cache->getTableColumns('records')))
        {
            $object_cache->saveTableColumns('records', $this->db->MetaColumnNames('records'));
            $records_columns = $object_cache->getTableColumns('records');
        }
            
        switch($this->get_field_type()) 
        {
            case "pick":
            // or
            case "pickid":
                // [1] is the table                
                if ($complete_removal) 
                {
                    if (pxdb::is_table_created($this->get_pick_table())) 
                    {
                        //remove the table
                        $sql = "DROP TABLE ".$this->get_pick_table()."";
                        if ($this->debug) 
                        {
                            pxdb::print_debug($sql, 'db-update'); 
                        }
                        $this->db->Execute($sql);
                    }
                    
                    // delete data from pickrel ? -- hmmm... yeah, why not.
                    $sql = "DELETE FROM pickrel WHERE f_id = ".$this->get_id();
                    if ($this->debug) 
                    {
                        pxdb::print_debug($sql, 'db-update'); 
                    }
                    $this->db->Execute($sql);
                        
                }
                    
                if (! empty( $records_columns[ $this->get_fname() ] ) )
                {
                    // delete the column from records table
                    $sql = 'ALTER TABLE records DROP COLUMN '.$this->get_fname().'';
                    if ($this->debug) 
                    {
                        pxdb::print_debug($sql, 'db-update'); 
                    }
                    $this->db->Execute($sql);						
                }
                                            
            break;
				
            default:
                if (! empty( $records_columns[ $this->get_fname() ] ) )
                {
                    // delete the column from records table
                    $sql = 'ALTER TABLE records DROP COLUMN '.$this->get_fname().'';
                    if ($this->debug) 
                    {
                        pxdb::print_debug($sql, 'db-update'); 
                    }
                    $this->db->Execute($sql);	
                }
            break;
        } // end switch			
	}

} // class

?>
