<?php
// $Id: pxdb_relationship.class.php,v 1.3 2006/01/27 16:58:47 omerida Exp $

pxdb_import('metadata.pxdb_basis');

/**
* This class represtents a row of the relationships table.
*
* @version $Revision: 1.3 $
* @package pxdb.metadata
* @author Hans Lellelid <hans@velum.net>
* @access public
*/
class pxdb_relationship extends pxdb_basis 
{
	
	/**
	* @var array
	* @access private
	*/
	var $_fields=array(); // store the row
		
	/**
	* Constructor: perform db query to get full row based on the relationship id.
	* @param $id A relationship relid.
	*/	
	function pxdb_relationship($id=null) 
	{
		parent::pxdb_basis();		
		
		// set class var from param
        $this->set_id($id);
		
		// read data
        $this->read();	// uses value of relid set above
	}
	
	/**
	* Updates class vars from the database.
	* @access public
	*/
	function read() 
	{						
		if ($this->get_id()) 
        {				
   			// could possibly be more than one result -- can be more than one relid
    		$sql = "select * from relationships where id = ".$this->get_id();
			$this->_fields = (array) $this->db->GetRow($sql);
			
			$this->set_dt1($this->db->GetCol("SELECT DISTINCT dt1 FROM datatypesrel WHERE relid = ".$this->get_id()));
			$this->set_dt2($this->db->GetCol("SELECT DISTINCT dt2 FROM datatypesrel WHERE relid = ".$this->get_id()));
		} // if $this->get_id()					
	}
	
	
	/** 
	* Load subclass: loads values from global scope into the class.
	* @access public
	*/
	function load() 
	{
		parent::load('relationships'); 
	}

	/**
	* Accessor: reliid.
	* @return integer
	* @access public
	*/
	function get_id() 
	{
		return $this->_fields['id'];
	}
	
	/**
	* sets the relationship id
	* @param integer $relid
	* @return void
	* @access public
	*/
	function set_id($relid) 
	{
		$this->_fields['id'] = $relid;	
	}
		
	/**
	* Alias to get_id() -- for legacy support
	* Accessor: reliid.
	* @return integer
	* @access public
	*/
	function get_relid() 
	{
		return $this->_fields['id'];
	}
	
	/**
	* Alias to set_id() -- for legacy support
	* @param integer $relid
	* @return void
	* @access public
	*/
	function set_relid($relid) 
	{
		$this->_fields['id'] = $relid;	
	}
	
	/**
	* Gets datatype 1
	* @return integer
	* @access public
	*/
	function get_dt1() 
	{
		return (array) $this->_fields['dt1'];
	}
	
	/**
	* Sets datatype 1
	* @param integer $dt1
	* @return void
	* @access public
	*/
	function set_dt1($dt1) 
	{
		$this->_fields['dt1'] = (array) $dt1;
	}
	
	/**
	* Gets the second datatype
	* @return integer
	* @access public
	*/
	function get_dt2() 
	{
		return (array) $this->_fields['dt2'];
	}
	
	/**
	* Sets the second datatype
	* @param integer $dt2
	* @return void
	* @access public
	*/
	function set_dt2($dt2) 
	{
		$this->_fields['dt2'] = (array) $dt2;
	}
	
	/**
	* Get the relationship description
	* @return string
	* @access public
	*/
	function get_description() 
	{
		return $this->_fields['description'];
	}
	
	/**
	* Set relationship description
	* @param string $description 
	* @return void
	* @access public
	*/
	function set_description($description) 
	{
		$this->_fields['description'] = $description;
	}
	
	/**
	* Get relationship weight
	* @return integer
	* @access public
	*/
	function get_weight() 
	{
		return $this->_fields['weight'];
	}
	
	/**
	* set relationship weight
	* @param integer $weight
	* @return void
	* @access public
	*/
	function set_weight($weight) 
	{
		$this->_fields['weight'] = $weight;
	}
	
	
	/**
	* Promote or demote a field.
	* Changes the orderby column for typesfields rows.
	* @param string $dir Direction ('up', 'down') to move.
	* @return void
	* @access public
	*/
	function move($dir) 
	{
			
		// 1) do we need to clean up order of fields?
		$sql = "select r.weight from relationships r inner join relationships r2 on r2.id <> r.id and r2.weight = r.weight";
		$test_rs = $this->db->Execute($sql);
		if (!$test_rs->EOF) 
        {
			$this->reorder();
		}

		// get the weight of the current field.
		// swap with next higher or lower field
		
		$params = array('id' => $this->get_id(),
						'table' => 'relationships',
						'constraint_sql' => '',
						'id_col' => 'id',
						'weight_col' => 'weight',
						'dir' => $dir 
						);
		
		// call the abstract method
		parent::move($params);
	} 

	
	/**
	* Re-orders the relationships.
	* @return void
	* @access public
	*/
	function reorder() 
	{

		// setup hash of parameters for parent method	
        $params = array('id' => $this->get_id(),
						'table' => 'relationships',
						'constraint_sql' => '',
						'id_col' => 'id',
						'weight_col' => 'weight',
						'dir' => null,
						);
						
		// invoke abstract parent method
        parent::reorder($params);

	} // reorder()
	

	/**
	* The main function to handle inserts & updates.
	* @return integer ID of record saved or created.
	* @access public
	*/
	function save() 
	{	
	
		// paramters have been loaded with load() method
		
		// A. Save / Add the 'relationships' table (metadata) record
		
		$this->debug=true;
		$this->db->debug=true;		
		$this->db->BeginTrans();	
		$ok=true;
		
		if ($this->get_id()) 
        {	// update existing record
			$sql = "UPDATE relationships SET 
							description = ".$this->db->Quote($this->get_description()).", 
							weight = ".$this->get_weight()."
					WHERE	id = ".$this->get_id();
					
			if ($this->debug) 
            {
                pxdb::print_debug($sql, 'db-update'); 
            }
			
            if ($ok) 
            {
                $ok = $this->db->Execute($sql);
            }		
		}
        else 
        {
			// 1) generate new weight
			$this->set_weight($this->_new_weight());		
			
			// 2) insert
            $sql = "INSERT INTO relationships (description, weight) VALUES (" . $this->db->Quote($this->get_description()) . ", " . $this->get_weight() . ")";
            
            if ($this->debug) 
            {
                pxdb::print_debug($sql, 'db-update'); 
            }
            
            if ($ok)
            {
                $ok = $this->db->Execute($sql);
            }
            
            if ($ok)
            {
                $this->set_id($this->db->Insert_ID());
            }
		
		}
		
		// B. Remove & Add the related datatypes to 'datatypesrel'		
        $dt1 = $this->get_dt1();
        $dt2 = $this->get_dt2();										
        $relid = $this->get_id();
            
        // oam fix: delete all datatypesrels associated with this relid and only save the ones
        // that have been sent to this method, the delete loop below is not working
        $sql = "DELETE FROM datatypesrel WHERE relid = $relid";
        if ($ok)
        {
            $ok = $this->db->Execute($sql);
        }

        foreach($dt1 as $dt1_piece) 
        {
            foreach($dt2 as $dt2_piece) 
            {
                //$sql = "DELETE FROM datatypesrel WHERE relid = $relid AND dt1 = $dt1_piece AND dt2 = $dt2_piece";
                //if($ok) $ok = $this->db->Execute($sql);
                    
                // insert relationship
                $sql = "INSERT INTO datatypesrel (relid, dt1, dt2) VALUES ($relid, $dt1_piece, $dt2_piece)";
                if ($this->debug)
                {
                    pxdb::print_debug($sql, 'db-update'); 
                }
                if ($ok) 
                {
                    $ok = $this->db->Execute($sql);
                }
            } // foreach($dt2)	
        } // foreach($dt1)
        
        $this->db->CommitTrans($ok);
		
		if (!$ok) 
        {
            return false;
        }
		else
        {
            return $this->get_id();
        }
	}
	
	/**
	* This function figures out what the next (max+5) weight should be.
	* @return integer
	* @access private
	*/
	function _new_weight() 
	{	
		$sql = "SELECT weight FROM relationships ORDER BY weight DESC";
		$rs = &$this->db->Execute($sql);
		return ($rs->fields['weight'] + 5);
	}	
	
	/**
	* Function to delete a relationship entry.
	* @param integer $id relid of rel to delete
	* @return integer The number of rows affected by delete statement.
	* @access public
	*/
	function delete($id=null) 
	{	
		$db = &pxdb::dbh();
		$id = ($id ? $id : $this->get_id());
		
		if (!$id) 
        {
			trigger_error("You must instantiate pxdb_relationship with the id of the field you wish to delete -- or pass an id to delete() method.", E_USER_WARNING);
			return false;
		}
		
		$ok = true;
		$this->db->BeginTrans();
			$sql = "DELETE FROM relationships WHERE id = $id"; // note: could be deleting more than one record.
			
            if ($this->debug) 
            { 
                pxdb::print_debug($sql, 'db-update'); 
            }
			if ($ok) 
            {
                $db->Execute($sql);
            }
			$sql = "DELETE FROM datatypesrel WHERE relid = $id"; // note: could be deleting more than one record.
			
            if ($this->debug) 
            {
                pxdb::print_debug($sql, 'db-update'); 
            }
            
			if ($ok)
            {
                $db->Execute($sql);
            }
		$this->db->CommitTrans($ok);
		
		return $ok;
	}
} // class
?>