<?php
// $Id: pxdb_typesfield.class.php,v 1.13 2006/02/17 16:59:54 omerida Exp $

pxdb_import('pxdb_privs');
pxdb_import('metadata.pxdb_basis');
pxdb_import('metadata.pxdb_dbfield');
pxdb_import('metadata.pxdb_datatype');

pxdb_import('metadata.object_cache');

/**
* This class represtents a row of the typesfields table.
*
* @version $Revision: 1.13 $
* @package pxdb.metadata
* @author Hans Lellelid <hans@velum.net>
* @access public
*/
class pxdb_typesfield extends pxdb_basis 
{
    /**
    * Datatype object for this typesfield.
    * @var object pxdb_datatype
    * @access private
    */
    var $dt;
    
    /**
    * Default privilege (for ugid 0) to assign to new fields.
    * @var string
    * @access public
    */
    var $default_priv='read,write';
        
    /**
    * Constructor: call super constructor and read in vars from db.
    * @param integer $id The id of typesfield row.
    * @access public
    */
    function pxdb_typesfield($id=null) 
    {        
        // call parent constructor    
        parent::pxdb_basis();        
        $this->set_id($id);        
        $this->read();                        
    }
    
    /**
    * Updates class vars from the database.
    * @access public
    */
    function read() 
    {    
        $object_cache = &object_cache::singleton();
        
        if ($typef = $this->get_id())
        {             
            if ($typesfield = $object_cache->getTypesFields($typef))
            {                
                $this->_fields = $typesfield;                    
            } 
            else 
            {            
                $sql = "select * from typesfields where id = ".$typef;
                $rs =& $this->db->Execute($sql);        
                $this->_fields = $rs->fields;                        
                                
                $object_cache->saveTypesFields($typef, $this->_fields);
            }
        }
    }

    /** 
    * Load subclass: loads values from global scope into the class.
    * @access public
    */
    function load() 
    {
        parent::load('typesfields'); 
    }
        
    /**
    * Gets ID of this typesfield
    * @access public
    */    
    function get_id() 
    {
        return $this->_fields['id'];
    }

    /**
    * Sets ID of this typesfield
    * @param integer $id
    * @return void
    * @access public
    */    
    function set_id($id) 
    {
        $this->_fields['id'] = $id;
    }

    /**
    * Gets the content type (datatype) ID this typesfield is for
    * @return integer
    * @access public
    */    
    function get_typeid() 
    {
        if (isset($this->_fields['typeid']))
        {
            return $this->_fields['typeid'];
        }
    }

    /**
    * Sets the content type (datatype) ID this typesfield is for
    * @param integer $typeid
    * @return void
    * @access public
    */    
    function set_typeid($typeid) 
    {
        $this->_fields['typeid'] = $typeid;
    }
    
    /**
    * Get the ID of the relationship this typesfield manages
    * @return integer
    * @access public
    */    
    function get_relid() 
    {        
        if (isset($this->_fields['relid']))
        {
            return $this->_fields['relid'];
        }
    }

    /**
    * Set the ID of the relationship this typesfield manages
    * @param integer $relid
    * @access public
    */    
    function set_relid($relid) 
    {
        $this->_fields['relid'] = $relid;
    }

    /**
    * Gets the content types (datatypes) this relationship handles.
    * Returns a comma-separated string of related content types (datatypes), so
    * even if you expect only one value to be returned, assume it's a string and
    * check for multiple values.
    * @return string Related datatype or comma-separated list of related datatypes.
    * @access public
    */    
    function get_otype() 
    {
        /* OAM: clean up what gets returned to be a unique set of values with no repeats */
        if (!empty($this->_fields['otype']))
        {        
            $val = explode(',' , $this->_fields['otype']);
            
            $val = array_unique($val);
            return join(',' , $val);
        }
        else
        {
            return $this->_fields['otype'];
        }
    }

    /**
    * Sets the content types (datatypes) this relationship handles.
    * The argument is a comma-separated string of related content types
    * (datatypes), so don't pass an array if you have multiple content types.
    * @param string $otype Related datatype or comma-separated list of related datatypes.
    * @access public
    */    
    function set_otype($otype) 
    {
        $this->_fields['otype'] = $otype;
    }
    
    /**
    * Gets the ID of the dbfield this typesfield uses
    * (for non-relationship typesfields, even multiple picks that use the
    * pickrel table)
    * @return integer
    * @access public
    */
    function get_fieldid() 
    {
        if (isset($this->_fields['fieldid']))
        {
            return $this->_fields['fieldid'];
        }
    }
    
    /**
    * Sets the ID of the dbfield this typesfield uses
    * (for non-relationship typesfields, even multiple picks that use the
    * pickrel table)
    * @param integer $fieldid
    * @access public
    */    
    function set_fieldid($fieldid) 
    {
        $this->_fields['fieldid'] = $fieldid;
    }

    /**
    * Get the default name to use in forms and the general module's detail page.
    * @return string
    * @access public
    */    
    function get_displayname() 
    {
        if (isset($this->_fields['displayname']))
        {
            return $this->_fields['displayname'];
        }
    }
    
    /**
    * Set the default name to use in forms and the general module's detail page.
    * @param string $displayname
    * @access public
    */    
    function set_displayname($displayname) 
    {
        $this->_fields['displayname'] = $displayname;
    }

    /**
    * Get an optional default description of the field to use in forms.
    * @return string
    * @access public
    */    
    function get_description() 
    {
        if (isset($this->_fields['description']))
        {
            return $this->_fields['description'];
        }
    }
    
    /**
    * Set an optional default description of the field to use in forms.
    * @param string $description
    * @return void
    * @access public
    */    
    function set_description($description) 
    {
        $this->_fields['description'] = $description;
    }
    
    /**
    * Get the unique text string that identifies this property of the content type (datatype).
    * Used to generate accessor function names in autogenerated classes, among
    * other things.
    * @return string
    * @access public
    */
    function get_identifier() 
    {
        if (isset($this->_fields['identifier']))
        {
            return $this->_fields['identifier'];
        }
    }

    /**
    * Set the unique text string that identifies this property of the content type (datatype).
    * Used to generate accessor function names in autogenerated classes, among
    * other things.
    * @param string $identifier
    * @return void
    * @access public
    */    
    function set_identifier($identifier) 
    {
        $this->_fields['identifier'] = $identifier;
    }

    /**
    * Gets the autogenerator, if any, this property uses, as well as any params the autogenerator may have.
    * @return string
    * @access public
    */
    function get_autogen() 
    {
        if (isset($this->_fields['autogen']))
        {
            return $this->_fields['autogen'];
        }
        else
        {            
            return null;
        }
    }

    /**
    * Sets the autogenerator, if any, this property uses, as well as any params the autogenerator may have.
    * @param string $autogen
    * @return void
    * @access public
    */    
    function set_autogen($autogen) 
    {
        $this->_fields['autogen'] = $autogen;
    }

    /**
    * Gets the validator, if any, this property uses, as well as any params the validator may have.
    * @return string
    * @access public
    */
    function get_validator() 
    {
        if (isset($this->_fields['validator']))
        {
            return $this->_fields['validator'];
        }
        else
        {            
            return null;
        }                
    }

    /**
    * Sets the validator, if any, this property uses, as well as any params the validator may have.
    * @param string $validator
    * @return void
    * @access public
    */    
    function set_validator($validator) 
    {
        $this->_fields['validator'] = $validator;
    }

    /**
    * Gets the processor, if any, this property uses, as well as any params the processor may have.
    * @return string
    * @access public
    */
    function get_processor() 
    {
        if (isset($this->_fields['processor']))
        {
            return $this->_fields['processor'];
        }
        else
        {
            return null;
        }
    }

    /**
    * Sets the processor, if any, this property uses, as well as any params the processor may have.
    * @param string $processor
    * @return void
    * @access public
    */    
    function set_processor($processor) 
    {
        $this->_fields['processor'] = $processor;
    }

    /**
    * Gets the widget, if any, this property uses, as well as any params the widget may have.
    * @return string
    * @access public
    */
    function get_widget() 
    {
        if (isset($this->_fields['widget']))
        {
            return $this->_fields['widget'];
        }
        else
        {            
            return null;
        }
    }

    /**
    * Sets the widget, if any, this property uses, as well as any params the widget may have.
    * @param string $widget
    * @return void
    * @access public
    */    
    function set_widget($widget) 
    {
        $this->_fields['widget'] = $widget;
    }

    /**
    * Returns whether or not (via string 'Y' or 'N') this property can have multiple values.
    * @return string 'Y' or 'N'
    * @access public
    */    
    function get_mul() 
    {
        if (isset($this->_fields['mul']))
        {
            return $this->_fields['mul'];
        }
        else
        {
            //assume false
            return 'N';
        }                    
    }
    
    /**
    * Sets whether or not (via string 'Y' or 'N') this property can have multiple values.
    * @param string $mul 'Y' or 'N'
    * @access public
    */
    function set_mul($mul) 
    {
        if ('Y' == strtoupper($mul) || 'N' == strtoupper($mul) )
        {
            $this->_fields['mul'] = $mul;
        }
        else if (is_bool($mul))
        {
            $this->_fields['mul'] = $mul ? 'Y' : 'N';
        }
        else
        {
            trigger_error('pxdb_typesfield::set_mul expects boolean or Y/N value.  Assuming "N".', E_USER_WARNING);
            $this->_fields['mul'] = 'N';
        }
    }
    
    /**
    * Returns whether or not (via string 'Y' or 'N') this property is required.
    * @return string 'Y' or 'N'
    * @access public
    */    
    function get_req() 
    {
        if (isset($this->_fields['req']))
        {
            return $this->_fields['req'];
        }
        else
        {
            //assume false
            return 'N';
        }        
    }

    /**
    * Sets whether or not (via string 'Y' or 'N') this property is required.
    * @param string $req 'Y' or 'N'
    * @access public
    */    
    function set_req($req) 
    {
        if ('Y' == strtoupper($req) || 'N' == strtoupper($req))
        {
            $this->_fields['req'] = $req;
        }
        else if (true === is_bool($req))
        {
            $this->_fields['req'] = $req ? 'Y' : 'N';
        }
        else
        {
            trigger_error('pxdb_typesfield::set_req expects boolean or Y/N value.  Assuming "N"', E_USER_WARNING);
            $this->_fields['req'] = 'N';
        }
    }

    /**
    * Returns whether or not (via string 'Y' or 'N') this property must be unique.
    * @return string 'Y' or 'N'
    * @access public
    */    
    function get_unique() 
    {
        if (isset($this->_fields['is_unique']))
        {
            return $this->_fields['is_unique'];
        }
        else
        {
            //assume false
            return 'N';
        }    
    }
    
    /**
    * Sets whether or not (via string 'Y' or 'N') this property must be unique.
    * @param string $unique 'Y' or 'N'
    * @access public
    */    
    function set_unique($unique) 
    {
        $this->_fields['is_unique'] = $unique;

        if ('Y' == strtoupper($unique) || 'N' == strtoupper($unique))
        {
            $this->_fields['is_unique'] = $unique;
        }
        else if (is_bool($unique))
        {
            $this->_fields['is_unique'] = $unique ? 'Y' : 'N';
        }
        else
        {
            trigger_error('pxdb_typesfield::set_unique expects boolean or Y/N value.  Assuming "N"', E_USER_WARNING);
            $this->_fields['is_unique'] = 'N';
        }
    }

    /**
    * This alias is needed because DBasis depends on function names being passed as args.
    * Sheesh.
    *
    * @return string 'Y' or 'N'
    * @access public
    */    
    function get_is_unique() 
    {
        return $this->get_unique();
    }
    
    /**
    * This alias is needed because DBasis depends on function names being passed as args.
    * Sheesh.
    *
    * @param string $unique 'Y' or 'N'
    * @return void
    * @access public
    */    
    function set_is_unique($unique) 
    {
        $this->set_unique($unique);
    }
    
    /**
    * Returns whether or not this property can have multiple values.
    * @return boolean
    * @access public
    */        
    function is_multiple() 
    {        
        return($this->get_mul() == "Y");    
    }

    /**
    * Returns whether or not this property is required.
    * @return boolean
    * @access public
    */        
    function is_required() 
    {        
        return($this->get_req() == "Y");    
    }
    
    
    /**
    * Returns whether or not this property must be unique.
    * @return boolean
    * @access public
    */    
    function is_unique() 
    {
        return ($this->get_unique() == "Y");
    }
    
        
    /**
    * Get the general type of field ('relationship' or 'simple')
    * @return string The field type (either 'relationship' or 'simple')
    * @access public
    */
    function get_field_type() 
    {
        if ($this->get_relid()) 
        {
            return "relationship";
        }
        else 
        {
            return "simple";
        }
    }
    
    /**
    * Gets the specific metatype of this field
    * Determined by the dbfield it uses unless it's a relationship.
    * @return string the Field metatype (e.g. 'ltxt', 'url', 'primaryrel')
    * @access public
    */
    function get_field_metatype() 
    {
        switch ($this->get_field_type()) 
        {
            case "simple":
            
                $dbf = &new pxdb_dbfield($this->get_fieldid());
                return $dbf->get_field_type();
                
            break;
            
            case "relationship":
                                
                return "primaryrel";
            
            break;
        }
    }
    
    
    /**
    * Replace tags with values from the class / db.
    * This is primarily for vartable (e.g. table '$(object)types' , but could be used almost anywhere.
    * @return string String with any dynamic tags replaced.
    */
    function replace_tags($txt) 
    {
        if (!$this->dt) 
        {        
            $this->dt = new pxdb_datatype($this->get_typeid());
        }
        
        // special case for object
        $txt = preg_replace("/^\\\$\(object([^\)]+)\)/ei", "\$this->dt->get_$1()", $txt);
        
        // catchall
        $txt = preg_replace("/^\\\$\(([^\)]+)\)/e", "\$this->get_$1", $txt);
                
        return $txt;
    }
    
    /**
    * Finds available (not used by this object) dbfields.
    * @param string $mask The mask to use for fname column.
    * @param string $constraint_sql WHERE-clause SQL to constrain results.
    * @return array id,fname of dbfields records found.
    * @access public
    */
    function arr_available_dbfields($mask='{fname}', $constraint_sql=null) 
    {
        $column = pxdb_sql::sqlize_mask($mask, 'df');
        
        $sql = "SELECT        df.*,
                            $column as custom_field
                FROM        dbfields df
                            LEFT JOIN typesfields tf ON tf.fieldid = df.id AND tf.typeid = ".$this->get_typeid()."
                WHERE        tf.id IS NULL ".($this->get_fieldid() ? "OR df.id = ".$this->get_fieldid() : "")."
                            ".($constraint_sql ? "AND $constraint_sql" : "")."
                ORDER BY    df.fname";        
        
        return $this->db->GetAll($sql);    
    }

    /**
    * Gets the datatypes related to a relid.
    * By convention the dt2 column contains the "related" datatypes.  This can include the current
    * datatype if it is so-specified in the relationships table.
    * This has changed now to be for a relid and datatype, if the dt is in dt1, return the dt2 column
    * , if that is empty look in dt2 and return dt1 column
    * @param integer $relid
    * @return array An array of datatypes that are related to current datatype by specified relid.
    * @access public
    */
    function get_related_datatypes($relid) 
    {    
        //OAM be smarter about how we return datatypes for relationships
        //if its in dt1, return all the datatypes in dt2
        $sql = "SELECT    dt2
                FROM    datatypesrel
                WHERE    relid = $relid and dt1=".$this->get_typeid();
                
        $dt1 = $this->db->GetCol($sql);        
        if (0 < count($dt1))
        {
            return $dt1;
        }
        else
        {
            //if dt1 is empty and its in dt2, return all the datatype in dt1
            $sql = "SELECT    dt1
                FROM    datatypesrel
                WHERE    relid = $relid and dt2=".$this->get_typeid();
                
            $dt2 = $this->db->GetCol($sql);
            
            if (0 < count($dt2))
            {
                return $dt2;
            }
            else
            {
                trigger_error('No related datatypes found for relid '.$relid.' and datatype '.$this->get_typeid(), E_USER_WARNING);
                return false;
            }
        }
    }

    /**
    * Finds available (not used by this object) relationships.
    * Also will make sure that relationships define current datatype as dt1 -- since convention
    * is that dt1 is the source object and dt2 is the related object.  If you want the relationship
    * to be bi-directional (i.e. you can choose the relationship from either object) then make sure
    * that you define it as such.
    * @param string $mask The mask to use for description column.
    * @param string $constraint_sql WHERE-clause SQL to constrain results.
    * @return array hash of relationships records found.
    * @access public
    */
    function arr_available_relationships($mask='{description}', $constraint_sql=null) 
    {    
        $column = pxdb_sql::sqlize_mask($mask, 'rel');

        /*
        // get distinct relids
        $sql = "SELECT        DISTINCT rel.*, $column AS custom_field    
                FROM        datatypesrel relx
                            INNER JOIN relationships rel ON rel.id = relx.relid
                            LEFT JOIN typesfields tf ON tf.relid = relx.relid AND tf.typeid = ".$this->get_typeid()."                            
                WHERE        tf.id IS NULL ".($this->get_relid() ? "OR relx.relid = ".$this->get_relid() : "")."
                            ".($constraint_sql ? "AND $constraint_sql" : "")."
                            AND relx.dt1 = ".$this->get_typeid()."
                ORDER BY    rel.weight";    
        */
        // OAM: updated this method to not care about directionality (source datatype in dt1)
        
        // 1) for this datatype, get a list of used relationships
        $sql = 'SELECT relid from typesfields WHERE typeid='.$this->get_typeid()." and relid>0";
        if ($current_relid = $this->get_relid())
        {
            // remove the current relid from the list of used relids
            // so that it shows up as an option when editing a typefield row
            $sql.=' and relid<>'.$current_relid;
        }
        
        $used = $this->db->GetCol($sql);

        $sql = "
SELECT DISTINCT
    rel.*, $column as custom_field
FROM
    datatypesrel relx
INNER JOIN
    relationships rel ON rel.id = relx.relid
WHERE
    (relx.dt1 = " . $this->get_typeid() . ' OR relx.dt2 = ' . $this->get_typeid() . ")\n";
              
        if (0 < count($used))
        {
            $sql.= '    AND rel.id NOT IN ('.join(',' , $used).")\n";         
        }
        
        return $this->db->GetAll($sql);        
        
    }
    
    /**
    * Saves a typesfield to the database.
    * @return integer ID of newly created or saved record.
    * @access public
    */
    function save() 
    {
    
        // we import everything that tells us which vars to write.

        // we're only writing a subset of the fields to the db:
        // others are written different ways:        
        
        if ($this->get_relid()) 
        {
            $otypes = implode(",", $this->get_related_datatypes($this->get_relid()));
        }
        else 
        {
            $otypes = null;
        }
        
        // params have been set with ->load() method
        
        $required = $this->get_req();
        $multiple = $this->get_mul();
        $unique   = $this->get_unique();
        $displayname = $this->get_displayname();
        $description = $this->get_description();
        $identifier  = $this->get_identifier();
        $relid   = $this->get_relid();
        $fieldid = $this->get_fieldid();
        $autogen = $this->get_autogen();
        $widget  = $this->get_widget();
        $validator = $this->get_validator();
        $processor = $this->get_processor();                
            
        if ($this->get_id()) 
        {
            // for now just update.
            $sql = "UPDATE ".PXSQL_L."typesfields".PXSQL_R." SET
                        ".PXSQL_L."displayname".PXSQL_R." = ".$this->db->Quote($displayname).",
                        ".PXSQL_L."description".PXSQL_R." = ".$this->db->Quote($description).",
                        ".PXSQL_L."identifier".PXSQL_R." = ".$this->db->Quote($identifier).",
                        ".PXSQL_L."autogen".PXSQL_R." = ".$this->db->Quote($autogen).",
                        ".PXSQL_L."relid".PXSQL_R." = ".$this->db->Quote($relid).",
                        ".PXSQL_L."otype".PXSQL_R." = ".$this->db->Quote($otypes).",
                        ".PXSQL_L."req".PXSQL_R." = ".$this->db->Quote($required).",
                        ".PXSQL_L."is_unique".PXSQL_R." = ".$this->db->Quote($unique).",
                        ".PXSQL_L."mul".PXSQL_R." = ".$this->db->Quote($multiple).",                        
                        ".PXSQL_L."fieldid".PXSQL_R." = ".$this->db->Quote($fieldid).",
                        ".PXSQL_L."widget".PXSQL_R." = ".$this->db->Quote($widget).",
                        ".PXSQL_L."validator".PXSQL_R." = ".$this->db->Quote($validator).",
                        ".PXSQL_L."processor".PXSQL_R." = ".$this->db->Quote($processor)."                        
                    WHERE ".PXSQL_L."id".PXSQL_R." = ".$this->get_id();
            
            $this->db->Execute($sql);
            
            return $this->get_id();
        }
        else 
        {
            $orderby = $this->_next_orderby();
                    
            // for now just update.
            $sql = "INSERT INTO ".PXSQL_L."typesfields".PXSQL_R." (
                            ".PXSQL_L."typeid".PXSQL_R.",
                            ".PXSQL_L."displayname".PXSQL_R.",
                            ".PXSQL_L."description".PXSQL_R.",
                            ".PXSQL_L."identifier".PXSQL_R.",
                            ".PXSQL_L."autogen".PXSQL_R.",
                            ".PXSQL_L."relid".PXSQL_R.",
                            ".PXSQL_L."otype".PXSQL_R.",
                            ".PXSQL_L."fieldid".PXSQL_R.",
                            ".PXSQL_L."req".PXSQL_R.",
                            ".PXSQL_L."is_unique".PXSQL_R.",
                            ".PXSQL_L."mul".PXSQL_R.",
                            ".PXSQL_L."orderby".PXSQL_R.",
                            ".PXSQL_L."widget".PXSQL_R.",
                            ".PXSQL_L."validator".PXSQL_R.",
                            ".PXSQL_L."processor".PXSQL_R."
                      ) VALUES (
                            ".$this->get_typeid().",
                            ".$this->db->Quote($displayname).",
                            ".$this->db->Quote($description).",
                            ".$this->db->Quote($identifier).",
                            ".$this->db->Quote($autogen).",
                            ".$this->db->Quote($relid).",
                            ".$this->db->Quote($otypes).",
                            ".$this->db->Quote($fieldid).",
                            ".$this->db->Quote($required).",
                            ".$this->db->Quote($unique).",
                            ".$this->db->Quote($multiple).",
                            ".$this->db->Quote($orderby).",
                            ".$this->db->Quote($widget).",
                            ".$this->db->Quote($validator).",
                            ".$this->db->Quote($processor)."
                      )";        
                        
            $this->db->Execute($sql);
            
            $this->set_id($this->db->Insert_ID());
            
            // add default privs for this field
            if ($this->default_priv) 
            {
                $this->grant_priv('0', $this->default_priv);
            }
            
        }
        
        $this->read();
        
        return $this->get_id();
                            
    }
    
    /**
    * Calculates the next weight value for this object.
    * @return int The new orderby value.
    * @access private
    */
    function _next_orderby() 
    {    
        // first reorder the fields
        $this->reorder();
        
        // get the new highest value for orderby
        $sql = "SELECT MAX(orderby) FROM typesfields WHERE typeid = ".$this->get_typeid();                
        // new one will be highest + 5
        $new_oby = $this->db->GetOne($sql) + 5;
        
        return $new_oby;        
    }

    /**
    * Remove a field from an pxdb object.
    * @param $id The field id we want to delete.
    * @return boolean true/false on success.
    * @access public
    */
    function delete($id=null) 
    {        
        
        $db =& pxdb::dbh();
        $id = ($id ? $id : $this->get_id());
        
        if (!$id) 
        {
            trigger_error("You must instantiate pxdb_typesfield with the id of the field you wish to delete -- or pass an id to delete() method.", E_USER_WARNING);
            return false;
        }
                
        // delete the row from typesfields
        
        $sql = "DELETE FROM typesfields WHERE id = $id";
        if ($this->debug)
        {
            pxdb::print_debug($sql,'db-update'); 
        }
        $db->Execute($sql);                
        
        // delete any associated privs from field_priv
        pxdb_privs::revoke('field', $id);
    
        return true;
    }            
    

    /**
    * Gets privileges for users and groups.
    * @return array Array of privs hash.
    * @access public
    */    
    function get_all_privs() 
    {    
        return pxdb_privs::get('field', $this->get_id());    
    }

    /**
    * Returns all the ugids (group or users) who have privs to this field
    *
    * @param integer datatype to test for privs: group | users
    * @access private
    */    
    function _fetch_ugid_fieldprivs($datatype)
    {
        $sql = 'SELECT r.id FROM field_priv p'
              .' INNER JOIN records r ON (r.id=p.ugid and r.datatype='.$datatype.')'
              .' WHERE p.fid='.$this->get_id()              
        ;
        
        // this returns the sql result as a 
        $rows = $this->db->GetCol($sql);
        
        if (0 < count($rows))
        {
            return $rows;
        }
        else 
        {
            return false;
        }
        
    }
    
    /**
    * Gets privileges for specified user.
    * @return Array (2-D) name=>user name, ugid => user id, priv => user privs.
    * @access public
    */    
    function get_user_privs() 
    {    
        static $dt_user;
        
        if (!isset($dt_user))
        {
            $dt_user = $this->prefs->get_list('dt_user');
        }
        /*
        $sql = "SELECT id FROM records WHERE datatype IN (".$this->prefs->get_list('dt_user').")";
        $ugids = $this->db->GetCol($sql);
        */
        // OAM: Improvement, before we lazily grabbed ALL the userids and sent them along
        // to pxdb_privs::get (below).  Now, we first lookup to see if there are any user level
        // privs for a field and only send the ugids that match
        if ($ugids = $this->_fetch_ugid_fieldprivs($dt_user))
        {        
            return pxdb_privs::get('field', $this->get_id(), $ugids);
        }
        else 
        {
            return false;
        }            
    }
    
    /**
    * Gets privileges for specified group.
    * @return Array (2-D) name=>group name, ugid => group id, priv => group privs.'
    * @access public
    */    
    function get_group_privs() 
    {                
        // OAM: Improvement, before we lazily grabbed ALL the userids and sent them along
        // to pxdb_privs::get (below).  Now, we first lookup to see if there are any user level
        // privs for a field and only send the ugids that match
        if ($ugids = $this->_fetch_ugid_fieldprivs($this->prefs->get_list('dt_group')))
        {        
            return pxdb_privs::get('field', $this->get_id(), $ugids);
        } 
        else 
        {
            return false;
        }    
    }
    
    
    /**
    * Gets privileges for ugid 0.
    * @return Array (1-D) name=>'Everyone', ugid => '0', priv => global privs.
    */
    function get_global_privs() 
    {
        // special case -- the '0' everyone case
        $privs = pxdb_privs::get('field', $this->get_id(), 0);
            
        if (sizeof($privs)) 
        {
            // there should only be one element at the intersection of id & ugid
            $global_privs = array_pop($privs);
            $global_privs['name'] = 'Everyone';
            return $global_privs;
        }                
        else
        {
            return null;
        }
    }
    
    /**
    * Function to grant privileges to typesfields.
    * 
    * By default it uses values in $_imported array, but can also be passed parameters -- for 
    * assigning privilegs internally.
    *
    * @param $ugid The user/group id (from records table)
    * @param $priv A string or array containing privilegs e.g. 'read,write' or array('read', 'write').
    * @return boolean True or False (with $errors[] added)
    */
    function grant_priv($ugid, $priv) 
    {
        // load parameters or values from import
        if (!is_array($priv))
        {
            $priv = explode(',',$priv);  //normalize
        }
        // knows which field we are dealing with from $_imported array.
        if (!$priv) 
        {
            trigger_error("You must give either read or write privs to the user/group you just added.", E_USER_WARNING);
            return false;        
        } 
        else 
        {            
            pxdb_privs::grant('field', $this->get_id(), $ugid, $priv);
            return true;
        }
    }
    
    /**
    * Function to revoke privileges to typesfields.
    * 
    * By default it uses values in $_imported array, but can also be passed parameters -- for 
    * revoking privilegs internally.
    *
    * @param $ugid The user/group id (from records table)
    * @param $priv The particular priv to revoke (def: all)
    * @return boolean True or False (with warnings)
    * @access public
    */
    function revoke_priv($ugid=null, $priv=null) 
    {
        // knows which field we are dealing with from $_imported array.                

        // load parameters or values from import
        $fid = $this->get_id();
        $ugid = ($ugid != null ? $ugid : $this->_imported['ugid']);
        
        if (!isset($ugid) || !isset($fid))
        {    
            trigger_error("You must specify both a ugid and a field id for the record you want to remove.", E_USER_WARNING);
            return false;
        } 
        else
        {
            return pxdb_privs::revoke('field', $fid, $ugid, $priv);
        }
    
    }
    

    /**
    * Promote or demote a field.
    * Changes the orderby column for typesfields rows.
    * @param string $dir Direction ('up', 'down') to move.
    * @return void
    * @access public
    */
    function move($dir) 
    {
        // get the weight of the current field.
        // swap with next higher or lower field
        $params = array('id' => $this->get_id(),
                     'table' => 'typesfields',
            'constraint_sql' => 'typeid = '.$this->get_typeid(),
                    'id_col' => 'id',
                'weight_col' => 'orderby',
                       'dir' => $dir 
                      );
        
        // call the abstract method
        parent::move($params);
    } 

    
    /**
    * Re-orders the typesfields.
    * @return void
    * @access public
    */
    function reorder() 
    {
        // setup hash of parameters for parent method    
        $params = array('id' => $this->get_id(),
                     'table' => 'typesfields',
            'constraint_sql' => 'typeid = '.$this->get_typeid(),
                    'id_col' => 'id',
                'weight_col' => 'orderby',
                       'dir' => null, // oam $dir was never set anyway 
                      );
                        
        // invoke abstract parent method
        parent::reorder($params);
    } // reorder()
    
    /**
    * Checks privileges for the given user to see if the field is readable
    * @param object pxdb_user
    * @return boolean
    * @access public
    */
    function is_readable($user)
    {
        return pxdb_privs::is_readable('field', $this->get_id(), $user->get_ugid_array());
    }
}
?>