<?php
// $Id: pxdb_view.class.php,v 1.8 2005/12/28 21:35:07 omerida Exp $

pxdb_import('metadata.pxdb_basis');
pxdb_import('metadata.pxdb_typesfield');
pxdb_import('metadata.pxdb_viewfield');

/**
* This class represtents a row of the views table.
*
* @version $Revision: 1.8 $
* @package pxdb.metadata
* @author Hans Lellelid <hans@velum.net>
* @access public
*/
class pxdb_view extends pxdb_basis 
{
			
	/**
	* Constructor: perform db query to get full row based on the relationship id.
	* @param $id The view id.
	*/	
	function pxdb_view($id=null) 
	{
		parent::pxdb_basis();		
		
		$this->set_id($id);
		$this->read();		
	}
	
    /**
    * Return a pxdb_view object based on a datatype and view name
    *
    * @param integer datatype id
    * @param string view name
    * @return pxdb_view object 
    */
    function &factory( $datatype, $view )
    {
        $sql = "SELECT id FROM ".PXSQL_L."views".PXSQL_R
              ." WHERE datatype='$datatype' and view='$view'"
        ;
        
        $dbh =& pxdb::dbh();
        $rs =& $dbh->Execute($sql);
        
        if ( 0 < $rs->RowCount() )
        {
            $row = $rs->FetchRow();
            $view = new pxdb_view( $row['id'] );             
            return $view; 
        } 
        else
        {
            trigger_error( "View '$view' for datatype $datatype not found", E_USER_WARNING );
            $r = false;
            return $r;
        }
    }
	/**
	* Updates class vars from the database.
	* @access public
	*/
	function read() 
	{	
		if ($this->get_id()) 
        {			    		
    		$sql = "SELECT * FROM ".PXSQL_L."views".PXSQL_R." WHERE id = ".$this->get_id();
    		$rs = &$this->db->Execute($sql);
			$this->_fields = $rs->fields;		
		}					
	}
	
	/** 
	* Load override: loads values from global scope into the class.
	* @access public
	*/
	function load() 
	{
		parent::load('views');
	}
	
	/**
	* @access public
	*/
	function get_id() 
	{
		return $this->_fields['id'];
	}
	
	function set_id($id) 
	{
		$this->_fields['id'] = $id;
	}
	
	/**
	* @return string
	* @access public
	*/
	function get_view() 
	{
		return $this->_fields['view'];
	}
	
	/**
	* @param string $view
	* @return void
	* @access public
	*/
	function set_view($view) 
	{
		$this->_fields['view'] = $view;
	}
	
	/**
	* @return integer
	* @access public
	*/
	function get_datatype() 
	{
        if ( isset( $this->_fields['datatype'] ) )
        {
            return $this->_fields['datatype'];
        }
        else
        {
            return false;
        }
	}
	
	/**
	* @param integer $datatype
	* @return void
	* @access public
	*/
	function set_datatype($datatype) 
	{
		$this->_fields['datatype'] = $datatype;
	}
	
	/**
	* @return string
	* @access public
	*/
	function get_label() 
	{
		return $this->_fields['label'];
	}
	
	/**
	* @param string $label
	* @return void
	* @access public
	*/
	function set_label($label) 
	{
		$this->_fields['label'] = $label;
	}

	/**
	* @return string
	* @access public
	*/
	function get_description() 
	{
		return $this->_fields['description'];
	}
	
	/**
	* @param string $description
	* @return void
	* @access public
	*/
	function set_description($description) 
	{
		$this->_fields['description'] = $description;
	}

	/**
	* Wrapper for parent method.
	* @param boolean $reset Whether to start over.
	* @return object pxdb_typesfield object
	*/
	function &fetch_viewfield($reset=null)
	{
		return parent::fetch_viewfield($this->get_id(), $reset);
	}
	
	/**
	* Returns an array of typesfields for this object.
	* Gets all typesfields that are referred to in the viewfields for the current view.  The typesfields
	* entries are ordered by the viewfields orderby column (not the typesfields column).
	* @param string $mask The value to return for custom_field.
	* @param string $constraint_sql WHERE-clause SQL to constrain results.
	* @return array 2-dim array of typsefields.
	* @access public
	*/
	function arr_typesfields($mask='{identifier}', $constraint_sql=null) 
	{
		$column = pxdb_sql::sqlize_mask($mask, 'tf');
		
		$sql = "SELECT		tf.*,
							$column AS custom_field
        		FROM		typesfields tf
							INNER JOIN viewfields vf ON vf.typesfield = tf.id
							INNER JOIN ".PXSQL_L."views".PXSQL_R." v ON v.id = vf.viewid
				WHERE		v.id = ".$this->get_id()."
							".($constraint_sql ? "AND $constraint_sql" : "")."
        		ORDER BY	vf.orderby";
		
		$rs =& $this->db->Execute($sql);
        //OAM return resultset with typefield id as key to array
        $typesfields = array();
        while ( $tf = $rs->fetchRow() )
        {
            $typesfields[ $tf['id'] ] = $tf; 
        }
        return $typesfields;
	}
	
	/**
	* Get all viewfields for this view.
	* @param string $mask The value to return for custom_field.
	* @param string $constraint_sql WHERE-clause SQL to constrain results.
	* @access public
	*/
	function arr_viewfields($mask='{label}', $constraint_sql=null) 
	{	
		$view_fields = parent::arr_viewfields($this->get_id(), $mask, $constraint_sql);        
        
        if ( 0 == count( $view_fields ) )
        {
            trigger_error( 'No fields found for view', E_USER_WARNING );
            return false;
        }
        else
        {
            $types_fields = $this->arr_typesfields();
            $fields = array(); // init to be returned;
            while( list( ,$view_field ) = each( $view_fields ) )        
            {
                //echo '<pre>'; print_r( $view_field ); echo '</pre>';
                //echo '<pre>'; print_r( $types_fields[ $view_field['typesfield'] ] ); echo '</pre>';
                
                if ( empty( $view_field['label'] ) )
                {
                    $view_field['label'] = $types_fields[ $view_field['typesfield'] ]['displayname'];
                }
                
                if ( !empty( $types_fields[ $view_field['typesfield'] ]['identifier'] ) )
                {
                    $view_field['identifier'] = $types_fields[ $view_field['typesfield'] ]['identifier'];
                }
                else 
                {
                    trigger_error( 'Could not find matching typesfield for view field: ' . $view_field['label'] . ' (' . $view_field['id'] . ')', E_USER_WARNING );
                }
                $fields[] = $view_field;
            }
            
            return $fields;
        }
	}
	
	/**
	* The main function to handle inserts & updates.
	* @return integer The id of saved or craeted view.
	* @access public
	*/
	function save() 
	{
        if ( !$this->get_datatype() )
        {
            trigger_error( 'view::save() called but datatype is not set', E_USER_ERROR );
            return false;
        }
        
		if ($this->get_id()) 
        {		
			$sql = "UPDATE ".PXSQL_L."views".PXSQL_R." SET
						".PXSQL_L."view".PXSQL_R." = ".$this->db->Quote($this->get_view()).",
						label = ".$this->db->Quote($this->get_label()).",
						description = ".$this->db->Quote($this->get_description()).",
						datatype = ".$this->get_datatype()."
					WHERE id = ".$this->get_id();
			$this->db->Execute($sql);		
		}
        else
        {
			$sql = "INSERT INTO ".PXSQL_L."views".PXSQL_R." (
						".PXSQL_L."view".PXSQL_R.", 
						datatype, 
						label, 
						description
					) VALUES (
						".$this->db->Quote($this->get_view()).", 
						".$this->get_datatype().",
						".$this->db->Quote($this->get_label()).",
						".$this->db->Quote($this->get_description())."
					)";
					
			$this->db->Execute($sql);						
			$this->set_id($this->db->Insert_ID());
		}
		
		if ($this->debug) 
        {
            pxdb::print_debug($sql, 'db-update'); 
        }
		
		$this->read();
		return $this->get_id();
	}
		
	/**
	* This function figures out what the next (max+5) weight should be.
	* @return integer New weight.
	* @access protected
	*/
	function _new_weight() 
	{	
		$sql = "select weight from datatypes order by weight desc";
		$rs  =& $this->db->Execute($sql);
		return ($rs->fields['weight'] + 5);
	}

	
	/**
	* Function to delete a relationship entry.
	* @param $id ID to remove.
	* @return boolean
	*/
	function delete($id=null) 
	{
		$db =& pxdb::dbh();
		$id = ($id ? $id : $this->get_id());

		if (!$id) 
        {
			trigger_error("You must instantiate pxdb_view with the id of the view you wish to delete -- or pass an id to delete() method.", E_USER_WARNING);
			return false;
		}
		
		// load parameters or values from import
					
		$sql = "DELETE FROM views WHERE id = $id"; // note: could be deleting more than one record.
		if ( $this->debug )
        {
            pxdb::print_debug($sql, 'db-update'); 
        }
		$db->Execute($sql);
		
		$sql = "SELECT id FROM viewfields WHERE viewid = $id";
		$rs = $db->Execute($sql);
		
		while ( !$rs->EOF ) 
        {
			pxdb_viewfield::delete($rs->fields['id']);	
			$rs->MoveNext();
		}
				
		return true;
		
	}

    /**
    * Alternate contruction method to get a pxdb_view object based on the 
    * datatype and view name.  Duplicates factory method.
    *
    * @param integer datatype id
    * @param string view name
    * @return view object
    * @static
    */
    function &fetch_object_view( $dt, $view )
    {   
        $dbh =& pxdb::dbh();
        
        $sql = "SELECT v.id
                FROM views v 
                WHERE v.view = ".$dbh->qstr( $view )."
                    AND v.datatype = ".$dt;

        $row = $dbh->GetAll( $sql );
        
        return new pxdb_view( $row[0]['id'] );
    }

} // class

?>
