<?php
// $Id: pxdb_viewfield.class.php,v 1.6 2005/12/28 21:36:10 omerida Exp $

pxdb_import('metadata.pxdb_basis');

/**
* This class represtents a row of the viewfields table.
* 
* @version $Revision: 1.6 $
* @package pxdb.metadata
* @author Hans Lellelid <hans@velum.net>
* @access public
*/
class pxdb_viewfield extends pxdb_basis 
{
    
    /**
    * Constructor: call super constructor and read in vars from db.
    * @param $id The id of viewfield row.
    */
    function pxdb_viewfield($id=false) 
    {        
        // call parent constructor    
        parent::pxdb_basis();
        $this->set_id($id);
        $this->read();
    }
    
    /**
    * Updates class vars from the database.
    * @access public
    */
    function read() 
    {    
        if ($this->get_id()) 
        {
            $sql = "select * from viewfields where id = ".$this->get_id();
            $rs = $this->db->Execute($sql);
            $this->_fields = $rs->fields;
        }
    }
    
    /** 
    * Load override: loads values from global scope into the class.
    * @return void
    * @access public
    */
    function load() 
    {
        parent::load('viewfields'); 
    }
    
    /**
    * @return integer
    * @access public
    */    
    function get_id() 
    {
        return $this->_fields['id'];
    }

    /**
    * @param integer $id
    * @return void
    * @access public
    */    
    function set_id($id) 
    {
        $this->_fields['id'] = $id;
    }

    /**
    * @return integer
    * @access public
    */    
    function get_viewid() 
    {
        return $this->_fields['viewid'];    
    }

    /**
    * @param integer $viewid
    * @return void
    * @access public
    */    
    function set_viewid($viewid) 
    {
        $this->_fields['viewid'] = $viewid;
    }
    
    /**
    * @return string 'Y' or 'N'
    * @access public
    */    
    function get_expand() 
    {
        $exp = (true === isset($this->_fields['expand']))
            ? $this->_fields['expand'] : null;
        if (!$exp)
        {
            $exp = 'N';
        }         
        return $exp;
    }

    /**
    * @param string $expand 'Y' or 'N'
    * @access public
    */    
    function set_expand($expand) 
    {
        $this->_fields['expand'] = $expand;
    }
    
    /**
    * @return boolean
    * @access public
    */
    function is_expandable() 
    {
        return ($this->get_expand() == "Y");
    }
            
    /**
    * @return boolean
    * @access public
    */    
    function get_subviewid() 
    {
        if (true === isset($this->_fields['subviewid']))
        {
            return $this->_fields['subviewid'];
        }
    }

    /**
    * @param integer $subviewid
    * @return void
    * @access public
    */    
    function set_subviewid($subviewid) 
    {
        $this->_fields['subviewid'] = $subviewid;
    }
    
    /**
    * @return string
    * @access public
    */    
    function get_label() 
    {
        if (true === isset($this->_fields['label']))
        {
            return $this->_fields['label'];
        }
    }

    /**
    * @param string $label
    * @return void
    * @access public
    */    
    function set_label($label) 
    {
        $this->_fields['label'] = $label;
    }

    /**
    * @return string
    * @access public
    */    
    function get_description() 
    {
        if (true === isset($this->_fields['description']))
        {
            return $this->_fields['description'];
        }
    }

    /**
    * @param string $description
    * @return void
    * @access public
    */    
    function set_description($description) 
    {
        $this->_fields['description'] = $description;
    }

    /**
    * @return integer
    * @access public
    */    
    function get_typesfield() 
    {
        return $this->_fields['typesfield'];
    }

    /**
    * @param integer $typesfield
    * @return void
    * @access public
    */    
    function set_typesfield($typesfield) 
    {
        $this->_fields['typesfield'] = $typesfield;
    }
    
    /**
    * Returns a typesfield object for this field
    * @return object
    * @access public
    */
    function fetch_typesfield()
    {
        if ( $tf = $this->get_typesfield() )
        {
            return new pxdb_typesfield($tf);
        }
        else
        {
            trigger_error( 'typefield property is empty or not set', E_USER_WARNING );
            return false;
        }
    }
    /**
    * @return integer
    * @access public
    */    
    function get_orderby() 
    {
        return $this->_fields['orderby'];
    }

    /**
    * @param integer $orderby 
    * @return void
    * @access public
    */    
    function set_orderby($orderby) 
    {
        $this->_fields['orderby'] = $orderby;
    }
    
    /**
    * Saves a viewfield to the database.
    * @return integer ID of newly created or saved record.
    * @access public
    */
    function save() 
    {
        // we import everything that tells us which vars to write.
        // we're only writing a subset of the fields to the db:
        // others are written different ways:        
        $typesfield = $this->get_typesfield();
        $label = $this->get_label();
        $expand = $this->get_expand();
        $subviewid = $this->get_subviewid();        
        $description = $this->get_description();
        
        if ($this->get_id()) 
        {
            // for now just update.
            $sql = "UPDATE ".PXSQL_L."viewfields".PXSQL_R." SET
                        ".PXSQL_L."label".PXSQL_R." = ".$this->db->Quote($label).",
                        ".PXSQL_L."description".PXSQL_R." = ".$this->db->Quote($description).",
                        ".PXSQL_L."typesfield".PXSQL_R." = ".$typesfield.",
                        ".PXSQL_L."expand".PXSQL_R." = ".$this->db->Quote($expand).",
                        ".PXSQL_L."subviewid".PXSQL_R." = ".$subviewid."
                    WHERE ".PXSQL_L."id".PXSQL_R." = ".$this->get_id();
            
            $this->db->Execute($sql);
            return $this->get_id();
        }
        else
        {
            $sql = "INSERT INTO ".PXSQL_L."viewfields".PXSQL_R." (
                            ".PXSQL_L."viewid".PXSQL_R.",
                            ".PXSQL_L."label".PXSQL_R.",
                            ".PXSQL_L."description".PXSQL_R.",
                            ".PXSQL_L."typesfield".PXSQL_R.",
                            ".PXSQL_L."expand".PXSQL_R.",
                            ".PXSQL_L."subviewid".PXSQL_R.",
                            ".PXSQL_L."orderby".PXSQL_R."
                        ) VALUES (
                            ".$this->get_viewid().",
                            ".$this->db->Quote($label).",
                            ".$this->db->Quote($description).",
                            ".$this->db->Quote($typesfield).",
                            ".$this->db->Quote($expand).",
                            ".$this->db->Quote($subviewid).",
                            ".$this->_next_orderby()."
                        )";        
                        
            $this->db->Execute($sql);
            $this->set_id($this->db->Insert_ID());
            
            // add default privs for this field
        }
        
        // re-read, just to be sure everythign is in order with this class
        // after we save contents.
        $this->read();
        return $this->get_id();
    }
    
    /**
    * Calculates the next weight value for this object.
    * @return int The new orderby value.
    * @access private
    */
    function _next_orderby() 
    {
        // first reorder the fields
        $this->reorder();
        
        // get the new highest value for orderby
        $sql = "SELECT MAX(orderby) as max_oby FROM viewfields WHERE viewid = ".$this->get_viewid();    
        $rs = &$this->db->Execute($sql);        
        
        // new one will be highest + 5
        $new_oby = $rs->fields['max_oby'] + 5;
        return $new_oby;
    }

    /**
    * Remove a field from an pxdb object.
    * @param $id The field id we want to delete.
    * @return boolean true/false on success.
    * @access public
    */
    function delete($id=null) 
    {        
        
        $id = ($id ? $id : $this->get_id());
        $db = &pxdb::dbh();
        
        if (!$id) 
        {
            trigger_error("You must instantiate pxdb_viewfield with the id of the field you wish to delete -- or pass an id to delete() method.", E_USER_WARNING);
            return false;
        }
        
        // delete the row from viewfields        
        $sql = "DELETE FROM viewfields WHERE id = $id";
        if ($this->debug) 
        {
            pxdb::print_debug($sql, 'db-update'); 
        }
        $db->Execute($sql);                
            
        return true;
    }            
    
    

    /**
    * Promote or demote a field.
    * Changes the orderby column for viewfields rows.
    * @param string $dir Direction ('up', 'down') to move.
    * @access public
    */
    function move($dir = 'down') 
    {
        // get the weight of the current field.
        // swap with next higher or lower field
        $params = array('id' => $this->get_id(),
                        'table' => 'viewfields',
                        'constraint_sql' => 'viewid = '.$this->get_viewid(),
                        'id_col' => 'id',
                        'weight_col' => 'orderby',
                        'dir' => $dir 
                        );
        
        // call the abstract method
        parent::move($params);
    } 

    
    /**
    * Re-orders the viewfields.
    * @access public
    */
    function reorder() 
    {
        // setup hash of parameters for parent method
        $params = array('id' => $this->get_id(),
                        'table' => 'viewfields',
                        'constraint_sql' => 'viewid = '.$this->get_viewid(),
                        'id_col' => 'id',
                        'weight_col' => 'orderby'
                        );
                        
        // invoke abstract parent method
        parent::reorder($params);
    } // reorder()

}
?>
