<?php
// $Id: pxdb.class.php,v 1.5 2005/10/17 05:40:29 sandys1 Exp $

pxdb_import('metadata.object_cache');
/**
* Core static utility class for storing system-wide methods.
* The original class design had all classes extending this basic top-level class. This class now exists
* to provide static methods for system-wide tasks:  e.g. triggering an error, printing debug code, etc.
* @package pxdb
* @version $Revision: 1.5 $
* @author Hans Lellelid <hans@velum.net>
* @access public
*/

require_once("PEAR.php");

class pxdb 
{
         
    /**
    * Set or get the current db instance.
    * DB object is stored as a static variable for this method, so you do not need to instantiate
    * this class to call this method / get the current db being used by the site.
    * @param object $db ADOConnection
    * @return object ADOConnection object
    * @access public
    */
    function &dbh($new_dbh = null)
    {
        static $dbh = null;
            
        if ($new_dbh !== null)
        {
            $dbh = $new_dbh;
        }
        
        return $dbh;
    }
    
    /**
    * Set or get the current Smarty template engine instance -- or instantiate if not.
    * Template engine is stored as a static variable for this method.
    * @param $tpl
    */
    function &tpl($new_tpl = null)
    {
        static $tpl;
        
        if ($new_tpl !== null)
        {        
            $tpl = $new_tpl;
        }
        
        if (!isset($tpl))
        {
            $tpl = new Smarty();            
            $tpl->left_delimiter = '<%';
            $tpl->right_delimiter = '%>';                                    
            $tpl->template_dir = PXDB_TPLPATH;
            $tpl->compile_dir = PXDB_TPLPATH . DIRECTORY_SEPARATOR . 'templates_c';
        }
        
        return $tpl;    
        
    }
    
    /**
    * Import data from global scope.
    * @param array $source The source values to import.
    * @return array The imported values.
    */
    function import($source = null) 
    {
    
        if ($source === null)
        { 
        
            $source = pxdb::import_var(PXDB_GLOBAL_VAR);             
            
            // if register globals is off, we *may* need to grab it from the $_REQUEST array
            // (it's also possible that this class is being instantiated from a point in the PHP
            // script where the global var would have been set.)
            
            if (!$source && !ini_get('register_globals') && isset( $_REQUEST[PXDB_GLOBAL_VAR] ))
            {
                 $source = $_REQUEST[PXDB_GLOBAL_VAR];
            }                    
        }
        
        return $source;                
    }    

    /**
    * Imports value from global scope.
    * This function takes a string representing a variable name and reads it in from global
    * scope.  This string could be an array:  e.g. $_REQUEST[f1db][desc1].
    * @param string $varname The name of variable to look for.
    * @return mixed The value corresponding to ${$varname};
    * @access private
    */
    function import_var($varname) 
    {
        if ($varname)
        {
            // get the variable part of the name
            // OAM: so what does this preg_match do? grab anything that is not an opening brace or dolalr sign?
            preg_match('/([^\[|$]+)/', $varname, $matches);        
            $var = $matches[1];
        
            global ${$var};        
            //eval("if (isset(\$$varname)) { \$value = \$$varname; } else { \$value = null; }");
            
            // oam, replace eval with straightup php
            if ( isset( $$varname) )
            {
                $value = $$varname;
            }
            else
            {
                $value = null;
            }
            
            return $value;
        }
        else
        {
            return null;
        }
    }
    
        
    /**
    * A function to print debug output -- if $debug var is set.
    * This is a tool used by classes that extend this class.
    * @param $msg The message (or array, etc) to print.
    * @param $type Type of error determines the color.
    * @access public
    */
    function print_debug($msg, $type = '') 
    {    
        
        switch ($type)
        {        
            case 'db-select':
                $color = 'green';
            break;
            
            case 'db-update':
                $color = 'red';
            break;
            
            case 'bookmark':
                $color = 'orange';
            break;
            default:
                $color = $type;
        }
        
        if (!$color)
        { $color = 'black'; }
        
        print "<font color='$color'><pre>"; print_r($msg); print "</pre></font>";
        
    }    
    
    /**
    * Wrapper for PHP's trigger_error() allows for flexible error handling in the future.
    * @param string $error_msg
    * @param integer $error_type
    * @param string $error_src The method name or file name of error origination.
    * @access public
    */
    function trigger_error($error_msg, $error_type = E_USER_WARNING, $error_src = null)
    {
        trigger_error("[PxDB".($error_src ? ": $error_src" : "")."] $error_msg", $error_type);
    }
    
    /**
    * Get all tables that are associated with Syntax.
    * This is a static lsit of tables that is set every time this function is called.
    * To add additional "hidden" tables to your Syntax  instance, add your tablename(s) as the
    * value of to the pxdb_prefs table -- with pref_name = 'pxdb_table', pref_key = '', pref_value='yourtablname'.
    * @return array Tables that are Syntax tables.
    * @access public
    */
    function get_pxdb_tables() 
    {        
        $prefs = &pxdb_prefs::singleton();    
        return $prefs->get_array('pxdb_table');
    }

    /**
    * Get the non-f1db tables in this instance.
    * Calculates the non-f1db tables by the difference between results from
    * get_f1db_tables() and get_tables() methods.
    * @return array The non-f1db tables.
    * @access public
    */
    function get_nonpxdb_tables() 
    {        
        $db = &pxdb::dbh();
        $prefs = &pxdb_prefs::singleton();
        $object_cache = &object_cache::singleton();
        if (! $tables = $object_cache->getDbTables() )
        {
            $object_cache->saveDBTables( $db->MetaTables() );
            $tables = $object_cache->getDbTables();
        }
        return array_diff($tables, pxdb::get_pxdb_tables());
    }
    
    /**
    * Get the default order by column for a pick table.
    * Will be a weight column if present, if not, will be a pick column.
    * @param string $table Pick table name
    * @param string $default The default column to use
    * @return string The column name to order by
    */
    function get_pick_orderby($table, $default = 'name')
    {
        $db = &pxdb::dbh();
        
        $object_cache = &object_cache::singleton();
        if (! $cols = $object_cache->getTableColumns( $table ) )
        {
            $object_cache->saveTableColumns($table, $db->MetaColumnNames($table));
            $cols = $object_cache->getTableColumns( $table );
        }
                                    
        if (! empty($cols['weight']))
        {
            return 'weight';
        }
        elseif (! empty($cols['orderby']))
        {
            return 'orderby';
        }
        else
        {
            return $default;
        }
        
    }
    
    /**
    * Checks whether a table exists.
    * @param string $table The name of the table we are looking for.
    * @return boolean
    * @access public    
    */
    function is_table_created($table) 
    {    
        $db = &pxdb::dbh();
        $object_cache = &object_cache::singleton();
        if (! $tables = $object_cache->getDbTables() )
        {
            $object_cache->saveDBTables( $db->MetaTables() );
            $tables = $object_cache->getDbTables();
        }
        return in_array($table, $tables);
    }    

    /**
    * Create a table to be used by pick/pickid dbfield types.
    * @param $table The name of the table to create.
    * @return boolean status
    * @access public
    */
    function create_pick_table($table) 
    {    
        $db = &pxdb::dbh();
        
        $object_cache = &object_cache::singleton();
        if (! $tables = $object_cache->getDbTables() )
        {
            $object_cache->saveDBTables( $db->MetaTables() );
            $tables = $object_cache->getDbTables();
        }
        if (in_array($table, $tables))
        {
            pxdb::trigger_error("Unable to create an already existing table: $table.", E_USER_WARNING, 'pxdb::create_pick_table()');
            return false;
        }
        
        $sql = sprintf(PXSQL_CREATE_PICKTABLE, $table);
            
        $db->Execute($sql);        

        return true;
    }
        
    /**
    * Get all records columns that are reserved / required by Syntax.
    * This is a static list of columns that is set every time this function is called.
    * To add additional "hidden" columns to your Syntax instance, add your fieldname(s) as the
    * value of to the pxdb_prefs table -- with pref_name = 'pxdb_field', pref_key = '', pref_value='yourtablname'.
    * @return array Tables that are Syntax tables.
    * @access public
    */
    function get_pxdb_reserved_fields() 
    {
        $prefs = &pxdb_prefs::singleton();    
        return $prefs->get_array('pxdb_field');
    }
    

    /**
    * Checks to see whether specified plugin can be found.
    * @param string $type The type of plugin (corresponds to filesystem directory)
    * @param string $plugin_data The plugin name (function name) we are looking for.
    * @return boolean
    * @access public
    */
    function plugin_exists($type, $plugin_data) 
    {
        $plugin_info = pxdb::parse_plugin($plugin_data);
        $plugin = $plugin_info['script'];
        return(file_exists(PXDB_PLUGPATH . "/$type/$plugin.php"));
    }    
    
    /**
    * Get the relative URL to the pxdb_www directory.
    * There could be more than one pxdb_www preference -- the first choice
    * is to find one where the key is the current hostname.  For example,
    * we could have the following in the database:
    *     - pxdb_www [dbasis.syntax.forumone.com] = "/data/pxdb_www"
    *     - pxdb_www [mysite.com] = "/pxdb_live"
    * @return string The best match from preferences table.
    * @access public
    */
    function get_pxdb_www() 
    {            
        $prefs = &pxdb_prefs::singleton();    
        
        // first look for a hostname-specific pref
        $pub_rel = $prefs->get_pref('pxdb_www', $_SERVER['HTTP_HOST']);        
        
        // if not found, try the default pref (we know this is set)
        if (!$pub_rel)
        {
            $pub_rel = $prefs->get_pref('pxdb_www', '');
            // SFS since the instructions insist you do a symlink in the top
            // level of public, no reason to not just assume that here.
            if (! $pub_rel)
            {
                $pub_rel = '/pxdb_www';
            }
        }
        
        return $pub_rel;
    } 
    
    /**
    * Handles reading / writing user-defined classes.
    * This has to be located in one function because we are using static variables to store
    * registered classes.
    * This class takes a variable number of parameters.  The first is the action.
    * @param string $action 'get' or 'register'
    * @return mixed Depends on $action
    * @access private
    */
    function _class($action)
    {
        static $classes;
        
        if (!isset($classes))
        {
            $classes = array();
        }
        
        if ($action == 'get')
        {
            // class to get equiv for
            $pxdb_class = func_get_arg(1);
            $option = func_get_arg(2);            
            if (isset($classes[$pxdb_class][$option]))
            {
                return $classes[$pxdb_class][$option];
            }
            else
            {
                return $pxdb_class; // fall through
            }
        }
        elseif ($action == 'register')
        {
            $user_class = func_get_arg(1);
            $pxdb_class = func_get_arg(2);
            $option = func_get_arg(3);
            $classes[$pxdb_class][$option] = $user_class;
            return true;
        }
        else
        {
            pxdb::trigger_error("Invalid action passed to pxdb::_class(): '$action'");
            return false;
        }
    }
    
    /**
    * Registers user classes to use in place of PxDB standard classes.
    * E.g. you may want to customize an authentication class or use a class
    * that extends the records class.
    * IMPORTANT:  It is assumed that the class you are registering has the same
    * constructor signature as PxDB equivalent class.
    * ALSO: not all classes will respect user-registered classes -- e.g. for overhead reasons
    * user-defined metadata data classes cannot be registered.
    * @param string $user_class The name of your class (e.g. Document)
    * @param string $pxdb_class The name of the pxdb class you are replacing (e.g. pxdb_record)
    * @param int $option Any class specific indexing option (i.e. provides ability to register many pxdb_record classes indexed by datatype)
    * @return boolean Success or failure to register
    * @access public
    */
    function register_class($user_class, $pxdb_class, $option = 0)
    {
        return pxdb::_class('register', $user_class, $pxdb_class, $option);
    }
    
    /**
    * Gets a user-defined class (if exists) for a corresponding PxDB class.
    * If not user-specified class is found, then it will return the pxdb class name.
    * @param string $pxdb_class
    * @param mixed $option
    * @return string Class name to use.
    * @access public
    */
    function get_class($pxdb_class, $option = 0)
    {
        return pxdb::_class('get', $pxdb_class, $option);
    }
    
    /**
    * Parses a plugin to extract any params and plugin script name.
    * @param string $plugin_data
    * @return array Hash with 'script' and 'params' keys.
    * @access public
    */
    function parse_plugin($plugin_data)
    {
        if (sizeof($plugin_parts = explode('::', $plugin_data)) > 1)
        {
            $plugin = $plugin_parts[0];
            
            // parse params which are in the following format:
            // plugin-name::width=100;height=200;
            $paramstr = $plugin_parts[1];
            // SFS: review this: why can't we split on ';'?
            $params_raw = preg_split("/(?<!\\\);/", $paramstr);
            $params = array();
            // while is faster
            while ( list( , $param ) = each( $params_raw ) )
            {    
                $param_pieces = explode('=', $param);
                if ( 2 == sizeof($param_pieces) )
                {
                    $param_name = trim($param_pieces[0]);
                    $param_val = $param_pieces[1];
                    $param_val = str_replace("\\;", ";", $param_val); // allow for escaped delim
                    $param_val = str_replace("\\\"", "\"", $param_val); // allow for escaped quotes
                    $params[$param_name] = trim($param_val, '" ');
                } // if sizeof == 2
            }
                                        
        }
        else
        {
            // no params specified
            $params = array();
            $paramstr = '';
            $plugin = $plugin_data;
        }

        return array('script'=>$plugin, 'params'=>$params, 'paramstr'=>$paramstr);
    }
}


?>