<?php
// $Id: pxdb_sql.class.php,v 1.2 2005/05/17 21:31:00 omerida Exp $

/**
* Static class of SQL utility functions.
*
* @author Hans Lellelid <hans@velum.net>
* @version $Revision: 1.2 $
* @since Syntax 0.0.3
* @package pxdb.tool
*/
class pxdb_sql 
{
    /**
    * Converts a mask into a CONCAT() statement for use in SQL.
    * A mask, in this case, is a string that has fieldnames enclosed in
    * curly brackets {}.  An example of a mask is: "{fname} - {ftype}"
    * @param string $mask The mask to model CONCAT() on.
    * @param string $alias The alias of the table we are selecting from (to prepend to column names).
    * @access private
    */
    function sqlize_mask($mask, $alias='')
    {
        $db =& pxdb::dbh();

        if ($alias)
        {
            $prefix = $alias.'.';
        }
        else
        {
            $prefix = '';
        }

        // get all the columns in the $mask

        preg_match_all("/{([^}]+)}/", $mask, $col_matches);

        $cols = $col_matches[1];

        $concat_elements = array();

        if (sizeof($cols))
        {
            preg_match("/^(.*?){".preg_quote($cols[0])."/", $mask, $lead);

            //print "<br>\$mask = $mask; \$cols[0] = $cols[0], \$lead = $lead[1]";

            if (!empty($lead[1]))
            {
                $concat_elements[] = $db->qstr($lead[1]); // this is text not a column name
            }

            $concat_elements[] = $prefix.$cols[0];

            // grab all the middle elements & text
            for ($i=0, $z=sizeof($cols)-1; $i < $z; $i++)
            {
                // grab the text between current and next elements
                preg_match("/".preg_quote($cols[$i])."}(.*?){".preg_quote($cols[$i+1])."/", $mask, $betwixt);
                $concat_elements[] = $db->qstr($betwixt[1]); // text
                $concat_elements[] = $prefix.$cols[$i+1]; // col
            }

            // grab any ending text
            preg_match("/".preg_quote($cols[$i])."}(.*?)\$/", $mask, $tail);
            if (!empty($tail[1]))
            {
                $concat_elements[] = $db->qstr($tail[1]);
            }

        }
        else
        {
            // there are no column names in mask
            $concat_elements[] = $db->qstr($mask);
        }

        if ( 1 == sizeof($concat_elements) )
        {
            $sql = $concat_elements[0];
        }
        else
        {
            // we're going to use eval, since ADOdb Concat() method would like to have a list
            // of params instead of an array
            $params = '"' . implode('", "', $concat_elements) . '"';
            eval("\$sql = \$db->Concat($params);");
        }
        return $sql;
    }

    /**
    * Gets the alias for a table from a SQL statement.
    * If no alias, then it returns the table name.
    * @param string $tablename The name of the table to check for an alias to. (first occur)
    * @param string $sql The SQL to parse for table alias.
    * @return string The found table alias (or tablename if none found)
    */
    function get_table_alias($tablename, $sql)
    {
        preg_match("/[\s]".$tablename."[\s]+([^,\s]+)[\s]*(,|where[\s]|inner[\s]|join[\s]|outer[\s]|left[\s]|right[\s])/i", $sql, $matches);
        // if found, will be in element 1 of matches array, otherwise use default 'records'
        $alias = ( isset($matches[1]) ? $matches[1] : $tablename);
        return $alias;
    }

    /**
    * Prepares a SQL statement. (not currently in use)
    * This is not the same as the ->Prepare() ADOdb method; this simply gets the SQL
    * ready for the database -- e.g. runing strtr() to swap in db-specific escaping, etc.
    * This allows us to use some meta-language in SQL statements in places.
    * @param string $sql
    * @return string The ready-to-go SQL
    */
    function prepare($sql)
    {
        // replace right and left quotes
        $sql = strtr($sql, "{/", PXSQL_L);
        $sql = strtr($sql, "/}", PXSQL_R);

        return $sql;
    }


} // class

?>