<?php
// $Id: pxdb_tools.class.php,v 1.4 2005/05/17 21:31:00 omerida Exp $

/**
* Class library of generic tools.
* This class should not be instantiated.
*
* @author Hans Lellelid <hans@velum.net>
* @version $Revision: 1.4 $
* @package pxdb.tool
*/
class pxdb_tools
{

    /**
    * Translates a string (using gettext).
    * -- I think that this could probably be replaced with sprintf() and variable swapping, but there
    * might be some reason to keep this method... If only for the fact that it's more elegant.
    * Takes a variable number of arguments, the first is the string to translate, subsequent ones
    * are values to replace {*} tags with.  This funtion preserves order relationship between tags & arguments
    * so that if tags are re-organized in translated text (very possible), it will still replace the correct values.
    * @param string $text The text to be translated.
    * @access public
    */
    function xlate($text) 
    {
        
        // 1) get the tags we are going to replace
        
        // get array of arguments in text
        preg_match_all("/{([^}]+)}/", $text, $var_matches);
        
        $replace_tags = $var_matches[1];
        
        // $replace_tags is an array of all tags we need to replace -- in original order.
        
        $num_args = func_num_args();
        $num_replace_tags = sizeof($replace_tags);
        
        if ( ($num_args - 1) < $num_replace_tags)
        {
            trigger_error("More replace tags than arguments in call to xlate() \"$text\"", E_USER_WARNING);
        }
        
        $replace_map = array();
        
        // build a hash of tags & values to replace them with.
        for ($c = 0; $c < $num_replace_tags; $c++)
        {
            $replace_map[ $replace_tags[$c] ] = func_get_arg($c+1);
        }
        
        
        // 2) translate the text
        
        $xlated = $text; //hnmmm...need to recompile my php
        //$xlated = gettext($text);
        
        // 3) re-insert the values from the hash
        
        $xlated = pxdb_tools::populate($xlated, $replace_map);
        
        // $xlated = preg_replace("/{([^\}]+)}/e", "\$replace_map[$1]" , $xlated); // \$replace_map[$1]
        
        return $xlated;

    }
    
    
    /**
    * Replaces {tags} in passed text with values in $replace_map.
    * @param string $text The text containing the tags we need to replace.
    * @param array $replace_map A hash where key is the tag in text and value is the value with which to replace that tag.
    * @param string $tag_open The character(s) that open(s) tags.
    * @param string $tag_close The character(s) that close(s) tags.
    * @return string The text that results after the find/replace.
    */
    function populate($text, $replace_map, $tag_open='{', $tag_close='}') 
    {    
        // {([^\}]+)}
        /*
        $regex = preg_quote($tag_open)."([^".preg_quote($tag_close)."]+)".preg_quote($tag_close);
        
        $text = preg_replace("/$regex/e", "\$replace_map['$1']" , $text); // \$replace_map[$1]
        */
        
        // oam what's wrong with strreplace?
        while ( list( $find, $replace ) = each( $replace_map ) )
        {
            $text = str_replace( $tag_open.$find.$tag_close, $replace, $text );
        }
        return $text;    
    }

    /**
    * Gets a list of search terms based on a passed search string.
    * @param string $search_data the string of keywords
    * @param string $prefix add to beginning of each term (e.g. '%')
    * @param string $suffix add to end of each term (e.g. '%')
    * @access public
    */
    function get_search_terms($search_data, $prefix=null, $suffix=null) 
    {
    
        // first get any quoted strings
        // match all
        preg_match_all("/\"(.*?)\"/", $search_data, $matches);
        $quoted_phrases = $matches[1];
        
        // now remove any quoted strings from the search keywords
        $search_data = preg_replace("/\".*?\"/", "", $search_data);
        
        // now change any spaces to commas
        $search_data = preg_replace("/[ ]+/", ",", $search_data);
        // remove any duplicate commas (i.e. what if they had put "love, war" -- then it would now be "love,,war")
        // SFS str_replace beats a preg_replace for performance
        $search_data = str_replace(',,', ',', $search_data);
        
        $keywords = explode(",", $search_data);
        
        // add the arrays together
        $all_terms = array_merge($keywords, $quoted_phrases);
        
        // remove any emtpy strings        
        $all_terms = array_diff($all_terms, array(''));
        
        if ( $prefix !== null || $suffix !== null )
        {
            $keys = array_keys($all_terms);
            // SFS removed another foreach
            while ( list( , $key ) = each( $keys ) )
            {
                $all_terms[$key] = $prefix . $all_terms[$key] . $suffix;
            }
        }
        
        return $all_terms;
    
    }

    /**
    * Takes a two-dimensional array and converts it to a one-dim array of values for given key.
    * @param string $key The key we want to extract.
    * @param array $data The input array.
    * @return array A 1-dim array with same number of elements of input array.
    * @access public
    */
    function array_thread($key, $data) 
    {
        $out = array();
        for ($z=sizeof($data), $i=0; $i < $z; $i++)
        {            
            $out[] = $data[$i][$key];        
        }    
        return $out;
    }
    
    /**
    * Generates a signature of ADOConnection.
    * Used for singleton instantiator functions.
    * @param ADOConnection $db The ADOdb Connection.
    */
    function db_signature(&$db) 
    {
        return md5($db->databaseType . $db->database . $db->host . $db->user . $db->password);
    }
    
    /**
    * Looks at the filesystem to find all the plugins (PHP files) for specified type.
    *
    * OAM: is this only used in dbasis?
    * 
    * @param string $type The plugin type we are looking for -- corresponds to filesystem directory.
    * @access public
    */
    function get_plugins($type) 
    {
        $dir = PXDB_PLUGPATH . "/$type";
        $d = dir($dir);
        $plugins = array();            
        while (false !== ($entry = $d->read()))
        {
            if (preg_match("/(.*)\.php\$/", $entry, $matches))
            {
                $plugins[] = $matches[1];
            }
        }
        $d->close(); 
        
        if ( !empty( $plugins ) && is_array( $plugins ) )
        {
            natcasesort($plugins);
        }
        
        return $plugins;
    }
    
    /**
    * Gets the time in 15 minute increments, to take advantage of MySQL's query caching
    *
    * @access public				
    * @return string ISO-formatted date to the previous 15 minute increment
    */
    function get_time_quarters()
    {
        // 5/5/2004 oam fix - get date from mysql database not webserver        
        $dbh = &pxdb::dbh();
        //must use %H in caps to get 24hr time (otherwise everything is AM) - AVC
        $sql = "SELECT DATE_FORMAT(NOW(), '%Y-%m-%d %H!%i') as time_now;";
        $q = $dbh->execute( $sql );
        $time = $q->getAll();
        
        $time_parts = explode('!', $time[0]['time_now']);
        $nowToHours = $time_parts[0].':';
        $mins = $time_parts[1];
        
        if ( $mins <15 )
        {
            $nowToHours .= '00';
        }
        else if ( $mins < 30 )
        {
            $nowToHours .= '15';
        }
        else if ( $mins < 45 )
        {
            $nowToHours .= '30';
        }
        else
        {
            $nowToHours .= '45';
        } // end if

        return $nowToHours;
    }

    /**
    * Encodes the URL_ID of a section so that URLs can be included in encoded
    * form in the URL_PATH of a section. Note that this involves a three step
    * encoding:
    *
    * Step 1: base64 encodes the entire string
    * Step 2: replaces the base64 '/' character with hash/pound sign '#'
    * Step 3: prepends an identifying label '*64'
    *
    * To decode an encoded link we use the decode_section_link() function
    *
    * @access public
    * @param string a URL to be encoded. Can be absolute or relative
    * @return string encoded version of input using modified base64 encoding
    */
    function encode_section_link($value)
    {
        // first base64 encode the string
        $value = base64_encode($value);
        
        // next alter the base64 slash characters to avoid
        // conflict with url path delimiters
        $value = str_replace("/", "#", $value);
        
        // Finally prepend a marker
        
        return "*64_".$value;
    }

    /**
    * Decodes a URL that has been encoded using the encode_section_link()
    * function. This essentially reverses the effect of running the encoding
    * function. It will return the decoded version of the input string or false
    * if the input string is not marked as an encoded URL (eg. has the *64_ prefix
    *
    * @access public
    * @param string encoded version of the string
    * @return mixed decoded version of input string or false if input is not encoded
    */
    function decode_section_link($value)
    {
        if (0 === strpos($value, '*64_'))
        {
            // Strip the prefix
            $value = substr($value, 4);
            // Turn back into base64 character set
            $value = str_replace("#", "/", $value);
            // decode base64
            return base64_decode($value);
        }
		
		return $value;
    }
}

?>
