<?php
// $Id: mysql-upgrade.php,v 1.18 2006/01/26 20:53:08 omerida Exp $

//  This file should only be included by the run-once.php script.
//  -------------------------------------------------------------
//
//  This file is responsible for making any changes to the database that
//  come about as a result of an upgrade to a newer Syntax instance.  The
//  file will do its best to bring things up-to-date.
//
//
//  $pxdb_prefs->get_pref('version') should return the database's version.  If
//  this pref is not set assume that either (1) it has been removed from the db 
//  or (2) the db is older than version 0.0.3.
//
//  The contant PXDB_VERSION should contain the current Syntax version.
//
//  We can assume that if this script is beng called, that there is a reason, but we
//  cannot make assumptions about which components are out of sync.  Careful testing
//  needs to precede any DB modification.
//
//
//  Also: this code is RDBMS specific, since it may involve creating tables, etc.
//
//

// this might take a little longer than usual:
set_time_limit( 60*5 );

$dbh = &${$PXDB_ADODB_VAR};  // alias -- for simplicity sake

//TODO: have upgrade modify .htacess to restrict access to only the
// upgrader's IP, ala serendipity:
//deny from all 
//allow from 207.32.101.193
//ErrorDocument 403 "Forum One is upgrading the site, please check back soon.
    
$to_dump_file = SITE_TMP_PATH.'/syntax_upgrade.'.DB.'.'.date('Ymdhis').'.sql';
$dump_cmd = 'mysqldump --add-drop-table -c -C -q -Q -u '.DB_USER.' -h '.DB_HOST.' -p'.DB_PASS.' '.DB;
//dump backup file to site temp
/*
$result = exec( "$dump_cmd > $to_dump_file && gzip $to_dump_file"); 
if ( !$result ) // silence is golden
{
    trigger_error( 'Saved database before upgrade to '.$to_dump_file, E_USER_NOTICE );
}
*/
// upgrade from 0.0.3 to 0.0.4

// OAM: todo, use mysqldump to get a backup snapshot before proceeding

if ( $_pxdb_db_version < 4 ) // 4 is 0.0.4
{
    // 0.0.3
    // Change `priv`.`rec_priv` from a SET to individual `read` and `write` columns
    // Change `field_priv`.`f_priv` from a SET to individual `read` and `write` columns
    
    // 1) Does the db qualify for this upgrade?
        
    $priv_columns = $dbh->MetaColumnNames('priv');
    // a) priv
    if (in_array('rec_priv', $priv_columns))
    {
    
        // Add the new fields
        if(!in_array('read', $priv_columns))
        {
            $sql = "ALTER TABLE `priv` ADD COLUMN `read` TINYINT NOT NULL";
            $dbh->Execute($sql);
            $sql = "ALTER TABLE `priv` ADD INDEX (`read`)";
            $dbh->Execute($sql);
        }
        
        if (!in_array('write', $priv_columns))
        {    
            $sql = "ALTER TABLE `priv` ADD COLUMN `write` TINYINT NOT NULL";
            $dbh->Execute($sql);
            $sql = "ALTER TABLE `priv` ADD INDEX (`write`)";
            $dbh->Execute($sql);
        }
                    
        // Update new fields        
        $sql = "UPDATE `priv` SET `read` = IF(FIND_IN_SET('read', rec_priv), 1, 0)";
        $dbh->Execute($sql);
        
        $sql = "UPDATE `priv` SET `write` = IF(FIND_IN_SET('write', rec_priv), 1, 0)";
        $dbh->Execute($sql);
        
        // drop the old column
        $sql = "ALTER TABLE `priv` DROP COLUMN `rec_priv`";
        $dbh->Execute($sql);
    
    } // if `rec_priv` column exists
    
    $field_priv_columns = $dbh->MetaColumnNames('field_priv');
    // b) field_priv
    if (in_array('f_priv', $field_priv_columns))
    {
    
        // Add the new fields
        if (!in_array('read', $field_priv_columns))
        {
            $sql = "ALTER TABLE `field_priv` ADD COLUMN `read` TINYINT NOT NULL";
            $dbh->Execute($sql);
            $sql = "ALTER TABLE `field_priv` ADD INDEX (`read`)";
            $dbh->Execute($sql);
        }
        
        if (!in_array('write', $field_priv_columns))
        {
            $sql = "ALTER TABLE `field_priv` ADD COLUMN `write` TINYINT NOT NULL";
            $dbh->Execute($sql);
            $sql = "ALTER TABLE `field_priv` ADD INDEX (`write`)";
            $dbh->Execute($sql);
        }
        
        // Update new fields        
        $sql = "UPDATE `field_priv` SET `read` = IF(FIND_IN_SET('read', f_priv), 1, 0)";
        $dbh->Execute($sql);
        
        $sql = "UPDATE `field_priv` SET `write` = IF(FIND_IN_SET('write', f_priv), 1, 0)";
        $dbh->Execute($sql);            
        
        // drop the old column
        $sql = "ALTER TABLE `field_priv` DROP COLUMN `f_priv`";
        $dbh->Execute($sql);
        
    } // if f_priv column exists
    
    
    if ($pxdb_prefs->get_pref('coltype','txt'))
    { // 'txt' is arbitrary, but a good test
        // if system is using old coltype pref style, then remove all the coltype prefs.
        $pxdb_prefs->del_pref('coltype');
    }
    
    
    // FIELD_PRIV
    // change `read` -> `priv_read`
    if (in_array('read', $field_priv_columns))
    {
        $sql = "ALTER TABLE field_priv CHANGE COLUMN `read` `priv_read` TINYINT";
        $dbh->Execute($sql);
    }
    // change `write` -> `priv_write`
    if (in_array('write', $field_priv_columns))
    {
        $sql = "ALTER TABLE field_priv CHANGE COLUMN `write` `priv_write` TINYINT";
        $dbh->Execute($sql);
    }
    
    // PRIV
    if (in_array('read', $priv_columns))
    {
        $sql = "ALTER TABLE priv CHANGE COLUMN `read` `priv_read` TINYINT";
        $dbh->Execute($sql);
    }
    // change `write` -> `priv_write`
    if (in_array('write', $priv_columns))
    {
        $sql = "ALTER TABLE priv CHANGE COLUMN `write` `priv_write` TINYINT";
        $dbh->Execute($sql);
    }
    
        
    // Changing relationships table
    //
    //
    
    $rel_columns = $dbh->MetaColumnNames('relationships');
    
    if (in_array('relid', $rel_columns))
    {
    
        $dbh->BeginTrans();
        $ok = true;
            
        // 1) Create new table: datatypesrel
            
        if (!in_array('datatypesrel', $dbh->MetaTables()))
        {
    
            $sql = "CREATE TABLE datatypesrel (
                        relid int NOT NULL ,
                        dt1 int NOT NULL ,
                        dt2 int NOT NULL,
                        PRIMARY KEY (relid,dt1,dt2)
                    )";
                    
            if ($ok) $ok = $dbh->Execute($sql);
            
        }
            
        // 2) Convert data
        
        $sql = "SELECT DISTINCT relid FROM relationships";
        $relids = $dbh->GetCol($sql);        
        
        if ($relids = array_diff($relids, array('')))
        {            
            
            // add a temporary column to relationships
            $sql = "ALTER TABLE relationships ADD COLUMN tmp_id INT";
            if ($ok) $ok = $dbh->Execute($sql);
            
            foreach ($relids as $relid)
            {
                                    
                $dt1 = array();
                $dt2 = array();                
                
                // find all datatypes for given relid
                $sql = "SELECT * FROM relationships WHERE relid = $relid";
                $rs = &$dbh->Execute($sql);
                
                // aggregate the datatypes
                while (!$rs->EOF)
                {
                    $description = $rs->fields['description']; // sure, keeps getting overwritten
                    $weight = $rs->fields['weight']; // overwritten
                    $dt1[] = $rs->fields['dt1']; // aggregate
                    $dt2[] = $rs->fields['dt2']; // aggregate
                    $rs->MoveNext();
                }                    
                            
                // add the new record with the ID field set to what used to be relid
                $sql = "
    INSERT INTO relationships (
        tmp_id,
        description, 
        weight
    ) VALUES (
        $relid, 
        ".$dbh->qstr($description).", 
        $weight)
                ";
    
                if ($ok) $ok = $dbh->Execute($sql);        
                
                // add datatypes to rel table
                foreach ($dt1 as $dt1_piece)
                {            
                    foreach ($dt2 as $dt2_piece)
                    {
                        $sql = "DELETE FROM datatypesrel WHERE relid = $relid AND dt1 = $dt1_piece AND dt2 = $dt2_piece";
                        if ($ok) $ok = $dbh->Execute($sql);
                        // insert relationship
                        $sql = "INSERT INTO datatypesrel (relid, dt1, dt2) VALUES ($relid, $dt1_piece, $dt2_piece)";
                        if ($ok) $ok = $dbh->Execute($sql);                                            
                    } // foreach($dt2)    
                } // foreach($dt1)
            
                // now delete the current records and create a new one
                $sql = "DELETE FROM relationships WHERE relid = $relid";
                if ($ok) $ok = $dbh->Execute($sql);                                        
                
            } // foreach
    
        } // if results
        
        // now set the ID to the tmp_id -- shouldn't be any duplicates ????        
        $sql = "UPDATE relationships SET id = tmp_id";
        if ($ok) $ok = $dbh->Execute($sql);
        
        // now drop the tmp column
        $sql = "ALTER TABLE relationships DROP COLUMN tmp_id";
        if ($ok) $ok = $dbh->Execute($sql);        
        
        // now drop the relid column -- all information is updated
        $sql = "ALTER TABLE relationships DROP COLUMN relid";
        if ($ok) $ok = $dbh->Execute($sql);
    
            // now drop the relid column -- all information is updated
        $sql = "ALTER TABLE relationships DROP COLUMN dt1";
        if ($ok) $ok = $dbh->Execute($sql);
        
                // now drop the relid column -- all information is updated
        $sql = "ALTER TABLE relationships DROP COLUMN dt2";
        if ($ok) $ok = $dbh->Execute($sql);        
        
        $dbh->CommitTrans($ok);    
    }    
    
    // 3) Change the typesfields table
    $typesfields_columns = $dbh->MetaColumnNames('typesfields');
    if (in_array('unique', $typesfields_columns))
    {
        // change column 'unique' to 'is_unique'
        $sql = "ALTER TABLE typesfields CHANGE `unique` is_unique ENUM('Y','N') DEFAULT 'N'";
        $dbh->Execute($sql);
    }
} // end of 0.0.3 -> 0.0.4 changes

// upgrade from 0.0.4 to 0.0.5
if ( $_pxdb_db_version < 100 ) // 4 is 0.0.4
{
    // 1) Create new table: datatype_privs            
    if (!in_array('datatype_privs', $dbh->MetaTables()))
    {        
        $sql = "CREATE TABLE `datatype_privs` (
                    `id` INT UNSIGNED NOT NULL AUTO_INCREMENT ,
                    `datatype_id` TINYINT UNSIGNED NOT NULL ,
                    `ugid` INT UNSIGNED NOT NULL ,
                    `priv_view` TINYINT UNSIGNED DEFAULT '0' NOT NULL ,
                    `priv_add` TINYINT UNSIGNED DEFAULT '0' NOT NULL ,
                    `priv_edit` TINYINT UNSIGNED DEFAULT '0' NOT NULL ,
                    `priv_approve` TINYINT UNSIGNED DEFAULT '0' NOT NULL ,
                    `priv_delete` TINYINT UNSIGNED DEFAULT '0' NOT NULL ,
                    PRIMARY KEY ( `id` ) ,  
                    KEY `datatype_id` (`datatype_id`,`ugid`),
                    KEY `datatype_ugid` (`datatype_id`,`ugid`)                    
               )";
                
        $dbh->Execute($sql);        
    }
    
    // 2) create admin group privs
    $admin_uid = $pxdb_prefs->get_pref('gid_admin');
    
    $sql = "SELECT id FROM datatypes WHERE expose='Y'";
    $datatypes = $dbh->getAll( $sql );
    $datatypes[] = array( 'id'=>9 ); // to keep sections editable by admin group    
    $privCols = $dbh->MetaColumns('datatype_privs' );
    $privCols = array_keys( $privCols );
    while( list(, $col) = each( $privCols ) )
    {
        $col = strtolower( $col );
        if ( FALSE!==strpos( $col, 'priv_' ) )
        {
            $privs[] = 1;            
        }
    }
   
    $privs = implode( ', ' , $privs );
    while ( list(,$dt) = each( $datatypes) )
    {
        $sql = 'INSERT INTO datatype_privs VALUES'
              ." ( '', ".$dt['id'].", $admin_uid, $privs )"
        ;
        
        $dbh->execute( $sql );
    }    
    
    
}

if ( $_pxdb_db_version < 110 ) // converting from x.x.x to xxx
{
    // change the noderel table
    $noderel_columns = $dbh->MetaColumnNames('noderel');
    if (! in_array('noderelid', $noderel_columns))
    {
        // add new fields and indices
        $sql = "
ALTER TABLE noderel
    ADD COLUMN noderelid TINYINT(3) UNSIGNED,
    ADD COLUMN weight INT UNSIGNED DEFAULT 0,
    ADD INDEX add_child (parent, noderelid),
    ADD INDEX add_parent (child, noderelid),
    ADD INDEX relid (noderelid),
    ADD INDEX relweight (parent, noderelid, weight),
    ADD INDEX weight (weight),
    DROP PRIMARY KEY
        ";
        $dbh->Execute($sql);
        
        // set default value to 0 for section to section relationships
        $sql = "SELECT id FROM records WHERE datatype = 9";
        $nodes = $dbh->GetCol($sql);
        $sql = "
UPDATE noderel
    SET noderelid = 0
    WHERE child IN (".implode(', ', $nodes).')';
        $dbh->Execute($sql);
        
        // set default value to 1 for section to non-section relationships,
        // if *not* featured
        $sql = "SELECT id FROM records WHERE datatype != 9 AND bool1 != 1";
        $unfeatured = $dbh->GetCol($sql);
        if ($unfeatured)
        {
            $sql = "
    UPDATE noderel
        SET noderelid = 1
        WHERE child IN (".implode(', ', $unfeatured).')';
            $dbh->Execute($sql);
        }
        
        // set default value to 2 for section to non-section relationships,
        // if *featured*
        $sql = "SELECT id FROM records WHERE datatype != 9 AND bool1 = 1";
        $featured = $dbh->GetCol($sql);
        if ($featured)
        {
            $sql = "
    UPDATE noderel
        SET noderelid = 2
        WHERE child IN (".implode(', ', $featured).')';
            $dbh->Execute($sql);
        }
        
        // performance upgrade: add an index to the priv table
        $sql = "
ALTER TABLE `priv` ADD INDEX (`recid`, `priv_read`, `ugid` )
        ";
        $dbh->Execute($sql);
        
        // OAM for default search module to use fulltext in records
        $sql = "
ALTER TABLE `records` ADD FULLTEXT (`name` ,`title` ,`desc1`);
        ";
        
        $dbh->Execute($sql);
    }
}

if ( $_pxdb_db_version < 120 ) // converting from x.x.x to xxx
{
    
    // create new url_path typesfield entry and give privs
    
    // TODO: OAM, check that this field doesn't already exist?
    $sql = "
INSERT INTO typesfields
    (typeid, identifier, fieldid, req, is_unique, displayname, widget)
VALUES
    (9, 'url_path', 60, 'N', 'N', 'URL PATH (automatic)', 'readonly::')
    ";
    $dbh->Execute($sql);
    // get ID of created field and give new field basic privs
    $sql = "SELECT id FROM typesfields WHERE identifier = 'url_path' AND typeid = 9";
    $id = $dbh->GetOne($sql);
    
    //oam: added check to make sure field_priv doesn't already exist
    $sql = "SELECT * from field_priv where fid=$id and ugid=0";
    $key_exists = $dbh->GetOne( $sql );
    
    if ( !empty( $key_exists ) )
    {
        $sql = "UPDATE field_priv SET priv_read=1, priv_write=1 WHERE fid=$id and ugid=0";
    }
    else 
    {
        $sql = "INSERT INTO field_priv (fid, ugid, priv_read, priv_write) VALUES ($id, 0, 1, 1)";
    }
    $dbh->Execute($sql);
    
    // Now the hard part: create the new paths for the sections
    
    // First, get ids of all the sections
    $sql = "SELECT id FROM records WHERE datatype = 9 order by created";
    $sections = $dbh->GetCol($sql);
    
    // Then set up the event handler that already has the relevant code
    pxdb_import('events.pxdb_event_listener');
    include_once(PXDB_CLASSPATH . DIRECTORY_SEPARATOR . '..' . DIRECTORY_SEPARATOR . 'events/syntax_section_handler.class.php');
    $handler = new syntax_section_handler();
    $handler->init();
        
    // oam, check first for sections that are parents of themselves or 
    // whose parents are their children (loops) or who are their own parents
    $sql = 'select n1.*, n2.* from noderel n1'
          .' inner join noderel n2 on (n1.child=n2.parent and n2.noderelid=0)'
          .' and (n2.child=n1.parent and n1.noderelid=0)'
    ;
    $results = $dbh->execute( $sql );
    while( $row = $results->FetchRow() )
    {                
        if( $row['parent'] == $row['child'] )
        {
            trigger_error( 'Removing noderel for section '.$row['parent'].', was set as its own child.', E_USER_NOTICE );
            
            $sql = 'delete from noderel where parent='.$row['parent'].' and child='.$row['parent'].' and noderelid=0';
            $dbh->Execute( $sql );            
        }
        else
        {
            // first we test that we haven't fixed the other part of this rel
            if ( $fixed[ $row['parent'] ] != $row['child'] )
            {
                trigger_error( 'Fixing section '.$row['parent'].', whose child '.$row['child'].' is also its parent.', E_USER_NOTICE );
                
                $sql = 'delete from noderel where parent='.$row['child'].' and child='.$row['parent'].' and noderelid=0';
                $dbh->execute( $sql );
                
                // fixed array keeps track of which we've fixed so we don't delete both relationshiops
                $fixed[ $row['child'] ] = $row['parent'];
            }
            else
            {
                trigger_error( 'This noderel was fixed', E_USER_NOTICE );
            }
        }
        
    }
    
    // loop through the sections and apply the logic to them
    $max = count($sections);
    for ($i = 0; $i < $max; $i++)
    {
        trigger_error( 'Updating '.$sections[$i], E_USER_NOTICE );
        $handler->updateSectionPath(array('id' => $sections[$i]));
    }
}

if ($_pxdb_db_version < 121) // converting from x.x.x to xxx
{
	// This update adds a new view for sections to support section links
	$sql = "INSERT INTO views (`datatype`, `view`, `label`, `description`)
			VALUES (9, 'links', 'Section Links','')";
	$dbh->Execute($sql);
	
	// Find out what the view id of the new view is
	$sql = "SELECT LAST_INSERT_ID() FROM views";
	$view_id = $dbh->GetOne($sql);
	
	if (isset($view_id))
	{
		// Now add the fields to the view
		$sql = "INSERT INTO viewfields (`viewid`, `typesfield`, `label`, `expand`, `subviewid`, `orderby`)
				VALUES ($view_id, 57, 'Name', 'N', 0, 0)";
		$dbh->Execute($sql);
	
		$sql = "INSERT INTO viewfields (`viewid`, `typesfield`, `label`, `expand`, `subviewid`, `orderby`)
				VALUES ($view_id, 171, 'Address (URL)', 'N', 0, 5)";
		$dbh->Execute($sql);
	}
	else
	{
		trigger_error("Unable to create section link view in SQL upgrade script", E_USER_WARNING);
	}
}

if (122 > $_pxdb_db_version)
{
    // Add support for explanatory labels for views and typesfield definitions
    $sql = "ALTER TABLE `viewfields` ADD COLUMN `description` LONGTEXT AFTER `label`";
    $dbh->Execute($sql);
    
    $sql = "ALTER TABLE `typesfields` ADD COLUMN `description` LONGTEXT AFTER `displayname`";
    $dbh->Execute($sql);
}

if (123 > $_pxdb_db_version)
{
    // convert single pickids to use pickrel table
    $sql = "SELECT id, fname FROM dbfields WHERE ftype LIKE 'pickid %'";
    $pickid_dbfields = $dbh->GetAll($sql);
    if (true === is_array($pickid_dbfields) && false === empty($pickid_dbfields))
    {
        // get records table column names, so we don't look for non-existent columns
        $records_column_names = array_values($dbh->MetaColumnNames('records'));
        while (list($row, $data) = each($pickid_dbfields))
        {
            if (true === in_array($data['fname'], $records_column_names))
            {
                $pickid_fields[$data['id']] = $data['fname'];
                $pickid_ids[] = $data['id'];
            }
        }
        // Get all single pickid fields
        $sql = "
SELECT
    fieldid, typeid
FROM
    typesfields
WHERE
    fieldid IN (" . join(', ', $pickid_ids) . ")
    AND mul = 'N'
        ";
        $typesinfo = $dbh->GetAll($sql);
        // loop through, grab the data, insert it into pickrel
        while (list($row, $data) = each($typesinfo))
        {
            $sql = "
SELECT
    id, " . $pickid_fields[$data['fieldid']] . "
FROM
    records
WHERE
    datatype = " . $data['typeid'] . "
    AND " . $pickid_fields[$data['fieldid']] . " != ''
            ";
            
            $records = $dbh->GetAll($sql);
            while (list($record_row, $record_data) = each($records))
            {
                if ( is_numeric( $record_data[$pickid_fields[$data['fieldid']]] ) )
                {
                    $my_value = $record_data[$pickid_fields[$data['fieldid']]];
                }
                else
                {
                    pxdb_import('metadata.pxdb_dbfield');
                    $my_value = $record_data[$pickid_fields[$data['fieldid']]];
                    $dbfield  = new pxdb_dbfield($data['fieldid']);
                    $table    = $dbfield->get_pick_table();
                    $column   = null == $dbfield->get_pick_column() ? 'name' : $dbfield->get_pick_column();
                    
                    // get the id
                    $pick_sql = "SELECT id FROM $table WHERE $column='$my_value'";
                    
                    $match = $dbh->GetAll($pick_sql);
                    $my_value = $match[0]['id'];                   
                }
                
                if ( !empty( $my_value ) )
                {
                    $sql = "
INSERT INTO
    `pickrel` (`f_id`, `p_id`, `recid`)
VALUES
    (
        " . $data['fieldid'] . ",
        " . $my_value . ",
        " . $record_data['id'] . "
    )
                    ";
                
                    $dbh->Execute($sql);
                }
                else
                {
                    trigger_error( 'Could not find matching id for value ' 
                    . $record_data[$pickid_fields[$data['fieldid']]] . ' in pick table ' . $table
                    .' for column '. $column, E_USER_WARNING );
                }
            }
        }
    }

    // Add typesfield for table pick
    $tables = $dbh->MetaTables();
    if (false === in_array('stx_section_templates', $tables))
    {
        // if we don't have it, add it and all necessary db stuff
        $sql = "
CREATE TABLE `stx_section_templates` (
  `id` int(10) unsigned NOT NULL auto_increment,
  `name` varchar(255) NOT NULL default '',
  `filename` varchar(255) default NULL,
  `weight` int(11) NOT NULL default '0',
  PRIMARY KEY  (`id`),
  UNIQUE KEY `id_2` (`id`),
  KEY `id` (`id`)
) TYPE=MyISAM
        ";
        $dbh->Execute($sql);
        // put in the dbfield
        $sql = "
INSERT INTO `dbfields`
    (`fname`, `ftype`)
VALUES
    ('section_template','pickid stx_section_templates')
        ";
        $dbh->Execute($sql);
        $sql = "
SELECT id FROM dbfields WHERE fname = 'section_template'
        ";
        $dbfield_id = $dbh->GetOne($sql);
        $sql = "
INSERT INTO `typesfields`
    (`typeid`, `identifier`, `fieldid`, `otype`, `relid`, `orderby`, `req`, `is_unique`, `mul`, `displayname`, `description`, `autogen`, `validator`, `processor`, `widget`)
VALUES
    (9,'template',$dbfield_id,'',0,55,'N','N','N','Template','Overrides the default behavior with this template. The filename column should have the complete path from your private root master templates directory.','','','','')
        ";
        $dbh->Execute($sql);
        // get ID of created field and give new field basic privs
        $sql = "SELECT id FROM typesfields WHERE identifier = 'template' AND typeid = 9";
        $id = $dbh->GetOne($sql);
        
        //oam: added check to make sure field_priv doesn't already exist
        $sql = "SELECT * from field_priv where fid=$id and ugid=0";
        $key_exists = $dbh->GetOne( $sql );
        
        if ( !empty( $key_exists ) )
        {
            $sql = "UPDATE field_priv SET priv_read=1, priv_write=1 WHERE fid=$id and ugid=0";
        }
        else 
        {
            $sql = "INSERT INTO field_priv (fid, ugid, priv_read, priv_write) VALUES ($id, 0, 1, 1)";
        }
        $dbh->Execute($sql);
    }
}

if (124 > $_pxdb_db_version)
{
    // Add stx_redirect for redirects
    $tables = $dbh->MetaTables();
    if (false === in_array('stx_redirects', $tables))
    {    
        $sql = "
CREATE TABLE `stx_redirects` (
  `id` int(10) unsigned NOT NULL auto_increment,
  `redirect` varchar(255) NOT NULL default '',
  `target` text NOT NULL,
  `root_section` varchar(255) NOT NULL default '',
  UNIQUE KEY `redirect` (`root_section`(64),`redirect`(64)),
  KEY `id` (`id`)
) TYPE=MyISAM;
        ";
        $dbh->Execute($sql);
    }
}

if (126 > $_pxdb_db_version)
{
    // Add stx_tags for redirects
    $tables = $dbh->MetaTables();
    if (false === in_array('stx_tags', $tables))
    {    
        $sql = "
        CREATE TABLE `stx_tags` (
  `record_id` int(11) NOT NULL,
  `tag` varchar(255) NOT NULL,
  UNIQUE KEY `record_id` (`record_id`,`tag`),
  KEY `record_id_2` (`record_id`)
) TYPE=MyISAM;
        ";
        $dbh->Execute($sql);
    }
}
// Once all updates have been done, update the preference to current version (so this script won't run again)
$pxdb_prefs->set_pref('version', PXDB_VERSION);

?>