<?php
/*
* Builds a hierarchical *looking* pick relationships based on 
* a picktable that has a column named 'parent' in it.
*
* initial assumptions about a hierarchical picklist
* - ordering is always by name field
* - node 1 is always the root node, has a parent of 0
*/

# IN: 	
#		$css_tag	Full CSS tag (e.g. 'class="error"' or 'style="font-family: Arial"')
#		$css_type	'style' vs. 'class'
#		$css_style	Style to use for the tag.
#		$db 		ADOdb connection
#		$params		Widget parameters
#
#		$form_name		[HTML] Form name.
#		$element_name	[HTML] Form element name (e.g. "pxdb[data][name]")
#		$name			Un-processed element name.
#		$id				Typesfield id (unique to this field).
#		$type			Type of field .... 
#       $multiple
#		etc, etc. -- see pxdb_input and pxdb_widget class.
#
# $plugin_info['params']['rows'] 
# OUT: $widget
pxdb_import( 'metadata.pxdb_typesfield' );
pxdb_import( 'metadata.pxdb_dbfield' );

$tf = new pxdb_typesfield( $id );
$mt = $tf->get_field_metatype();

ob_start();
if ( 'pickid' != $mt && 'pick' != $mt )
{
    trigger_error( 'Hierarchy_pick: this typefield does not appear to be a pick or pickid: '.$name, E_USER_WARNING );
    return false;
}

if ( !function_exists( 'hierpick_get_children' ) )
{
    function hierpick_get_children( $src, $parent )
    {
        $dbh =& pxdb::dbh();

        $sql = "select * from $src where parent='$parent' order by name";
        $rs = $dbh->GetAll( $sql );
        return $rs;        
    }
 
    // returns array of options that have children
    function hierpick_get_breeder_ids( $src )
    {
        $dbh =& pxdb::dbh();

        $sql = "select distinct parent from $src order by parent";
        $rs = $dbh->GetAll( $sql );
        return $rs;
    }
 
    function hierpick_draw_opt( $child, $passthru, $depth )
    {
        ob_start();
        
        $style = 'padding-left: '.(2*$depth).'em;';
        if ( 1 == $passthru[ 'multiple' ] )
        { // checkbox
            if ( !empty( $passthru['value'] ) && in_array( $child['id'], $passthru['value'] ) )
            {
                $sel=' checked ';
            } else {
                $sel = '';
            }
            
?><div style="<?=$style?>"><input type="checkbox" name="<?=$passthru['element_name']?>[]" value="<?=$child['id']?>"<?=$sel?>/> <?=$child['name']?>
        </div>

<?php
        } else {
        //radio button        
            if ( $child['id']==$passthru['value'] )
            {
                $sel=' selected ';
            } else {
                $sel = '';
            }        
?><div style="<?=$style?>"><input type="radio" name="<?=$passthru['element_name']?>[]" value="<?=$child['id']?>"<?=$sel?>/><?=$child['name']?>
        </div>

<?php
        }
        $c = ob_get_contents();
        ob_end_clean();
        return $c;
    }
    
    // recursive function to display the options for a level of the tree
    function hierpick_disp_opts( $pickid, &$children, $passthru, $depth=0)
    {        
        
        $my_kids = $children[ $pickid ];
        
        while( list( ,$child ) = each( $my_kids ) )
        {
            echo hierpick_draw_opt( $child, $passthru, $depth );
            
            //recursive call here
            if ( !empty( $children[ $child['id'] ] ) )
            {                
                echo hierpick_disp_opts( $child['id'], $children, $passthru, $depth+1 );
            }
        }
        
    }
    
}

$passthru['multiple'] = $multiple;
$passthru['element_name'] = $element_name;
$passthru['value'] = $value;

$dbf = new pxdb_dbfield( $tf->_fields['fieldid'] );
$source = $dbf->get_pick_table();

$breeders = hierpick_get_breeder_ids( $source );

while( list( ,$bred ) = each( $breeders ) )
{
    $children[ $bred['parent'] ] = hierpick_get_children( $source, $bred['parent'] );    
}
?>
<input type="hidden" name="<?=$passthru['element_name']?> value="">
<?=hierpick_disp_opts( 0, $children, $passthru );?>

<?php
$widget = ob_get_contents();
ob_end_clean();
?>
