<?php
// $Id: pxdbTest.php,v 1.1.1.1 2004/09/10 19:27:15 omerida Exp $

require_once("PHPUnit.php");
require_once("./configure.php");

// Test global vars
${$PXDB_GLOBAL_VAR}['foo'] = 'bar';
$test_global = 'foobar';

/**
* This PHPUnit test will test the pxdb class in pxdb.class.php
*/
class pxdbTest extends PHPUnit_TestCase
{
    var $dbh;

    function pxdbTest($name)
    {
        $this->PHPUnit_TestCase($name);
    }

    function testDbh()
    {
        // Set the value to the ADOdb Connection object created in setUp()
        pxdb::dbh($this->dbh);
        
        // Now get the database handle back
        $dbh = pxdb::dbh();

        // Test this is a database object
        $expected = 'adodb_'.ADODB_DRIVER;
        $this->assertEquals(get_class($dbh), 
                            $expected);
    }

    function testTpl()
    {
        require_once(SMARTY_DIR."/Smarty.class.php");
        // Set the value to the Smart template engine instance
        pxdb::tpl(); // this should create a new Smarty instance

        // Now get the new Smarty instance handle.
        $tpl = pxdb::tpl();

        // Test this is a Smarty object
        $this->assertEquals(get_class($tpl), 'smarty');
    }

    function testImportVar()
    {
        $result = pxdb::import_var('test_global');
        
        $this->assertEquals($result, 'foobar');
    }

    function testImport()
    {
        // Import the default global var idenitified by $PXDB_GLOBAL_VAR
        pxdb::import();
        $result = pxdb::import();
        
        $this->assertEquals($result['foo'], 'bar');
    }

    function testGetPxdbTables()
    {
        $tbls = pxdb::get_pxdb_tables();

        $expect = array('datatypes',
                        'dbfields',
                        'field_priv',
                        'noderel',
                        'pickrel',
                        'primaryrel',
                        'priv',
                        'pxdb_prefs',
                        'records',
                        'relationships',
                        'secondaryrel',
                        'typesfields',
                        'usergroup',
                        'viewfields',
                        'views',
                        'weights');

        $diffs = array_diff($tbls, $expect);
        $this->assertTrue(0 == count($diffs));
    }

    function testGetNoPxdbTables()
    {
        $tbls = pxdb::get_nonpxdb_tables();

        $expect = array('category',
                        'country',
                        'datatypesrel',
                        'documenttypes',
                        'filetypes',
                        'organizationtypes',
                        'state');

        $diffs = array_diff($tbls, $expect);
        $this->assertTrue(0 == count($diffs));                
    }

    function testPickOrderby()
    {
        $result = pxdb::get_pick_orderby('country', 'id');
        $this->assertEquals($result, 'id');
    }

    function testIsTableCreatedTrue()
    {
        $this->assertTrue(true == pxdb::is_table_created('documenttypes'));
    }

    function testIsTableCreatedFalse()
    {
        $this->assertTrue(false == pxdb::is_table_created(uniqid('test_pxdb')));
    }

    function testCreatePickTable()
    {
        $db = pxdb::dbh();
        $table_name = uniqid('test_pxdb');
        $success = pxdb::create_pick_table($table_name);

        if ($success)
        {
            $result = pxdb::is_table_created($table_name);
            $db->Execute("DROP TABLE $table_name");
        }
        else
        {
            $result = false;
        }

        $this->assertTrue($result);
    }

    function testGetPxdbReservedFields()
    {
        $results = pxdb::get_pxdb_reserved_fields();
        
        $expected = array('approved',
                          'created',
                          'datatype',
                          'id',
                          'modified');
        $diff = array_diff($results, $expected);
        
        $this->assertTrue(0 == count($diff));
    }

    function testPluginExists()
    {
        $result = pxdb::plugin_exists('generators', 'userid');
        $this->assertTrue($result);
    }

    function testGetPxdb_www()
    {
        $results = pxdb::get_pxdb_www();
        if ($results[0]==='/pxdb_www' && $results['db.dev.pxdb.org'] === '/pxdb_www')
        {
            $result = true;
        }
        else
        {
            $result = false;
        }
        
        $this->assertTrue($result);   
    }

    function testRegisterClass()
    {
        $this->assertTrue(pxdb::register_class('my_auth', 'pxdb_auth'));    
    }

    function testGetClass()
    {
        $this->assertEquals(pxdb::get_class('pxdb_auth'), 'my_auth');
    }

    function testParsePlugin()
    {
        $plugin_data = "plugin-name::width=100;height=200";
        $plug = pxdb::parse_plugin($plugin_data);

        if (($plug[script]=='plugin-name') &&
            ($plug[params][width]==100) && 
            ($plug[params][height]==200) &&
            ($plug[paramstr]=='width=100;height=200'))
        {
            $result = true;
        }
        else
        {
            $result = false;
        }

        $this->assertTrue($result);
    }
}
