<?php
require_once(dirname(__FILE__).'/documentInfo.php');
/**
* Accessory class for use in debugging documentInfo.  Provides extra methods to allow you to set or get things not necessary for production use.
*/
class documentInfo_debug extends documentInfo
{

/** 
* Debugging use: Return an array of the methods that were tried while attempting to determine filetype.  'FAILED' will be the last element if methods were exhausted without returning a filetype.  A realistic example might be 'head', 'extension', 'FAILED'.  Returns FALSE if none have been tried (get_filetype() has not yet been called).
* @param void
* @return arrray/boolean array of methods attempted, or FALSE if none have been used
* @access public
* @see get_filesize
*/
function get_filetype_methods_used()
	{
		if (isset($this->filetype_methods_used))
			return $this->filetype_methods_used;
		else
			return false;
	} // method


/** 
* Debugging use: Return an array of the methods that were tried while attempting to determine filesize.  'FAILED' will be the last element if methods were exhausted without returning a filetype.  A realistic example might be 'head', 'download', 'FAILED'.  Returns FALSE if none have been tried (you haven't yet called get_filesize().
* @param void
* @return arrray/boolean array of methods attempted, or FALSE if none have been used
* @access public
* @see get_filesize
*/
function get_filesize_methods_used()
	{
		if (isset($this->filesize_methods_used))
			return $this->filesize_methods_used;
		else
			return false;
	} // method

/**
* Debugging use: return the head of the document that has been fetched with a remote function.
* Note that this function will not cause the remote head to be fetched -- just
* return it if we already have it.
* @param void
* @return string/boolean head of the remote document, FALSE if it doesn't exist.
* @access public
*/
function get_file_head()
	{
		if (isset($this->file_head))
			return $this->file_head;
		else return false;
	}
	
/**
* FOR DEVELOPER USE. Returns a multidimensional array of all the different filetypes we've defined.  FALSE if that array is empty.  This is really intended to get this valuable list out of this app and into somewhere else.
* @param void
* @return array/boolean  see the documentation for $filetype_array to see what you'll be getting
* @access public
* @see filetype_array (variable)
*/
function get_all_filetypes()
	{
		if (count($this->filetypes_array)>0)
			return $this->filetypes_array;
		else return false;
	}

/**
* Debugging: Returns the remainder of the get_extension function -- whatever remains beyond extension.
* @param void
* @return string/boolean contents if exists, FALSE if not
* @access public
*/
function get_post_extension()
	{
		if (isset($this->post_extension))
			return $this->post_extension;
		else return false;
	}
/**
* Debugging: Return the actual value of $file.  For local files, this may be the true path on the local filesystem, not what was entered in set_file().  Do not use for public display!
* @param void
* @return string $file the true filepath
* @access public
*/
function get_file()
	{
		return $this->file;
	}

} //class
?>