<?php
/**
* Get the HTTP head of a document, which may then be used for Content-Length and Content-Type.
* @author Jason Roberts
* @version 0.5
*/
class fetchRemoteHead
{
/**
* Maximum number of characters of the head to fetch.
* @access private
*/
var $max_head_length;

/**
* Maximum amount of time before fetch_fetchRemoteHead will timeout
* @access private
*/
var $timeout;

/**
* Actual amount of time taken by fetch_fetchRemoteHead.
* @access private
*/

var $elapsed_time;

/**
* Constructor, set default values.
* @access public
* @return void
*/
function fetchRemoteHead()
	{
		$this->set_max_head_length(6000);
		$this->set_timeout(5);
	}

/**
* Set the maximum number of charcters of the head to fetch.
* @param integer $length length expressed in bytes
* @access public
* @return void
*/
function set_max_head_length($length)
	{
		$this->max_head_length=$length;
	}

/**
* Set the maximum time (in seconds) allowed for the complete operation.
* Because there is a problem with how the timeout on fsockopen() works
* (does not include time for DNS lookup), it is possible that the operation
* can take longer than the specified timeout.
* @param float $seconds
* @access public
* @return void
*/
function set_timeout($seconds)
	{
		$this->timeout = $seconds;
	}

/**
* Parse a url and return various parts.  Currently, returns 'host' and 'path'.  'path' is defined to include ANYTHING that follows the host.
* @param string $url
* @return array | 'host'=>host, 'path'>path
* @access private
*/
function get_url_parts($url)
		{
			$url_parts=parse_url($url);
			$regex_string="/".$url_parts['host']."(.*)/i";
			preg_match($regex_string, $url, $matches); // will actually get path + query string (i hope)
			$parts['path']= $matches[1];
			if ($parts['path']=='') $parts['path']='/'; // to deal with 'http://www.example.com'
			$parts['host'] = $url_parts['host'];
			return $parts;
		}
/**
* Make a socket connection to the remote server and fetch the HEAD of the file.  Uses the value set in set_max_head_length to determine the number of bytes.  Uses the value of set_timeout to determine how long we're willing to wait for the entire operation.  Because the timeout on fsockopen() does not seem to work, the actual time may exceed the timeout.
* @param string $url the file you're interested in.
* @param float $timeout OPTIONAL timeout, otherwise uses the default or value set
* in set_timeout()
* @return string/boolean the remote head, or FALSE if fails
* @access public
*/

function get_remote_head($url, $timeout=NULL)
	{
		if (!isset($timeout)) $timeout = $this->timeout;
		if ($timeout == 0)
			return false;
		else
		{
			$url_parts = $this->get_url_parts($url);
			$file_host=$url_parts['host'];
			$file_path=$url_parts['path'];

			// make the connection, receive errors by reference
			$start_time = timeLibrary::get_microtime();
			$fp = fsockopen($file_host, 80, &$errno, &$errstr, $timeout);
			// see how long it took to open & record
			$elapsed_time = timeLibrary::get_interval($start_time);
			if($fp && $elapsed_time < $timeout)
			{
				// get fetch timeout based on time remaining, use function to split into seconds/usecs
				$fto = timeLibrary::split_time($timeout - $elapsed_time);
				socket_set_timeout($fp, $fto['s'], $fto['us']);
				$msg = "HEAD $file_path HTTP/1.0\nHost:$file_host\n\n";
				// this fputs should be nearly instantaneous, since it does not rely on the remote server. it's just us sending the message
				fputs($fp,$msg);
				$response=fread($fp, $this->max_head_length);
				fclose($fp);
			}
			else; // what do we do if there's an error?  for now that will just cause the whole thing to return false, which is fine.
			$elapsed_time = timeLibrary::get_interval($start_time);
			$this->elapsed_time = $elapsed_time;
			if (!empty($response))
				return $response;
			else return false;
		} // end if $timeout = 0
	} // end function

/**
* Debugging use.  Return the total amount of time taken by fetch_fetchRemoteHead.  This will either be however long it needed, or whatever it was constrained to by the timeout value.  Since the fsockopen() timeout does not seem to currently work, it may be more than the timeout value.  :(
* @param void
* @access public
*/
function get_elapsed_time()
	{
		return $this->elapsed_time;
	}
} // end class

?>