<?php
/**
* Methods for doing various time calculations not included in default PHP installation.
*/
class timeLibrary
{
/**
* Return current time expressed in unix time with decimal precision to 1 microsecond.  If the optional $time parameter is passed, uses that instead of now().
* @param string $time optional: time expressed as usec sec
* @return float $time time (seconds) with decimal precision to 1 microsecond
* @access private
*/
function get_microtime($time=NULL)
	{
		if (!isset($time)) $time=microtime();
		list($usec, $sec) = explode(" ",$time);
		return ((float)$usec + (float)$sec);
	}


/**
* Return the number of seconds elapsed since the specified start time.  Return is in seconds with decimals.
* @param float $start the time (unix time) to measure since
* @param float $finish optional: the ending time
* @return float $elapsed the elapsed time, in seconds with decimals
* @access private
*/
function get_interval($start, $finish=NULL)
	{
		if (!isset($finish)) $finish=timeLibrary::get_microtime();
		$elapsed_time = $finish - $start;
		return $elapsed_time;
	}

/**
* Take a decimal time in seconds, and return it split into an array seconds and microseconds.
* @param float $time
* @return array $time | 's'=>seconds, 'us'=>microseconds
* @access private;
*/
function split_time($time)
	{
		$seconds = floor($time);
		$microseconds = round(($time-$seconds)*1000000);
		return (array('s'=>$seconds, 'us'=>$microseconds));
	}
} // end class