<?php
/**
* This class supplants part of the old Request class to parse the current URL
* for various bits of information.
*
* @author Sandy Smith <ssmith@forumone.com>
* @date 4/17/03
* @access public
* @package SyntaxCMS
*/
class PathInfo
{

    /**
    * @var string type of request being made in the URL ('section' or 'module')
    * @access private
    */
    var $requestType;
    
    /**
    * @var string Name of the current request (last item in the URL)
    * @access private
    */
    var $currentRequest;
    
    /**
    * Constructor. Parses $_SERVER['REQUEST_URI'] and stores the info.
    *
    * @access public
    */
    function PathInfo()
    {
        // strip args
        $path = preg_replace('/\?.*$/', '', $_SERVER['REQUEST_URI']);
        // ditch the ending /
    	$path = preg_replace('/\/$/', '', $path);
    	// ditch the beginning /
    	$path = preg_replace('/^\//', '', $path);
    	// build an array
        $path_items = split('/', $path);
        
        // first part of the URL
        $this->_setRequestType($path_items[0]);
        
        // last part of the URL
        $last_path_item = count($path_items) - 1;
        $this->_setCurrentRequest($path_items[$last_path_item]);
    }
    
    /**
	* Instantiate one copy of this object.
	* @access public
	* @return object PathInfo
	*/
	function &singleton()
	{
		static $instance;
		if (!isset($instance))
		{
			$instance = new PathInfo();
		}
		return $instance;
	}
	
	/**
	* sets the request type
	*
	* @access private
	* @param string The request type
	*/
	function _setRequestType($type)
	{
		$this->requestType = $type;
	}
	
	/**
	* Returns the request type
	*
	* @access public
	* @return string The Request type
	*/
	function getRequestType()
	{
		return $this->requestType;
	}
	
	/**
	* Sets the current request
	*
	* @access private
	* @param string The current request
	*/
	function _setCurrentRequest($request)
	{
		$this->currentRequest = $request;
	}
    
    /**
    * Returns the current request (section or module url_id)
    *
    * @access public
    * @return string url_id of the current request
    */
    function getCurrentRequest()
    {
    	return $this->currentRequest;
    }
    
    /**
    * Tests whether a given string is the current request (section or module)
    *
    * @access public
    * @return boolean
    * @param string url_id to be tested
    */
    function isCurrentRequest($url_id)
    {
    	return ($url_id == $this->getCurrentRequest()) ? true : false;
    }
    
    /**
    * Tests whether a url_id is in the current path or not. Used mainly with sections.
    *
    * @access public
    * @return boolean
    * @param string url_id of section or module to be tested.
    */
    function isInPath($url_id)
    {
    	return (stristr($_SERVER['REQUEST_URI'], '/' . $url_id . '/') or preg_match('/\/' . $url_id . '$/', $_SERVER['REQUEST_URI'])) ? true : false;
    }
    
    /**
    * Tests if a url_id is the immediate parent of the current section
    *
    * @access public
    * @return boolean
    * @param string url_id of section to be tested.
    */
    function isParentOfCurrentRequest($url_id)
    {
    	return (preg_match('/' . $url_id . '\/' . $this->getCurrentRequest() . '/', $_SERVER['REQUEST_URI'])) ? true : false;
    }
}
?>