<?Php

/**
* This is a skeleton class for a class that represents a record of type Section
* You can use this in place of using the pxdb_record class directly.
*
* Provided are some rough guesses of methods you might want.  Customize & add as needed!
*
* Note that this class uses Camels Hump capitalization so that methods of this
* class can be easily distinguished from the all-lowercase methods of the parent
* classes.
*
* This class does not handle writing data to a record.  You could add this if you wanted,
* but that is not part of the pxdb_record functionality -- that is handled by the pxdb_commit
* (and often pxdb_input for form building) classes.
*
* This class provides methods for accessing record values.  You can use this class in 
* conjunction with the pxdb_collection class by either
* 
* 1) using the auto-generated SectionCollection class, which will return this object
* when you invoke the fetchRecord() method.  or
*
* 2) specify this object when you fetch a record from  pxdb_collection:
*	while($rec = &$content->fetch_record('Section')) { 
*		print $rec->getName(); // e.g., not necessary a method of this class.
*	}
*
* @version 1.1
* @author DBasis :-)
* @author Sandy Smith <ssmith@forumone.com>
* @author Oscar Merida <omerida@forumone.com>
* @package SyntaxCMS
*/
class Section extends pxdb_record
{
    /**
    * Public navigation path for the section
    */
    var $_navPath;
    
	/**
	* Constructor. 
	* Calls parent constructor.
	* @param mixed $data Record ID or record row (array)
	*/
	function Section($data)
	{
		parent::pxdb_record($data);
	}
	
	/**
	* Get the value for field `url_id`.
	* @return mixed Value or array of values.
	*/
	function getUrlId()
	{		
		return $this->get_field('url_id');				
	}
	
	/**
	* Get the value for field `crumb`.
	* @return mixed Value or array of values.
	*/
	function getCrumb()
	{		
		return $this->get_field('crumb');				
	}
	
	/**
	* Get the value for field `name`.
	* @return mixed Value or array of values.
	*/
	function getName()
	{		
		return $this->get_field('name');				
	}
	
	/**
	* Get the value for field `description`.
	* @return mixed Value or array of values.
	*/
	function getDescription()
	{		
		return $this->get_field('description');				
	}
	
	/**
	* Get the value for field `content`.
	* @return mixed Value or array of values.
	*/
	function getContent()
	{		
		return $this->get_field('content');				
	}
	
	/**
	* Get the value for field `grouping`.
	* @return mixed Value or array of values.
	*/
	function getGrouping()
	{		
		return $this->get_field('grouping');				
	}
	
	/**
	* Get the value for field `weight`.
	* @return mixed Value or array of values.
	*/
	function getWeight()
	{		
		return $this->get_field('weight');				
	}

	/**
	* Get the value for field `meta_keywords`.
	* @return mixed Value or array of values.
	*/
	function getMetaKeywords()
	{
		return $this->get_field('meta_keywords');
	}

	/**
	* Get the value for field `meta_description`.
	* @return mixed Value or array of values.
	*/
	function getMetaDescription()
	{
		return $this->get_field('meta_description');
	}

    /**
    * Get the value for field `url_path`.
    * @return string Path from top-level Section to this one.
    */
    function getTemplate( $field = 'filename')
    {
        return $this->get_field('template', $field);
    }
    
	/**
	* Get the value for field `url_path`.
	* @return string Path from top-level Section to this one.
	*/
	function getUrlPath()
	{
		return $this->get_field('url_path');
	}
		
    /**
    * WARNING: This function no longer works, get rid of it in your code.
    * @deprecated
    */
    
    function getByUrlId( $urlid, $show_unapproved = FALSE )
    {
        trigger_error('This function no longer works as of Syntax CMS 1.2.0.  Use getByUrlPath() instead.', E_USER_WARNING);
        return false;
        /*
        $filter = &pxdb_search::filter(DATATYPE_SECTION);   
        $filter->add_value('idnum', $urlid );
        
        $sections  = &new pxdb_collection(DATATYPE_SECTION);
        $sections->set_show_unapproved( $show_unapproved );
        $sections->find($filter);
        
        $sections->execute();
        
        return $sections->fetch_record();
        */
    }
		
    /**
    * Get a section by its URL path
    * @access public
    * @param string $url_path Full section path from top node to the section
    * @pa
    * @return object pxdb_record
    * @author Sandy Smith <ssmith@forumone.com>
    */
    function getByUrlPath( $url_path , $show_unapproved=FALSE)
    {
        pxdb_import( 'content.output.pxdb_search' );
        pxdb_import( 'content.output.pxdb_collection' );
        $filter = &pxdb_search::filter(DATATYPE_SECTION);   
        $filter->add_value('url_path', $url_path);
        
        $sections  =& new pxdb_collection(DATATYPE_SECTION);
        if ( true == $show_unapproved )
        {
            $sections->set_show_unapproved( TRUE );
        }
        
        $sections->find($filter);
        $sections->execute();
        
        return $sections->fetch_record();
    }
    
    /**
    * Returns the navigation path for the section
    *
    * Internally caches the result to avoid multiple calls to SectionNavigator
    * 
    * @access public
    * @return string 
    */
    function getNavPath()
    {
        // get home
        if (empty($this->_navPath))
        {
            // OAM: let SectionNavigator handle this again since
            // it has the logic for testing if it is an external link or not. 
            $navigator =& SectionNavigatorOptimized::singleton();
            $this->_navPath = $navigator->getNavPath( $this );
        }
   
        return $this->_navPath;
    }
    
    /**
    * Uses SectionNavigator to return a pxdb_collection of the section's ancestors.
    *
    * @return pxdb_collection
    * @access public
    * @author Oscar Merida <omerida@forumone.com>
    */
    function &getAncestorCollection()
    {
        if ( empty( $this->ancestors ) )
        {
            require_once( SITE_LIB_PATH.'/syntaxcms/SectionNavigatorOptimized.php' );
            $nav = SectionNavigatorOptimized::singleton();
            $nav->setSection( $this );            
            $this->ancestors = $nav->getAncestorCollection();
        }
        
        return $this->ancestors;
    }
    
    /**
    * Returns the parent section object for the current section, if it has one.
    *
    * @return section object
    * @access public
    * @author Oscar Merida
    */
    function &getParentSection()
    {
        $url_path = explode( '/', $this->getUrlPath() );
        array_pop( $url_path );
        $url_path = join( $url_path, '/' );
        return Section::getByUrlPath( $url_path );
    }
    
    /**
    * Returns whether a section is a link type or not
    *
    * @access public
    * @return boolean
    */
    function isLink()
    {
        return (0 === strpos($this->getUrlId(), '*64_'));
    }
    
    /**
    * Checks if the user can read this section AND all of its ancestors
    *
    * @access public
    * @return boolean
    */
    function isAuthorized()
    {
        if ( false === $this->is_authorized() )
        {
            return false;
        }
        else
        {
            //check ancestors
            if  ( $ancestors = $this->getAncestorCollection() )
            {
                $ancestors->execute();
                while ( $ancestor = $ancestors->fetch_record() )
                {
                    if ( false === $ancestor->is_authorized() )
                    {
                        return false;
                    }                
                }
            }
            return true;
        }
        
    }
    
} // end Section class

?>