<?php

/**
* Provides a wrapper for the SectionNavigator optimizing the database calls
* for Syntax CMS distributions that use the NoderelID column in the noderel
* table.
*
* @author Sandy Smith <ssmith@forumone.com>
* @date 5/10/04
* @package SyntaxCMS
*/
class SectionNavigatorOptimized extends SectionNavigator
{
    
    /**
	* Instantiate one copy of this object.
	* @access public
	* @return object SectionNavigator
	* @param object pxdb_record
	*/
	function &singleton($section = null)
	{
		static $instance;
		if (!isset($instance))
		{
			$instance = new SectionNavigatorOptimized($section);
		}
		return $instance;
	}		
    
    /**
    * Controls wether we return hidden children sections
    *
    * @param bool True=return hidden sections ( default )
    * @access public
    */
    function setShowHiddenSections( $show = true )
    {
        $this->_showHiddenSections = $show;
    }
    
    /**
     * Generates the pxdb_collection for the getChildren method,
     * used to abstract it for overloading.
     *
     * @access private
     * @return object Reference to the pxdb_collection class containing the children
     * @param string $url_path The URL PATH* of the section *Change
     *
     */
    function &_getChildrenCollection($url_path = null )
    {
        if (null === $url_path)
        {
            trigger_error('_getChildrenCollection needs a url_path to be passed.', E_USER_WARNING);
            return false;
        }
             
        $sql = "
SELECT DISTINCT
     r.approved, r.created, r.datatype, r.id, r.modified, r.`idnum` AS `url_id`,
     r.`desc3` AS `url_path`, r.`title` AS `crumb`, r.`name`,
     r.`desc1` AS `description`, r.`desc2` AS `content`,
     r.`type` AS `grouping`, r.`weight`, r.`text1` AS `meta_keywords`,
     r.`text2` AS `meta_description`, r.`bool5` AS `members_only`,
     r.`bool1` AS `inherit_template`
FROM
    records r
WHERE
    r.desc3 LIKE '$url_path%'
    AND r.desc3 REGEXP '^$url_path/[^/]+$'";
    
    if (false === $this->_showHiddenSections) 
    {        
        $sql .= "
    AND LEFT(r.idnum,1)<>'_'";     
    }
    
$sql.="
    AND r.approved = 1						
ORDER BY
    r.weight
        ";
        $children =& new pxdb_collection(DATATYPE_SECTION);
        
        $children->find($sql);
        return $children;
    }
    
    /**
    * Gets a Section by its URL Path**, optimized for MySQL db performance
    * ** Changed from URL ID
    *
    * @access private
    * @return object pxdb_record Reference to a pxdb_record object or false on failure
    * @param string $url_path The URL PATH* of the section you're trying to get *Change
    */
    function &_getSectionByUrlPath($url_path)
    {
    	if (true === empty($url_path) || false === is_string($url_path))
    	{
    	   trigger_error(E_USER_WARNING, 'A URL ID string must be passed to _getSectionbyUrlId');
    	   return false;
    	}
        $dbh =& pxdb::dbh();
        $sql = "
SELECT DISTINCT
     r.approved, r.created, r.datatype, r.id, r.modified, r.idnum AS `url_id`,
     r.`desc3` AS `url_path`, r.`title` AS `crumb`, r.`name`,
     r.`desc1` AS `description`, r.`desc2` AS `content`,
     r.`type` AS `grouping`, r.`weight`, r.`text1` AS `meta_keywords`,
     r.`text2` AS `meta_description`, r.`bool5` AS `members_only`,
     r.`bool1` AS `inherit_template`
FROM
    records r
WHERE
    r.desc3 = '$url_path'
    AND r.datatype = " . DATATYPE_SECTION . "
    AND r.approved = 1						
LIMIT 1
        ";
        $array =& $dbh->GetRow($sql);
        
        $section = new pxdb_record($array);
        return $section;
    }
    
    
    /**
     * Abstraction method for implementation of getting the collection with a Parent in it.
     *
     * @access private
     * @return object pxdb_collection Reference to the PxDB collection containing the parent
     * @param string $url_path The URL ID of the section that is a child of the parent you want.
     *
     */
    function &_getParentCollection($url_path = null)
    {
    	if (null === $url_path)
    	{
    	    trigger_error('_getParentCollection requires URL ID to be passed', E_USER_WARNING);
            return false;
        }
        // get rid of last item so it starts from its parent
        $url_path = preg_replace("/\/[^\/]+$/", '', $url_path);
        $sql = "
SELECT DISTINCT
     r.approved, r.created, r.datatype, r.id, r.modified, r.idnum AS `url_id`,
     r.`desc3` AS `url_path`, r.`title` AS `crumb`, r.`name`,
     r.`desc1` AS `description`, r.`desc2` AS `content`,
     r.`type` AS `grouping`, r.`weight`, r.`text1` AS `meta_keywords`,
     r.`text2` AS `meta_description`, r.`bool5` AS `members_only`,
     r.`bool1` AS `inherit_template`
FROM
    records r
WHERE
    r.desc3 = '$url_path'
    AND r.approved = 1						
LIMIT 1
        ";
        
        $parents =& new pxdb_collection(DATATYPE_SECTION);
        $parents->find($sql);
        
        return $parents;
    }
}
?>