<?php 
/**
* Class to store and access attributes of the current instance.
*
* @author Sandy Smith
* @author Oscar Merida
* @package SyntaxCMS
*/

class StxPrefs 
{
    /**
    * Facets holds an array describing fields that span content types 
    * that are meant to be used for facetted classification.  It is
    * primarily used in modules/general/list.php
    */
    var $facets;
    
    /**
    * Instantiate one copy of this object.
    * @access public
    * @return object Site
    */
    function &singleton()
    {
        static $instance;
        if (!isset($instance))
        {
            $instance = new Site();
        }
        return $instance;
    }
    
    /**
    * Sets the facet array to use globally.
    * @access public
    * @static
    */
    function setFacets($facet_array)
    {
        $StxPrefs = &StxPrefs::singleton();
        $StxPrefs->facets = $facet_array;
    }
    
    /**
    * Returns the global facet array.
    * @access public
    * @static
    */
    function getFacets()
    {
        $StxPrefs = &StxPrefs::singleton();
        if (!isset($StxPrefs->facets))
        {
            trigger_error('Site facets property is not set', E_USER_WARNING);
            return false;
        }
        else if (empty($StxPrefs->facets))
        {
            trigger_error('Site facets property is empty', E_USER_WARNING);
            return false;
        }
        else
        {
            return $StxPrefs->facets;
        }
    }
}
?>