<?php

/**
* Truncates a block of text to the nearest end-of-sentence character after
* the character count (currently period, question mark, and exclamation point),
* or the whole string if it is too short.
* Ex: stx_truncate_to_sentence('Hello World! How are you?', 8) would return 'Hello World!'
* Ex: stx_truncate_to_sentence('Hello World! How', 80) would return 'Hello World! How'
*
* @author AVC
* @param string text
* @param int length
* @deprecated
*/
function stx_truncate_to_sentence($text, $length)
{
	return preg_replace('/(^.{'.$length.'}.*?[.?!]).+/is', '$1', $text);
}

/**
* Creates an a hreg tag given a url and its label
*
* @author OAM
* @param string url
* @param string link text
* @param string link title (optional)
* @deprecated
*/
function stx_create_link($url, $text, $title = null)
{
    if (null == $title)
    {
        $title = $text;
    }
    
    return '<a href="' . $url . '" title="' . htmlentities($title) . '">' . $text . '</a>';
}

/**
* Alias for stx_link
*
* @author OAM
* @param string url
* @param string link text
* @param string link title (optional)
* @deprecated
*/
function _link($url, $text, $title = null)
{
    return stx_create_link($url, $text, $title);
}

/**
* Formats text for display.
* This is for descriptions & longer text bodies.  Is smart
* about whether to use nl2br() or to simply display raw.
*
* @param string $text String to be formatted
* @return string Formatted $text
* @deprecated
*/
function stx_format_text($text)
{
    if (!preg_match("/<p[^>]*>|<br[^>]*>|<li[^>]*>|<table[^>]*>/i", $text))
    {
        $text = nl2br($text);
    }
    $text = str_replace('�', '&rsquo;', $text);
    return $text;
}

/**
* Alias for stx_text()
*
* @param string $text String to be formatted
* @return string Formatted $text
* @deprecated
*/
function _text($text)
{
    return stx_format_text($text);
}

/**
* Formats a timestamp into a date.
* @param integer $ts Unix Timestamp
* @param string $fmt Time formatting string
* @return string formatted date
* @deprecated
*/
function stx_format_timestamp($ts, $fmt = LOCALE_DATE_STD)
{
    if ($ts)
    {        
        return trim(strftime($fmt, $ts));
    }
    else
    {
        return null;
    }    
}

/**
* Alias for stx_date()
* @param integer $ts Unix Timestamp
* @param string $fmt Time formatting string
* @return string formatted date
* @deprecated
*/
function _date($ts, $fmt = LOCALE_DATE_STD)
{
    return stx_format_timestamp($ts, $fmt);
}

/**
* Formats time string (such as a SQL formatted date) into a standard date format.
* @param integer $time SQL Timestamp
* @param string $format Time formatting string
* @return string formatted date
* @deprecated
*/
function stx_format_date_string($time, $format = LOCALE_DATE_STD)
{
    return _date(strtotime($time), $format);
}

/**
* Alias for stx_sql_date()
* @param integer $time SQL Timestamp
* @param string $format Time formatting string
* @return string formatted date
* @deprecated
*/
function _sql_date($time, $format = LOCALE_DATE_STD)
{
    return stx_format_date_string($time, $format);
}

/**
* A sort of factory method for the Request object, that looks
* at a path and loads the correct Request module.  Very site-specicific
* (because of path conventions) -- so dont' want this to be in Request
* classes [yet].
* @param string $path
* @return object reference to new Request instance
* @deprecated
*/
function &stx_request_factory($path)
{
    $matches = array();
    
    if ( '/' == $path || preg_match("/\/?(section|content)(\/[^\$\?]+)/", $path, $matches))
    {        
        
        if (isset($matches[1]) && $matches[1] == 'section' || $path == '/') 
        {
            $_c = 'SectionRequest';
        }
        elseif ($matches[1] == 'content')
        {
            $_c = 'ModuleRequest';
        }
        
        include_once(SITE_LIB_PATH . "/syntaxcms/Request/$_c.class.php");
        $Request = &new $_c();
                
        //print  "<br>Setting path to $matches[2]";
        if ($path == '/')
        {
            $Request->setPath($path);
        }
        else
        {
            $Request->setPath($matches[2]);        
        }
        
    }
    else
    {
        return false;
    }

    return $Request;
}

/**
* Alias for &stx_request_factory()
* @param string $path
* @return object reference to new Request instance
* @deprecated
*/
function &_request_factory($path)
{
    $var = stx_request_factory($path);
    return $var;
}

/**
* Takes a Section object and looks for its template. Put here
* so the Section object can be regenerated if its properties change.
* Useful for inheriting templates from parents.
*
* @access public
* @return string 
* @param object Section
* @deprecated
*/
function stx_get_section_template(&$section)
{    
    $section_navigator =& SectionNavigatorOptimized::singleton();
    $section_navigator->setSection($section);
    
    // enable template inheritance
    while (false === $section_navigator->isHomeSection() 
            && 1 == $section->get_field('inherit_template'))
    {
        $section =& $section_navigator->getParent();
        $section_navigator->setSection($section);
    }
    
    // home section is handled specially as usual
    if (false === $section_navigator->isHomeSection() || NAV_ROOT_EL != $section->get_field('url_path'))
    {
        $section_specific_template =  preg_replace("/\/$/", '', $section_navigator->getNavPath()) . '.tpl';
    } 
    else 
    {     
        $section_specific_template = '/section/' . NAV_ROOT_EL . '.tpl';
    }
    
    if (file_exists(SITE_TPL_PATH . $section_specific_template))
    {
        $template = $section_specific_template;
        if (DEBUG)
        {
            Log::entry(F1_DEBUG_LOG, 'section.php', __FILE__, __LINE__, "Found section-specific template: $template");
        }
    }
    else
    {
        $template = '/section.tpl';
        if (DEBUG) 
        {
            Log::entry(F1_DEBUG_LOG, 'section.php', __FILE__, __LINE__, "Using generic template: $template");
        }
    }
    return $template;
}

/**
* Alias for stx_get_section_template
*
* @access public
* @return string 
* @param object Section
* @deprecated
*/
function get_section_template(&$section)
{
    return stx_get_section_template($section);
}

/**
* Finds the last date of a numerically given month and year
*
* @access public
* @return int last date of given month
* @param int $month to have last date found for
* @param int $year to make sure leap year is calculated correctly
* @author Sandy Smith <ssmith@forumone.com>
* @deprecated
*/
function stx_find_last_date($month, $year)
{
    if (! is_integer($month) || ! is_integer($year))
    {
        trigger_error('You must pass integers for month and year.', E_USER_WARNING);
        return false;
    }
    if (12 < $month || 1 > $month)
    {
        trigger_error('Your month is out of range.', E_USER_WARNING);
        return false;
    }
    $string_month = (string) $month;
    if (4 <> strlen($string_month))
    {
        trigger_error('You must pass a 4-digit month', E_USER_WARNING);
        return false;
    }
    $month = (int) $month;
    $last_dates = array(
        1 => 31,
        2 => (0 == $year % 4 && (0 != $year % 100 || 0 == $year % 400)) ? 29 : 28,
        3 => 31,
        4 => 30,
        5 => 31,
        6 => 30,
        7 => 31,
        8 => 31,
        9 => 30,
        10 => 31,
        11 => 30,
        12 => 31
   );
    
    return $last_dates[$month];
}

/**
* Alias for stx_find_last_date()
*
* @access public
* @return int last date of given month
* @param int $month to have last date found for
* @param int $year to make sure leap year is calculated correctly
* @deprecated
*/
function find_last_day($month, $year)
{
    return stx_find_last_date($month, $year);
}

/**
* Encodes ASCII text using numeric entities.
* Useful for obfuscating e-mail addresses to
* prevent them from being harvested by spambots.
* Note--this will not work well with entities or non-ASCII characters.
*
* @access public
* @return string obfuscated text
* @param string $text Plain ASCII text to be obfuscated
* @deprecated
*/
function stx_obfuscate_ascii($text)
{
    $text = (string) $text; // make sure it's a string
    for ($i = 0; $i < strlen($text); $i++)
    {
        $decimal = ord($text{$i});
        $obfuscated_ascii .= "&#$decimal;";
    }
    return $obfuscated_ascii;
}

/**
* Alias for stx_obfuscate_ascii()
*
* @access public
* @return string obfuscated text
* @param string $text Plain ASCII text to be obfuscated
* @deprecated
*/
function obfuscate_ascii($text)
{
    return stx_obfuscate_ascii($text);
}

// ***********************************************************************
//
// is_valid_email(): an e-mail validation utility routine
// Version 1.1.1 -- September 10, 2000
//
// Written by Michael A. Alderete
// Please send bug reports and improvements to: <michael@aldosoft.com>
//
// This function matches a proposed e-mail address against a validating
// regular expression. It's intended for use in web registration systems
// and other places where the user is inputting their e-mail address and
// you want to check that it's OK.
//
// It does NOT attempt to prove that the address exists, only that it
// COULD exist. Validators which look up the host, try to validate an MX
// record, etc. can significantly slow the response time of your
// application; network connections are significantly more expensive than
// regular expressions, especially on days when the Internet is slow.
//
// It also doesn't attempt to ensure that the TLD is a known valid TLD.
// Doing a lookup against a table isn't too expensive, performance-wise,
// but given the explosion in Internet usage, keeping a lookup table
// up-to-date would be problematic.
//
// The validating regular expression is significantly more accurate than
// many others I've seen in example code. It still won't prevent users
// from entering bogus entries like "a@b.com", but then, if the user
// really doesn't want to give your their e-mail address, there's nothing
// you can do about it, other than make a confirmation e-mail a
// requirement to registering.
//
// You should clean up the input e-mail address in your code before you
// validate it, with trim() and strtolower(), if you're planning to save
// it into a database or otherwise preserve it beyond the one-time use.
// The validator doesn't depend on it, but you'll want to save clean data!
//
// Example of usage: (assumes the user entry form variable is $f_email)
//
//    $f_email = strtolower(trim($f_email));
//    if (is_valid_email($f_email)) {
//           // Do database save here
//           // Or send them a confirmation e-mail
//    }
//
// Implementation notes:
//
// The routine uses the Perl Compatible Regular Expressions (pcre) routine
// preg_match(), which my benchmarking indicates is about twice as fast
// as the standard ereg() routines in this usage (10 seconds vs. 21
// seconds to do 50,000 validations). If your installation of PHP doesn't
// have the pcre package compiled in, you can switch to using the
// is_valid_email_eregi() version, also provided.
//
// Notes about the expression:
//   - The user name needs to be pretty tolerant; you'd be amazed at
//     what can be in this portion of a valid address
//   - The expression allows an unlimited number of sub-domains, e.g.:
//     user@my.really.deep.sub.domain.com is allowed
//   - The only four-character TLD that's valid is "arpa", but the
//     expression doesn't know that, and allows any four-character TLD.
//
// ***********************************************************************
function is_valid_email($address)
{

    return (preg_match(
        '/^[-!#$%&\'*+\\.\/0-9=?A-Z^_`{|}~]+'.   // the user name
        '@'.                                     // the ubiquitous at-sign
        '([-0-9A-Z]+\.)+' .                      // host, sub-, and domain names
        '([0-9A-Z]){2,4}$/i',                    // top-level domain (TLD)
        trim($address)));
}

/**
* takes an image, finds the width and height, allows you to set a maximum width or height
* and returns an HTML img tag. Only works on local images.
*
* @access public
* @param string $url The local URL (everything after the domain)
* @param array $restrict Associative array. Valid keys are 'by' (string 'width' or 'height') and 'max' (int)
* @param string $class CSS class
* @param string $alt Alt text
* @return string The img tag.
* @deprecated
*/
function stx_get_image_link($url, $restrict = array('by' => 'width', 'max' => '200'), $class = null, $alt = null)
{
    $fullpath = SITE_PATH.$url;
    $info = getimagesize($fullpath);

    $dim_check = ($restrict['by'] == 'width') ? 0 : 1;
    $dim_label = ($restrict['by'] == 'width') ? 'width' : 'height';

    if ($info[$dim_check] > $restrict['max'] ) 
    {
        $img = '<img src="' . $url . '" ' . $dim_label . '="' . $restrict['max'] . '"';
        
    }
    else
    {
        $img = '<img src="' . $url . '" ' . $info[3];
    } // end if
    
    if ($class)
    {
        $img .=' class="' . $class . '" ';   
    }
    
    if (null !== $alt)
    {
        $img .= ' alt="' . htmlentities($alt) . '" ';
    }
    return $img.'>';
}

/**
* Alias for stx_get_image_link
*
* @access public
* @param string $url The local URL (everything after the domain)
* @param array $restrict Associative array. Valid keys are 'by' (string 'width' or 'height') and 'max' (int)
* @param string $class CSS class
* @param string $alt Alt text
* @return string The img tag.
* @deprecated
*/
function get_image_link($url, $restrict = array('by' => 'width', 'max' => '200'), $class = null, $alt = null)
{
    return stx_get_image_link($url, $restrict, $class, $alt);
}

/**
* single function call for getting a listWidget object based on 
* a Syntax pick table object.
*
* @param string, name of list widget
* @param string, name of pick table
* @param array, optional array of values to prepend to options array
* @return object ListWidget
* @access public
* @deprecated
*/
function &stx_list_from_picktable($name, $picktable, $prepend = null)
{
    pxdb_import('content.output.pxdb_picktable');
    
    $pickstable = new pxdb_picktable($picktable);
    $picks = $pickstable->get_listwidget_array();
    
    if (null !== $prepend)
    {
        array_unshift($picks, $prepend);
    }
    
    return new ListWidget($picks, $name);
}

/**
* alias for &stx_list_from_picktable()
*
* @param string, name of list widget
* @param string, name of pick table
* @param array, optional array of values to prepend to options array
* @return object ListWidget
* @access public
* @deprecated
*/    
function &list_from_picktable($name, $picktable, $prepend = null)
{
    return stx_list_from_picktable($name, $picktable, $prepend);
}

/**
* single function call for getting a listWidget object based on 
* a Syntax pick table object.
*
* @param string, name of list widget
* @param string, name of pick table
* @param array, optional array of values to prepend to options array
* @return object ListWidget
* @access public
* @deprecated
*/
function &stx_list_from_picktable_tree($name, $picktable, $prepend = null, $depth_label = null)
{
    pxdb_import('content.output.pxdb_picktable_tree');
    
    $pickstable = new pxdb_picktable_tree($picktable);
    $picks = $pickstable->get_listwidget_array(null, $depth_label);
    
    if (null !== $prepend)
    {
        array_unshift($picks, $prepend);
    }
    
    return new ListWidget($picks, $name);
}

/**
* Alias for &stx_list_from_picktable_tree()
*
* @param string, name of list widget
* @param string, name of pick table
* @param array, optional array of values to prepend to options array
* @return object ListWidget
* @access public
* @deprecated
*/    
function &list_from_picktable_tree($name, $picktable, $prepend = null, $depth_label = null)
{
    return stx_list_from_picktable_tree($name, $picktable, $prepend, $depth_label);
}

/**
* Calculates a sort direction for a field.
*
* @param string $field FIeld to sort by
* @param object $Request the request object
* @return string The SQL direction for sorting
* @deprecated
*/
function stx_get_sort_direction($field, &$Request)
{
    if ($Request->getVar('orderby_field') == $field)
    {		
        if ($Request->getVar('orderby_direction') == 'ASC')
        {
            return 'DESC';
        }
        else
        {
            return 'ASC';
        }
    }
    else
    {
        return 'ASC';
    }			
}

/**
* Alias for stx_get_sort_direction()
*
* @param string $field FIeld to sort by
* @param object $Request the request object
* @return string The SQL direction for sorting
* @deprecated
*/
function _direc($field, &$Request)
{
    return stx_get_sort_direction($field, $Request);
}

/**
* Tells whether a link is external to the current site
*
* @param string $url the url to test
* @return bool true if external, false if on this site
* @deprecated
*/
function stx_is_url_external($url)
{
    return (false !== strpos($url, '://')) && (false === strpos($url, $_SERVER['HTTP_HOST']));
}

?>