<?php


/**
* This is a collection class for of type AdBanner.
* This class extends pxdb_collection and provides object-specific methods which invoke
* parent generic methods.  This class provides some methods you might find useful --
* and more importantly, it provides a base from which you can build your own.
*
* IMPORTANT: If you change your object definition this class may no longer be applicable.
* So make sure your object is stable before implementing this class in your code.
*
* Note that this class uses CamelHump capitalization so that methods of this
* class can be easily distinguished from the all-lowercase methods of the parent
* classes.
*
* Here's an example of how to use these classes (not based on this class):
*
* $myDocs = new DocumentList();
* $myDocs->findByAuthor($user->get_uid()); // current logged-in user's id.
* 
* while($doc = &$myDocs->fetchRecord()) { 
*	// fetchRecord() returns a Document object --
*	// which extends the pxdb_record object and provides individual methods
*	// for getting attributes. 
*	print "<br>".$doc->getTitle();
* }
*
* If you need to only get a subset or a paged version of the query, you will 
* want to execute the query manually (otherwise default execute() is run when 
* you fetchRecord()).
*
* $myDocs->execute_page($rows_per_page, $current_page);
* while($doc = &$myDocs->fetchRecord()) { 
* ...
* OR
* $myDocs->execute_limit($num_rows);
* while($doc = &$myDocs->fetchRecord()) { 
* ...
*
* Remember also that the ADOdb recordset is returned from the ->execute() method.
* $rs = &$myDocs->execute();
* print "Rows found: ".$rs->RecordCount();
* while(!$rs->EOF) {
*	print "Document name: ".$rs->fields['name'];
*	$rs->MoveNext();
* }
*
* @version 1.1
* @author DBasis :-)
*/
class AdBannerCollection extends pxdb_collection
{

	/**
	* Constructor. 
	* Calls parent constructor and sets a default 'order by' value for the
	* results sets this class will produce.
	*/
	function AdBannerCollection()
	{
		parent::pxdb_collection(DATATYPE_AD_BANNER);
		// Default order will be the 'name' column
		// you can override this, of course
		$this->orderBy('name'); 
	}
	
	/**
	* Wrapper for parent's set_order method.
	* @param string $order Column to order by.
	*/
	function orderBy($order)
	{
		return parent::set_order($order);
	}
	
	/**
	* Calls parent fetch_record class, specifying to use the AdBanner class.
	* @return object AdBanner
	*/
	function &fetchRecord()
	{
        $record = &parent::fetch_record('AdBanner'); 
		return $record;
	}
	
	/**
	* Find records matching `size` pick list -- or finds all Ad Banner sorted by size.
	* If the $size parameter is left blank, it is assumed you want to find all of the 
	* Ad Banner records, organized (sorted) by size.
	* @param mixed $size Pick table `id` col -- or `name` col
	* @return void
	*/
	function findBySize($size=null)
	{
		if($size == null) {
			$this->orderBy('size, name');
			$this->find();
		} else {
			$pickFilter = &pxdb_search::filter(DATATYPE_AD_BANNER);
			$pickFilter->add_value('size', $size);
			$this->find($pickFilter);					
		}
	}
	
} // end AdBannerCollection class



?>
