<?php
// Include module-specific init file.
include(dirname(__FILE__) . '/init/init.php');

// check request trail for topic or archive 
if ( !$Request->isIncluded() && null !== $Request->getPathElement(2) )
{
    $path_el = strtolower($Request->getPathElement(2));
    switch ( $path_el ) 
    {
        case 'topic':
            pxdb_import( 'content.output.pxdb_picktable' );
            $topic = $Request->getPathElement(3);
            $table = &new pxdb_picktable('topics');
            $label = $table->lookup_label( $topic );
            
            $Request->addTrailPoint( array( 'crumb'=>$label ) );
            
            $topic = $Request->getPathElement(3);
            if ( !empty( $topic ) && is_numeric( $topic ) )
            {                
                $Request->setVar( 'topics', $topic );
            }
        break;
        
        case 'month':
            $date = $Request->getPathElement(3);
            $Request->addTrailPoint( array( 'crumb'=>strftime( LOCALE_DATE_MONTHYEAR, strtotime($date.'01') ). ' Archives' ) );
            
            if ( !empty($date) )
            {
                $Request->setVar( 'month', $date );
            }
        break;
    } // end switch    
}

if ( !$Request->isIncluded() )
{
    if ( null == $Request->getVar('page') )
    {
        $Request->setVar('page', 1);
    }
}
        
$filter =& pxdb_search::filter(DATATYPE_BLOG_POST);

// filter by sunrise/sunset (string is already quoted)
$filter->add_value('sunset', DB_TIME_STAMP, '>=', null, false);
$filter->add_value('sunrise', DB_TIME_STAMP, '<=', null, false);

// filter archives to month by sunset
if ( null !== $Request->getVar('month') )
{
    $month = $Request->getVar('month');
    if ( preg_match("/^[0-9]{6}$/", $month ) )
    {
        $start = date( 'Y-m-d', strtotime($month.'01' ) ).' 00:00';
        $end = date( 'Y-m-t', strtotime($start) ).' 23:59';
        
        $filter->add_value( 'sunrise', $start, '>=', null, true );
        $filter->add_value( 'sunrise', $end, '<=', null, true );
        
    }
}
// filter by Topics (pickid) if needed
if (null !== $Request->getVar('topics'))
{
    $filter->add_value('topics', $Request->getVar('topics'));
}        
            
// filter by section if needed
if (null !== $Request->getVar('in_section'))
{
    // add to main filter
    $filter->add_parent($Request->getVar('in_section'));
}

// create collection
$Records = new BlogPostCollection();

        
if ($Request->getVar('orderby'))
{
    $Records->set_order($Request->getVar('orderby'));
}
else
{
    // set a default order by, usually sunrise
    $Records->set_order('sunrise DESC');
} // end if

// bind the filter first
$Records->find($filter);

// limit recordset results
if (null !== $Request->getVar('limit'))
{
    if (false === $Records->execute_limit($Request->getVar('limit')))
    {
        trigger_error('Execute Limit failed', E_USER_WARNING);
    }    
} // paginate recordset
else if (null !== $Request->getVar('page') && defined('BLOG_POSTS_PER_PAGE'))
{    
    if (false === $Records->execute_page(BLOG_POSTS_PER_PAGE, $Request->getVar('page')))
    {
        trigger_error('Execute Page failed', E_USER_WARNING);
    }
// show all records    
}
else
{
    $Records->execute();
}

// generate output
if (!$Request->isIncluded())
{
    include_once(SITE_TPL_PATH . '/header.tpl');
} // end if included

// if there are no records do not output anything at all.
if (0 < $Records->num_records())
{
    if (null !== $Request->getVar('template'))
    {    
        include($MODULE_TPL_PATH . '/' . $Request->getVar('template') . '.tpl');
    }
    else
    {
        include($MODULE_TPL_PATH . '/list.tpl');
    } // end if no template var
} // end if no records

if (!$Request->isIncluded())
{    
    include_once(SITE_TPL_PATH . '/footer.tpl');
} // end if included

?>
