<?php
/**
* Basic Calendar data and display
*
* @author Oscar Merida
* @created Jan 18 2004
*/
class Calendar
{
    var $year;
    var $month;    
    var $monthNameFull;
    var $monthNameBrief;
    var $startDay;
    var $endDay;  
    /**
    * Constructor
    *
    * @param integer, year
    * @param integer, month
    * @return object
    * @access public
    */
    function Calendar($year, $month)
    {
        $this->year    = $year;
        $this->month   = (int) $month;
        
        $this->startTime = strtotime("$year-$month-01 00:00");
        
        $this->endDay = date('t', $this->startTime);
        
        $this->endTime   = strtotime("$year-$month-".$this->endDay." 23:59");
         
        $this->startDay    = date('D', $this->startTime);
        $this->startOffset = date('w', $this->startTime) - 1;
        
        if ($this->startOffset < 0)
        {        
            $this->startOffset = 6;
        }
        
        $this->monthNameFull = strftime('%B', $this->startTime);
        $this->monthNameBrief= strftime('%b', $this->startTime);
        
        $this->dayNameFmt = '%a';
        $this->tblWidth="*";
    }
    // ==== end Calendar ================================================
    
    function getStartTime()
    {
        return $this->startTime;   
    }
    
    function getEndTime()
    {    
        return $this->endTime;    
    }
    
    function getYear()
    {
        return $this->year;   
    }
    
    function getFullMonthName()
    {
        return $this->monthNameFull;   
    }
    
    function getBriefMonthName()
    {
        return $this->monthNameBrief;   
    }
    
    function setTableWidth($w)
    {
        $this->tblWidth = $w;   
    }
    
    function setYear($year)
    {    
        $this->year = $year;   
    }
    
    function setMonth($month)
    {
        $this->month = $month;   
    }
    /**
    * Any valid strftime format for display weekday names
    *
    * %a - abbreviated, %A - full, %u as number with 1==Monday
    */
    function setDayNameFormat($f)
    {
        $this->dayNameFmt = $f;   
    }
    /**
    * Returns markup for displaying the calendar.
    *
    * @return string
    * @access public
    */
    function display()
    {
        ob_start();
?>
    <table border="0" cellspacing="0" cellpadding="0" width="<?= $this->tblWidth ?>">
        <?= $this->dspDayNames() ?>
        <?= $this->dspDayCells() ?>
    </table>
<?php    
        $c = ob_get_contents();
        ob_end_clean();
        return $c;
    }
    // ==== end display ================================================
    
    /**
    * Displays the row of day names.
    *
    * @return string
    * @access private
    */
    function dspDayNames()
    {
        $names = array(
            '2004-10-25', '2004-10-26', '2004-10-27', '2004-10-28',
            '2004-10-29','2004-10-30','2004-10-31'
        );
        
        ob_start();
?>
        <tr>
<?php    
        for ($i=0; $i<7; $i++) 
        {
?>
            <th width="14%">
                <?= strftime($this->dayNameFmt, strtotime($names[$i])) ?>
            </th>
<?php
        }        
?>
        </tr>
<?php    
        $c = ob_get_contents();
        ob_end_clean();
        return $c;
    }
    // ==== end dspDayNames ================================================
    
    /**
    * Displays all day cells for the month
    *
    * @return string
    * @access private
    */
    function dspDayCells()
    {
        $i = 0; // cell counter
        ob_start();
?>
        <tr>
<?php    

        // first display empty cells based on what weekday the month starts in]
        for($c = 0; $c < $this->startOffset; $c++)
        {
            $i++;
?>        
        <td class="notInMonth">&nbsp;</td>
<?php
        } // end offset cells
    
        // write out the rest of the days, at each sunday, start a new row.
        for ($d = 1; $d <= $this->endDay; $d++)
        {
            $i++;
?>        
        <?= $this->dspDayCell($d) ?>        
<?php
            if (($i % 7) == 0)
            { 
?>
        </tr>        
<?php   
            }
        
            if (($d < $this->endDay) && (($i % 7) == 0))
            {
?>      <tr>       
<?php   
            }
        }
        
        // fill in the final row
        $left = 7 - ($i % 7);
        
        if ($left < 7)  
        {
            for ($c = 0; $c < $left; $c++)
            {
?>
            <td class="notInMonth">&nbsp;</td>
<?php
            }
?>
        </tr>
<?php
        }    
    
        $c = ob_get_contents();
        ob_end_clean();
        return $c;        
    }
    
    // ==== end dspDayCells ================================================
    
        
    /**
    * outputs the contents for a given day
    *
    * @param integer, day
    * @abstract
    */
    function dspDayCell($day)
    {
        return '<td>'.$day.'</td>';
    }
    // ==== end dayCell ================================================    
    
} // end class

/**
* Displays events in a given month for a Syntax Collection
*
* @author Oscar Merida <oscarAtoscarm.org>
* @created Jan 18 2004
* @package  goCoreLib
*/
class Calendar_SyntaxEvents extends Calendar
{
    /**
    * EventCollection object
    */
    var $EventCollection;
    
    /**
    * Array holding the current month's event links
    */
    var $event_links;
    
    function dspDayCell($day)
    {
        
        if ($events = $this->getDaysEvents($day))
        {
?>
    <td>
        <div class="dayNum"><?= $day ?></div>
        <ul>
<?php     
            foreach ($events as $i => $e)
            {
?>
            <li><a href="<?= $e['link'] ?>"><?= $e['title'] ?></a></li>
<?php   
            }
?>  
        </ul>
    </td>
<?php 
        }
        else 
        {
?>
        <td class="dayNumNoEvents"><?= $day ?></td>
<?php
        } // end if
    }
    
    /**
    * Sets the syntax EventCollection to use to build links
    *
    * @param object EventCollection
    * @access public
    */
    function setEventCollection($collection)
    {
        $this->EventCollection = $collection; 
    }
    
    /**
    * Parses the event collection to grab events for the current month.
    *
    * @return
    * @access private
    */
    function parseEvents()
    {
        if (isset($this->EventCollection))
        {
            
            $this->EventCollection->reset_iterator();
            $this->events_links = array();
            while($event = $this->EventCollection->fetch_record())
            {                
                // NOTE: come back and add better handling for start end date ranges.
                // if the start date is in the current month, we add it to our array                
                $month_start = date('Ym', strtotime($event->getStartDate()));
                $month_end   = date('Ym', strtotime($event->getEndDate()));
                
                if ($month_start == $month_end && $month_start == date('Ym', $this->startTime))
                {
                    // this event starts and ends within this month
                    $day_start = (int)date('d', strtotime($event->getStartDate()));
                    $day_end   = (int)date('d', strtotime($event->getEndDate()));
                    
                }
                else if ($month_start == date('Ym', $this->startTime))
                {
                    // this event starts this month but ends sometime next month
                    $day_start = (int)date('d', strtotime($event->getStartDate()));
                    $day_end   = (int)date('t', $this->startTime);
                    
                }
                else if ($month_end == date('Ym', $this->startTime))
                {
                    // this event ends this month but started last month;
                    $day_start = 1;
                    $day_end = (int)date('d', strtotime($event->getEndDate()));
                }
                else if ($month_start < date('Ym', $this->startTime) && $month_end > date('Ym', $this->startTime))
                {
                    // this event startedin a previous month and ends after this month
                    $day_start = 1;
                    $day_end   = (int)date('t', $this->startTime);
                }
                else
                {
                    //echo "<br><br>this month: ".$month_start." - ".$month_end;
                    //echo "<br>this event ".date('Ym', $this->startTime).' - '.date('Ym',$this->endTime);
                }
                
                if (isset($day_start))
                {
                    for($i = $day_start; $i <= $day_end; $i++)
                    {                                        
                        $this->event_links[$i][] = array(
                            'title' => $event->getTitle(),
                            'link' => '/content/calendar/detail/' . $event->get_id() . '/' 
                       );
                    }
                }
            }            
            return true;
            
        }
        else
        {
            trigger_error('parseEvents called but EventCollection is empty', E_USER_WARNING);
            return false;
        }
    }
    // ==== end addEvent ================================================
    
    /**
    * Returns an array of the events on a day.
    *
    * @return array
    * @access public
    */
    function getDaysEvents($day)
    {
     
        if (isset($this->event_links[$day]) && 0 < count($this->event_links[$day]))
        {            
            return $this->event_links[$day];
        }
        else 
        {
            return FALSE;   
        }
    }
    
    /**
    * Wrapper around parent method, make sure we parse out relevant events.
    *
    * @access public
    */
    function display()
    {
        if ($this->parseEvents())
        {
            return parent::display();
        }
    }
    // ==== end getDaysEvents ================================================

} // end class
?>