<?php

// Include class file for base abstract class
require_once(dirname(__FILE__) . "/AbstractEvent.class.php");

// Register this class as the pxdb_record class to use for datatype DATATYPE_EVENT
pxdb::register_class('Event', 'pxdb_record', DATATYPE_EVENT);

/**
* This is a collection class for objects of type Event.
*
* This class extends the abstract class which in turn extends the pxdb_collection
* and provides object-specific methods which invoke
* parent generic methods.  This class provides some methods you might find useful --
* and more importantly, it provides a base from which you can build your own.
*
* Note that this class uses CamelHump capitalization so that methods of this
* class can be easily distinguished from the all-lowercase methods of the parent
* classes.
*
* Here's an example of how to use these classes (with hypothetical user functions):
*
* $event_collection = new EventCollection();
* $event_collection->findByAuthor($user->get_uid()); // current logged-in user's id.
*
* while ($Record =& $event_collection->fetchRecord())
* {
*   // fetchRecord() returns a Event object --
*   // which extends the pxdb_record object and provides individual methods
*   // for getting attributes.
*   print "<br>".$Record->getTitle();
* }
*
* If you need to only get a subset or a paged version of the query, you will
* want to execute the query manually (otherwise default execute() is run when
* you fetchRecord()).
*
* $event_collection->execute_page($rows_per_page, $current_page);
* while ($Record =& $event_collection->fetchRecord())
* {
* ...
* OR
* $event_collection->execute_limit($num_rows);
* while ($Record =& $event_collection->fetchRecord())
* {
* ...
*
* Remember also that the ADOdb recordset is returned from the ->execute() method.
* $rs  =& $event_collection->execute();
* print "Rows found: ".$rs->RecordCount();
* while (!$rs ->EOF)
* {
*   print "Document name: ".$rs->fields['name'];
*   $rs->MoveNext();
* }
*
* @version build_user_class 1.2
* @author AutoGenerated (DBasis)
*/
class EventCollection extends AbstractEventCollection
{
    /**
    * Constructor.
    * Calls parent constructor
    * results sets this class will produce.
    */
    function EventCollection()
    {
        parent::init();
    }
    
} // end EventCollection class

/**
* This is a skeleton class for a class that represents
* a record of type Event* You can use this in place of using the pxdb_record class directly.
*
* @version build_class 1.2
* @author AutoGenerated (DBasis)
*/
class Event extends AbstractEvent
{
    /**
    * Constructor.
    * Calls parent constructor.
    * @param mixed $data Record ID or record row (array)
    */
    function Event($data = null)
    {
        parent::init($data);
    }
    
    /**
    * Initialization method
    */
    function init($data)
    {
        parent::pxdb_record($data);
    }
    
    /**
    * Returns a pretty printed version of the start and end dates for an event
    */
    function echoDateRange()
    {
        $start = strtotime( $this->getStartDate() );
        $end   = strtotime( $this->getEndDate() );
        
        $start_year = date( 'Y', $start );
        $end_year    = date( 'Y', $end );
        
        $start_month = date( 'Y-m', $start );
        $end_month    = date( 'Y-m', $end );
        
        $start_day = date( 'Y-m-d', $start );
        $end_day   = date( 'Y-m-d', $end );
        
        if ( $start_day == $end_day )
        {                        
            return strftime(LOCALE_DATE_LONG, strtotime($start_day));
        }
        else if ( $start_month == $end_month )
        { 
            // a multiple day event within one month
            return strftime( '%B %e', strtotime($start_day)) . ' - ' . strftime('%e, %Y', strtotime($end_day));
        }
        else if ( $start_year == $end_year )
        {   
            // a multiple day event across months within a given year
            return strftime( '%B %e', strtotime($start_day)) . ' - ' . strftime('%B %e, %Y', strtotime($end_day));            
        }
        else
        {
            return strftime(LOCALE_DATE_LONG, strtotime($start_day)) . ' - ' . strftime(LOCALE_DATE_LONG, strtotime($end_day));
        }        
        
    }

} // end Event class

?>