<?php
// Include module-specific init file.
include(dirname(__FILE__) . '/init/init.php');

$end_set = false;
$filter =& pxdb_search::filter(DATATYPE_EVENT);

// filter by sunrise/sunset (string is already quoted)
$filter->add_value('sunset', DB_TIME_STAMP, '>=', null, false);
$filter->add_value('sunrise', DB_TIME_STAMP, '<=', null, false);

// filter by Type (pickid) if needed
if (null !== $Request->getVar('type'))
{
    $filter->add_value('type', $Request->getVar('type'));
}   

// filter by section publication if needed
if (null !== $Request->getVar('in_section'))
{
    // add to main filter
    if (1 == $Request->getVar('featured'))
    {
        $filter->add_parent($Request->getVar('in_section'), NODEREL_SECTION_TO_FEATURED, 'ASC');
    }
    else
    {
        //assume publication & order by section weight
        $filter->add_parent($Request->getVar('in_section'), NODEREL_SECTION_TO_CONTENT, 'ASC');
    }
    $node_weighted = true;
}
else
{
    $node_weighted = false;
}

// Now we figure out what events you are looking for
if ('past' == $Request->getVar('segment'))
{
    if (!$Request->isIncluded())
    {
        $Request->setTrailPoint(1, array('url'=>'/content/calendar/', 'crumb'=>'Events'));        
        $Request->addTrailPoint(array('url'=>'', 'crumb'=>'Past events'));
    }
    
    $filter->add_value('start_date', date('Y-m-d 00:00'), '<', null, TRUE);
    $Request->setVar('orderby', 'start_date DESC');
}
else if (null != $Request->getVar('month'))
{
    $month = strtotime($Request->getVar('month'));
    $filter->add_value('start_date', date('Y-m-t 23:59', $month), '<=', null, TRUE);
    $filter->add_value('end_date', date('Y-m-01 00:00', $month), '>=', null, TRUE);
    $end_set = true;
}
// if you specify upcoming or past, you're not going to want more specific dates for start and end
else if (null !== $Request->getVar('start_date'))
{
    // on specified day
    $filter->add_value('start_date', $Request->getVar('start_date'), '>=', null, TRUE);    
}
else if ( null == $Request->getVar('segment') )
{
    // if you don't specify a segment or start date, then it'll default to today, at midnight
    $filter->add_value('start_date', date('Y-m-d 00:00'), '>=', null, TRUE);
    $startTs = time();
}

if (false === $end_set && null !== $Request->getVar('end_date'))
{
    $end_date =$Request->getVar('end_date');
    // you could pass an empty string as the end date to make
    // sure you have NO end date in the filter.  Since 
    // the default end date is to get dates before the beginning of next month
    if (!empty($end_date))
    {
        $filter->add_value('end_date', $end_date, '<', null, TRUE);
    }    
}
// default to upcoming
else if ((false == $end_set && null == $Request->getVar('segment')) || 'upcoming' == $Request->getVar('segment'))
{    
    $filter->add_value('start_date', date('Y-m-d 00:00'), '>=', null, TRUE);
    if (!$Request->isIncluded())
    {
        $Request->setTrailPoint(1, array('url'=>'/content/calendar/', 'crumb'=>'Events'));
        $Request->addTrailPoint(array('url'=>'', 'crumb'=>'Upcoming events'));
    }
    if ('upcoming' != $Request->getVar('segment'))
    {
        $Request->setVar('segment', 'upcoming');
    }
    $Request->setVar('orderby', 'start_date ASC');    
    
}

// Find all events within date range

$Records = new EventCollection();

if ($Request->getVar('orderby'))
{
    $Records->set_order($Request->getVar('orderby'));
}
else
{
    // set a default order by, usually sunrise
    // oam don't clobber order if its already been set
    if ( false == $Records->get_order() )
    {        
        $Records->set_order('start_date ASC');
    }
} // end if

$Records->find($filter);

// limit recordset results
if (null !== $Request->getVar('limit'))
{
    if (false === $Records->execute_limit($Request->getVar('limit')))
    {
        trigger_error('Execute Limit failed', E_USER_WARNING);
    }
} // paginate recordset
else if (!$Request->isIncluded() && null !== $rows_per_page)
{    
    if (null == $Request->getVar('page'))
    {
        $Request->setVar('page', 1);
    }
    
    if (false === $Records->execute_page($rows_per_page, $Request->getVar('page')))
    {
        trigger_error('Execute Page failed', E_USER_WARNING);
    }
// show all records
}
else
{
    $Records->execute();
}

if (!$Request->isIncluded()) 
{
    include_once(SITE_TPL_PATH . '/header.tpl');
}

// if there are no records do not output anything at all.
if (true == $Request->getVar('always_display') || 0 < $Records->num_records() || !$Request->isIncluded())
{
    
    if (null !== $Request->getVar('template'))
    {    
        include($MODULE_TPL_PATH . '/' . $Request->getVar('template') . '.tpl');
    }
    else
    {
        include($MODULE_TPL_PATH . '/list.tpl');
    } // end if no template var
} // end if no records

if (!$Request->isIncluded()) 
{
    include_once(SITE_TPL_PATH . '/footer.tpl');
}
?>