<?php

/**
* This is a collection class for the Person and Organisation datatypes.
* @version 1.1
* @author hans lellelid, hans@forumone.com
*/
class DirectoryCollection extends pxdb_collection
{
	
	/**
	* @var object
	*/
	var $coreFilter=null;
	
	/**
	* Constructor. 
	* Calls parent constructor and sets a default 'order by' value for the
	* results sets this class will produce.
	*/
	function DirectoryCollection()
	{
		parent::pxdb_collection(Modules::getAttribute('directory', 'datatype'));
		// Default order will be the 'name' column
		// you can override this, of course
		$this->orderBy('name'); 
	}
	
	/**
	* Wrapper for parent's set_order method.
	* @param string $order Column to order by.
	*/
	function orderBy($order)
	{
		return parent::set_order($order);
	}
	
	/**
	* Calls parent fetch_record class, specifying to use the Person class.
	* @return object Person
	*/
	function &fetchRecord()
	{
		return parent::fetch_record();
	}
	
	/**
	* @param object
	*/
	function setCoreFilter(&$f)
	{
		$this->coreFilter = &$f;
	}

	/**
	* Find records matching `topic` pick list -- or finds all Person sorted by topic.
	* If the $topic parameter is left blank, it is assumed you want to find all of the 
	* Person records, organized (sorted) by topic.
	* @param mixed $topic Pick table `id` col -- or `name` col
	* @return void
	*/
	function findByTopic($topic=null)
	{
		if($topic == null) {
			$this->orderBy('topic, name');
			$this->find();
		} else {
			$pickFilter = &pxdb_search::filter($this->get_datatype());
			$pickFilter->add_value('topic', $topic);
			$this->find($pickFilter);					
		}
	}
	
	/**
	* Find records matching `issue` pick list -- or finds all Person sorted by issue.
	* If the $issue parameter is left blank, it is assumed you want to find all of the 
	* Person records, organized (sorted) by issue.
	* @param mixed $issue Pick table `id` col -- or `name` col
	* @return void
	*/
	function findByIssue($issue=null)
	{
		if($issue == null) {
			$this->orderBy('issue, name');
			$this->find();
		} else {
			$pickFilter = &pxdb_search::filter($this->get_datatype());
			$pickFilter->add_value('issue', $issue);
			$this->find($pickFilter);					
		}
	}
	
	/**
	* Keyword search.
	* @param string $keywords Data entered in search form.
	* @return void
	*/
	function findKeywordSearch($keywords) 
	{		
		// converts input into an array of keywords
		$terms = pxdb_tools::get_search_terms($keywords);		
		
		$search = &pxdb_search::search($this->get_datatype());
		$search->set_value_conj('AND');
		$search->add_value('name', $terms, 'REGEXP');
		$search->add_value('city', $terms, 'REGEXP');
		$search->add_value('country', $terms, 'REGEXP');
        
        
		$this->find($search);
		//now filter
		$this->find($this->coreFilter->filter_sql($this->sql));			
	}
	
	/**
	* Alpha search.
	* @param string $initial
	* @return void
	*/
	function findAlphaSearch($initial) 
	{		
		// converts input into an array of keywords
		$search = &pxdb_search::search($this->get_datatype());
		$search->set_value_conj('AND');
		$search->add_value('name', "^$initial", 'REGEXP');
        $search->add_value('lname', "^$initial", 'REGEXP');
        
		$this->find($search);			
		//now filter
		$this->find($this->coreFilter->filter_sql($this->sql));			
	}
	
	
} // end DirectoryCollection class


?>