<?php
// Include module-specific init file.
include(dirname(__FILE__) . '/init/init.php');

$dbh = &pxdb::dbh();

if (!$Request->getVar('id')) 
{
    $Request->setVar('id', $Request->getPathElement(2)); // could be /module/detail/1232
}

if (!$Request->getVar('id')) 
{
    trigger_error('Call to (_content) detail action w/o ID specified. (referer: '.$_SERVER[HTTP_REFERER].')', E_USER_WARNING);
    return false;
}


// FILTERING
// ---------
// use a filter to get the record -- because even though we know the ID,
// we need to make sure that it is not restricted content, etc.

$content_filter = &pxdb_search::filter(CONTENT_DATATYPES_LIST);
$content_filter->add_value('id', $Request->getVar('id'));

/*
10/26/04 OAM: Starting with Syntax 1.1.0, record privs are a much better way to 
restrict content to specific groups.  By default, pxdb_collection should be applying privs
when used.  
*/

$Records = new pxdb_collection(CONTENT_DATATYPES_LIST);
$Records->find($content_filter); // max of 1 recrd

// END FILTERING

//pxdb::print_debug($Records->get_sql(), 'red');
    
if (!($Record = &$Records->fetch_record())) 
{
    // there's only 1 record
    trigger_error('Error fetching document (ID: '.$Request->getVar('id').') -- probably user does not have privs.', E_USER_WARNING);
    return false;
}

$Request->setVar('datatype', $Record->get_datatype());

/*
Breadcrumb is a challenge here.  Because of all the different ways that a detail
page can be reached, the breadcrumb is build based on the HTTP_REFERER, which is
kinda messy & does rely on that value being passed by the browser [but that
should be fine 99% of the time], especially considering other things that will
break if using such an old / non-standard browser.
*/    
if ('sectionrequest' == strtolower(get_class($Request)) && isset($_SERVER['HTTP_REFERER']))
{    
    $ref_path = str_replace('http://' . $_SERVER['HTTP_HOST'], '', $_SERVER['HTTP_REFERER']);
}
else
{     
    $ref_path = explode('/', $_SERVER['REQUEST_URI']);
    $ref_path = array_slice($ref_path, 0, 3);
    $ref_path = join('/', $ref_path);    
}

$TrailRequest = &_request_factory($ref_path);

if (false !== $TrailRequest) 
{
    if ('sectionrequest' == strtolower(get_class($TrailRequest)))
    {
        $Request->setTrail($TrailRequest->getTrail());
        $Request->setVar('requesttype', 'sectionrequest');
    }
    else
    {
        $Request->setTrail($TrailRequest->getTrailStart());                
        // behavior is more complex for module requests
        if ('browse' == $TrailRequest->getAction() && $TrailRequest->getModule() == Modules::getGenericModule()) 
        {
            // special exception for generic module being used in 'browse' mode    
            $Request->addTrailPoint(array('crumb' => 'Browsing Content', 'url'=>$_SERVER[HTTP_REFERER]));
        }
        elseif ('search' == $TrailRequest->getAction() && $TrailRequest->getModule() == Modules::getGenericModule()) 
        {
            $Request->addTrailPoint(array('crumb' => 'Search Results', 'url'=>$_SERVER[HTTP_REFERER]));
        }
        else
        {
            $trail_modules = Modules::findModules('datatype', $Record->get_datatype());
            $trail_module = $trail_modules[0]; // grab the first
            if ($parent_urlpath = Modules::getAttribute($trail_module, 'parent_url_path'))
            {
                if ($parentSection = Section::getByUrlPath($parent_urlpath))
                {
                    $Request->addTrailPoint(array(
                        'url' => $parentSection->getNavPath() . '/',
                        'crumb' => $parentSection->getCrumb()
                   ));
                }
                else
                {
                    trigger_error('parent_url_path does not exist: '.$parent_urlpath, E_USER_WARNING);
                }
            }
            elseif ('general' != $trail_module)
            {
                // Add the current module to the breadcrumb
                $Request->addTrailPoint(array(
                    'url' => '/content/' . $trail_module . '/',
                    'crumb' => Modules::getAttribute($trail_module, 'label')
                ));
            }
        } // if module requests    
    }
} // otherwise couldn't figure out the referer ... so I guess we should just build a trail from "Home > "

$Request->addTrailPoint(array('crumb' => $Record->get_field('name'), 'url' => ''));                                                                                                
$Request->setSidebarScript($MODULE_PATH . '/detail-sidebar.php');
$Request->setPageTitle($Record->get_field('name'));

// Display the template      
if (!$Request->isIncluded())
{
    include_once(SITE_TPL_PATH . '/header.tpl');
}
            
// 1) Find which (if any) modules handle this datatype
$matches = Modules::findModules('datatype', $Record->get_field('datatype'));
$tpl_basename = 'detail.tpl';

// (if it found more than one match we'll just use the first match.)            
$module_specific_template = SITE_MOD_PATH . "/{$matches[0]}/templates/$tpl_basename";

// check if matches is empty to prevent file_exists from triggering an error
if (false === empty($matches[0]) && file_exists($module_specific_template)) 
{
    // use that template
    include($module_specific_template);
}
else
{
    // use our generic template
    include($MODULE_TPL_PATH . '/'. $tpl_basename);
}                    
    
if (!$Request->isIncluded())
{
    include_once(SITE_TPL_PATH . '/footer.tpl');
}
?>