<?php    

// Include module-specific init file.
include(dirname(__FILE__) . '/init/init.php');

//$dbh =& pxdb::dbh();

if (null !== CONTENT_DATATYPES_LIST && '' !== CONTENT_DATATYPES_LIST)
{
    
    // restrict by datatype
    if (null !== $Request->getVar('datatype'))
    {
        // create a content filter and collection for just that datatype
        $datatypes = $Request->getVar('datatype');        
    }
    // Test if we are providing the list capability as a fallback for another 
    // module call. If so, we need to restrict by that module's datatypes.
    else if ( 'general' !== $Request->getModule() )
    {
        $module = $Request->getModule();
        $datatypes = Modules::getAttribute( $module, 'datatype' );
    }
    else
    {
        // get all publically exposed content types
        $datatypes = CONTENT_DATATYPES_LIST;
    }

    $Records = new pxdb_collection($datatypes);
    $content_filter = &pxdb_search::filter($datatypes);
    
    if ($Request->getVar('orderby'))
    {
        $Records->set_order($Request->getVar('orderby'));
    }
    else
    {        
        $Records->set_order('sunrise DESC, name ASC');
    }
    
    // FILTERING
    // ---------
    
    // section_id is set if this is being included from
    // within a section page (which is the usual way things work)
    $content_filter->add_value('sunset', DB_TIME_STAMP, '>=', $opt=null, $quote=false);
    $content_filter->add_value('sunrise', DB_TIME_STAMP, '<=', $opt=null, $quote=false);
    
    /*
    OAM 12-28-2005: members_only field is obsolete
    
    $User = &Site::getUser();
    if (!$User || !$User->is_member(AUTH_MEMBER_GROUPS))
    {
        $content_filter->add_value('members_only', 0);
    }        
    */
    // Account for different kinds of grouping:
    
    // restrict by module affiliation
    if (null !== $Request->getVar('group_module'))
    {        
        // this is a request for content grouped by module
        $datatypes = implode(',', (array) Modules::getAttribute($Request->getVar('group_module'), 'datatype'));        
        $content_filter->add_value('datatype', "$datatypes", 'IN');
    }
    
    // restrict by user defined factes ( see config/user.conf.php )    
    if ( $facets = StxPrefs::getFacets() )
    {
        while ( list($key, $facet) = each ($facets) )
        {
            if ( empty($facet['field']) || !isset($facet['field']) )
            {
                trigger_error( "Facet: $key field can not be empty", E_USER_WARNING );
            }
            else
            {
                if (null !== $Request->getVar( $facet['field']) )
                {
                    $content_filter->add_value( $facet['field'], $Request->getVar($facet['field']));
                }
            }
            
        }
        
    }
    
    // restrict by section
    if (null !== $Request->getVar('section'))
    {
        // filter based on nodrel relationship to that section:        
        $section_filter =& pxdb_search::filter(DATATYPE_SECTION);
        $section_filter->add_value('id', $Request->getVar('section'));
    
        // restrict by featured
        if (null !== $Request->getVar('featured'))
        {
            $noderel_id = (true === $Request->getVar('featured')) ? NODEREL_SECTION_TO_FEATURED : NODEREL_SECTION_TO_CONTENT;
            
            $content_filter->add_parent($section_filter, $noderel_id, 'ASC'); //asc enforces noderel ordering
        }
        else
        {
            $content_filter->add_parent($section_filter);
        }
    }
    
    // restrict by type
    if (null !== $Request->getVar('type'))
    {
        $content_filter->add_value('type', $Request->getVar('type'));
    }
    $Records->find($content_filter);

    // END FILTERING
    
    if (null !== $Request->getVar('limit'))
    {
        if (false === $Records->execute_limit($Request->getVar('limit')))
        {
            trigger_error('Execute Limit failed', E_USER_WARNING);
        }
        
    }
    elseif (null !== $Request->getVar('page') && null !== $Request->getVar('rows_per_page'))
    {
        if (false === $Records->execute_page($Request->getVar('rows_per_page'), $Request->getVar('page')))
        {
            trigger_error('Execute Page failed', E_USER_WARNING);
        }
    }
    else
    {
    
        // just get 'em all.        
        if (false === $Records->execute()) 
        {
            trigger_error('Execute failed', E_USER_WARNING);
        }        
    }
}

if (!$Request->isIncluded())
{
    include_once(SITE_TPL_PATH . '/header.tpl');
}
    
// We only display the "No records found" message is this module
// is not being included in a section page -- why?  -- so that nothing
// is returned if no content is found (so that the section page can test
// whether module request found anything)
if (null === CONTENT_DATATYPES_LIST || '' === CONTENT_DATATYPES_LIST)
{
    if (! $Request->isIncluded())
    {
        include($MODULE_TPL_PATH . '/no_content.tpl');
    }
}
elseif (!$Records->num_records() && !$Request->isIncluded())
{
    include($MODULE_TPL_PATH . '/no_content.tpl');
}
elseif (0 < $Records->num_records())
{    
    
    if (null == $Request->getVar('template'))
    {
        include($MODULE_TPL_PATH . '/list.tpl');
    }
    else
    {
        $matches = Modules::findModules('datatype', $Records->get_datatype());
        // (if it found more than one match we'll just use the first match.)
        $module_specific_template = SITE_MOD_PATH . '/' . $matches[0] .'/templates/' . $Request->getVar('template') . '.tpl';
        
        
        if ( file_exists($module_specific_template ) )
        {
            include($module_specific_template);
        }
        else
        {
            include($MODULE_TPL_PATH . '/' . $Request->getVar('template') . '.tpl');
        }
    }
} // if no content


if (!$Request->isIncluded()) 
{
    include_once(SITE_TPL_PATH . '/footer.tpl');
}
?>