<?php
/**
* HOW DOES SEARCH CONSTRUCT A QUERY?
*
* 1) Gets datatypes for all modules that have the searchable attribute set to true

* @author Oscar Merida
*/
require_once (dirname(__FILE__).'/init/init.php');

$searchable = Modules::aggregateAttribute('datatype', Modules::findModules('searchable', true));
pxdb_import('metadata.pxdb_datatype');

if ($dt = $Request->getVar('datatype'))
{
    if (in_array($dt, $searchable))
    {
        $searchable = array($dt);
        $datatype_search = true;
        $limit_results = null;
    }
    else
    {
        trigger_error('Non-searchable datatype specified', E_USER_ERROR);
    }
}
// fill in output arrays as long as they are not empty for the output template
$total_found = 0;
$search_query = $Request->getVar('search_query');

/**
* Patch XSS Vulnerability
* SFS
*/
$search_query = strip_tags($search_query);
$search_query = htmlentities($search_query);
$Request->setVar('search_query', $search_query);

if (2 < strlen($search_query))
{
    //also show the "Best Bets" based on simple name matching
    // to not deal with hidden sections and what not, make them not searchable
    $BestFilter = &pxdb_search::filter(array_diff($searchable, array(DATATYPE_SECTION)));
    // filter by sunrise/sunset (string is already quoted)
    $BestFilter->add_value('sunset', DB_TIME_STAMP, '>=', null, false);
    $BestFilter->add_value('sunrise', DB_TIME_STAMP, '<=', null, false);
    $BestFilter->add_value('name', "%$search_query%", 'LIKE', null, true);
    
    
    $BestRecords = &new pxdb_collection($searchable);
    $BestRecords->find($BestFilter);
    $BestRecords->execute_limit(10);
}

while (list(, $dt) = each($searchable))
{
    // lookup datatype info
    $dt_info = new pxdb_datatype($dt);
    
    // This returns a custom dbsearch object named 'dbsearch_datatypelable' (for example dbsearch_publications)
    // if one has been loaded for that datatype (usually in lib the module that handles it)
    // If none is found, then it defaults to a generic class that simply searches the name, title, desc1 fields.
    // You can customize behavior per datatype by sub-classing and changing the get_search_sql and displayLinks methods. 
    
    $search =& DBSearch::factory($dt, $dt_info->get_type()); 
    $search->setTerm($search_query);
    
    // returns null if there are no results
    if ($matches = $search->results($limit_results))
    {        
        $count = $search->countResults();
        $total_found += $count;
        
        if (isset($limit_results) && !empty($limit_results) )
        {
            $key = $dt . ';' .$dt_info->get_label() . ';' . $count .';' .$limit_results;
        }
        else
        {
            $key = $dt . ';' .$dt_info->get_label() . ';' . $count;
        }
        
        $results[ $key ] = $matches;
    }
    unset ($dt_info);
    unset ($search);        
}

$Request->setPageTitle('Search Results');

if (!$Request->isIncluded())
{   
    include_once(SITE_TPL_PATH . '/header.tpl');
}

if (isset($BestRecords) && 0 < $BestRecords->num_records())
{
    include(dirname(__FILE__) . '/templates/best_matches.tpl');
}

if (0 < $total_found)
{
    include(dirname(__FILE__) . '/templates/results.tpl');
}
else
{
    include(dirname(__FILE__) . '/templates/none_found.tpl');
}

if (!$Request->isIncluded())
{
    include_once(SITE_TPL_PATH . '/footer.tpl');
}
?>