<?php
/**
* Defines generic interface for database searches.  You can customize behavior
* by sub-classing and changing the get_search_sql and displayLinks methods. 
*
* TODO: integrate with Syntax and Collections object better instead of treating
*   get_search_sql as a wrapper function to a collection object.
*   For now, this means you can customize search behavior per object by either
*   writing straight up sql in get_search_sql or creaitng a collection object
*   within that method and return the collection's get_sql result.
*
* @author Oscar Merida
*/
class DBSearch_section extends DBSearch {

    /**
    * Creator
    *
    * Should be called by DBSearch::factory
    */
    function DBSearch_section ( $typeid, $classname ) 
    {
        parent::DBSearch( $typeid, $classname );
        $this->dbfields = array( 'name', 'title', 'desc1' );
        $this->getFields = array( 'id', 'name', 'title', 'idnum', 'created', 'modified');
        $this->orderby = 'created DESC';
    } // end func
    
    
    /**
    * Cachable method, can be called statically
    *
    * @access private
    */
    function _getSectionIds( $timestamp, $server )
    {
        require_once( SITE_LIB_PATH . '/syntaxcms/SectionNavigatorOptimized.php' );        
        $section_navigator =& new SectionNavigatorOptimized();        
        
        $section =& $section_navigator->getHomeSection();        
        $section_navigator->setSection($section);
        
        $section_navigator->setShowHiddenSections( FALSE ); // ignore hidden sects
        $ids = $section_navigator->getDescendantIds();
        
        return $ids;
    }
    
    /*
    * Figure out what sections to include in the site
    *
    * @access private
    */
    function getSectionIds()
    {
        // OAM use content_last_update as param to method call to 
        // get descentd ids function so that its cached but 
        // updated whenever that key changes.
        
        $last_update = f1cms::getContentUpdated( 'nav' );
        
        require_once ('Cache/Function.php');
        $cache = new Cache_Function( 'file', array('cache_dir'=> SITE_TMP_PATH.'/cache/'), 3600*24 );
                
        $ids = $cache->call( 'DBSearch_section::_getSectionIds', $last_update, $_SERVER['SERVER_NAME'] );        
        return $ids;
    }
    /*
    * Override these methods to customize search results per datatype
    */
    function getSearchSql () 
    {

        /* OAM: restrict to the current site NAV_ROOT_EL */
        $child_ids = $this->getSectionIds();
                
        /*
        * OAM NOTE: noticed the fulltext doesn't return matches for 'about' maybe it is too
        * common a word?
        * Yup, its not indexed: http://www.databasejournal.com/features/mysql/article.php/1578331
        */
        $sql = 'SELECT '.implode(', ', $this->getFields )
              .' from records where '
              ."   ( MATCH(`name`,`salutation`,`title`,`fname`,`lname`,`city`,`desc1`,`desc2`,`desc3`,`text1`,`text2`,`text3`,`text4`,`text5`) AGAINST ('".addslashes($this->getTerm())."') )>0"
              .' and datatype='.$this->datatype.' and approved=1'
              .' and id in('. join( ',' , $child_ids ) .')';
        ;        
        return $sql;
    } // end func

    /**
    * Short description.
    *
    * @param     
    * @return
    * @access public
    */
    function displayLinks() {        
        ob_start();
    ?>
        <ol>
    <?  while ( $r =  $this->query->fetchRow() ) 
    {
            $path = $this->getSectionPath( $r['id'] );
    ?>      <li><a href="<?=$path?>"><?=stripslashes($r['name'])?></a></li>
    
    <?
    } // end while
    ?>
        </ol>
    <?
        $c = ob_get_contents();
        ob_end_clean();
        return $c;
    } // end func
    
    /**
    * @private
    */
    function getSectionPath( $sectId )
    {
        // make the section object
        $section = new Section( $sectId );
        
        $section_navigator =& SectionNavigator::singleton();
        $section_navigator->setSection($section);        
        return $section_navigator->getNavPath();
    }
    
} // end class

?>
