<?php

/*
* This is the logic file for handling section displaying.  It's very
* simple and not customized -- unlike the logic for Modules.
*
*
*/

$dbh =& pxdb::dbh();

// lookup section information
$Section =& $Request->getSection();
if (false === $Section)
{
    header('HTTP/1.0 404 Not Found');
    if ('/error.php' == $_SERVER['PHP_SELF'])
    {
        include('http://' . $_SERVER['HTTP_HOST'] . '/error.php?error=error');
    }
    else
    {
        include('http://' . $_SERVER['HTTP_HOST'] . '/error.php');
    }
    exit();
}

if ( !$Section->isAuthorized() )
{
    header('Location: ' . $_SERVER['SCRIPT_URI'] . '?login=1'); 
    exit();
    
}
else
{
    $template = StxUtilities::getSectionTemplate($Section);
    
    /*
    * What do we need to know?
    * $grouping, $groups
    * so we need an array of "topics" "datatypes", etc.
    * we want all datatypes where expose =1 -- or all topics or all issues...
    * and then we just pass $grouping => $groups[$i]
    */
    $grouping = $Section->getGrouping();
    if (!$grouping)
    {
        $grouping = 'datatype';
    }
    
    switch ($grouping)
    {
        case 'topic':
    
            $sql = "SELECT id, name FROM topics ORDER BY weight";
            $groups = $dbh->GetAll($sql);
    
        break;
    
        case 'issue':
    
            $sql = "SELECT id, name FROM issues ORDER BY weight";
            $groups = $dbh->GetAll($sql);
    
        break;
    
        case 'datatype':
    
            // SFS handle case where all datatypes are not publicly exposed
            if (null !== CONTENT_DATATYPES_LIST && '' !== CONTENT_DATATYPES_LIST)
            {
                $sql = "SELECT id, label as name FROM datatypes WHERE expose = 'Y' AND id IN (".CONTENT_DATATYPES_LIST.") ORDER BY weight";
                $groups = $dbh->GetAll($sql);
            }
            else
            {
                $groups = array();
            }
    
        break;
    
        case 'module':
    
            // $groups needs to be array of hash containing 'id' and 'name'
            // in this case 'id' will just be the module 'name' (not 'label')
            $modules = Modules::findModules('public_exposure', true);
            $groups = array();
            $module_keys = array_keys($modules);
            while (list(, $key) = each($module_keys))
            {
                $groups[] = array('id' => $modules[$key], 'name' => Modules::getAttribute($modules[$key], 'content_label'));
            }
            $grouping = 'group_module'; // because 'module' is reserved
    
        break;
    }
    
    // perform one query to see if there is any content that has been associated with this section
    // that is expired -- this will determine whether to show the ">>More Information" link on the section
    // template.
    
    // SFS handle case where all datatypes are not publicly exposed
    if (null !== CONTENT_DATATYPES_LIST && '' !== CONTENT_DATATYPES_LIST)
    {
        $archived_filter =& pxdb_search::filter(CONTENT_DATATYPES_LIST);
        
        // EXPIRED
        $archived_filter->add_value('sunrise', DB_TIME_STAMP, '<=', null, false);
        $archived_filter->add_value('sunset', DB_TIME_STAMP, '<', null, false);
        
        $section_filter =& pxdb_search::filter(DATATYPE_SECTION);
        $section_filter->add_value('id', $Section->get_id());
        $archived_filter->add_parent($section_filter);
        
        $ArchivedRecords =& new pxdb_collection(CONTENT_DATATYPES_LIST);
        $ArchivedRecords->find($archived_filter);
        $ArchivedRecords->execute();
        
        $has_archived_content = ($ArchivedRecords->num_records() > 0);
    }
    else
    {
        $has_archived_content = false;
    }
    
    include(SITE_TPL_PATH . $template);
} // end if not authorized
?>