<?php
/**
* Generates the php code for an abstract classes for a datatype.
*
* @author Oscar Merida
*/

class SyntaxCodeTemplate_BaseClass extends SyntaxCodeTemplate
{
    
   
    function generate()
    {
        $this->vals['datatype_label'] = $this->datatype->get_label();
        $this->vals['datatype_id']    = $this->datatype->get_id();
        $this->vals['basename']       = $this->convert_case($this->datatype->get_type()); // 'document' -> 'Document'        
        $this->vals['basename']       = str_replace( '_', '', $this->vals['basename'] ); 
        
        $this->vals['basevarname']    = $this->convert_case($this->datatype->get_type());
        $this->vals['define']         = 'DATATYPE_' . strtoupper($this->datatype->get_type());
        
        ob_start();
            $this->startAbsBlock();
            $this->fieldsBlock();
            $this->endAbsCollect();
            
            $this->startAbsRecord();
            $this->fieldsRecord();
            $this->endAbsRecord();
            
        $c = ob_get_contents();
        ob_end_clean();
        
        return $c;

    }

    function startAbsBlock()
    {
?>
/*
IMPORTANT NOTICE

This class definition file is autogenerated. You SHOULD NOT modify this code.
Instead always use a subclass of this class and add or override methods as
necessary.
*/

// Include class dependencies

    pxdb_import('content.output.pxdb_collection');
    pxdb_import('content.output.pxdb_record');
    pxdb_import('content.output.pxdb_search');

// Set up a constant to use in place of datatype number

    define('<?= $this->vals['define'] ?>', <?= $this->vals['datatype_id'] ?>);

/**
* This is a collection class for of type <?= $this->vals['basename'] ?>.
* This class extends pxdb_collection and provides object-specific methods which invoke
* parent generic methods.  This class provides some methods you might find useful --
* and more importantly, it provides a base from which you can build your own.
*
* IMPORTANT: You SHOULD NOT implement this class directly. It is an abstract class.
* Instead you should use the subclassed 'user' class.
*
* @version build_class 1.2
* @author AutoGenerated (DBasis)
*/
class Abstract<?= $this->vals['basename'] ?>Collection extends pxdb_collection
{

    /**
    * Constructor. 
    * Calls parent constructor and sets a default 'order by' value for the
    * results sets this class will produce.
    */
    function Abstract<?= $this->vals['basename'] ?>Collection()
    {
        $this->init();
    }
    
    /**
    * Initialization function
    * Called by subclasses to initialize the object
    */
    function init()
    {
        parent::pxdb_collection(<?= $this->vals['define'] ?>);
        // Default order will be the 'name' column
        // you can override this, of course
        $this->orderBy('name'); 
    }
    
    /**
    * Wrapper for parent's set_order method.
    * @param string $order Column to order by.
    */
    function orderBy($order)
    {
        return parent::set_order($order);
    }
    
    /**
    * Calls parent fetch_record class, specifying to use the <?= $this->vals['basename'] ?> class.
    * @return object <?= $this->vals['basename'] ?>
    
    */
    function &fetchRecord()
    {
        return parent::fetch_record('<?= $this->vals['basename'] ?>');
    }    
<?php 
    } // end notice block
    

    
    function fieldsBlock()
    {
        // get all typesfields for that datatype
        
        while ($objf =& $this->datatype->fetch_typesfield())
        {
            $field = $objf->get_identifier();
            $field_basefuncname = $this->convert_case($field);
            $field_basevarname = $this->convert_case($field, false);
        
            if ($field)
            {
                if ('relationship' == $objf->get_field_type()) 
                {
                    $this->primaryrelBlock($field, $field_basefuncname, $field_basevarname);
                }   
        
                if ($objf->get_field_metatype() == 'pickid' || $objf->get_field_metatype() == 'pick') 
                {
                    $this->pickrelBlock($field, $field_basefuncname, $field_basevarname);
                }
            } // if field
        } // while ()        
                
    }
    
    function primaryrelBlock($field, $field_basefuncname, $field_basevarname)
    {
?>      
    /**
    * Find based on a filter for relationship (field `<?= $field ?>`)
    *
    * @param integer $<?= $field_basevarname ?>ID Related record ID
    * @return void
    */
    function findBy<?= $field_basefuncname ?>($<?= $field_basevarname ?>ID)
    {       
        $<?= $this->vals['basevarname'] ?>Filter =& pxdb_search::filter(<?= $this->vals['define'] ?>);
        $<?= $this->vals['basevarname'] ?>Filter->add_value('<?= $field ?>', $<?= $field_basevarname ?>ID);
        $this->find($<?= $this->vals['basevarname'] ?>Filter);               
    }
            
              
<?php         
    }
    
    function pickrelBlock ($field, $field_basefuncname, $field_basevarname)
    {        
?>        
    /**
    * Find records matching `<?= $field ?>` pick list -- or finds all <?= $this->vals['datatype_label'] ?> sorted by <?= $field ?>.
    *
    * If the $<?= $field ?> parameter is left blank, it is assumed you want to find all of the 
    * <?= $this->vals['datatype_label'] ?> records, organized (sorted) by <?= $field ?>.
    * @param mixed $<?= $field ?> Pick table `id` col -- or `name` col
    * @return void
    */
    function findBy<?= $field_basefuncname ?>($<?= $field ?> = null)
    {
        if (null === $<?= $field ?>)
        {
            $this->orderBy('<?= $field ?>, name');
            $this->find();
        }
        else
        {
            $pickFilter =& pxdb_search::filter(<?= $this->vals['define'] ?>);
            $pickFilter->add_value('<?= $field ?>', $<?= $field ?>);
            $this->find($pickFilter);                 
        }
    }
<?php             
                
    }
    
    function endAbsCollect()
    {
?>
} // end Abstract<?= $this->vals['basename'] ?>Collection class

<?php       
    }
    
    function startAbsRecord()
    {
?>
/**
* This is a skeleton class for a class that represents a record of type <?= $this->vals['basename'] ?>
* You can use this in place of using the pxdb_record class directly.
* It is an abstract class that SHOULD NOT be implemented directly. Instead you should
* use a subclass (such as the user skeleton class provided in DBasis).
*
* @version build_class 1.2
* @author AutoGenerated (DBasis)
*/
class Abstract<?= $this->vals['basename'] ?> extends pxdb_record
{

    /**
    * Constructor. 
    * Calls parent constructor.
    * @param mixed $data Record ID or record row (array)
    */
    function Abstract<?= $this->vals['basename'] ?>($data = null)
    {
        init($data);
    }
    
    /**
    * Initialization method
    */
    function init($data)
    {
        parent::pxdb_record($data);
    }
    
    /**
    * Return the ID for the object.
    * This is "aliased" because of how frequently it is used.
    * @return integer ID
    */
    function getId()
    {
        return parent::get_id();    
    }
<?php    
   
    }
    
    function fieldsRecord()
    {
        // now for each typesfield ...
        $i = 0;
        while ($objf =& $this->datatype->fetch_typesfield((!$i++ ? true : null)))
        {
            $field = $objf->get_identifier();
            $field_basefuncname = $this->convert_case($field);
            $field_basevarname = $this->convert_case($field, false);
            
            if ($field)
            {
                if ('relationship' == $objf->get_field_type()) 
                {
                    $this->relFieldRecord($field, $field_basefuncname);            
                }
                else
                {
                    $this->stdFieldRecord($field, $field_basefuncname);
                
                    if ($objf->get_field_metatype() == 'boolean')
                    {
                        $this->boolFieldRecord($field, $field_basefuncname);
                    } // if boolean
                } // if field type is relationship
            } // if field
        } // while        

    }
    
    function relFieldRecord($field, $field_basefuncname)
    {
?>
    /**
    * Get the related object for record `<?= $field ?>` -- or value, if $lookup_field is specified.
    * @param string $lookup_field Field to lookup on related record
    * @return mixed Object, array of objects -- or value or array of values (if $lookup_field)
    */
    function get<?= $field_basefuncname ?>($lookup_field = null)
    {               
        return $this->get_field('<?= $field ?>', $lookup_field);             
    }
    
<?php         
    }
    
    function stdFieldRecord($field, $field_basefuncname)
    {
?>
    /**
    * Get the value for field `<?= $field ?>`.
    * @return mixed Value or array of values.
    */
    function get<?= $field_basefuncname ?>()
    {       
        return $this->get_field('<?= $field ?>');             
    }
    
<?php 
    }
    
    function boolFieldRecord($field, $field_basefuncname)
    {
?>
    /**
    * Get whether or not `<?= $field ?>` is non-zero.
    * @return boolean True for non-zero value, false if zero.
    */
    function is<?= $field_basefuncname ?>()
    {       
        if ($this->get_field('<?= $field ?>'))
        {
             return true; 
        }
        else
        { 
            return false;
        }             
    }
    
<?php 
    }

    function endAbsRecord()
    {
?>

} // end Abstract<?= $this->vals['basename'] ?> class

<?php 
    }
} // end class
?>
