<?php
/**
* Generates the php code for a user class 
*
* @author Oscar Merida
*/

class SyntaxCodeTemplate_UserClass extends SyntaxCodeTemplate
{    
    function generate()
    {  
        $this->vals['datatype_label'] = $this->datatype->get_label();
        $this->vals['datatype_id']    = $this->datatype->get_id();
        $this->vals['basename']       = $this->convert_case($this->datatype->get_type()); // 'document' -> 'Document'
        $this->vals['basevarname']    = $this->convert_case($this->datatype->get_type());
        $this->vals['define']         = 'DATATYPE_' . strtoupper($this->datatype->get_type());
        
        ob_start();
            $this->classBlock();
            
        $c = ob_get_contents();
        ob_end_clean();
        
        return $c;
        
    }
    
    function classBlock()
    { 
?>

// Include class file for base abstract class
require_once(dirname(__FILE__) . "/Abstract<?= $this->vals['basevarname'] ?>.class.php");

// Register this class as the pxdb_record class to use for datatype <?= $this->vals['define'] ?>

pxdb::register_class('<?= $this->vals['basevarname'] ?>', 'pxdb_record', <?= $this->vals['define'] ?>);

/**
* This is a collection class for objects of type <?= $this->vals['basename'] ?>.
*
* This class extends the abstract class which in turn extends the pxdb_collection
* and provides object-specific methods which invoke
* parent generic methods.  This class provides some methods you might find useful --
* and more importantly, it provides a base from which you can build your own.
*
* Note that this class uses CamelHump capitalization so that methods of this
* class can be easily distinguished from the all-lowercase methods of the parent
* classes.
*
* Here's an example of how to use these classes (with hypothetical user functions):
*
* $<?= strtolower($this->vals['basevarname']) ?>_collection = new <?= $this->vals['basevarname'] ?>Collection();
* $<?= strtolower($this->vals['basevarname']) ?>_collection->findByAuthor($user->get_uid()); // current logged-in user's id.
* 
* while ($Record =& $<?= strtolower($this->vals['basevarname']) ?>_collection->fetchRecord())
* { 
*   // fetchRecord() returns a <?= $this->vals['basevarname'] ?> object --
*   // which extends the pxdb_record object and provides individual methods
*   // for getting attributes. 
*   print "<br>".$Record->getTitle();
* }
*
* If you need to only get a subset or a paged version of the query, you will 
* want to execute the query manually (otherwise default execute() is run when 
* you fetchRecord()).
*
* $<?= strtolower($this->vals['basevarname']) ?>_collection->execute_page($rows_per_page, $current_page);
* while ($Record =& $<?= strtolower($this->vals['basevarname']) ?>_collection->fetchRecord())
* { 
* ...
* OR
* $<?= strtolower($this->vals['basevarname']) ?>_collection->execute_limit($num_rows);
* while ($Record =& $<?= strtolower($this->vals['basevarname']) ?>_collection->fetchRecord())
* { 
* ...
*
* Remember also that the ADOdb recordset is returned from the ->execute() method.
* $rs  =& $<?= strtolower($this->vals['basevarname']) ?>_collection->execute();
* print "Rows found: ".$rs->RecordCount();
* while (!$rs ->EOF)
* {
*   print "Document name: ".$rs->fields['name'];
*   $rs->MoveNext();
* }
*
* @version build_user_class 1.2
* @author AutoGenerated (DBasis)
*/

class <?= $this->vals['basevarname'] ?>Collection extends Abstract<?= $this->vals['basevarname'] ?>Collection 
{ 
    /**
    * Constructor. 
    * Calls parent constructor
    * results sets this class will produce.
    */
    function <?= $this->vals['basevarname'] ?>Collection()
    {
        parent::init();
    }
    
} // end <?= $this->vals['basevarname'] ?>Collection class

/**
* This is a skeleton class for a class that represents 
* a record of type <?= $this->vals['basename'] ?>
* You can use this in place of using the pxdb_record class directly.
*
* @version build_class 1.2
* @author AutoGenerated (DBasis)
*/
class <?= $this->vals['basename'] ?> extends Abstract<?= $this->vals['basevarname'] ?> 
{
    /**
    * Constructor. 
    * Calls parent constructor.
    * @param mixed $data Record ID or record row (array)
    */
    function <?= $this->vals['basename'] ?>($data = null)
    {
        parent::init($data);
    }
    
    /**
    * Initialization method
    */
    function init($data)
    {
        parent::pxdb_record($data);
    }
    
} // end Abstract<?= $this->vals['basevarname'] ?> class

<?php 
    }
} 
?>