<?php
/**
* Base class for Syntax code generating templates in dbasis
*
* @author Oscar Merida
*/

class SyntaxCodeTemplate {
  
    /**
    * Datatype we are working with
    */
    
    var $datatype; 
    
    /**
    * Set Datatype we are working 
    */
    function setDatatype( $dt )
    {
        pxdb_import( 'metadata.pxdb_datatype' );
        $this->datatype = new pxdb_datatype( $dt );
    }
    
    function prettyPrint() {
        
        $code = $this->generate();

        highlight_string( "<?php\n".$code."\n?>" );    
        
    }
    
    function convert_case($name, $ucfirstword=true)
    {
        $pieces = preg_split("/[^a-zA-Z0-9]/", $name);
        $res = ''; $i=0;
        for($i=0, $z=sizeof($pieces); $i<$z; $i++) {
            $piece = strtolower($pieces[$i]); 
            if(!($ucfirstword==false && !$i)) {
                $piece = ucfirst($piece);
            }
            $res .= $piece;
        }
        return $res;
    }
    
    /**
    * Get Datatype we are working with, as a pxdb_datatype object
    */ 
    function getDatatype( $dt )
    {        
        return $this->datatype;
    }
    
    /**
    * Returns an array of the child code templates available from the same directory as this class
    *
    * @access public
    * @static
    * @return array
    */
    function getAvailable() {
        
        if ( $handle = opendir( dirname(__FILE__) ) ) 
        {
            while (false !== ($file = readdir($handle)))
            {
                if ($file != "." && $file != "..") 
                {
                    if ( preg_match( '/^class\.SyntaxCodeTemplate_(.+)\.php/', $file, $match ) )
                    {
                        $avail[] = $match[1];                        
                    }                    
                }
            }
            closedir($handle);
        }

        return $avail;
    }
    
    /**
    * Tries to instantiate a child class of the given name
    *
    * @param string name
    * @return object
    * @access public
    */
    function factory( $name, $dt ) 
    {
        $class_name = 'SyntaxCodeTemplate_'.$name;
        
        require_once( dirname(__FILE__).'/class.'.$class_name.'.php' );
        
        $b = new $class_name;
        $b->setDatatype( $dt );
        return $b;
    }
    
    /**
    * Abstract method to be overriden by child classes.   Generates the actual code.
    */
    function generate() 
    {
        trigger_error( 'Abstract method getCode() not implemented by child class.', E_USER_WARNING );
    }
}

?>
