<?php
/**
* Generates the php code for a classes detail template
*
* @author Oscar Merida
*/

class SyntaxCodeTemplate_DetailTemplate extends SyntaxCodeTemplate
{
    
   
    function generate()
    {
        $this->vals['datatype_label'] = $this->datatype->get_label();
        $this->vals['datatype_id']    = $this->datatype->get_id();
        $this->vals['basename']       = $this->convert_case($this->datatype->get_type()); // 'document' -> 'Document'
        $this->vals['basevarname']    = $this->convert_case($this->datatype->get_type());
        $this->vals['define']         = 'DATATYPE_' . strtoupper($this->datatype->get_type());
        
        ob_start();
            $this->startDetailTemplate();
            $this->fieldsRecord();
        $c = ob_get_contents();
        ob_end_clean();
        
        $c = str_replace( '{!', '<?php', $c );
        $c = str_replace( '!}', '?>', $c );
        $c = str_replace( ')?>', ') ?>', $c );
        $c = str_replace( '{=', '<?= ', $c );
        
        return $c;

    }

    
    function fieldsRecord()
    {
        // now for each typesfield ...
        $i = 0;
        while ($objf =& $this->datatype->fetch_typesfield((!$i++ ? true : null)))
        {
            $field = $objf->get_identifier();
            $field_basefuncname = $this->convert_case($field);
            $field_basevarname = $this->convert_case($field, false);
            
            
            
            if ($field)
            {
                switch( $objf->get_field_type() )
                {           
                                       
                    case 'relationship':
                    
                        $this->relField($field, $field_basefuncname, $field_basevarname);            
                    
                    break;
                
                    case 'simple':
                        $fieldtype = $objf->get_field_metatype();
                        $mul=$objf->get_mul();
                        
                        // clean up picks                        
                        if ( 'N' == $mul && 'pick'==$fieldtype )
                        {
                            // single picks can be handled directly 
                            $fieldtype = 'txt';
                        }
                        //clean up images
                        if ( preg_match( '/^image/', $field_basevarname ) )
                        {
                            $fieldtype = 'image';                            
                        }
                        switch( $fieldtype )
                        {
                            case 'pass2':
                            case 'phone':
                            case 'fax':
                            case 'phone':                            
                            case 'txt':
                                $this->textField( $objf->get_DisplayName(), $field_basefuncname, $field_basevarname );
                            break;
                            
                            case 'ltxt':
                                $this->longTextField( $objf->get_DisplayName(), $field_basefuncname, $field_basevarname );
                            break;
                            
                            case 'date':
                                $this->dateField( $objf->get_DisplayName(), $field_basefuncname, $field_basevarname );
                            break;
                            
                            case 'boolean':
                                $this->boolField( $objf->get_DisplayName(), $field_basefuncname, $field_basevarname );
                            break;
                            
                            case 'email':
                                $this->emailField( $objf->get_DisplayName(), $field_basefuncname, $field_basevarname );
                            break;
                            
                            case 'image':
                                $this->imageField( $objf->get_DisplayName(), $field_basefuncname, $field_basevarname );
                            break;
                            
                            case 'pickid':
                            case 'pick':
                                $this->pickField( $objf->get_DisplayName(), $field_basefuncname, $field_basevarname );
                            break;
                            
                            case 'upload':
                                $this->fileUploadField( $objf->get_DisplayName(), $field_basefuncname, $field_basevarname );
                            break;
                            
                            case 'url':
                                $this->UrlField( $objf->get_DisplayName(), $field_basefuncname, $field_basevarname );
                            break;
                            
                            default:
                                echo "($field) Field type: ".$objf->get_field_type()."\n";
                                echo "\n".$objf->get_field_metatype()." field type not implement yet.\n\n";
                            break;
                        }
                        
                    break;
                }
            } // if field
        } // while        

    }
        
    function textField( $field, $func, $var )
    {
?>        
{! 
    if ( null != $Record->get<?=$func?>() ) 
    {
!}  
        <p><?=ucwords($field)?></p>
        <p>{=$Record->get<?=$func?>()!}</p>
{!
    } // end get <?=$var?>
    
!}

<?php         
    } // end textField

    
    function longTextField( $field, $func, $var )
    {
?>        
{! 
    if ( null != $Record->get<?=$func?>() ) 
    {
!}  
        <p><?=ucwords($field)?></p>
        <p>{=StxUtilities::formatText( $Record->get<?=$func?>() )!}</p>
{!
    } // end get <?=$var?>
    
!}

<?php         
    } // end longTextField

    
    function pickField( $field, $func, $var )
    {
?>        
{! 
    // pick field <?=$var?>
    i have 
    // returns an array of picks
    if ( $picks = $Record->get<?=$func?>() ) 
    {
!}  
        <p><?=ucwords($field)?></p>
        <p>{=join(', ', $picks )!}</p>
{!
    } // end get <?=$var?>
    
!}

<?php         
    } // end textField

    
    function emailField( $field, $func, $var )
    {
?>        
{! 
    if ( null != $Record->get<?=$func?>() ) 
    {
!}  
        <p><?=ucwords($field)?></p>
        <p><a href="mailto:{=StxUtilities::obfuscateAscii($Record->get<?=$func?>())!}">{=StxUtilities::obfuscateAscii($Record->get<?=$func?>())!}</a></p>
{!
    } // end get <?=$var?>
    
!}

<?php         
    } // end textField

    function imageField( $field, $func, $var )
    {
?>        
{! 
    if ( null != $Record->get<?=$func?>() ) 
    {
!}  
        <p><?=ucwords($field)?></p>
        <p><img src="{=$Record->get<?=$func?>()!}" /></p>
{!
    } // end get <?=$var?>
    
!}

<?php         
    } // end imageField

    
    function dateField( $field, $func, $var )
    {
?>        
{! 
    if ( null != $Record->get<?=$func?>() ) 
    {
!}  
        <p><?=ucwords($field)?></p>
        <p>{=StxUtilities::formatDateString( $Record->get<?=$func?>() )!}</p>
{!
    } // end get <?=$var?>
    
!}

<?php         
    } // end dateField 

    function boolField( $field, $func, $var )
    {
?>        
{! 
    if ( null!==$Record->get<?=$func?>() ) 
    {
        $lbl = ( true == $Record->get<?=$func?>() ) ? 'Yes' : 'No';        
!}  
        <p><?=ucwords($field)?></p>
        <p>{=$lbl!}</p>
{!
    } // end get <?=$var?>
    
!}

<?php         
    } // end dateField 
    

    function relField( $field, $func, $var )
    {
?>        
{! 
    // relationship field '<?=$var?>'
    
    // you will usually want to customize this some more to display
    // other fields for an object, point links to the right module, etc...    
    if ($rel_objects = $Record->get<?= $func ?>('name')) 
    {
        $rel_ids = $Record->get<?= $func ?>('id');
        $links = array();
        for ($count = count($rel_ids), $i = 0; $i < $count; $i++)
        {
            $links[] = '<a href="/content/general/detail/' . $rel_ids[$i] . '">' . $rel_objects[$i] . '</a>';
        }
!}  
        <p><?= ucwords($field) ?></p>
        <p>{= join(', ', $links) !}</p>
{!
    } // end get <?=$var?>
    
!}
<?php 
    }

    function fileUploadField( $field, $func, $var )
    {
?>
{! 
    // file upload field '<?=$var?>'
            
    if ( $file = $Record->get<?=$func?>() ) 
    {
        require_once( SITE_LIB_PATH . '/ext/documentInfo/documentInfo.php' );
        $file_info = documentInfo::stxFactory( $file );
!}  
        <p><?=ucwords($field)?></p>

        <p><a href="{= $file !}">Download ( {= $file_info->get_filetype() !}, {= $file_info->get_filesize() !})</a></p>
{!
    } // end get <?=$var?>
    
!}
<?php 
    }
    
    function urlField( $field, $func, $var )
    {
?>
{! 
    // url field '<?=$var?>'
            
    if ( $<?=$var?> = $Record->get<?=$func?>() ) 
    {        
!}  
        <p><?=ucwords($field)?></p>

        <p><a href="{= $<?= $var ?> !}">{= $<?= $var ?> !}</a></p>
{!
    } // end get <?=$var?>
    
!}
<?php 
    }
    
    function startDetailTemplate()
    {
?>
/*    Detail template   */
<?php     
    echo"\n?>\n"; 
    }
} // end class
?>