<?php
/**
* Generates the php code needed to handle an input form 
*
* @author Oscar Merida
*/

class SyntaxCodeTemplate_ListCapability extends SyntaxCodeTemplate
{
    function generate()
    {
        ob_start();
        
            $this->initBlock();
                        
            $this->filterBlock();
            $this->collectionBlock();
            $this->orderByBlock();
            $this->executeBlock();
            $this->templateBlock();
            
        $c = ob_get_contents();
        ob_end_clean();
        
        return $c;
    }
       
    function initBlock() 
    {
?>
// Include module-specific init file.
include(dirname(__FILE__) . '/init/init.php'); 

<?php
    } // end errorBlock

    function filterBlock() 
    {
        $dt_define = 'DATATYPE_' . strtoupper($this->datatype->get_type());
?>        
$filter =& pxdb_search::filter(<?= $dt_define ?>); 

// filter by sunrise/sunset (string is already quoted)
$filter->add_value('sunset', DB_TIME_STAMP, '>=', null, false);
$filter->add_value('sunrise', DB_TIME_STAMP, '<=', null, false);

<?php
        $this->fieldFiltersBlock();
?>
// filter by section publication if needed
if ( null !== $Request->getVar( 'in_section' ) )
{
    // add to main filter
    if( 1 == $Request->getVar( 'featured' ) )
    {
        $filter->add_parent( $Request->getVar( 'in_section' ), NODEREL_SECTION_TO_FEATURED, 'ASC' );
    } 
    else
    {
        //assume publication & order by section weight
        $filter->add_parent( $Request->getVar( 'in_section' ), NODEREL_SECTION_TO_CONTENT, 'ASC' );
    }
    $node_weighted = true;
}
else
{
    $node_weighted = false;
}

<?php
    }
    
    function orderbyBlock() 
    {
?>        
if ($Request->getVar('orderby'))
{
    $Records->set_order($Request->getVar('orderby'));
}
else
{
    // set a default order by, usually sunrise
    $Records->set_order('sunrise');
} // end if

<?php         
    }
    
    function collectionBlock() 
    {
        $name = $this->datatype->get_type();
        $name = ucwords( str_replace( '_', ' ', $name ) );
        $name = str_replace( ' ', '', $name );
        $collection_define = $name . 'Collection';
?>
// create collection
$Records = new <?= $collection_define ?>();

<?php
    }
    
    function executeBlock() 
    {
?>
// bind the filter first 
$Records->find($filter);

// limit recordset results
if (null !== $Request->getVar('limit'))
{
    if (false === $Records->execute_limit($Request->getVar('limit')))
    {
        trigger_error('Execute Limit failed', E_USER_WARNING);
    }    
} // paginate recordset
else if (null !== $Request->getVar('page_no') && null !== $rows_per_page) 
{
    if (false === $Records->execute_page($rows_per_page, $Request->getVar('current_page')))
    {
        trigger_error('Execute Page failed', E_USER_WARNING);
    }
// show all records    
}
else
{
    $Records->execute();
}

<?php
    }
    
    function templateBlock() 
    {
?>
// generate output
if (!$Request->isIncluded())
{
    include_once(SITE_TPL_PATH . '/header.tpl');
} // end if included

// if there are no records do not output anything at all.
if (0 < $Records->num_records())
{
    if (null !== $Request->getVar('template')) 
    {    
        include($MODULE_TPL_PATH . '/' . $Request->getVar('template') . '.tpl');
    }
    else
    {
        include($MODULE_TPL_PATH . '/list.tpl');
    } // end if no template var
} // end if no records

if (!$Request->isIncluded())
{    
    include_once(SITE_TPL_PATH . '/footer.tpl');
} // end if included
<?php
    }
    
    function fieldFiltersBlock() 
    {
        //echo '<pre>'; print_r($this->datatype); echo '</pre>';
        while($objf =& $this->datatype->fetch_typesfield()) 
        {
            //echo '<pre>'; print_r($objf); echo '</pre>';
            $field = $objf->get_identifier();
            $fieldtype = $objf->get_field_metatype();
            
            switch($fieldtype) 
            {
                case 'primaryrel':
                case 'pick':
                case 'pickid':
                case 'boolean':
                    $this->varFilterBlock($field, $fieldtype);
                break;
                
                default:
                    // do nothing
                    //echo "$field $fieldtype\n";
                break;
            } // end switch 
        } // end while
    }
    
    function varFilterBlock($field, $type)
    {
?>
// filter by <?= ucwords($field) ?> (<?= $type ?>) if needed
if (null !== $Request->getVar('<?= $field ?>'))
{
    $filter->add_value('<?= $field ?>', $Request->getVar('<?= $field ?>'));
}        
            
<?php         
    }
}
?>