<?php
// $Id: mysql-upgrade.php,v 1.1.1.1 2004/09/10 19:27:31 omerida Exp $

#  This file should only be included by the run-once.php script.
#  -------------------------------------------------------------
#
#  This file is responsible for making any changes to the database that
#  come about as a result of an upgrade to a newer Syntax instance.  The
#  file will do its best to bring things up-to-date.
#
#
#  $pxdb_prefs->get_pref('version') should return the database's version.  If
#  this pref is not set assume that either (1) it has been removed from the db 
#  or (2) the db is older than version 0.0.3.
#
#  The contant PXDB_VERSION should contain the current Syntax version.
#
#  We can assume that if this script is beng called, that there is a reason, but we
#  cannot make assumptions about which components are out of sync.  Careful testing
#  needs to precede any DB modification.
#
#
#  Also: this code is RDBMS specific, since it may involve creating tables, etc.
#
#

$dbh = &${$PXDB_ADODB_VAR};  // alias -- for simplicity sake



# 0.0.3
# Change `priv`.`rec_priv` from a SET to individual `read` and `write` columns
# Change `field_priv`.`f_priv` from a SET to individual `read` and `write` columns

// 1) Does the db qualify for this upgrade?
	
	$priv_columns = $dbh->MetaColumnNames('priv');
	// a) priv
	if(in_array('rec_priv', $priv_columns)) {
	
		// Add the new fields
		if(!in_array('read', $priv_columns)) {
			$sql = "ALTER TABLE `priv` ADD COLUMN `read` TINYINT NOT NULL";
			$dbh->Execute($sql);
			$sql = "ALTER TABLE `priv` ADD INDEX (`read`)";
			$dbh->Execute($sql);
		}
		
		if(!in_array('write', $priv_columns)) {	
			$sql = "ALTER TABLE `priv` ADD COLUMN `write` TINYINT NOT NULL";
			$dbh->Execute($sql);
			$sql = "ALTER TABLE `priv` ADD INDEX (`write`)";
			$dbh->Execute($sql);
		}
					
		// Update new fields		
		$sql = "UPDATE `priv` SET `read` = IF(FIND_IN_SET('read', rec_priv), 1, 0)";
		$dbh->Execute($sql);
		
		$sql = "UPDATE `priv` SET `write` = IF(FIND_IN_SET('write', rec_priv), 1, 0)";
		$dbh->Execute($sql);
		
		// drop the old column
		$sql = "ALTER TABLE `priv` DROP COLUMN `rec_priv`";
		$dbh->Execute($sql);

	} // if `rec_priv` column exists

	$field_priv_columns = $dbh->MetaColumnNames('field_priv');
	// b) field_priv
	if(in_array('f_priv', $field_priv_columns)) {
	
		// Add the new fields
		if(!in_array('read', $field_priv_columns)) {
			$sql = "ALTER TABLE `field_priv` ADD COLUMN `read` TINYINT NOT NULL";
			$dbh->Execute($sql);
			$sql = "ALTER TABLE `field_priv` ADD INDEX (`read`)";
			$dbh->Execute($sql);
		}
		
		if(!in_array('write', $field_priv_columns)) {
			$sql = "ALTER TABLE `field_priv` ADD COLUMN `write` TINYINT NOT NULL";
			$dbh->Execute($sql);
			$sql = "ALTER TABLE `field_priv` ADD INDEX (`write`)";
			$dbh->Execute($sql);
		}
		
		// Update new fields		
		$sql = "UPDATE `field_priv` SET `read` = IF(FIND_IN_SET('read', f_priv), 1, 0)";
		$dbh->Execute($sql);
		
		$sql = "UPDATE `field_priv` SET `write` = IF(FIND_IN_SET('write', f_priv), 1, 0)";
		$dbh->Execute($sql);			
		
		// drop the old column
		$sql = "ALTER TABLE `field_priv` DROP COLUMN `f_priv`";
		$dbh->Execute($sql);
		
	} // if f_priv column exists

	
	if($pxdb_prefs->get_pref('coltype','txt')) { // 'txt' is arbitrary, but a good test
		// if system is using old coltype pref style, then remove all the coltype prefs.
		$pxdb_prefs->del_pref('coltype');
	}
	
	
	# FIELD_PRIV
	# change `read` -> `priv_read`
	if(in_array('read', $field_priv_columns)) {
		$sql = "ALTER TABLE field_priv CHANGE COLUMN `read` `priv_read` TINYINT";
		$dbh->Execute($sql);
	}
	# change `write` -> `priv_write`
	if(in_array('write', $field_priv_columns)) {
		$sql = "ALTER TABLE field_priv CHANGE COLUMN `write` `priv_write` TINYINT";
		$dbh->Execute($sql);
	}
	
	# PRIV
	if(in_array('read', $priv_columns)) {
		$sql = "ALTER TABLE priv CHANGE COLUMN `read` `priv_read` TINYINT";
		$dbh->Execute($sql);
	}
	# change `write` -> `priv_write`
	if(in_array('write', $priv_columns)) {
		$sql = "ALTER TABLE priv CHANGE COLUMN `write` `priv_write` TINYINT";
		$dbh->Execute($sql);
	}
	
		
	# Changing relationships table
	#
	#
	
	$rel_columns = $dbh->MetaColumnNames('relationships');
	if(in_array('relid', $rel_columns)) {
	
		$dbh->BeginTrans();
		$ok=true;
			
		// 1) Create new table: datatypesrel
			
			if(!in_array('datatypesrel', $dbh->MetaTables())) {

				$sql = "CREATE TABLE datatypesrel (
							relid int NOT NULL ,
							dt1 int NOT NULL ,
							dt2 int NOT NULL,
							PRIMARY KEY (relid,dt1,dt2)
						)";
						
				if($ok) $ok = $dbh->Execute($sql);
				
			}
			
		// 2) Convert data
		
		$sql = "SELECT DISTINCT relid FROM relationships";
		$relids = $dbh->GetCol($sql);		
		
		if($relids = array_diff($relids, array(''))) {			
			
			// add a temporary column to relationships
			$sql = "ALTER TABLE relationships ADD COLUMN tmp_id INT";
			if($ok) $ok = $dbh->Execute($sql);
			
			foreach($relids as $relid) {
									
				$dt1 = array();
				$dt2 = array();				
				
				// find all datatypes for given relid
				$sql = "SELECT * FROM relationships WHERE relid = $relid";
				$rs = &$dbh->Execute($sql);
				
				// aggregate the datatypes
				while(!$rs->EOF) {
					$description = $rs->fields['description']; // sure, keeps getting overwritten
					$weight = $rs->fields['weight']; // overwritten
					$dt1[] = $rs->fields['dt1']; // aggregate
					$dt2[] = $rs->fields['dt2']; // aggregate
					$rs->MoveNext();
				}					
							
				// add the new record with the ID field set to what used to be relid
				$sql = "INSERT INTO relationships (
							tmp_id,
							description, 
							weight
						) VALUES (
							$relid, 
							".$dbh->qstr($description).", 
							$weight)";

				if($ok) $ok = $dbh->Execute($sql);		
				
				// add datatypes to rel table
				foreach($dt1 as $dt1_piece) {			
					foreach($dt2 as $dt2_piece) {
						$sql = "DELETE FROM datatypesrel WHERE relid = $relid AND dt1 = $dt1_piece AND dt2 = $dt2_piece";
						if($ok) $ok = $dbh->Execute($sql);
						// insert relationship
						$sql = "INSERT INTO datatypesrel (relid, dt1, dt2) VALUES ($relid, $dt1_piece, $dt2_piece)";
						if($ok) $ok = $dbh->Execute($sql);											
					} // foreach($dt2)	
				} // foreach($dt1)
			
				// now delete the current records and create a new one
				$sql = "DELETE FROM relationships WHERE relid = $relid";
				if($ok) $ok = $dbh->Execute($sql);										
				
			} // foreach

		} // if results
		
		// now set the ID to the tmp_id -- shouldn't be any duplicates ????		
		$sql = "UPDATE relationships SET id = tmp_id";
		if($ok) $ok = $dbh->Execute($sql);
		
		// now drop the tmp column
		$sql = "ALTER TABLE relationships DROP COLUMN tmp_id";
		if($ok) $ok = $dbh->Execute($sql);		
		
		// now drop the relid column -- all information is updated
		$sql = "ALTER TABLE relationships DROP COLUMN relid";
		if($ok) $ok = $dbh->Execute($sql);
	
			// now drop the relid column -- all information is updated
		$sql = "ALTER TABLE relationships DROP COLUMN dt1";
		if($ok) $ok = $dbh->Execute($sql);
		
				// now drop the relid column -- all information is updated
		$sql = "ALTER TABLE relationships DROP COLUMN dt2";
		if($ok) $ok = $dbh->Execute($sql);		
		
		$dbh->CommitTrans($ok);	
	}	

	
	# Once all updates have been done, update the preference to current version (so this script won't run again)
	
	$pxdb_prefs->set_pref('version', PXDB_VERSION);
	
?>
