<?php
// $Id: tabset.class.php,v 1.2 2005/06/03 20:44:50 sandys1 Exp $

/**
* This is a class to manage a set of tabs.
*
* This class works in conjunction with the tab class to provide an flexible system
* to create and display tabs.
*
* Synopsis
* --------
*
* # Setting up the tabs
* # -------------------
*
* $tabs = new tabset()
*
* // use this function if all tabs are going to have the same link type
* // otherwise you can pass a link to the add_tab() method -- or use the set_link() method of a tab object.
* $tabs->set_default_link("javascript: loadTab('[tab_name]', [tabset_depth])");
*
* // set display options for the tabset
* $tabs->set_css('color: #ffcc00; background-color: #000000', 'row');
* $tabs->set_css('color: #ffcc00; background-color: #000000', 'tab', 'active'); // or use a CSS class name!
* $tabs->set_css('color: #ffcc00; background-color: #000000', 'tab', 'default');
* $tabs->set_css('color: #ffcc00', 'link', 'active');
* $tabs->set_css('color: #ffcc00', 'link', 'default');
*
* // add tabs to this tabset
* $tabs->add_tab('datatypes');
* $obj_tab =& $tabs->add_tab('object'); //NOTE: we capture the tab object here so we can add a subtabset later.
* $tabs->add_tab('dbfields', 'Fields');
* $tabs->add_tab('relationships');
* $tabs->add_tab('preferences', 'Preferences', '/prefs.php');
*
* // set display options for sub-tabset
* $obj_subtabs->set_css('color: #ffcc00; background-color: #cccccc', 'row');
* $obj_subtabs->set_css('color: #ffcc00; background-color: #cccccc', 'tab', 'active');
* $obj_subtabs->set_css('color: #ffcc00; background-color: #cccccc', 'tab', 'default');
* $obj_subtabs->set_css('color: #ffcc00', 'link', 'active');
* $obj_subtabs->set_css('color: #ffcc00', 'link', 'default');
*
* // add tabs to the object subtab
* $obj_subtabs =& $obj_tab->add_subtabs();
* $obj_subtabs->add_tab('fields');
* $obj_subtabs->add_tab('privs');
*
*
* # Displaying the tabs
* # -------------------
*
* // once everything is setup, just use show() method.
* // which will show the top level (of object) tabset and any sub tabsets
* <?=$tabs->show()?> // or print $tabs->show(); from within PHP context.
*
*/
class tabset
{
    var $parent; // if this is a subtab, this holds parent
    var $tabs;
    var $jscript_printed; // true/false, has javascript been printed (can only be done once);
    var $depth;

    // display stuff
    var $row_css_type;
    var $row_css_style;

    var $sel_css_type;
    var $sel_css_style;

    var $gap;

    /**
    * @var int
    * @access private
    */
    var $gap_width;


    var $path_separator = ":";

    function tabset()
    {
        $this->depth = 0;

        $this->tabs = array();

        $this->gap = 5;
    }

    /**
    * Display: Sets the space between tabs.
    * @param $gap The gap (px) between tabs.
    */
    function set_gap_width($gap)
    {
        $this->gap_width = $gap;
    }

    /**
    * Sets the CSS for a component of the tabset.
    * @param $state Whether we are setting a CSS for active or default tab. (i.e. 'active' / 'default')
    * @param $element Which element we are setting CSS for -- e.g. link or tab.
    * @param $css The actual style text or CSS class name.
    */
    function set_css($css, $element, $state=false)
    {
        if ($state === false) $state = 'default';


        if (preg_match("/[\s]|[:]/",$css))
        {
            $this->css_type[$element][$state] =  "style";
        }
        else
        {

            $this->css_type[$element][$state] = "class";
        }

        $this->css_style[$element][$state] = $css;

    }


    /**
    * Create a new tab object.
    * @return object Tab we just created.
    */
    function &add_tab($name, $label=false, $link=false)
    {
        // create the instance

        $tab =& new tab($name, $label, $link);

        // set link

        if ($link === false)
        {
            $link = $this->default_link;
        }

        $tab->set_link($link);

        // specify that this set owns the tab we created

        $tab->set_owner($this);

        // add it to our array

        $this->tabs[] =& $tab;

        return $tab;

    }

    /**
    *
    */
    function replace_tags($str, &$tab)
    {
        // replace any occurences of [tab_*] with $tab->get_*()
        //-- e.g. [tab_name] is replaced by the value returned by $tab->get_name()
        $str = preg_replace("/{tab\->([^\]]+)}/ie", "\$tab->get_$1()", $str);

        $str = preg_replace("/{tabset\->([^\]]+)}/ie", "\$this->get_$1()", $str);

        return $str;
    }

    /**
    * Sets a default link to add to any links we add.
    */
    function set_default_link($link)
    {

        $this->default_link = $link;
    }

    /**
    * Find a tab matching a given name.
    * @return Object A Reference to the tab that was found.
    */
    function &find_tab($name)
    {
        // can't use foreach if we want to actually return a reference to the desired tab
        for($i=0; $i < sizeof($this->tabs); $i++)
        {
            $tab =& $this->tabs[$i];
            if ($tab->get_name() == $name)
            {
                return $tab;
            }
        }
        return false; // not found
    }

    /**
    * Set the depth (i.e. 0 = top tab, 1 = first subtab) of current tabset.
    * @param $depth The depth to set.
    */
    function set_depth($depth)
    {
        $this->depth = $depth;
    }

    /**
    * Sets the parent [Object tab] tab if this is not the top-level.
    */
    function set_parent(&$parent)
    {
        $this->parent =& $parent;
    }


    /**
    * Gets the depth of this tabset.
    */
    function get_depth()
    {

        return $this->depth;
    }


    /**
    * Outputs the current tabset.
    */
    function show_tabset($selected)
    {


        $row_css = '';
        $col_css = '';
        if (true === isset($this->css_type['row']['default']))
        {

            $row_css = $this->css_type['row']['default']."=\"".$this->css_style['row']['default']."\"";
        }

        $out = "<tr $row_css><td><table cellspacing=\"0\" cellpadding=\"5\" width=\"100%\"><tr>";

        for($i=0; $i<sizeof($this->tabs); $i++)
        {


            $tab =& $this->tabs[$i];

            if (true === isset($this->css_type['tab']['default']))
            {

                $col_css = $this->css_type['tab']['default']."=\"".$this->css_style['tab']['default']."\"";
            }

            if ($this->css_type['link']['default'])
            {

                $link_css = $this->css_type['link']['default']."=\"".$this->css_style['link']['default']."\"";
            }

            // override the values we set with the ones for "active" -- if defined...

            if ($tab->get_name() == $selected)
            {


                if ($this->css_type['tab']['active'])
                {

                    $col_css = $this->css_type['tab']['active']."=\"".$this->css_style['tab']['active']."\"";
                }

                if ($this->css_type['link']['active'])
                {

                    $link_css = $this->css_type['link']['active']."=\"".$this->css_style['link']['active']."\"";
                }


            }


            $out .= '<td align="left" valign="bottom"' . (true === is_numeric($this->gap_width) ? ' width="' . $this->gap_width . '"' : '') . "></td>\n";
            $out .= "<td align=\"center\" $col_css>\n";
            $out .= "<a href=\"".$tab->get_link()."\" $link_css>".$tab->get_tab()."</a></td>\n";
            $out .= "<td align=\"right\" valign=\"bottom\"" . (true === is_numeric($this->gap_width) ? ' width="' . $this->gap_width . '"' : '') . "></td>";

        }

        $out .= "</tr></table></td></tr>";

        return $out;

    }


    /**
    * Outputs the current tabset and any sub-tabsets.
    * We could move this to using a templating system -- but that's a lot
    * of overhead for not much more flexibility.
    *
    * @param $subtabs Array of currently-selected tabs.
    */
    function show($subtabs=false)
    {


        // subtabs will be like 'object', 'fields'

        // we need to show the current tabset

        $out = '<table cellspacing="0" cellpadding="0" width="100%">';

        $ts =& $this; // start out with current object

        for($i=0; $ts !== false; $i++)
        {


            $out .= $ts->show_tabset($subtabs[$i]);
            //print "<pre>"; print_r($ts); print "</pre>";

            if ($subtabs[$i])
            {

                $sel =& $ts->find_tab($subtabs[$i]);
                $ts =& $sel->get_subtabs();
            }
            else
            {

                $ts = false;
            }

        }


        $out .= "</table>";

        // then we need to show any subtabs which are in the subtabs array

        return $out;
    }


}


/**
* Class representing a single tab in a set of tabs.
* This class is instantiated by the tabset class, and referenced by the tabset class for display.
* @access private
*/
class tab
{
    var $label;
    var $image;
    var $name;
    var $link;

    var $owner; // this tab belongs to an instance of TabSet

    var $sub; // this tab may also be the parent to some subtabs.

    function tab($name, $label, $link)
    {
        // vars for this tab
        $this->label = $label;
        $this->name = $name;
    }

    /**
    * Recursive function to return the path of the current tab.
    *
    */
    function get_path($node=false)
    {
        $path = array();

        if ($node===false) $node =& $this;

        if ($node->owner->parent)
        {
            $path = $this->get_path(&$node->owner->parent).$node->owner->path_separator.$node->get_name();
        }
        else
        {
            //print "toplevel: ".$node->get_name();
            $path = $node->get_name();
        }

        return $path;
    }

    function get_name()
    {
        return $this->name;
    }

    function get_image()
    {
        return $this->image;
    }

    function get_label()
    {
        if ($this->label === false)
        {
            return $this->name;
        }
        else
        {
            return $this->label;
        }
    }

    function get_link()
    {
        return $this->owner->replace_tags($this->link, $this);
    }

    function set_owner(&$owner)
    {
        $this->owner =& $owner;
    }

    function set_link($link)
    {

        $this->link = $link;
    }

    function set_image($image)
    {

        $this->image = $image;
    }

    /**
    * gets the label or image (if set) for the tab.
    */
    function get_tab()
    {
        if ($this->get_image())
        {
            return $this->get_image();
        }
        else
        {
            return $this->get_label();
        }
    }

    function &add_subtabs()
    {
        $sub = new tabset();
        $this->sub =& $sub;
        $sub->set_parent($this);
        $sub->set_depth($this->owner->depth + 1);
        $sub->set_default_link($this->owner->default_link);

        $sub->path_separator = $this->owner->path_separator;

        return $sub;
    }

    function &get_subtabs()
    {
        return ($this->sub ? $this->sub : false);
    }
}

?>