<?php 
/**
* simple library to generate output filters for display
*
* put here to corral outside of the variable scope
*
* @author Oscar Merida <omerida@forumone.com>
*/
class DspListFilters 
{
    /**
    * Extracts all pickid type fields from the view for use as filter widgets
    *
    * @param object view object
    * @param array current pick values 
    */
    function getFromView($view, $current_values, $user)
    {
        $typesfields = $view->arr_typesfields();
        
        pxdb_import('content.input.pxdb_input');
        $input = new pxdb_input($view->get_datatype() );
        
        $widgets = array();
        while (list(, $typesfield_info) = each ($typesfields)) 
        {
            $typesfield = new pxdb_typesfield($typesfield_info['id']);
            
            if ($typesfield->is_readable($user))
            {
                switch ($typesfield->get_field_metatype())
                {
                    case 'pick':
                    
                        //picked value will be in $_SESSION[object_name]['admin_filters'], $identifier
                        $input->_get_options($typesfield);     
                        $identifier = $typesfield->get_identifier();
                        $current = $current_values[$typesfield->get_identifier()];                
                        $widget_list = new ListWidget($input->_options[$typesfield->get_id()], $identifier, $current);
                        $widget_list->setDefaultElement(array('value' => '', 'label' => 'Any  ' . $typesfield_info['displayname'] . ""));        
                        $widgets[$identifier] = array('label' => $typesfield_info['displayname'], 'object' => $widget_list->selectList());
                    
                    break;
                    
                    case 'pickid':
                    
                        //picked value will be in $_SESSION[object_name]['admin_filters'], $identifier
                        $input->_get_options($typesfield);     
                        $identifier = $typesfield->get_identifier();
                        $current = $current_values[$typesfield->get_identifier()];                
                        $widget_list = new ListWidget($input->_options[$typesfield->get_id()], $identifier, $current);
                        $widget_list->setDefaultElement(array('value' => '', 'label' => 'Any  ' . $typesfield_info['displayname'] . ""));        
                        $widgets[$identifier] = array('label' => $typesfield_info['displayname'], 'object' => $widget_list->selectList());
                    
                    break;
                    
                    case 'primaryrel':
                    
                        //picked value will be in $_SESSION[object_name]['admin_filters'], $identifier
                        $input->_get_options($typesfield);     
                        $identifier = $typesfield->get_identifier();
                        $current = isset($current_values[$typesfield->get_identifier()]) 
                                    ? $current_values[$typesfield->get_identifier()] : null;
                        $widget_list = new ListWidget($input->_options[$typesfield->get_id()], $identifier, $current);
                        $widget_list->setDefaultElement(array('value' => '', 'label' => 'Any  ' . $typesfield_info['displayname'] . ""));        
                        $widgets[$identifier] = array('label' => $typesfield_info['displayname'], 'object' => $widget_list->selectList());
                    
                    break;
                    
                    case 'boolean':
                    
                        $opts[0]['label'] = $typesfield_info['displayname'] . ' - yes';
                        $opts[0]['value'] = 1;
                        $opts[1]['label'] = $typesfield_info['displayname'] . ' - no';
                        $opts[1]['value'] = 0;
                        
                        $identifier = $typesfield->get_identifier();
                        $current = $current_values[$typesfield->get_identifier()];
                        
                        $widget_list = new ListWidget($opts, $identifier, $current);
                        $widget_list->setDefaultElement(array('value' => null, 'label' => 'Any  ' . $typesfield_info['displayname'] . ""));        
                        $widgets[$identifier] = array('label' => $typesfield_info['displayname'], 'object' => $widget_list->selectList());
                        
                    break;
                    
                    default:
                    
                        trigger_error('Unsupported field type in view: ' . $typesfield->get_field_metatype(), E_USER_WARNING);
                    
                    break;
                }
            }
        }
        
        return $widgets;
    }
    
    /**
    * Extracts all pickid type fields from the object's fields for use as filter widgets
    *
    * @param integer datatype id
    * @param array current pick values
    * @param array current pick values
    * @return an array of widgets
    */
    function getFromTypesfield($dt, $current_values, $user)
    {
        pxdb_import('content.input.pxdb_input');
        
        $dtinfo = new pxdb_datatype($dt);
        $input = new pxdb_input($dt);
                
        $widgets = array(); //init       
        while ($typesfield = $dtinfo->fetch_typesfield()) 
        {
            if ($typesfield->is_readable($user))
            {
                switch ($typesfield->get_field_metatype())
                {
                    case 'pick':
                    
                        //picked value will be in $_SESSION[object_name]['admin_filters'], $identifier
                        $input->_get_options($typesfield);     
                        $identifier = $typesfield->get_identifier();
                        $current = (true === isset($current_values[$typesfield->get_identifier()]))
                            ? $current_values[$typesfield->get_identifier()] : null;
                        $widget_list = new ListWidget($input->_options[$typesfield->get_id()], $identifier, $current);
                        $widget_list->setDefaultElement(array('value' => '', 'label' => 'Any  ' . $typesfield->get_displayname()));
                        $widgets[$identifier] = array('label' => $typesfield->get_displayname(), 'object' => $widget_list->selectList());
                    
                    break;
                    
                    case 'pickid':
                    
                        //picked value will be in $_SESSION[object_name]['admin_filters'], $identifier
                        $input->_get_options($typesfield);     
                        $identifier = $typesfield->get_identifier();
                        $current = (true === isset($current_values[$typesfield->get_identifier()]))
                            ? $current_values[$typesfield->get_identifier()] : null;
                        $widget_list = new ListWidget($input->_options[$typesfield->get_id()], $identifier, $current);
                        $widget_list->setDefaultElement(array('value' => '', 'label' => 'Any  ' . $typesfield->get_displayname()));
                        $widgets[$identifier] = array('label' => $typesfield->get_displayname(), 'object' => $widget_list->selectList());
                    
                    break;
                    
                    case 'boolean':
                    
                        $opts[0]['label'] = $typesfield_info['displayname'] . ' - yes';
                        $opts[0]['value'] = 1;
                        $opts[1]['label'] = $typesfield_info['displayname'] . ' - no';
                        $opts[1]['value'] = 0;
                        
                        $identifier = $typesfield->get_identifier();
                        $current = $current_values[$typesfield->get_identifier()];
                        
                        $widget_list = new ListWidget($opts, $identifier, $current);
                        $widget_list->setDefaultElement(array('value' => null, 'label' => "Any " . $typesfield->get_displayname()));        
                        $widgets[$identifier] = array('label'=> $typesfield->get_displayname(), 'object' => $widget_list->selectList());
                    
                    break;
                }
            }
        }       
        
        return $widgets;
    }
}
?>