<?php

if (!defined('DT_NODE'))
{
    define('DT_NODE', 9);
}

if (!defined('END_OF_TIME_STR'))
{
    define('END_OF_TIME_STR', '3000-01-01 00:00:00');
}

$v = &pxdb_validator::singleton();

function check_date($params, &$validator)
{
    $error = '';
    // print "<br>built = ".date("Y-m-d H:i:s", strtotime($params['value']));
    // print "<br>test = ".$params['value'];
    
    if (
        false === strpos(date("Y-m-d H:i:s", strtotime($params['value'])), $params['value']) && 
        false === strpos(date("Y-n-j H:i:s", strtotime($params['value'])), $params['value']) &&        
        false === strpos(END_OF_TIME_STR, $params['value'])
    )
    {
        $error = 'The date you entered is not a valid date.';
    }
    return $error;
}

$v->register_type_validator('date', 'check_date');

/**
* Post-processor to get rid of any non alphanumeric chars.
*/
function minimize($params, &$validator)
{
    extract($params);
    $value = trim($value);
    
    // if this is a section link check to see if it needs to be encoded
    if ($_REQUEST['is_link']) 
    {
	    // Only encode it if it hasn't already been encoded
	    if (!preg_match('/^\*64_/', $value))
	    {
		    $value = pxdb_tools::encode_section_link($value);
	    }
    }
    else
    {
	    // If it's a section page then replace any non-alphanumeric characters
	    $value = strtolower(preg_replace("/[^\w]+/", "_", $value));
    }
    return $value;
}

$v->register_field_processor('url_id', 'minimize', DT_NODE);

function urlize($params, &$validator)
{
	extract($params);
	$value = trim($value);
    
    if ($value && 0 !== strpos($value, '/')) 
    {
		if(!preg_match("/http[s]?:\/\//i", $value)) {
			$value = 'http://' . $value;
		}
	}    
    //encode just spaces, using urlencode or rawurlencode wipes out / and other chars
    $value = str_replace( ' ', '%20', $value);
	return $value;
}

$v->register_type_processor('url', 'urlize');


function check_url($params, &$validator)
{
    extract($params);
    $error = '';
    if (preg_match("/[\s\"\<>']/i", $value))
    {
        $error = 'The URL you entered contains invalid characters.';
    }
    return $error;
}

$v->register_type_validator('url', 'check_url');

function maxlen_255($params, &$validator)
{
    $error='';
    if (255 < strlen($params['value']))
    {
        $error = 'You cannot enter more than 255 characters for this value.';
    }
    return $error;
}

?>