<?php
// record privs
pxdb_import('record_privs');

/**
* Lookup datatype info based on name.
* @param string $datatype The datatype name to lookup.
*/
function lookup_datatype($datatype)
{
	$dbh = &pxdb::dbh();
	$sql = "SELECT * FROM datatypes WHERE type LIKE ".$dbh->qstr($datatype);
	$rs = $dbh->Execute($sql);
	return $rs->fields;
}

/**
* Gets the fields for given datatype.
* @param $datatype The datatype name or id.
*/
function get_datatype_fields($datatype)
{
	$dbh = &pxdb::dbh();
    $sql = "SELECT tf.id, tf.identifier, tf.displayname AS label
            FROM	datatypes dt
            INNER JOIN typesfields tf ON tf.typeid = dt.id
            WHERE	dt.".(!is_numeric($datatype) ? 'type' : 'id')." like ".$dbh->qstr($datatype)."
            ORDER BY tf.orderby";

    $rs = $dbh->Execute($sql);
		
    return $rs->GetArray();
}

/**
* Formats a field value for display.
* @param mixed $val The field value to format.
* @return string Formatted field value.
*/
function format_field($val)
{				
	if (is_array($val)) 
    {
		// array of pxdb_record objects (i.e. related)?
		// for now standard handling (this can be extended later)        
		if ( isset($val[0]) && is_object($val[0])) 
        {
			$txt = '';
			for ($c=0, $d=0, $z=sizeof($val); $d < $z; $d++) 
            {
				$obj = &$val[$d];
				$txt .= ($c++ ? ', ' : '').$obj->get_field('name');
			}
			$val = $txt;
		} 
        else 
        {
			$val = implode(', ', $val); 
		}				
	}
	return $val;
}


/**
* Template function for displaying a sortable label.
* @param string $field
* @param string $prev_direction 'DESC' or 'ASC'
* @return string 'ASC' or 'DESC'
*/
function sort_direction($field, $prev_direction = 'DESC') 
{
	// global $Persist;	
	
	// basically this function will create something like <a href="...?orderby_field=$field">$label</a>
    /*
	if($Persist->get_param('orderby_field') != $field) {
		$new_direction = "ASC";	
	} else {	
		$new_direction = ($Persist->get_param('orderby_direction') == "ASC" ? "DESC" : "ASC");
	}
	*/    
	return ('ASC' == $prev_direction) ? 'DESC' : 'ASC';
	// return $new_direction;
}

/**
* Reorder the weights
* @param integer $relid
* @parem integer $parent the parent ID
*/
function reorder($relid, $parent) 
{	
	$dbh = &pxdb::dbh();
	// for curr datatype

	$sql = "SELECT 	r.id
			FROM 	records parent	
					INNER JOIN primaryrel xref ON xref.relid = $relid AND xref.id1 = parent.id
					INNER JOIN records r ON r.id = xref.id2
					LEFT JOIN weights ON weights.relid = xref.relid AND weights.recid = r.id
			WHERE	parent.id = $parent
			ORDER BY weights.weight ASC, r.name ASC";
				
	$rs = &$dbh->Execute($sql);

	for($oby=0; !$rs->EOF; $oby += 5) {																							
		$sql = "REPLACE INTO weights (recid, relid, parentid, weight) VALUES (".$rs->fields['id'].", $relid, $parent, $oby)";
		# print "<br>$sql";
		$dbh->Execute($sql);			
		$rs->MoveNext();
	}
	$rs->close();
}

/**
* Promote / demote record for given relationship.
* @param integer $id
* @param integer $relid
* @param string $dir 'up' / 'down'
*/
function move($id, $relid, $dir, $parent) 
{	
	
	$dbh = &pxdb::dbh();
	
	// 0) make sure there are entries for this relationship
	$sql = "select count(*) as found from weights where relid = $relid and recid = $id and parentid = $parent";
	$test_rs = $dbh->Execute($sql);
	if($test_rs->fields['found'] == 0) {
		reorder($relid, $parent);
	}

	// swap with next higher or lower field

	$sql = "select weight from weights where recid = $id AND relid = $relid and parentid = $parent";
	$curr_rs = $dbh->Execute($sql);	
	$curr_oby = (integer) $curr_rs->fields['weight'];

	switch ($dir) 
    {

		case 'up':				
			$new_sql = "SELECT recid, relid, parentid, weight FROM weights WHERE weight < $curr_oby ORDER BY weight DESC";				
		break;			

		case 'down':			
			$new_sql = "SELECT recid, relid, parentid, weight FROM weights WHERE weight > $curr_oby ORDER BY weight ASC";				
		break;

	}

	$new_rs = $dbh->Execute($new_sql);

	if (!$new_rs->EOF) 
    {
		// if it found a higher or lower order by value, then it
		// should swap them.
		$sql = "REPLACE INTO weights (recid, relid, parentid, weight) VALUES (".$new_rs->fields['recid'].",".$new_rs->fields['relid'].", ".$new_rs->fields['parentid'].",$curr_oby)"; 
		#print "<br>$sql";
		
		$dbh->Execute($sql);
		$sql = "REPLACE INTO weights (recid, relid, parentid, weight) VALUES ($id, $relid, $parent, ".$new_rs->fields['weight'].")"; 
		#print "<br>$sql";
		$dbh->Execute($sql);

		return true;
	}
    else 
    {
		// first check to make sure that there aren't any duplicates
		$sql = "SELECT COUNT(*) AS count FROM weights WHERE weight = $curr_oby";
		#print "<br>$sql";
		$check_rs =& $dbh->Execute($sql);
		
		// if dup $obycol nums were found -- then re-organize
		if ($check_rs->fields['count'] > 1) 
        {

			reorder($relid, $parent);

			// and run this function again ...  RECURSIVE
			move($id, $relid, $dir);

			return true;
        }
        else 
        {
			// it didn't move the field
			return false;
		}
	} 
}

/**
* Get the next weight
* @param integer $relid
*/
function _new_oby($relid)
{
	$dbh = &pxdb::dbh();
	$sql = "select weight from weights where relid = $relid order by weight desc";
	$rs = $dbh->Execute($sql);
	return ($rs->fields['weight'] + 5);	
}


/**
* Recursive function to get sub-nodes
*/
function build_nodes(&$parent, $select='multiple', $depth=0, $edit_node_id = 0, $descendantsOfCurrentNode = false)
{
    global $user;
	$pad = ($depth * 15 + 5) . 'px';
    $noderel_count = count( array_keys( $GLOBALS['noderel_data'] ) );
    
    $num_cells = ('single' == $select) 
                 ? $noderel_count
                 : $noderel_count;    
    ob_start();
?>
    <tr>

<?php
	// ensure we can't assign a node to itself and we have privs
	if (($_REQUEST['mode'] !== 'add' && $descendantsOfCurrentNode) 
         || false === record_privs::hasPriv($parent->get_id(), $user->ugidarray, 'write'))
    {
?>
        <td class="sectNavDisabled" style="padding-left:<?= $pad ?>"><?= $parent->get_field('name') ?></td>

<?php
        for ($i = 0; $i < $num_cells; $i++)
        {
?>
        <td>&nbsp;</td>

<?php
        } // end for
?>
<!-- end sectNavDisabled -->

<?php
        if ($_REQUEST['mode'] !== 'add' && $descendantsOfCurrentNode)
        {
            $descendantsOfCurrentNode = true;
        }
    }
    else if ( $_REQUEST['mode'] !== 'add' && $parent->get_id() == $edit_node_id && 'single'==$select)
    {        
        //$widget .= ' &nbsp; &nbsp; &nbsp; &nbsp;<span style="color: red">'.$parent->get_field('name')." (current)</span>\n";
        // fix for padding
        $pad = ( $depth * 6 + 19 ) . 'px';
?>        
        <td class="currSectionNoEdit" style="padding-left:<?=$pad?>"><?=$parent->get_field('name')?> (current)</td>

<?php
        for ($i = 0; $i < $num_cells; $i++)
        {
?>
        <td class="sectNavOpt">&nbsp;</td>

<?php
        } // end for
?>
<!-- end currSectionNoEdit -->

<?php        
        $descendantsOfCurrentNode = true;
    }
    else
	{        
        if ($select == 'single')
        {
            $type = 'radio';
            $name = 'node_parent[0]';
            if ( is_array( $GLOBALS['CurrSelectedNavigation'][0] ) )
            {
               $checked = in_array($parent->get_id(), $GLOBALS['CurrSelectedNavigation'][0]) ? ' checked' : '';
            }
            else
            {
               $checked = ( $GLOBALS['CurrSelectedNavigation'][0] == $parent->get_id() ) ? ' checked' : '';           
            }
?>
        <td class="sectNavOpt" style="padding-left:<?=$pad?>"><?=$parent->get_field('name');?></td>
        <td class="sectNavOpt"><input type="<?=$type?>" name="<?=$name?>" value="<?=$parent->get_id()?>"<?= $checked ?>></td>
        <!-- end sectNavOpt -->

<?php        
        }
        else
        {
            $type = 'checkbox';
?>
        <td class="sectNavLabel" style="padding-left:<?= $pad ?>"><?= $parent->get_field('name'); ?></td>

<?php
            foreach ($GLOBALS['noderel_data'] AS $noderel_id => $noderel_label)
            {
                $name = 'node_parent['.$noderel_id.'][]';
                if ( is_array( $GLOBALS['CurrSelectedNavigation'][$noderel_id] ) )
                {
                    $checked = in_array($parent->get_id(), $GLOBALS['CurrSelectedNavigation'][$noderel_id]) ? ' checked' : '';
                }
                else
                {
                    $checked = ( $GLOBALS['CurrSelectedNavigation'][$noderel_id] == $parent->get_id() ) ? ' checked' : '';           
                }
?>
        <td class="sectNavOpt" id="section_<?= $noderel_id . '_' . $parent->get_field('id') ?>"><input type="<?=$type?>" name="<?=$name?>" value="<?=$parent->get_id()?>"<?= $checked ?>></td>

<?php
            } // end foreach
?>
<!-- end sectNavOpt -->

<?php
        } // end if not single
    } // end if mode is add
?>
    </tr>

<?php
	$children = $parent->get_children(DT_NODE, 'weight');
	// dt param restricts it to children of type NODE				
	for ($i=0, $z = sizeof($children); $i < $z; $i++)
	{
		echo build_nodes($children[$i], $select, $depth+1, $edit_node_id, $descendantsOfCurrentNode);
	}
    
    $widget = ob_get_contents();
    ob_end_clean();
	return $widget;	
}

/**
* Function to build the widget for display nodes.
* @param integer id of node to start tree from
* @param string $selection 'single' or 'multiple' -- determines whether to use radio buttons or checkboxes
* @return string The HTML form widget.
*/
function get_navigation_widget( $id, $select='multiple' )
{				
	pxdb_import('content.output.pxdb_collection');
	pxdb_import('content.output.pxdb_search');
	pxdb_import('content.output.pxdb_record');			
	        
	$dbh = &pxdb::dbh();
	
	$GLOBALS['CurrSelectedNavigation'] = array(); // populated below
	
	// get all parent IDs for current record
	if ( isset($id)  )
	{
		if ( !isset($GLOBALS['node_parent']) || !$GLOBALS['node_parent'])
		{
		    if ('single' == $select)
		    {
    			$sql = "
SELECT
    nrel.parent 
FROM
    noderel nrel
WHERE
    nrel.child = $id AND nrel.noderelid = 0
                ";		        
			    $GLOBALS['CurrSelectedNavigation'][0] = (array) $dbh->GetCol($sql);
            }
            else
            {
                foreach (array_keys($GLOBALS['noderel_data']) as $noderel_id)
                {
                    $sql = "
SELECT 	nrel.parent 
FROM 	noderel nrel
WHERE 	nrel.child = $id AND nrel.noderelid = $noderel_id";
                    $GLOBALS['CurrSelectedNavigation'][$noderel_id] = (array) $dbh->GetCol($sql);
                }
            }
		}
		else
		{
			foreach (array_keys($GLOBALS['noderel_data']) as $noderel_id)
                {
                    $sql = "
SELECT 	nrel.parent 
FROM 	noderel nrel
WHERE 	nrel.child = $id AND nrel.noderelid = $noderel_id";
                    $GLOBALS['CurrSelectedNavigation'][$noderel_id] = (array) $dbh->GetCol($sql);
                }
		}
	}    

	// get all top-level nodes ... which are nodes (DT_NODE) that aren't children of
	// any other node.

    
    if ( isset( $_SESSION['pub_node_parent'] ) ||  isset( $_REQUEST['pub_node_parent'] ) )
    { // we don't show the whole nav tree here
        $top_nodes = new pxdb_collection(DT_NODE);
        $sql = "SELECT DISTINCT r.id, r.datatype"
              ." FROM records r"
              ." WHERE r.id='".$GLOBALS['node_parent']."'";
        $top_nodes->find($sql);
    } 
    else 
    {
        // here we reuse the top nodes function from section navigator
        require_once( SITE_ROOT.'/modules/navigation/lib.php' );
        $top_nodes = get_top_nodes();        
    }
    
    
    
    
	$widget = "
<table border=\"0\" width=\"100%\">
    <tr>
        <th>Section Name</th>\n";
    if ('single' == $select)
    {
        $widget .= "
        <th>Publish to:</th>\n";
    }
    else
    {        
        foreach ($GLOBALS['noderel_data'] as $noderel_id => $noderel_label)
        {
            $widget .= "
        <th class=\"noderel_label\">$noderel_label</th>\n";
        }
    }
    $widget .= "
    </tr>\n";
    
	while( $node =& $top_nodes->fetch_record() ) 
    {	
		// loop through top-level nodes ...
        
		$widget .= build_nodes($node, $select, null, $id, false);		
	}	
	$widget .= "</table>\n";
	return $widget;
	
}


/**
 * Prepare a URI to have arguments appended.
 * Returns &amp; or ? depending on the existence of a GET argument already
 *
 * @access public
 * @return string the processed URI
 * @param string $uri The URI to be processed.
 * @param boolean $href Is the output going to an href? if so, entity encode &
 *
 */
function prep_uri_for_args($uri, $href = true)
{
    $ampersand = (true === $href) ? '&amp;' : '&';
    return (strstr($uri,'?')) ? $uri.$ampersand : '?';
}

/**
* Counts the number of unapproved records for a given datatpye
*
* @param integer, datatype id
*/
function count_unapproved( $dt )
{
    //oam: show the number of items waiting for approval in bold     
    $sql = 'SELECT COUNT( r.id ) as count_unapproved'
          .' FROM records r'          
          ." WHERE r.datatype=$dt and r.approved=0"          
    ;          
    
    $dbh = &pxdb::dbh();
    $rows= $dbh->GetAll( $sql );        
    
    return $rows[0]['count_unapproved'];
    
}

/**
* Formats a time string
*
* @param string $time Time in some random format, usually ISO
* @return string Formatted time.
*/
function apFormatTime( $time )
{
    // handle end of time
    if (END_OF_TIME == $time)
    {
        return 'Never';
    }
    else
    {
        return date( 'M j, Y', strtotime( $time ) ).'<br />'.date( 'g:i A', strtotime( $time ) );
    }
}

/**
* Returns the query string for sorting the current list view.
* @param string name of sort parameter, field to sort on
* @param string direction of sort parameter
*/
function get_sort($sort_param, $direction='ASC')
{
    // go through _REQUEST to persist any filters/search
    // because list filters use post
    if ( !isset($_GET) || empty($_GET))
    {
        $params['orderby_field'] = $sort_param;
        $params['orderby_direction'] = $direction;
    }
    else
    {
        // params to ignore
        $ignore = array('object');
        
        // oam had to use
        reset($_GET);
        while ( list($var,$value) = each($_GET))
        {
            if (!in_array($var, $ignore))
            {
                $params[$var] = urldecode($value);
            }
        }
        reset($_GET);
        $params['orderby_field'] = $sort_param;
        $params['orderby_direction'] = $direction;
    }
    
    // now build the query string
    $qs = array();
    while (list($param,$value) = each($params))
    {
        $qs[] = $param . '=' . urlencode($value);
    }
    
    $qs = join( '&amp;', $qs);
    return '?'.$qs;
}
?>
