<?php 
/**
* Class to simplify workings of the Image Manager module
*
* @author oscar merida
*/
class ImageManager 
{
    
    var $dirs;
    var $files;
    var $topDirectory;
    var $currentDirectory;
    var $thumbSize;
    var $publicDirectory;
    /**
    * @var array of handled photo extensions, defined in config.php
    * @access private
    */
    var $allowedExtensions = array ('gif', 'jpg', 'jpeg', 'png');
    
    /**
    * Sets the allowable extensions
    *
    * @access public
    * @param array of extension strings
    */
    function setAllowedExtensions($extensions)
    {
        $this->allowedExtensions = $extensions;
    }
    
    
    /**
    * Sets path to image web root
    *
    *@param string
    */
    function setPublicDir($dir)
    {
        $this->publicDirectory = $dir;        
    }
    
    /**
    * Sets the image size to trigger thumbnail making
    *
    * @param integer
    */
    function setThumbSize($size)
    {
        $this->thumbSize = $size;
    }
        
    /**
    * Sets the Toppost Directory, the root of the images folder
    *
    * @param string
    */
    function setTopDir($dir)
    {
        if (is_dir($dir))
        {
            $this->topDirectory = $dir;
            return true;
        }
        else
        {
            trigger_error($dir . ' is not a directory', E_USER_WARNING);
            return false;
        }
    }

    /**
    * Sets the Current Directory, holds the files we will list
    *
    * @param string
    */    
    function setCurrentDir($dir)
    {
        if (is_dir($dir))
        {
            $this->currentDirectory = $dir;            
            return true;
        }
        else
        {
            trigger_error($dir . ' is not a directory', E_USER_WARNING);
            return false;
        }
        
    }
    
    /**
    * Returns the current directory, false if none is set
    */
    function getCurrentDir()
    {
        if (isset($this->currentDirectory) && !empty($this->currentDirectory))
        {
            return $this->currentDirectory;
        }
        else 
        { 
            return false;
        }
    }
    
    function _read()
    {        
        if ($dir = $this->getCurrentDir())
        {
            if ($handle = opendir($dir))
            {
               /* This is the correct way to loop over the directory. */
               while (false !== ($file = readdir($handle))) 
               {
                   if (is_dir($dir . '/' . $file))
                   {
                       if ('.' == $file)
                       {
                           //do nothing
                       }
                       else if ('..' == $file && $this->currentDirectory != $this->topDirectory)  
                       {
                           // remember $file = ..
                           $here = realpath($dir . '/.');                           
                           $up_one = realpath($here . '/..');
                                                      
                           $this->_dirs[$file] = array(
                                'label'     => '.. (to parent directory)',
                                'filename'  => $here,
                                'fullpath'  => $here,
                                'localpath' => str_replace($this->topDirectory, '', $up_one).'/',
                          );
                       }
                       else if ('..' != $file)
                       {
                           $this->_dirs[$file] = array(
                                'label'     => $file,
                                'filename'  => $file,
                                'fullpath'  => $dir.'/'.$file,
                                'localpath' => str_replace($this->topDirectory, '', $dir.'/'.$file),
                          );
                           
                       }
                       
                   }
                   else if (0 !== strpos($file, '.') && is_file($dir . '/' . $file) && true === $this->fileHasAllowedExtension($file))
                   {
                       $img = getimagesize($dir.'/'.$file);
                       $localpath = str_replace($this->topDirectory, '', $dir . '/' . $file);
                       $this->_files[$file] = array(
                            'label'     => $file,
                            'filename'  => $file,
                            'filesize'  => filesize($dir . '/' . $file),
                            'fullpath'  => $dir . '/' . $file,
                            'localpath' => $localpath,                            
                            'img' => array('height' => $img[1], 'width' => $img[0], 'tags' => $img[3], 'mime' => $img['mime']),
                      );
                       
                   }
                   else 
                   {
                       //do nothing
                   }
               }
               closedir($handle);
            }
        }
        
        if (0 > count($this->_files))
        {
            asort($this->_dirs);
        }
        
        if (0 > count($this->_files))
        {
            asort($this->_files);
        }
        return true;
    }
    
    
    function getDirs()
    {
        if (empty($this->_dirs))
        {
            $this->_read();
        }
        
        return $this->_dirs;
    }
    
    function getFiles()
    {
        if (empty($this->_files))
        {
            $this->_read();            
        }
        
        return $this->_files;
        
    }
    
    function byte_format($input, $dec = 0)
    {
        $prefix_arr = array(" B", "K", "M", "G", "T");
        $value = round($input, $dec);
        $i=0;
        while ($value > 1024)
        {
            $value /= 1024;
            $i++;
        }
        $return_str = round($value, $dec) . $prefix_arr[$i];
        return $return_str;
    }
    
    function isInTopDir()
    {
        return ($this->currentDirectory == $this->topDirectory);
    }
    
    /**
    * Tests whether a file's extension is in the array of allowed extensions
    *
    * @access public
    * @return boolean
    * @param string The file name
    */
    function fileHasAllowedExtension($file)
    {
        // make the test look for alternatives at the end of the string,
        // case-insensitively
        $preg_test = '/' . join('$|', $this->allowedExtensions) . '$/i';
        return (1 === preg_match($preg_test, $file)) ? true : false;
    }
    
}
?>