<?php
// record privs
pxdb_import('record_privs');


/**
* Reorder the weights within a heirarchy
* @param integer $parent the parent ID
*/
function node_reorder($parent)
{
    $dbh =& pxdb::dbh();

    // for curr datatype
    $sql = "
SELECT     r.id
FROM     records r
INNER JOIN noderel nrel ON nrel.child = r.id
WHERE    nrel.parent = $parent AND r.datatype = " . DT_NODE . "
ORDER BY r.weight ASC, r.name ASC
    ";

    $rs = $dbh->Execute($sql);

    for ($weight = 0; !$rs->EOF; $weight += 5)
    {
        $sql = "UPDATE records SET weight = $weight WHERE id = ".$rs->fields['id'];
        $dbh->Execute($sql);
        $rs->MoveNext();
    }
    $rs->close();
}

/**
* Promote / demote node within heirarchy
* This code assumes that you do not need to check type of parent records related through noderel.
* @param integer $id
* @param string $dir 'up' / 'down'
* @param integer $parent The parent node
*/
function node_move($id, $dir, $parent)
{
    $dbh = &pxdb::dbh();

    // 0) make sure there are weights for this node
    $sql = "
SELECT COUNT(*)
FROM records r
INNER JOIN noderel nrel ON nrel.child = r.id
WHERE nrel.parent = $parent AND r.weight <> 0 AND r.datatype = " . DT_NODE;
    if (0 == $dbh->GetOne($sql))
    {
        node_reorder($parent);
    }

    // 1) check to make sure there aren't any duplicates for this part of the tree

    $sql = "
SELECT COUNT(*)
FROM records r1
INNER JOIN noderel nrel1 ON nrel1.child = r1.id
INNER JOIN records r2 ON r2.weight = r1.weight
INNER JOIN noderel nrel2 ON nrel2.child = r2.id
WHERE
    nrel1.parent = $parent
    AND nrel2.parent = $parent
    AND r1.datatype = " . DT_NODE . "
    AND r2.datatype = " . DT_NODE . "
    AND r1.id <> r2.id
    ";

    if (0 < $dbh->GetOne($sql))
    {
        node_reorder($parent);
    }

    // swap with next higher or lower field
    $sql = "
SELECT r.weight
FROM records r
INNER JOIN noderel nrel ON nrel.child = r.id
WHERE nrel.parent = $parent AND r.id = $id
    ";
    $curr_rs = $dbh->Execute($sql);
    $curr_oby = (integer) $curr_rs->fields['weight'];

    switch ($dir)
    {
        case 'up':
            $new_sql = "
SELECT r.id, r.weight
FROM records r
INNER JOIN noderel nrel ON nrel.child = r.id
WHERE
    r.weight < $curr_oby
    AND nrel.parent = $parent
    AND r.datatype = " . DT_NODE . "
ORDER BY r.weight DESC
            ";

        break;

        case 'down':

            $new_sql = "
SELECT r.id, r.weight
FROM records r
INNER JOIN noderel nrel ON nrel.child = r.id
WHERE
    r.weight > $curr_oby
    AND nrel.parent = $parent
    AND r.datatype = " . DT_NODE . "
ORDER BY r.weight ASC
            ";

        break;
    }

    $new_rs = $dbh->Execute($new_sql);

    if (!$new_rs->EOF)
    {
        // if it found a higher or lower order by value, then it
        // should swap them.
        $sql = "
UPDATE records
SET weight = $curr_oby
WHERE id = " . $new_rs->fields['id'];
        $dbh->Execute($sql);

        $sql = "
UPDATE records
SET weight = " . $new_rs->fields['weight'] . "
WHERE id = $id";
        $dbh->Execute($sql);

        return true;
    }
    else
    {
        // first check to make sure that there aren't any duplicates
        $sql = "
SELECT COUNT(*)
FROM records r
INNER JOIN noderel nrel ON nrel.child = r.id
WHERE
    r.weight = $curr_oby
    AND nrel.parent = $parent
    AND r.datatype = " . DT_NODE;

        // if dup $obycol nums were found -- then re-organize
        if (1 < $dbh->GetOne($sql))
        {
            node_reorder($parent);

            // and run this function again ...  RECURSIVE
            node_move($id, $dir, $parent);
            return true;
        }
        else
        {
            // it didn't move the field
            return false;
        }
    } // end if
}

/**
* Recursive function to get sub-nodes
*/
function build_nodes_list(&$curr, $parent, $depth = 0)
{
    global $dbh;
    global $datatype_privs;
    global $user;
    global $modified;

    $is_section = ($curr->get_field('datatype') == DT_NODE);

    if ($is_section)
    {
        $is_hidden = (0 === strpos($curr->get_field('url_id'), '_')) ? true : false;
        $is_browse_hidden = (0 === strpos($curr->get_field('crumb'), '_')) ? true : false;
		$is_link = (0 === strpos($curr->get_field('url_id'), '*64_')) ? true : false;
		$stx_view = ($is_link) ? '&amp;stx_view=links&amp;is_link=1' : '';
    }
    else
    {
        $is_hidden = TRUE;
        $is_browse_hidden = TRUE;
    }
    
    // build URL to this Section
    if ($is_section && NAV_ROOT_EL == $curr->get_field('url_id'))
    {
        $link_url = '/';
    }
    elseif ($is_link)
    {
		$link_url = pxdb_tools::decode_section_link($curr->get_field('url_id'));        
    }
    else if ( !$is_hidden )
    {
        $link_url = str_replace(NAV_ROOT_EL, '', $curr->get_field('url_path'));
        if (0 === strpos($link_url, '/'))
        {
            $link_url = '/section' . $link_url . '/';
        }
        else
        {
            $link_url = '/section/' . $link_url . '/';
        }
    }
    

    global $lace, $row_classes;
    $pad = ($depth * 17) . 'px' ;

    $widget = '<tr class="' . $row_classes[$lace++ % 2] . '">';
    if ($datatype_privs->is_allowed(DT_NODE, DATATYPE_PRIV_EDIT))
    {
        $widget .= '
    <td class="action" width="60" valign="middle">';
        if ($parent && record_privs::hasPriv($parent->get_id(), $user->ugidarray, 'write'))
        {
            $widget .= '
        <a href="' . prep_uri_for_args($_SERVER['REQUEST_URI']) . 'mode=move_down&amp;id=' . $curr->get_id() . '&amp;parent=' . $parent->get_id() . '"><img src="' . SITE_REL . '/images/gnome_icons/stock_down-16.png" border="0" title="Move down"></a>
        <a href="' . prep_uri_for_args($_SERVER['REQUEST_URI']) . 'mode=move_up&amp;id=' . $curr->get_id() . '&amp;parent=' . $parent->get_id() . '"><img src="' . SITE_REL . '/images/gnome_icons/stock_up-16.png" border="0" title="Move up"></a>
            ';
        }
    }

    $widget .= '</td>
    <td class="action" width="25">';
    if (record_privs::hasPriv($curr->get_id(), $user->ugidarray, 'write'))
    {
        $widget .= '
        <a href="' . SITE_REL . '/db.php?mode=edit&amp;dt=' . DT_NODE . $stx_view .'&amp;id=' . $curr->get_id() . '&amp;ref=' . urlencode($_SERVER['SCRIPT_URI']) . '"><img src="' . SITE_REL . '/images/gnome_icons/stock_edit-16.png" border="0" title="Edit section"></a>
        ';
    }
    $widget .= '
    </td>
    <td class="action" width="25">';

    if (record_privs::hasPriv($curr->get_id(), $user->ugidarray, 'write'))
    {
        $widget .= '
        <a href="' . SITE_REL . '/db.php?mode=add&amp;dt=' . DT_NODE . '&amp;node_parent=' . $curr->get_id() . '&amp;ref=' . urlencode($_SERVER['SCRIPT_URI']) . '">';
        
        if ( !$is_link ) 
        {
            $widget .= '<img src="' . SITE_REL . '/images/plus.gif" border="0" title="Insert section below ' . $curr->get_field('name') . '"></a>';
        }
    }
    $widget .= '
    </td>
    <td width="*" style="padding-left:' . $pad . '">' . $curr->get_field('name');
    if ($is_section && isset($link_url) )
    {
        $widget .= '&nbsp;<a href="' . $link_url . '" target="_blank"><img src="' . SITE_REL . '/images/gnome_icons/stock_insert_url-16.png" border="0" title="URL" align="middle"></a>';
    }

    $widget .= '</td>';
    if ($datatype_privs->is_allowed(DT_NODE, DATATYPE_PRIV_DELETE))
    {
        $widget .= '
    <td class="action" width="25">
        ';
        if (record_privs::hasPriv($curr->get_id(), $user->ugidarray, 'write') && false === $is_hidden)
        {
           $widget .= '
        <a href="' . SITE_REL . '/db.php?mode=del&amp;dt=' . DT_NODE . '&amp;id=' . $curr->get_id() . '&amp;ref=' . urlencode($_SERVER['SCRIPT_URI']) . '" onclick="return confirmDelete(\'Site Section\')"><img src="' . SITE_REL . '/images/gnome_icons/stock_delete-16.png" border="0" title="Delete Section"></a></td>
            ';
        }
        $widget .= '
    </td>
        ';
    }

    $widget .= "</tr>\n";

    if (!empty($_REQUEST['max_depth']) && null!==$_REQUEST['max_depth'] && $depth==$_REQUEST['max_depth'])
    {
        return $widget;
    }
    else
    {
        if($children = $curr->get_children_noderelid(NODEREL_SECTION_TO_SECTION, 'weight'))
        {
            // noderelid param restricts it to children of type NODE
            for ($i = 0, $z = count($children); $i < $z; $i++)
            {
                $widget .= build_nodes_list($children[$i], $curr, $depth+1);
            }
        }
    }

    return $widget;
}

/**
* Improved sql for finding top nodes
*
* @author Oscar Merida
*/
function &get_top_nodes()
{
    $dbh = &pxdb::dbh();
    // first find all the nodes that have a parent
    // oam: mysql will prob be faster here
    $sql = "
SELECT r.id, nrel.parent
FROM records r
LEFT JOIN noderel nrel on nrel.child = r.id
WHERE r.datatype = " . DT_NODE . " and nrel.parent IS NULL
    ";

    if (defined('DISPLAY_TOP_NODES'))
    {
        $disp_nodes = "'" . DISPLAY_TOP_NODES . "'";
        if (false !== strpos($disp_nodes, ','))
        {
            $disp_nodes = explode(',' , $disp_nodes);
            $disp_nodes = join("', '", $disp_nodes);
        }

        // only show the ones whose url_ids match above
        $sql.= "
    AND r.desc3 IN (" . $disp_nodes . ")
        ";
    }

    $root_ids = $dbh->getCol($sql);

    if (0 < count($root_ids))
    {
        //$query = $dbh->execute($sql);
        $id = join(',' , $root_ids);

        $top_nodes = new pxdb_collection(DT_NODE);

        $sql = "
SELECT  DISTINCT " . $top_nodes->sql_aliased_columns('r') . "
FROM     records r
WHERE    r.id IN (" . $id . ") AND r.datatype = " . DT_NODE . "
ORDER BY r.weight, r.name";

        $top_nodes->find($sql);

        return $top_nodes;
    }
    else
    {
        trigger_error('No top nodes found.', E_USER_WARNING);
    }

}

/**
* Returns the given parentId as a collection of sections.
*
* The collection is used to restric the top nodes shown to the user.  See list.disp.php:87
*
* @param integer section id
* @return pxdb_collection
*/
function &get_parent_node($parentId)
{
    $top_nodes = new pxdb_collection(DT_NODE);

    $sql = "
SELECT  DISTINCT r.id, r.datatype
FROM     records r
WHERE r.id = " . $parentId . "
ORDER BY r.weight, r.name
    ";

    $top_nodes->find($sql);

    return $top_nodes;
}

/**
* Old function for getting top nodes.
* @obsolete
*/
function &old_get_top_nodes()
{
    $top_nodes = new pxdb_collection(DT_NODE);

    $sql = "
SELECT  DISTINCT ".$top_nodes->sql_aliased_columns('r')."
FROM     records r
LEFT JOIN noderel nrel ON nrel.child = r.id
WHERE    nrel.parent IS NULL
    AND r.datatype = " . DT_NODE . "
ORDER BY r.weight, r.name
    ";
    $top_nodes->find($sql);
    return $top_nodes;
}

/*
* Non-recursive build node function that takes advantage of url_path for speed
*/
function build_nodes_list_opt( $current, $parent=null, $max_depth=null)
{
    $base_url_path = $current->get_field('url_path');
    
    $sql = 'SELECT r.id, r.name, r.idnum as url_id, r.title as crumb, r.desc3 as url_path'
          .' , n.parent as parent, r.weight as weight'      
          .' , paren.name as parent_name, paren.desc3 as parent_url_path'
          .' FROM records r'
          .' LEFT JOIN noderel n ON (n.child=r.id and n.noderelid='.NODEREL_SECTION_TO_SECTION.')'
          .' LEFT JOIN records paren on (paren.id=n.parent)'
          .' WHERE r.datatype='.DT_NODE
          ." and ( r.desc3 = '$base_url_path' or paren.desc3 LIKE '$base_url_path%')"
     //     .' ORDER BY r.desc3';
    ;
    
    //get a sortable array from the sql above
    $rows = _prepare_nodes($sql, $max_depth);
    
    reset($rows);
    while( list(,$row) = each($rows) )
    {
        echo dsp_node_row($row);
    }
    
}

function _prepare_nodes($sql, $max_depth=null)
{
    global $dbh;
    $query = $dbh->execute($sql);
    $rows = array();
        
    $weights = array();
    
    while( $row = $query->FetchRow() )
    {
        if ( empty($row['url_path']) )
        {
            // must be a section link - fake a urlpath so
            // that we can make it sortable
            $row['url_path'] =  $row['parent_url_path'].'/'.$row['url_id'];
        }
        
        $weights[ $row['url_path'] ] = str_pad($row['weight'], '4', '0', STR_PAD_LEFT);

        if ( null!= $max_depth )
        {
            $depth = substr_count( $row['url_path'], '/');
            if ( $depth <= $max_depth )
            {
                $rows[] = $row;
            }
        }
        else
        {
            $rows[] = $row;
        }
    }
    
    $sortable = array();
    while( list(,$row) = each($rows) )
    {
        $index = _calc_weight( $row, $weights );
        $sortable[ $index ] = $row;
    }
    ksort($sortable);
    return $sortable;
}

function _calc_weight( $row, $weights )
{
    $path_parts = explode('/', $row['url_path'] );
        
    $weight = array();
    $root ='';
    while( list(, $path) = each( $path_parts ) )
    {
        $root.= $path;
        if ( isset($weights[$root]) )
        {
            $weight[] =$weights[$root];
        }
        else
        {
            $weight[] = '0000';
        }
        $root.='/';
    }
    reset($path_parts);
    $max = count($path_parts);
    $index = array();
    for( $i=0; $i<$max; $i++ )
    {
        $index[] = $weight[$i].','.$path_parts[$i];
    }
    $index = join('/', $index);
    return $index;
    
}

function dsp_node_row( $row )
{
    global $datatype_privs;
    global $user;
    global $modified;
    
    $is_hidden = (0 === strpos($row['url_id'], '_')) ? true : false;
    $is_browse_hidden = (0 === strpos($row['crumb'], '_')) ? true : false;
    $is_link = (0 === strpos($row['url_id'], '*64_')) ? true : false;
    $stx_view = ($is_link) ? '&amp;stx_view=links&amp;is_link=1' : '';

    
    // build URL to this Section
    if (NAV_ROOT_EL == $row['url_id'] )
    {
        $link_url = '/';
    }
    elseif ($is_link)
    {
		$link_url = pxdb_tools::decode_section_link($row['url_id']);        
    }
    else if ( !$is_hidden )
    {
        $link_url = str_replace(NAV_ROOT_EL, '', $row['url_path']);
        if (0 === strpos($link_url, '/'))
        {
            $link_url = '/section' . $link_url . '/';
        }
        else
        {
            $link_url = '/section/' . $link_url . '/';
        }
    }
    
    //echo '<pre>'; print_r( $section ); echo '</pre>';
    ob_start();
    
    global $lace, $row_classes;
    $depth = substr_count( $row['url_path'], '/');    
    $pad = ($depth * 17 + 5) . 'px' ;
    
    $row_class = $row_classes[$lace++ % 2];
    
    // this lets us set two styles and if the second doesn't exist it'll cascade to just tr.on tr.off
    // ie class="on depth_1_on"
    $row_class = $row_class.' '.$row_class.'_depth_'.$depth;
?>
    <tr class="<?= $row_class ?>">
<?php     
    if ($datatype_privs->is_allowed(DT_NODE, DATATYPE_PRIV_EDIT))
    {
?>                
    <td class="action" width="60" valign="middle">
<?php     
        if ($row['parent'] && record_privs::hasPriv($row['parent'], $user->ugidarray, 'write'))
        {
?>                        
        <a href="<?= prep_uri_for_args($_SERVER['REQUEST_URI']) ?>mode=move_down&amp;id=<?= $row['id'] ?>&amp;parent=<?= $row['parent'] ?>"><img src="<?= SITE_REL ?>/images/gnome_icons/stock_down-16.png" border="0" title="Move down"></a>
        <a href="<?= prep_uri_for_args($_SERVER['REQUEST_URI']) ?>mode=move_up&amp;id=<?= $row['id'] ?>&amp;parent=<?= $row['parent'] ?>"><img src="<?= SITE_REL ?>/images/gnome_icons/stock_up-16.png" border="0" title="Move up"></a>
<?php             
        }
    }
?>
    </td>
    <td class="action" width="25">
<?php     
    if (record_privs::hasPriv($row['id'], $user->ugidarray, 'write'))
    {
?>        
        <a href="<?= SITE_REL ?>/db.php?mode=edit&amp;dt=<?= DT_NODE . $stx_view ?>&amp;id=<?= $row['id'] ?>&amp;ref=<?= urlencode($_SERVER['SCRIPT_URI']) ?>"><img src="<?= SITE_REL ?>/images/gnome_icons/stock_edit-16.png" border="0" title="Edit section"></a>
<?php         
    }
?>        
    </td>
<?php
    // build pull down options
    $section_actions = array();
    
    // create a subsection
    if (!$is_link && record_privs::hasPriv($row['id'], $user->ugidarray, 'write'))
    {
        $section_actions[] = array('url'=> SITE_REL . '/db.php?mode=add&amp;dt=' . DT_NODE . '&amp;node_parent=' 
                                    . $row['id'] . '&amp;ref=' . urlencode($_SERVER['SCRIPT_URI'])
                                  , 'desc' => 'Insert section below ' .$row['name']
                                  , 'icon' => SITE_REL . '/images/gnome_icons/stock_attach-16.png');
    }
    
    // work here option
    $section_actions[] = array( 'url' => '?mode=set_parent_node&amp;pub_node_parent=' . $row['id']
                              , 'desc' => 'Show only this section and below'
                              , 'icon' => SITE_REL . '/images/gnome_icons/stock_open-16.png' );
    
?>
    <td width="*" style="padding-left:<?= $pad ?>"><?= $row['name'] ?></td>
    <td width="150" class="section-tools">
<?php

// view live page, if its not hidden
    if ( isset( $link_url ) )
    {
        if ( !$is_link )
        {
            $link_url = 'http://' . $_SERVER['SERVER_NAME'] . $link_url;
        }
?>
        <a href="<?= $link_url ?>"><img src="<?= SITE_REL ?>/images/gnome_icons/stock_insert_url-16.png" border="0"  title="View live page"/></a>

<?php
    }

    while ( list(,$option) = each( $section_actions) )
    {
?>
           &nbsp;<a href="<?= $option['url'] ?>"><img src="<?= $option['icon'] ?>" border="0" title="<?= $option['desc'] ?>" /></a>
<?php
    }
?>
    </td>
<?php 
    if ($datatype_privs->is_allowed(DT_NODE, DATATYPE_PRIV_DELETE))
    {
?>
    <td class="action" width="25">
<?php
        if (record_privs::hasPriv($row['id'], $user->ugidarray, 'write') && false === $is_hidden)
        {
?>
        <a href="<?= SITE_REL ?>/db.php?mode=del&amp;dt=<?= DT_NODE ?>&amp;id=<?= $row['id'] ?>&amp;ref=<?= urlencode($_SERVER['SCRIPT_URI']) ?>" onclick="return confirmDelete('<?= $row['name'] ?>')"><img src="<?= SITE_REL ?>/images/gnome_icons/stock_delete-16.png" border="0" title="Delete Section"></a></td>
<?php 
        }
?>
    </td>
<?php 
    }    

    $c = ob_get_contents();
    ob_end_clean();
    
    return $c;
    
    
}
?>